unit UAddTabs;

interface

uses
  Windows, Classes, SysUtils;

const
  SearchLines =
    '(^ *Galaxy Game \w+ \w+ \w+ \w+)|(^ *Status of Players)|(^ *Your Ship Types)|' +
    '(^ *\w+ +Ship Types)|(^ *Battle at \w+)|' +
    '(^ *Your Routes)|' +
    '(^ *\w+ +Groups)|(^ *Bombings)|(^ *Unidentified Planets)|' +
    '(^ *\w+ +Planets)|(^ *Uninhabited Planets)|(^ *Fleet)|' +

    '(^ *Your Systems)|' +
    '(^ *Blind Galaxy game \w+ Report for \w+)|' +
    '(^ *Ships in Production)|' +
    '(^ *Uninhabited Systems)|' +
    '(^ *Options:)|' +
    '(^ *Unknown Systems)|' +
    '(^ *\w+ +Systems)|(^ *GalaxyNG-Blind Game \w+ \w+ \w+ \w+)|' +

    '(^ *Ships In Production)|(^ *Bulletins for Galaxy Game)|(^ *Your Options)|' +
    '(^ *Your Planets)|(^ *Your Groups)|(^ *Incoming Groups)';

procedure AddTabsToReportFile(strCurReportName: string);
procedure ComponentToFile(const component: TComponent; const FileName: string);
procedure ComponentFromFile(const component: TComponent; const FileName: string);

implementation

uses
  mkRegEx, HyperStr, Dialogs, StabUtil;

const
  MAX_MKGAL_TAB_PATTERNS = 0;

procedure AddTabsToReportFile(strCurReportName: string);
var
  report: TStringList;
  mkreExpr: TmkreExpr;
  FileStr: string;
  i, j: integer;
begin
  if not FileExists(strCurReportName) then Exit;
  CopyFile(PChar(strCurReportName), PChar(strCurReportName + '.TabBackup'), True);
  mkreExpr := TmkreExpr.Create(nil);
  report := TstringList.Create;
  try
    MakeFileReadOnly(strCurReportName, False);
    report.LoadFromFile(strCurReportName);
    FileStr := report.GetText;
    ReplaceC(FileStr, #$A0, #$20);
    report.SetText(PChar(FileStr));
    mkreExpr.Pattern := SearchLines;
    for i := 0 to report.Count - 1 do begin
      mkreExpr.Str := report[i];
      if mkreExpr.DoSearch(1) > 0 then
        report[i] := #9#9 + Trim(report[i]);
    end;
    try
      report.SaveToFile(strCurReportName);
    except
      ShowMessage(Format('I was not able to save %s to disk, please check if it is write protected, read only or if the disk is full', [strCurReportName]));
    end;
  finally
    report.Free;
    mkreExpr.Free;
  end;
end;

procedure ComponentToFile(const component: TComponent; const FileName: string);
var
  FileStream: TFileStream;
begin
  try
    FileStream := nil;
    try
      FileStream := TFileStream.Create(FileName, fmCreate);
      FileStream.WriteComponent(component);
    finally
      FileStream.Free;
    end;
  except
    ;
  end;
end;

procedure ComponentFromFile(const component: TComponent; const FileName: string);
var
  FileStream: TFileStream;
begin
  if FileExists(FileName) then
  try
    FileStream := nil;
    try
      FileStream := TFileStream.Create(FileName, fmOpenRead);
      FileStream.ReadComponent(component);
    finally
      FileStream.Free;
    end;
  except
    DeleteFile(FileName);
  end;
end;

end.
