unit UBatSim2;

interface

uses Classes, SysUtils;

const
  MAX_REPEAT = 10;

type
  TBatGroup = class;
  TBatGroup = class
    next: TBatGroup;
    number: integer;
    temporary: integer;
    number_survived: integer;
    max_number_survived: integer;
    min_number_survived: integer;
    attacks: integer;
    weapons: Single;
    shield: single;
    drive_tech: Single;
    weapons_tech: Single;
    shield_tech: Single;
    cargo_load: Single;
    eff_shield: Single;
    eff_weapons: Single;
    eff_range: Single;
    total_mass: single;
      name: string;
  end;

  PLongIntArray = ^TLongIntArray;
  TLongIntArray = array[0..0] of LongInt;
  PBatGroupArray = ^TBatGroupArray;
  TBatGroupArray = array[0..0] of TBatGroup;

  TBatGroupArray2 = array of TBatGroup;

  PBatPlayer = ^TBatPlayer;
  TBatPlayer = record
    groups: TBatGroup;
    group_array: PBatGroupArray;
    bat_array: PLongIntArray;
    number_of_ships: integer;
      number_of_armed_ships: integer;
      selected_armed_ship: integer;
      bat_array_size: integer;
      cur_bat_array_size: integer;
      group_array_size: integer;
  end;

  TBatSim = class(TThread)
  private
    function Load_Fleet_Information(var p: TBatPlayer; groups: TList): boolean;

    function Alloc_Bat_array(var the_groups: TBatGroup; var size: integer): PLongIntArray;
    function Alloc_Group_array(var the_groups: TBatGroup; var size: integer): PBatGroupArray;
    function Init_Bat_array({var } bat_array: PLongIntArray; var the_groups: TBatGroup): integer;
    procedure Randomize_Armed_Ships(bat_array: PLongIntArray; number_of_armed_ships: integer);

    procedure Probe(var p1, p2: TBatPlayer);
    function Do_Battle(var p1, p2: TBatPlayer): integer;
    procedure Fight(var p1, p2: TBatPlayer; var al, sl: integer);
    function Select_a_Ship(var p: TBatPlayer): integer;
    function Select_Armed_Ship(var p: TBatPlayer): integer;
    function Max_Weapons(var p: TBatPlayer): TBatGroup;
    function Min_Shield(var p: TBatPlayer): TBatGroup;
    function Is_A_Draw(var p1, p2: TBatPlayer): boolean;
    function Can_Destroy(g1, g2: TBatGroup): boolean;

    procedure Init_Survival_Count(cur_Group: TBatGroup);
    procedure Count_Survivors(var p: TBatPlayer);
    procedure Print_Survival_Count(cur_group: TBatGroup);

    procedure FlushOutPut;
//    procedure FreePlayer(p: TBatPlayer);
  public
    verb: boolean;
    debug: boolean;
    max_battle: integer;

    p1, p2: TBatPlayer;
    al1, al2, sl1, sl2: integer;

    OutPut: TStringList;
    groupA,
      groupD: TList;
    RoundResults: TStrings;
    constructor Create;
    destructor Destroy; override;
    procedure Execute; override;
  end;

var
  BattleSim: TBatSim;

implementation

uses UTypes, Math, Dialogs, Controls;

function My_log(x: single): single;
begin
  if (x <= 0) then raise Exception.CreateFmt('ERROR: My_Log(%f)', [x]);
  Result := ln(x);
end;

function My_log2(x: single): single;
begin
  if (x <= 0) then result := -1
  else Result := ln(x);
end;


constructor TBatSim.Create;
begin
  inherited Create(True);
  OutPut := TStringList.Create;
end;

destructor TBatSim.Destroy;
begin
  inherited Destroy;
end;

{procedure TBatSim.FreePlayer(p: TBatPlayer);
var
  groupold, group: TBatGroup;
begin
  groupold := p.groups;
  while groupold <> nil do
  begin
    group := groupold.next;
    groupold.Free;
    groupold := group;
  end;
  if p.group_array <> nil then Freemem(p.group_array);
  if p.bat_array <> nil then Freemem(p.bat_array);
end;}

procedure TBatSim.FlushOutPut;
begin
  RoundResults.AddStrings(Output);
  Output.Clear;
end;

procedure TBatSim.Execute;
begin
  p1.groups := nil;
  p1.group_array := nil;
  p1.bat_array := nil;
  p2.groups := nil;
  p2.group_array := nil;
  p2.bat_array := nil;
  if Load_Fleet_Information(p1, groupA) and
    Load_Fleet_Information(p2, groupD) then
  begin
    Probe(p1, p2);
  end;

  Synchronize(FlushOutPut);
end;

function TBatSim.Load_Fleet_Information(var p: TBatPlayer; groups: TList): boolean;
var
  armed_groups,
    unarmed_groups,
    cur_group: TbatGroup;
  i: integer;
  group: TGroup;
begin
  armed_groups := nil;
  unarmed_groups := nil;
  output.Add('');
  output.Add('==== Loading Player Data ====');
  for i := 0 to groups.Count - 1 do
  begin
    group := TGroup(groups.items[i]);
    cur_group := TbatGroup.Create;
    cur_group.number := group.Aantal;
    cur_group.attacks := group._Type.Attacks;
    cur_group.weapons := group._Type.Weapons;
    cur_group.shield := group._Type.Shields;
    cur_group.drive_tech := group.DriveTech;
    cur_group.weapons_tech := group.WeaponsTech;
    cur_group.shield_tech := group.ShieldsTech;
    cur_group.cargo_load := group.Lading;
    cur_group.eff_shield := group.GetEffShieldWithLoad(group.Lading);
    cur_group.eff_weapons := group.EffWeapon;
    cur_group.eff_range := group.EffSpeedNow;
    cur_group.total_mass := group._Type.Mass + group.Lading;
    cur_group.name := group._type.name;
    if cur_group.attacks > 0 then
    begin
      cur_group.next := armed_groups;
      armed_groups := cur_group;
    end
    else
    begin
      cur_group.next := unarmed_groups;
      unarmed_groups := cur_group;
    end;
  end;
  if armed_groups <> nil then
  begin
    cur_group := armed_groups;
    while cur_group.next <> nil do cur_group := cur_group.next;
    cur_group.next := unarmed_groups;
    p.groups := armed_groups;
  end
  else
    p.groups := unarmed_groups;
  p.bat_array := Alloc_Bat_array(p.groups, p.bat_array_size);
  p.group_array := Alloc_Group_array(p.groups, p.group_array_size);
  Result := True;
end;

function TBatSim.Alloc_Bat_array(var the_groups: TBatGroup; var size: integer): PLongIntArray;
var
  cur_group: TBatGroup;
  number_of_ships: integer;
begin
  Result := nil;
  output.Add('-- Allocating Battle Array...');
  cur_group := the_groups;
  number_of_ships := 0;
  while cur_group <> nil do
  begin
    inc(number_of_ships, cur_group.number);
    cur_group := cur_group.next;
  end;
  output.Add(format('-- Total number of ships %d.', [number_of_ships]));
  size := number_of_ships + 1;
  Reallocmem(Result, size * sizeof(integer));
end;

function TBatSim.Alloc_Group_array(var the_groups: TBatGroup; var size: integer): PBatGroupArray;
var
  cur_group: TBatGroup;
  cur_elem: integer;
  number_of_groups: integer;
begin
  Result := nil;
  output.Add('-- Allocating Group Array...');
  cur_group := the_groups;
  number_of_groups := 0;
  while cur_group <> nil do
  begin
    inc(number_of_groups);
    cur_group := cur_group.next;
  end;
  output.Add(format('-- Total number of groups %d.', [number_of_groups]));
  size := number_of_groups + 1;
  Reallocmem(Result, size * sizeof(pointer));
  if Result <> nil then
  begin
    cur_group := the_groups;
    Result[0] := nil;
    cur_elem := 1;
    while cur_group <> nil do
    begin
      Result[cur_elem] := cur_group;
      inc(cur_elem);
      cur_group := cur_group.next;
    end;
  end;
end;

function TBatSim.Init_Bat_array({var } bat_array: PLongIntArray; var the_groups: TBatGroup): integer;
var
  cur_group: TBatGroup;
  cur_group_id,
    number_of_armed_ships,
    number_of_ships, cur_ship, i: integer;
begin
  if debug then output.Add('-- Initializing Batlle Array.');
  cur_group := the_groups;
  cur_group_id := 1;
  cur_ship := 1;
  bat_array[0] := 0;
  number_of_armed_ships := 0;
  while cur_group <> nil do
  begin
    number_of_ships := cur_group.number;
    if cur_group.attacks > 0 then inc(number_of_armed_ships, number_of_ships);

    for i := 0 to number_of_ships - 1 do
      bat_array[cur_ship + i] := cur_group_id;
    inc(cur_ship, number_of_ships);
    cur_group := cur_group.next;
    inc(cur_group_id);
  end;
  Result := number_of_armed_ships;
end;

procedure TBatSim.Randomize_Armed_Ships(bat_array: PLongIntArray; number_of_armed_ships: integer);
var
  i, ship1, ship2, temp: integer;
begin
  if debug then
  begin
    output.Add('-- Randomizing Armed Ships.');
    output.Add(Format('   Number of Armed Ships %d', [number_of_armed_ships]));
  end;
  for i := 0 to number_of_armed_ships - 1 do
  begin
    ship1 := random(number_of_armed_ships) + 1;
    ship2 := random(number_of_armed_ships) + 1;
    temp := bat_array[ship1];
    bat_array[ship1] := bat_array[ship2];
    bat_array[ship2] := temp;
  end;
  if debug then output.Add('   Ready...');
end;

(****** WOPR.c/Probe [1.0a] *
*
*  NAME
*    Probe -- Compute the win-chance of player 1 (Attacker)
*  SYNOPSIS
*    result = int Probe (struct player *P1, struct player *P2)
*  FUNCTION
*    Figth max_battle battles and use the results to compute
*    the chance of winning for player 1.
*  INPUTS
*    P1  -- player structure of player 1
*    P2  -- player structure of player 2 (Defender)
*  BUGS
*
*  SEE ALSO
*
**********
*)

procedure TBatSim.Probe(var p1, p2: TBatPlayer);
var
  battle, w_p1, w_p2: integer;
  res: integer;
begin
  Output.Add('');
  Output.Add('');
  Output.Add('====== Battle Simulation =====');
  Init_Survival_Count(p1.groups);
  Init_Survival_Count(p2.groups);
  Synchronize(FlushOutPut);
  w_p1 := 0;
  w_p2 := 0;
  battle := 0;
  while battle < max_battle do
  begin
    if (battle and 1) <> 0 then
      res := Do_Battle(p1, p2)
    else
    begin
      res := Do_Battle(p2, p1);
      if res = 1 then res := 2
      else
        if res = 2 then res := 1;
    end;
    case res of
      0: if verb then output.Add('Draw');
      1: begin
          inc(w_p1);
          if verb then output.Add('Attacker Wins.');
        end;
      2: begin
          inc(w_p2);
          if verb then output.Add('Defender Wins.');
        end;
      3: begin
          if verb then output.Add('Battle stopped.');
        end;
    end;
    if res <> 3 then
    begin
      Count_Survivors(p1);
      Count_Survivors(p2);
      inc(battle);
    end;
    if verb then Synchronize(FlushOutPut);
    if terminated then
      max_battle := battle;
  end;
  output.Add('');
  output.Add('');
  output.Add('-- Ready...');
  output.Add('');
  output.Add('');
  output.Add('======== Battle Report =======');
  output.Add('Battles fought: ' + IntToStr(max_battle));
  output.Add(Format('Attacker wins %d battles', [w_P1]));
  output.Add(Format('Defender wins %d battles', [w_P2]));
  if (w_p1 + w_p2) > 0 then
    output.Add(Format('Attacker has a %f %1s chance to win.',
      [((100 * w_p1) / max_battle), #37]))
  else
    output.Add('Game is a draw.');
  output.Add('');
  output.Add('== Attacker Survival Count ==');
  Print_Survival_Count(p1.groups);
  output.Add('');
  output.Add('== Defender Survival Count ==');
  Print_Survival_Count(p2.groups);
end;


(****** WOPR.c/Do_Battle [1.0a] *
*
*  NAME
*    Do_Battle -- fight a battle between players.
*  SYNOPSIS
*    result = int Do_Battle (struct player *P1,struct player *P2)
*  FUNCTION
*
*    Algorithm:
*      Repeat until there either players hasn't any ships left or
*             the game is a draw.
*      {
*        Randomly select a ship, S1, of player 1 (Attacker) that hasn't fired yet
*        For all the attacks of S1
*        {
*          Select a ship S2 of player 2 (Defender) and fire on it
*        }
*        Randomly select a ship, S2, of player 2 that hasn't fired yet
*        For all the attacks of S2
*        {
*          Select a ship S1 of player 1 and fire on it
*        }
*      }
*
*    To adjust for the fact that Player 1 always starts firing, the
*    players are swaped randomly in the functions that calls this
*    function.
*  INPUTS
*    P1  -- player structure of player 1
*    P2  -- player structure of player 2 (Defender)
*  RESULT
*    0  -- Battle is a draw.
*    1  -- Player 1 wins.
*    2  -- Player 2 wins.
*  BUGS
*
*  SEE ALSO
*
**********
*)

function TBatSim.Do_Battle(var p1, p2: TBatPlayer): integer;
var
  stop_battle,
    draw: boolean;
  round: integer;
begin
  round := 1;
  p1.number_of_Armed_Ships := Init_Bat_array(p1.bat_array, p1.groups);
  p2.number_of_Armed_Ships := Init_Bat_array(p2.bat_array, p2.groups);
  if (p1.number_of_Armed_Ships > 0) and (p2.number_of_armed_ships > 0) then
  begin
    Randomize_Armed_Ships(p1.bat_array, p1.number_of_armed_ships);
    Randomize_Armed_Ships(p2.bat_array, p2.number_of_armed_ships);
    p1.cur_bat_array_size := p1.bat_array_size;
    p2.cur_bat_array_size := p2.bat_array_size;

    if debug then Output.Add('-- Fighting');
    sl1 := 1;
    sl2 := 1;
    al1 := 1;
    al2 := 1;
    draw := False;
    while (sl1 > 0) and (sl2 > 0) and not draw do
    begin
      p1.selected_armed_ship := 1;
      p2.selected_armed_ship := 1;
      stop_battle := False;
      while not stop_battle do
      begin
        Fight(p1, p2, al1, sl2);
        Fight(p2, p1, al2, sl1);
        if (sl1 = 0) or (sl2 = 0) then stop_battle := True;
        if (al1 = 0) and (al2 = 0) then stop_battle := True;
      end;
      if (sl1 <> 0) and (sl2 <> 0) then draw := Is_A_Draw(p1, p2);
      if debug then Synchronize(FlushOutput);
      inc(round);
      if round = 10000 then
      begin
        if MessageDlg('10000 rounds. Continue with this battle?', mtConfirmation, [mbYes, mbNo], 0) = mrNo then
        begin
          Result := 3;
          Exit;
        end;
        round := 1;
      end;
    end;
    if (sl1 <> 0) and (sl2 <> 0) then Result := 0
    else
      if (sl1 <> 0) then Result := 1
    else Result := 2;
  end
  else
  begin
    Output.Add('One of the players has no armed ships!');
    if (p1.number_of_armed_ships = 0) and (p2.number_of_armed_ships = 0) then
      Result := 0
    else
      if p1.number_of_armed_ships > 0 then Result := 1
    else Result := 2;
  end;
end;

(****** WOPR.c/Fight [1.0a] *
*
*  NAME
*    Fight -- fight a battle between ships
*  SYNOPSIS
*    Fight (struct player *P1, struct player *P2, int *al, int *sl)
*  FUNCTION
*    Select a armed ship of player one.
*    for each of its attack select a ship of player two and fire on it.
*  INPUTS
*    P1  -- player structure of player 1
*    P2  -- player structure of player 2 (Defender)
*  RESULT
*    al -- flag: attacking ships left
*    sl -- flag: ships left
*  BUGS
*
*  SEE ALSO
*
**********
*)

procedure TBatSim.Fight(var p1, p2: TBatPlayer; var al, sl: integer);
var
  att_ship, def_ship: integer;
  attack: integer;
  att_group,
    def_group: TBatGroup;
  P_Kill: single;
  sLine: string;
begin
  att_ship := Select_Armed_Ship(p1);
  al := att_ship;
  if al <> 0 then
  begin
    att_group := p1.group_array[p1.bat_array[att_ship]];
    attack := att_group.attacks;
    sl := 1;
    while (attack > 0) and (sl <> 0) do
    begin
      def_ship := Select_a_Ship(p2);
      sl := def_ship;
      if sl > 0 then
      begin
        def_group := p2.group_array[p2.bat_array[def_ship]];
        if verb then sLine := format('%20s [%3d] attacks %-20s',
            [att_group.name, attack, def_group.name]);
        if att_group.eff_weapons = 0 then output.Add('ERROR Attacker without weapons!!!');
        if def_group.eff_shield > 0 then
        begin
          P_Kill := ((my_log(att_group.eff_weapons / def_group.eff_shield) / 1.386294) + 1) / 2; // 3.1072325 is al inbegrepen in effective shield
          if P_Kill >= 1 then
          begin
            p2.bat_array[def_ship] := 0;
            if verb then output.add(sLine + ' -- Destroyed');
          end
          else
            if P_Kill > 0 then
          begin
            if Random < P_Kill then
            begin
              p2.bat_array[def_ship] := 0;
              if verb then output.add(sLine + ' -- Destroyed');
            end
            else
              if verb then output.add(sLine + ' -- Survived');
          end
          else
            if verb then output.add(sLine + ' -- Survived');
        end
        else
        begin
          p2.bat_array[def_ship] := 0;
          if verb then output.add(sLine + ' -- Destroyed');
        end;
      end;
      dec(attack);
    end;
  end;
end;

(****** WOPR.c/Select_a_Ship [1.0a] *
*
*  NAME
*    Select_a_Ship -- randomly select a ship.
*  SYNOPSIS
*    int Select_a_Ship (struct player *P)
*  FUNCTION
*    Randomly selects a non zero entry in the battle array.
*  INPUTS
*    P -- player structure
*  RESULT
*    0 -- no ships left, otherwise index number of the ship.
*  BUGS
*  SEE ALSO
*
**********
*)

function TBatSim.Select_a_Ship(var p: TBatPlayer): integer;
var
  rep, i: integer;
  old_ship, new_ship, cur_ship,
    new_number_of_armed_ships,
    new_selected_armed_ship: integer;
  sLine: string;
begin
  if p.cur_bat_array_size = 0 then
    raise Exception.Create('ERROR: internal error -- bat_array_size == 0');
  if p.cur_bat_array_size = 1 then Result := 0
  else
  begin
    rep := 0;
    result := 0;
    while (rep < MAX_REPEAT) and (p.bat_array[Result] = 0) do
    begin
      Result := random(p.cur_bat_array_size - 1) + 1;
      inc(rep);
    end;
    if p.bat_array[Result] = 0 then
    begin
      if debug then
      begin
        output.Add('Could not locate a ship, compacting battle array');
        output.Add(Format('Cur battle array size %d', [p.cur_bat_array_size]));
        for i := 0 to p.cur_bat_array_size - 1 do
          sLine := sLine + format('[%d]', [p.bat_array[i]]);
        output.Add(sLine);
      end;
      cur_ship := 1;
      new_number_of_armed_ships := 0;
      while cur_ship <= p.number_of_armed_ships do
      begin
        if p.bat_array[cur_ship] <> 0 then inc(new_number_of_armed_ships);
        inc(cur_ship);
      end;
      old_ship := 1;
      new_ship := 0;
      new_selected_armed_ship := P.selected_armed_ship;
      while old_ship < p.cur_bat_array_size do
      begin
        if p.bat_array[old_ship] <> 0 then
        begin
          inc(new_ship);
          p.bat_array[new_ship] := p.bat_array[old_ship];
        end
        else
          if old_ship < p.selected_armed_ship then
          dec(new_selected_armed_ship);
        inc(old_ship);
      end;
      p.cur_bat_array_size := new_ship + 1;
      p.number_of_armed_ships := new_number_of_armed_ships;
      p.selected_armed_ship := new_selected_armed_ship;
      if debug then
        output.Add(Format('New battle array size %d', [p.cur_bat_array_size]));
      if p.cur_bat_array_size = 1 then Result := 0
      else
      begin
        Result := random(p.cur_bat_array_size - 1) + 1;
        if p.bat_array[Result] = 0 then
        begin
          output.Add('ERROR: Internal error -- cant find ship after comp.');
          output.Add(Format(' ship = %d', [Result]));
          for i := 0 to new_ship - 1 do
            sLine := sLine + format('[%d]', [p.bat_array[i]]);
          output.Add(sLine);
          Result := 0;
        end;
      end;
    end;
  end;

end;

(****** WOPR.c/Select_Armed_Ship [1.0a] *
*
*  NAME
*    Select_Armed_Ship -- select an armed ship.
*  SYNOPSIS
*    ship = int Select_Armed_Ship (struct player *P)
*  FUNCTION
*    Select the next ship that hasn't fired yet, skip the ships
*    that have been destroyed.
*  INPUTS
*    P  -- player structure of a player
*  RESULT
*    ship -- selected ship,  0 if there are none left.
*  BUGS
*
*  SEE ALSO
*
**********
*)

function TBatSim.Select_Armed_Ship(var p: TBatPlayer): integer;
begin
  Result := p.selected_armed_ship;
  while (result <= p.number_of_armed_ships) and
    (p.bat_array[result] = 0) do inc(result);
  if Result <= p.number_of_armed_ships then
    p.selected_armed_ship := Result + 1
  else
    Result := 0;
end;

function TBatSim.Max_Weapons(var p: TBatPlayer): TBatGroup;
var
  cur_ship: integer;
  max_weapons: single;
  cur_group: TBatGroup;
  index: integer;
begin
  cur_ship := 1;
  max_weapons := 0;
  Result := nil;
  while cur_ship < p.cur_bat_array_size do
  begin
    index := p.bat_array[cur_ship];
    if index > 0 then
    begin
      cur_group := p.group_array[index];
      if cur_group.eff_weapons >= max_weapons then
      begin
        max_weapons := cur_group.eff_weapons;
        Result := cur_group;
      end;
    end;
    inc(cur_ship);
  end;
end;

function TBatSim.Min_Shield(var p: TBatPlayer): TBatGroup;
var
  cur_ship: integer;
  min_shield: single;
  cur_group: TBatGroup;
  index: integer;
begin
  cur_ship := 1;
  min_shield := 999999.99;
  Result := nil;
  while cur_ship < p.cur_bat_array_size do
  begin
    index := p.bat_array[cur_ship];
    if index > 0 then
    begin
      cur_group := p.group_array[index];
      if cur_group.eff_shield <= min_shield then
      begin
        min_shield := cur_group.eff_shield;
        Result := cur_group;
      end;
    end;
    inc(cur_ship);
  end;
end;

(****** WOPR.c/Is_A_Draw [1.0a] *
*
*  NAME
*    Is_A_Draw -- Test if situation is a draw.
*  SYNOPSIS
*    result = int Is_A_Draw (struct player *P1, struct player *P2)
*  FUNCTION
*    Test if either of the players can't destroy the other:
*  INPUTS
*    P1  -- player structure of player 1
*    P2  -- player structure of player 2 (Defender)
*  RESULT
*    TRUE or FALSE.
*  BUGS
*
*  SEE ALSO
*
**********
*)

function TBatSim.Is_A_Draw(var p1, p2: TBatPlayer): boolean;
var
  G1mw, G2mw, G1ms, G2ms: TBatGroup;
  sLine: string;
begin
  Result := False;
  G1mw := Max_Weapons(p1);
  G2mw := Max_Weapons(p2);
  G1ms := Min_Shield(p1);
  G2ms := Min_Shield(p2);
  if (G1mw <> nil) and (G2mw <> nil) and (G1ms <> nil) and (G2ms <> nil) then
  begin
    if debug then
    begin
      output.Add('G1mw ' + G1mw.name);
      output.Add('G1ms ' + G1ms.name);
      output.Add('G2mw ' + G2mw.name);
      output.Add('G2ms ' + G2ms.name);
      sLine := 'G1mw > G2ms ';
      if Can_Destroy(G1mw, G2ms) then sLine := sLine + 'True'
      else sLine := sLine + 'False';
      output.Add(sLine);
      sLine := 'G2mw > G1ms ';
      if Can_Destroy(G2mw, G1ms) then sLine := sLine + 'True'
      else sLine := sLine + 'False';
      output.Add(sLine);
    end;
    if not Can_Destroy(G1mw, G2ms) and not Can_Destroy(G1mw, G2mw) and
      not Can_Destroy(G2mw, G1mw) then Result := True
    else
      if not Can_Destroy(G2mw, G1ms) and not Can_Destroy(G2mw, G1mw) and
      not Can_Destroy(G1mw, G2mw) then Result := True
  end;
end;

(****** WOPR.c/Can_Destroy [1.0a] *
*
*  NAME
*    Can_Destroy -- Test whether one ship can destroy another.
*  SYNOPSIS
*    int Can_Destroy (struct group *g1, struct group *g2)
*  FUNCTION
*    Test whether one type of ship can destroy another type of ship.
*    this that: is P(kill) > 0
*  INPUTS
*    g1  -- the group the attacking ship belongs to.
*    g2  -- the group the defending ship belongs to.
*  RESULT
*    TRUE or FALSE.
*  BUGS
*
*  SEE ALSO
*
**********
*)

function TBatSim.Can_Destroy(g1, g2: TBatGroup): boolean;
begin
  if (g2.eff_shield = 0) and (g1.eff_weapons > 0) then Result := True
  else
    if (g2.eff_shield = 0) and (g1.eff_weapons = 0) then Result := False
  else
    if (((My_log2(g1.eff_weapons / g2.eff_shield) / 1.386294) + 1) / 2) > 0 then
    Result := True
  else
    Result := False
end;


procedure TBatSim.Init_Survival_Count(cur_Group: TBatGroup);
begin
  while cur_Group <> nil do
  begin
    cur_group.number_survived := 0;
    cur_group.max_number_survived := 0;
    cur_group.min_number_survived := cur_group.number;
    cur_group := cur_group.next;
  end;
end;

procedure TBatSim.Count_Survivors(var p: TBatPlayer);
var
  ship: integer;
  cur_group: TBatGroup;
begin
  if debug then Output.Add('Counting survivors...');

  cur_group := p.groups;
  while cur_group <> nil do
  begin
    cur_group.temporary := 0;
    cur_group := cur_group.next;
  end;
  for ship := 1 to p.cur_bat_array_size - 1 do
  begin
    if p.bat_array[ship] <> 0 then
    begin
      inc((p.group_array[p.bat_array[ship]]).temporary);
      if debug then output.Add(format('S: %s', [(p.group_array[p.bat_array[ship]]).name]));
    end;
  end;
  if debug then Synchronize(FlushOutput);
  cur_group := p.groups;
  while cur_group <> nil do
  begin
    cur_group.number_survived := cur_group.number_survived + cur_group.temporary;
    if cur_group.temporary > cur_group.max_number_survived then
      cur_group.max_number_survived := cur_group.temporary;
    if cur_group.temporary < cur_group.min_number_survived then
      cur_group.min_number_survived := cur_group.temporary;

    cur_group := cur_group.next;
  end;
end;

function CompareItems(Item1, Item2: Pointer): Integer;
begin
  Result := CompareText(TBatGroup(Item1).Name, TBatGroup(Item2).Name);
end;

procedure TBatSim.Print_Survival_Count(cur_group: TBatGroup);
var
  tot_mass,
    tot_sur_mass: single;
  sLine: string;
  aBatGroupArray: TBatGroupArray2;
  aCount: integer;
  aOld_CurGroup: TBatGroup;
  i,j: integer;
  aList: TList;

begin
  aOld_CurGroup := cur_group;

  // count the number of groups
{  aCount := 0;
  while cur_group <> nil do begin
    inc(aCount);
    cur_group := cur_group.Next;
  end;
  cur_group := aOld_CurGroup;
  // Create a array for it
  SetLength(aBatGroupArray, aCount);
  i := 0;
  while cur_group <> nil do begin
    aBatGroupArray[i] := cur_group;
    inc(i);
    cur_group := cur_group.Next;
  end;
  cur_group := aOld_CurGroup;
  for i := 0 to}
  aList := TList.Create;
  try
    while cur_group <> nil do begin
      aList.Add(cur_group);
      cur_group := cur_group.Next;
    end;
    cur_group := aOld_CurGroup;
    aList.Sort(CompareItems);
    Output.Add('');
    Output.Add('   # #Att    Ef W    Ef S                 T Surv Rate  Min    Max');
    tot_mass := 0;
    tot_sur_mass := 0;
  {  while cur_group <> nil do
    begin}
    for i := 0 to aList.Count -1 do begin
      cur_group := aList[i];
      sLine := format('%4d %4d %7.2f %7.2f ', [cur_group.number,
        cur_group.attacks,
          cur_group.eff_weapons,
          cur_group.eff_shield]);
      tot_mass := tot_mass + cur_group.number * cur_group.total_mass;
      sLine := sLine + format('%20s', [cur_group.name]);
      sLine := sLine + Format(' %3.2f', [cur_group.number_survived /
        (cur_group.number * max_battle)]);
      tot_sur_mass := tot_sur_mass + cur_group.number * cur_group.total_mass *
        (cur_group.number_survived /
        (cur_group.number * max_battle));
      sLine := sLine + Format('      %4d %4d', [cur_group.min_number_survived,
        cur_group.max_number_survived]);
      Output.Add(sLine);
  {    cur_group := cur_group.next;}
    end;
    output.Add(format('Total fleet mass     %6.0f', [tot_mass]));
    output.Add(format('Total surviving mass %6.0f', [tot_sur_mass]));
    output.Add(format('Total loss           %6.0f', [tot_mass - tot_sur_mass]));
    output.Add('');
  finally
    aList.Free;
  end;
end;

end.

