unit UDevelop;

interface

uses
  Classes, UFiles, UTypes, ExtListView;

const
  MAXDEVELOPTURNS = 20;
  OCMD_PRODUCE = 1;
  OCMD_SEND = 2;
  OCMD_LOAD = 3;
  OCMD_BREAK = 4;
  OCMD_INTERCEPT = 5;
  OCMD_ROUTE = 6;
  OCMD_UNLOAD = 7;
  OCMD_UPGRADE = 8;
  OCMD_DESIGN = 9;

type
  TDevPlanet = class(Tobject)
  public
    orgPlanet: Tplanet; // verwijzing naar originele planeet
    devCOL,
      devCAP,
      devMAT,
      devPopulation,
      devIndustry: array[0..MAXDEVELOPTURNS] of single;
    devProduce: array[0..MAXDEVELOPTURNS] of Integer;
    devAvailableIndustry: array[0..MAXDEVELOPTURNS] of single;
    developThisPlanet: boolean;
    function GetEffIndustry(devTurn: integer): single;
    procedure GetdevTurn(var population, industry: integer);
    procedure CalculateDevelopment(blnWithLoadTurn1: boolean);
  end;

  TdevType = (dtNone, dtToDo, dtLoadedSend, dtOnRoute);

  TDevGroup = class(Tobject)
    orgGroup: TGroup;
    devFrom,
      devTo: array[0..MAXDEVELOPTURNS] of TPlanet;
    devToGo: array[0..MAXDEVELOPTURNS] of single; // distance to go before arriving in first dev turn
    devLoadedWith: integer;
    devQuantity: single;
    devAantal: integer;
    Calculated,
      Ignore,
      Upgraded: boolean;

    PlanetCOL: single;
    PlanetCAP: single;
    PlanetDist: single;
    PlanetDest: TPlanet;
    devType: TDevType;
    devCargo: single;
    devPlanet: Tdevplanet; // for use in comparing

    function GetSpeed: single;
  end;

var
  devPlanets: TStringList;
  devGroups: TList;

function AnalyseerOrders(blnWithLoadTurn1, blnUseRoutesForDev: boolean): boolean;
procedure AnalyseerOrdersWithoutSending;
procedure DoOutput;
procedure SetOrderTstrings(Lines: TStrings);
procedure SetDevelopingListview(listview: TdfsExtListview);
procedure SetToTransportListview(listview: TdfsExtListview);
procedure SetToDoListview(listview: TdfsExtListview);
procedure SetLoadedSendListview(listview: TdfsExtListview);
function GetPlanetFromName(strPlanet: string): TPlanet;
function GetPlanetFromOrderLine(strLine: string): Tplanet;
function FindDevPlanet(planet: TPlanet): TdevPlanet;
function FindGroupFromString(s: string): TGroup;

implementation

uses
  SysUtils, UfrmOrders, Dialogs, HyperStr, comctrls, mkStrUtils,
  UfrmAutoCargoSettings;

var
  ordersTString: TStrings;
  loadedSendListview,
    toDoListview,
    toTransportListview,
    toDevelopingListview: TdfsExtListview;

procedure PutGroupsOnRoutes; forward;

function FindDevPlanet(planet: TPlanet): TdevPlanet;
var
  i: integer;
begin
  Result := nil;
  if devplanets.Find(planet.Name, i) then
    Result := TdevPlanet(devPlanets.Objects[i]);
end;

function FindRoute(planet: TPlanet): TRoute;
var
  i: integer;
begin
  for i := 0 to routes.Count -1 do begin
    Result := TRoute(routes.objects[i]);
    if Result.PlanetFrom = planet then Exit;
  end;
  Result := nil;
end;

function GetPlanetFromName(strPlanet: string): TPlanet;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to devPlanets.count - 1 do
  begin
    if CompareText(TdevPlanet(devPlanets.Objects[i]).orgplanet.Name, strPlanet) = 0 then
      Result := TdevPlanet(devPlanets.Objects[i]).orgplanet;
  end;
end;

procedure DeleteNewShiptypes;
var
  i: integer;
  shiptype: TShiptype;
begin
  i := 0;
  while i < qreportplayer.ShipTypes.Count do begin
    shiptype := TShiptype(qreportplayer.ShipTypes.objects[i]);
    if shiptype.IsNew then begin
      shiptype.Free;
      qreportplayer.ShipTypes.Delete(i);
    end else
      inc(i);
  end;
end;

procedure ResetRoutes;
var
  i: integer;
begin
  for i := 0 to routes.Count - 1 do
    TRoute(routes.objects[i]).RestoreOrgValues;
end;

procedure SetDevelopingListview(listview: TdfsExtListview);
begin
  toDevelopingListview := listview;
end;

procedure SetToTransportListview(listview: TdfsExtListview);
begin
  toTransportListview := listview;
end;

procedure SetToDoListview(listview: TdfsExtListview);
begin
  toDoListview := listview;
end;

procedure SetLoadedSendListview(listview: TdfsExtListview);
begin
  loadedSendListview := listview;
end;

procedure SetOrderTstrings(Lines: TStrings);
begin
  ordersTString := Lines;
end;

procedure CleardevPlanets;
var
  i: integer;
begin
  for i := 0 to devplanets.Count -1 do
    TdevPlanet(devPlanets.Objects[i]).Free;
  devPlanets.Clear;
end;

procedure CleardevGroups;
var
  i: integer;
begin
  for i := 0 to devGroups.Count -1 do
    TdevGroup(devGroups.Items[i]).Free;
  devGroups.Clear;
end;

function SetNullDevTurn: boolean;
var
  group: TGroup;
  planet: Tplanet;
  devGroup: TdevGroup;
  devPlanet: TdevPlanet;
  i, j, k: integer;
begin
  Result := True;
  CleardevPlanets;
  CleardevGroups;
  // Reportplayer moet speler zijn
  for i := 0 to qReportPlayer.groups.Count - 1 do
  begin
    group := TGroup(qReportPlayer.groups.Items[i]);
    if group.EffCargo > 0 then
    begin
      devGroup := TdevGroup.Create;
      devGroups.Add(devGroup);
      devGroup.orgGroup := group;
      devGroup.devType := dtNone;
      devGroup.devAantal := group.Aantal;
      if group.Range > 0 then
      begin
        devGroup.devFrom[0] := group.LocPlanet { nil};
        devGroup.devTo[0] := group.Planet;
        devGroup.devToGo[0] := group.Range { + group.EffSpeedNow};
        k := 1;
        while (k < MAXDEVELOPTURNS) and (devGroup.devToGo[k - 1] > 0) do
        begin
          devGroup.devFrom[k] := nil;
          devGroup.devTo[k - 1] := nil;
          devGroup.devToGo[k] := devGroup.devToGo[k - 1] - group.EffSpeedNow;
          if devGroup.devToGo[k] <= 0 then devGroup.devToGo[k] := 0;
          devGroup.devTo[k] := group.Planet;
          inc(k);
        end;
        devGroup.Calculated := True;
      end
      else
      begin
        devGroup.devFrom[0] := group.Planet;
        devGroup.devTo[0] := nil;
        devGroup.devToGo[0] := 0;
        devGroup.devFrom[1] := nil;
        devGroup.devTo[1] := nil;
        devGroup.devToGo[1] := 0;
        devGroup.Calculated := False;
      end;
      devGroup.devQuantity := group.Lading;
      devGroup.devLoadedWith := group.LadingType;
    end;
  end;
  for i := 0 to planets.Count - 1 do
  begin
    planet := TPlanet(planets.Objects[i]);
    planet.ProduceType := planet.OrgProduceType;
    if planet.PlayerCur = qReportPlayer then
    begin
      devPlanet := TdevPlanet.Create;
      devPlanets.AddObject(planets.Strings[i], devPlanet);
      devPlanet.orgPlanet := planet;
      devPlanet.devPopulation[0] := planet.PopulationCur;
      devPlanet.devIndustry[0] := planet.IndustryCur;
      devPlanet.devCOL[0] := planet.ColonistCur;
      devPlanet.devCAP[0] := planet.CapitalCur;
      devPLanet.devMAT[0] := planet.MaterialCur;
      devPlanet.developThisPlanet := False;
      devPlanet.devAvailableIndustry[0] := planet.EffIndustryCur;
      for j := 0 to MAXDEVELOPTURNS do
        devPlanet.devProduce[j] := planet.ProducingCur;
    end;
  end;
end;

procedure BreakGroup(var devGroup: TdevGroup; aantal: integer);
var
  newDevGroup: TdevGroup;
  i: integer;
begin
  newDevGroup := TdevGroup.Create;
  devGroups.Add(newDevGroup);
  newDevGroup.orgGroup := devGroup.orgGroup;
  for i := 0 to MAXDEVELOPTURNS do
  begin
    newDevGroup.devFrom[i] := devGroup.devFrom[i];
    newDevGroup.devTo[i] := devGroup.devTo[i];
    newDevGroup.devToGo[i] := devGroup.devToGo[i];
  end;
  newDevGroup.devLoadedWith := devGroup.devLoadedWith;
  newDevGroup.devQuantity := devGroup.devQuantity;
  newDevGroup.devAantal := aantal;
  devGroup.devAantal := devGroup.devAantal - aantal;
  devGroup := newDevGroup; // return the new created group
end;

function GetOrderPlaneet(var strLine: string; var devPlanet: TdevPlanet;
  AllowCreateDevPlanet: boolean): boolean;
var
  ipos, found, i: integer;
begin
  Result := False;
  if Length(strLine) = 0 then Exit;
  ipos := pos(' ', strLine);
  if ipos = 0 then ipos := Length(strLine)
  else dec(ipos);
  found := devPlanets.IndexOf(copy(strLine, 1, ipos));
  if found <> -1 then
  begin
    devPlanet := TdevPlanet(devPlanets.Objects[found]);
    Result := true;
  end
  else
    if AllowCreateDevPlanet then
    begin
      if planets.Find(lowercase(copy(strLine, 1, ipos)), found) then
      begin
        devPlanet := TdevPlanet.Create;
        devPLanets.AddObject(planets.Strings[found], devPlanet);
        devPlanet.orgPlanet := TPlanet(planets.Objects[found]);
        for i := 0 to MAXDEVELOPTURNS do
        begin
          devPlanet.devCOL[i] := 0;
          devPlanet.devCAP[i] := 0;
          devPlanet.devMAT[i] := 0;
          devPlanet.devPopulation[i] := 0;
          devPlanet.devIndustry[i] := 0;
          devPlanet.devProduce[i] := 0;
          devPlanet.devAvailableIndustry[i] := 0;
        end;
        Result := true;
      end;
    end;
  strLine := trim(copy(strLine, ipos + 1, Length(strLine)));
end;

function GetOrderProduce(var strLine: string; var produce: integer; var produceStr: string): boolean;
var
  ipos: integer;
begin
  Result := False;
  if Length(strLine) = 0 then Exit;
  ipos := pos(' ', strLine);
  if ipos = 0 then ipos := Length(strLine)
  else dec(ipos);
  produceStr := copy(strLine, 1, ipos);
  if lowercase(produceStr) = 'mat' then
    produce := P_MAT
  else
    if lowercase(produceStr) = 'cap' then
      produce := P_CAP
    else
      if lowercase(produceStr) = 'col' then
        produce := P_COL;
  Result := (produce <> 0);
  strLine := trim(copy(strLine, ipos + 1, Length(strLine)));
end;

function GetOrderGroup(var strLine: string; var devGroup: TdevGroup): boolean;
var
  ipos: integer;
  nr: integer;
  i: integer;
begin
  Result := False;
  if Length(strLine) = 0 then Exit;
  ipos := pos(' ', strLine);
  if ipos = 0 then ipos := Length(strLine)
  else dec(ipos);
  if IsNum(copy(strLine, 1, ipos)) then
  begin
    try
      nr := StrToInt(copy(strLine, 1, ipos));
      i := 0;
      while (i < devGroups.Count) and (devGroup = nil) do
      begin
        if TdevGroup(devGroups.Items[i]).OrgGroup.Nr = nr then
          devGroup := TdevGroup(devGroups.Items[i]);
        inc(i);
      end;
    except
      ;
    end;
  end
  else
  begin
    if lowercase(copy(strLine, 1, ipos)) = 'max' then
      if devGroups.Count > 0 then
        devGroup := TdevGroup(devGroups.Items[devGroups.count - 1]);
  end;
  Result := (devGroup <> nil);
  strLine := trim(copy(strLine, ipos + 1, Length(strLine)));
end;

function GetOrderAantal(var strLine: string; var aantal: integer): boolean;
var
  ipos: integer;
begin
  Result := False;
  if Length(strLine) = 0 then Exit;
  ipos := pos(' ', strLine);
  if ipos = 0 then ipos := Length(strLine)
  else dec(ipos);
  if (Length(strLine) > 0) and (strLine[1] in ['0'..'9']) then begin
    try
      aantal := StrToInt(copy(strLine, 1, ipos));
      Result := True;
    except
      aantal := 0;
    end;
  end else
    aantal := 0;
  strLine := trim(copy(strLine, ipos + 1, Length(strLine)));
end;

procedure CreateShiptypeFromLine(SLine: string);
var
  strName: string;
  drive: Single;
  attacks: integer;
  weapons: Single;
  shields: Single;
  cargo: Single;
  shiptype: TShipType;
  CharTable: string;
begin
  CharTable := BuildTable(#$A0' '#9);
  strName := Fetch(SLine, CharTable, 2, False);
  shiptype := FindShipType(qreportplayer, strName);
  if not assigned(shiptype) then begin
    try
      drive := StrToFloat(Fetch(SLine, CharTable, 3, False));
      attacks := mkAToI(Fetch(SLine, CharTable, 4, False));
      weapons := StrToFloat(Fetch(SLine, CharTable, 5, False));
      shields := StrToFloat(Fetch(SLine, CharTable, 6, False));
      cargo := StrToFloat(Fetch(SLine, CharTable, 7, False));

      shiptype := TShipType.Create;
      qreportplayer.ShipTypes.AddObject(lowercase(strName), shiptype);
      shiptype.Name := strName;
      shiptype.Drive := round(drive * 100) / 100;
      shiptype.Attacks := attacks;
      shiptype.Weapons := weapons;
      shiptype.Shields := shields;
      shiptype.Cargo := cargo;
      shiptype.IsNew := true;
    except
      ;
    end;
  end;
end;

function FindGroupFromString(s: string): TGroup;
var
  CharTable: string;
  i, Nr: integer;
begin
  CharTable := BuildTable(#$A0' '#9);
  Nr := mkAtoI(Fetch(s, CharTable, 2, False));
  if NR <> 0 then begin
    for i := 0 to qReportplayer.Groups.Count -1 do begin
      Result := TGroup(qReportplayer.Groups.Items[i]);
      if Result.Nr = Nr then Exit;
    end;
  end;
  Result := nil;
end;

procedure CreateRouteFromLine(SLine: string);
var
  strFromName: string;
  strToName: string;
  strCargoName: string;
  CharTable: string;
  index: integer;
  planetFrom, planetTO: TPlanet;
  route: TRoute;
begin
  CharTable := BuildTable(#$A0' ');
  strFromName := Fetch(SLine, CharTable, 2, False);
  strCargoName := lowercase(Fetch(SLine, CharTable, 3, False));
  strToName := Fetch(SLine, CharTable, 4, false);
  planetFrom := FindPlanetWithName(strFromName);
  planetTO := FindPlanetWithName(strToName);


  index := routes.IndexOf(lowercase(strFromName));
  if (index = -1) and Assigned(planetTO) then begin
    route := TRoute.Create(planetFrom);
    routes.AddObject(lowercase(strFromName), route);
  end else
    route := TRoute(routes.objects[index]);
  if strCargoName = 'col' then
    route.COLTo := planetTO
  else
    if strCargoName = 'cap' then
      route.CAPTo := planetTO
    else
      if strCargoName = 'mat' then
        route.MATTo := planetTO
      else
        if strCargoName = 'emp' then
          route.EMPTo := planetTO;
end;

function ParseOrderLine(strLine: string; var command: integer; var devPlanet: TdevPlanet;
  var devGroup: TdevGroup; var produce: integer;
  var produceStr: string;
  var aantal: integer; AllowCreateDevPlanet: boolean): boolean;
begin
  Result := False;
  devPlanet := nil;
  devGroup := nil;
  produce := 0;
  aantal := 0;
  command := 0;
  strLine := trim(strLine);
  if Length(strLine) = 0 then Exit;
  case strLine[1] of
    'P', 'p': command := OCMD_PRODUCE; // p planeetnaam whattoproduce
    'S', 's': command := OCMD_SEND; // s group planeetnaam [aantal]
    'I', 'i': command := OCMD_INTERCEPT; // i group planeetnaam [aantal]
    'B', 'b': command := OCMD_BREAK; // b group aantal
    'L', 'l': command := OCMD_LOAD; // l group whattoload [aantal]
    'R', 'r': command := OCMD_ROUTE; // r planet Whattoship [planetdest]
    'U', 'u': command := OCMD_UNLOAD; // u group [aantal]
    'G', 'g': command := OCMD_UPGRADE; // g group [aantal]
    'D', 'd': command := OCMD_DESIGN; // d Name drive attacks weapons shields cargo
  end;
  strLine := copy(strLine, 2, Length(strLine));
  strLine := trim(strLine);
  if Length(strLine) = 0 then Exit;
  case command of
    OCMD_PRODUCE:
      begin
        if GetOrderPlaneet(strLine, devPlanet, AllowCreateDevPlanet) then
        begin
          Result := true;
          GetOrderProduce(strLine, produce, produceStr);
        end;
      end;
    OCMD_SEND:
      begin
        if GetOrderGroup(strLine, devGroup) then
          if GetOrderPlaneet(strLine, devPlanet, AllowCreateDevPlanet) then
          begin
            GetOrderAantal(strLine, aantal);
            Result := true;
          end;
      end;
    OCMD_INTERCEPT:
      begin
        if GetOrderGroup(strLine, devGroup) then
          if GetOrderPlaneet(strLine, devPlanet, AllowCreateDevPlanet) then
          begin
            GetOrderAantal(strLine, aantal);
            Result := true;
          end;
      end;
    OCMD_BREAK:
      begin
        if GetOrderGroup(strLine, devGroup) then
          if GetOrderAantal(strLine, aantal) then
            Result := true;
      end;
    OCMD_LOAD:
      begin
        if GetOrderGroup(strLine, devGroup) then
          if GetOrderProduce(strLine, produce, produceStr) then begin
            GetOrderAantal(strLine, aantal);
            Result := true;
          end;
      end;
    OCMD_ROUTE: // r planet Whattoship [planetdest]
      begin
        Result := true;
      end;
    OCMD_UNLOAD: // u group [aantal]
      begin
        if GetOrderGroup(strLine, devGroup) then
        begin
          GetOrderAantal(strLine, aantal);
          Result := true;
        end;
      end;
    OCMD_UPGRADE:
      begin
        GetOrderGroup(strLine, devGroup);
        GetOrderAantal(strLine, aantal);
        Result := true;
      end;
    OCMD_DESIGN: Result := true;
  end;

end;

function GetProducingOrders: boolean;
var
  curLine, i: integer;
  strTemp: string;
  command, produce, aantal: integer;
  produceStr: string;
  devPlanet: TdevPlanet;
  devGroup: TdevGroup;
  group: TGroup;
begin
  Result := True;
  for curLine := 0 to ordersTString.Count - 1 do
  begin
    strTemp := ordersTString.Strings[curLine];
    if Length(strTemp) > 0 then
    begin
      if strTemp[1] = ';' then
      begin
        strTemp := trim(copy(strTemp, 2, length(strTemp)));
        if ParseOrderLine(strTemp, command, devPlanet, devGroup, produce, produceStr, aantal, True) then
        begin
          case command of
            OCMD_PRODUCE: if devPlanet <> nil then
                devPlanet.developThisPlanet := True;
          end;
        end;
      end
      else
      begin
        if ParseOrderLine(strTemp, command, devPlanet, devGroup, produce, produceStr, aantal, False) then
        begin
          case command of
            OCMD_PRODUCE:
              begin
                if devPlanet <> nil then
                  for i := 1 to MAXDEVELOPTURNS do
                    devPlanet.devProduce[i] := produce;
                if Assigned(devPlanet) and Assigned(devPlanet.orgPlanet) then
                  devPlanet.orgPlanet.ProduceType := ProduceStr;
              end;
            OCMD_DESIGN:
              begin
                CreateShiptypeFromLine(strTemp);
              end;
            OCMD_ROUTE:
              begin
                CreateRouteFromLine(strTemp);
              end;
            OCMD_UPGRADE:
              begin
                if Assigned(devGroup) and Assigned(devGroup.orgGroup.Planet) then begin
                  devPlanet := FindDevPlanet(devGroup.orgGroup.Planet);
                  group := devGroup.OrgGroup;
                end else begin
                  group := FindGroupFromString(strTemp);
                  if Assigned(group) then
                    devPlanet := FindDevPlanet(group.Planet);
                end;
                if Assigned(devPlanet) then begin
                  if Assigned(devPlanet) then begin
                    if Assigned(devGroup) then begin
                      if (aantal = 0) or (devGroup.devAantal < aantal )then
                        aantal := devGroup.devAantal;
                      group := devGroup.OrgGroup;
                    end else begin
                      if (aantal = 0) or (group.Aantal < aantal) then
                        aantal := group.aantal;
                    end;
                    devPlanet.devAvailableIndustry[0] := devPlanet.devAvailableIndustry[0] - Group.UpgradeCost * aantal;
                    if devPlanet.devAvailableIndustry[0] < 0 then
                      devPlanet.devAvailableIndustry[0] := 0;
                  end;
                end;
              end;
            OCMD_UNLOAD:
              begin
                if Assigned(devGroup) and Assigned(devGroup.orgGroup.Planet) then begin
                  devPlanet := FindDevPlanet(devGroup.orgGroup.Planet);
                  group := devGroup.OrgGroup;
                  if Assigned(devPlanet) and (group.Range = 0) then begin
                    case devGroup.devLoadedWith of
                      P_COL: devPlanet.devCOL[0] := devPlanet.devCOL[0] +
                               devGroup.devAantal * devGroup.devQuantity;
                      P_CAP: devPlanet.devCAP[0] := devPlanet.devCAP[0] +
                               devGroup.devAantal * devGroup.devQuantity;
                      P_MAT: devPlanet.devMAT[0] := devPlanet.devMAT[0] +
                               devGroup.devAantal * devGroup.devQuantity;
                    end;
                    devGroup.devQuantity := 0;
                    devGroup.devLoadedWith := 0;
                  end;
                end;
              end;
          end;
        end;
      end;
    end;
  end;
end;

function GetGroupOrders(blnUseRoutesForDev: boolean): boolean;
var
  curLine, i, k: integer;
  strTemp: string;
  command, produce, aantal: integer;
  produceStr: string;
  devPlanet: TdevPlanet;
  devGroup: TdevGroup;
  distance: single;
  planet: TPlanet;
  vX1: Single;
  vY1: Single;
  vX2: Single;
  vY2: Single;
begin
  Result := True;
  for curLine := 0 to ordersTString.Count - 1 do
  begin
    strTemp := ordersTString.Strings[curLine];
    if Length(strTemp) > 0 then
    begin
      if strTemp[1] = ';' then
      begin
        strTemp := trim(copy(strTemp, 2, length(strTemp)));
        command := 0;
        devPlanet := nil;
        devGroup := nil;
        produce := 0;
        aantal := 0;
        ParseOrderLine(strTemp, command, devPlanet, devGroup, produce, produceStr, aantal, False);
        case command of
          OCMD_SEND: if Assigned(devGroup) then devGroup.Ignore := true;
        end;
      end
      else
      begin
        if ParseOrderLine(strTemp, command, devPlanet, devGroup, produce, produceStr, aantal, {False} True) then
        begin
          case command of
            OCMD_LOAD:
              begin
                if devGroup = nil then continue;
                for i := 0 to devPlanets.Count - 1 do
                  if TdevPlanet(devPlanets.Objects[i]).OrgPlanet = devGroup.devFrom[0] then
                    devPlanet := TdevPlanet(devPlanets.Objects[i]);
                if devPLanet = nil then continue;
                if aantal > 0 then if aantal < devGroup.devAantal then
                    BreakGroup(devGroup, aantal);
                devGroup.devLoadedWith := produce;
                devGroup.devQuantity := 0;
                case produce of
                  P_COL: begin
                      if devPLanet.devCOL[0] > 0 then
                      begin
                        if devGroup.devAantal * devGroup.orgGroup.EffCargo >=
                          devPLanet.devCOL[0] then
                        begin
                          devGroup.devQuantity := devPLanet.devCOL[0] / devGroup.devAantal;
                          devPLanet.devCOL[0] := 0;
                        end
                        else
                        begin
                          devPLanet.devCOL[0] := devPLanet.devCOL[0] -
                            devGroup.devAantal * devGroup.orgGroup.EffCargo;
                          devGroup.devQuantity := devGroup.orgGroup.EffCargo;
                        end;
                      end;
                    end;
                  P_CAP: begin
                      if devPLanet.devCAP[0] > 0 then
                      begin
                        if (devGroup.devAantal * devGroup.orgGroup.EffCargo) >=
                          devPLanet.devCAP[0] then
                        begin
                          devGroup.devQuantity := devPLanet.devCAP[0] / devGroup.devAantal;
                          devPLanet.devCAP[0] := 0;
                        end
                        else
                        begin
                          devPLanet.devCAP[0] := devPLanet.devCAP[0] -
                            devGroup.devAantal * devGroup.orgGroup.EffCargo;
                          devGroup.devQuantity := devGroup.orgGroup.EffCargo;
                        end;
                      end;
                    end;
                      P_MAT: begin
                             if devPLanet.devMAT[0] > 0 then
                             begin
                               if devGroup.devAantal * devGroup.orgGroup.EffCargo >=
                                  devPLanet.devMAT[0] then
                               begin
                                 devGroup.devQuantity := devPLanet.devMAT[0] / devGroup.devAantal;
                                 devPLanet.devMAT[0] := 0;
                               end
                               else
                               begin
                                 devPLanet.devMAT[0] := devPLanet.devMAT[0] -
                                   devGroup.devAantal * devGroup.orgGroup.EffCargo;
                                 devGroup.devQuantity := devGroup.orgGroup.EffCargo;
                               end;
                             end;
                           end;
                end;
              end;
            OCMD_SEND:
              begin
                if devGroup = nil then continue;
                if aantal > 0 then
                begin
                  if aantal > devGroup.devAantal then
                    ShowMessage(Format('Can''t break group %d!', [devGroup.orgGroup.Nr]))
                  else
                    if aantal < devGroup.devAantal then
                      BreakGroup(devGroup, aantal);
                end;
                if not devGroup.Calculated then
                  devGroup.devTo[1] := devPLanet.Orgplanet;
              end;
            OCMD_BREAK:
              begin
                if devGroup = nil then continue;
                if aantal > 0 then
                begin
                  if aantal > devGroup.devAantal then
                    ShowMessage(Format('Can''t break group %d!', [devGroup.orgGroup.Nr]))
                  else
                    if aantal < devGroup.devAantal then
                      BreakGroup(devGroup, aantal);
                end;
              end;
            OCMD_UPGRADE:
              begin
                if devGroup = nil then continue;
                if aantal > 0 then
                begin
                  if aantal > devGroup.devAantal then
                    ShowMessage(Format('Can''t break group %d!', [devGroup.orgGroup.Nr]))
                  else
                    if aantal < devGroup.devAantal then
                      BreakGroup(devGroup, aantal);
                end;
                devGroup.Upgraded := True;
              end;
          end;
        end;
      end;
    end;
  end;

  if blnUseRoutesForDev then
    PutGroupsOnRoutes;

  for i := 0 to devGroups.Count - 1 do
  begin
    devGroup := TdevGroup(devGroups.items[i]);

    if (devGroup.devTo[1] <> nil) then
    begin
      if devGroup.devToGo[0] > 0 then
      begin
        devGroup.devToGo[1] := devGroup.devToGo[0] - devGroup.GetSpeed;
        if devGroup.devToGo[1] < 0 then
          devGroup.devToGo[1] := 0;
      end
      else
      begin
        planet := devGroup.devTo[1];
        vX1 := devGroup.devFrom[0].x;
        vY1 := devGroup.devFrom[0].y;
        vX2 := devGroup.devTo[1].x;
        vY2 := devGroup.devTo[1].y;
        distance := (sqrt((vX1 - vX2) * (vX1 - vX2) +
          (vY1 - vY2) * (vY1 - vY2)));
        devGroup.devToGo[0] := distance / 100;
        k := 1;
        while (k < MAXDEVELOPTURNS) and (devGroup.devToGo[k - 1] > 0) do
        begin
          devGroup.devFrom[k] := nil;
          devGroup.devTo[k - 1] := nil;
          devGroup.devToGo[k] := devGroup.devToGo[k - 1] - devGroup.GetSpeed;
          if devGroup.devToGo[k] <= 0 then devGroup.devToGo[k] := 0;
          devGroup.devTo[k] := Planet;
          inc(k);
        end;
        devGroup.Calculated := True;
      end;
    end;
  end;
end;

function GetPlanetFromOrderLine(strLine: string): Tplanet;
var
  devPlanet: TdevPlanet;
  devgroup: TdevGroup;
  command,
    produce,
    aantal: integer;
  produceStr: string;
begin
  Result := nil;
  if ParseOrderLine(strLine, command, devPlanet, devGroup, produce, produceStr, aantal, False) then
    if devPlanet <> nil then
      Result := devPlanet.orgPlanet
    else
      if devGroup <> nil then
        Result := devgroup.orgGroup.Planet;
end;

function TDevGroup.GetSpeed: single;
begin
  Result := 20 * (orgGroup._Type.Drive * orgGroup.DriveTech) /
    (orgGroup._Type.Mass + (devQuantity / orgGroup.CargoTech));
end;

procedure TDevPlanet.CalculateDevelopment(blnWithLoadTurn1: boolean);
var
  i, j, x: integer;
  devGroup: TdevGroup;
  size: single;
  capProd: Single;
  tmpCap1, tmpCap2: Single;

  function LoadShipWithCol(devGroup: TdevGroup; i: integer): boolean;
  begin
    Result := True;
    if devCOL[i] > 0 then begin
      if devGroup.orgGroup.EffCargo > devCOL[i] then
        devCOL[i] := 0
      else
        devCOL[i] := devCOL[i] - devGroup.orgGroup.EffCargo;
    end
    else
      Result := False;
  end;

  function LoadShipWithCap(devGroup: TdevGroup; i: integer): boolean;
  begin
    Result := True;
    if devCAP[i] > 0 then begin
      if devGroup.orgGroup.EffCargo > devCAP[i] then
        devCAP[i] := 0
      else
        devCAP[i] := devCAP[i] - devGroup.orgGroup.EffCargo;
    end
    else
      Result := False;
  end;

begin
  // all groups must be known where they go and the quantity
  for i := 1 to MAXDEVELOPTURNS do
  begin
    size := orgPlanet.Size;
    // use resource stacks left of previous turn
    devCOL[i] := devCOL[i - 1];
    devCAP[i] := devCAP[i - 1];
    devMAT[i] := devMAT[i - 1];
    // produce this turn
    case devProduce[i] of
      P_CAP: begin // when producing cap calculate how many
          if orgPlanet.Resources > 0 then begin
//            devCAP[i] := devCAP[i] + devAvailableIndustry[i - 1] / (5 + 1 / orgPlanet.Resources);
//            devCAP[i] := devCAP[i] + GetEffIndustry(i - 1) / (5 + 1 / orgPlanet.Resources);
             capProd := (devAvailableIndustry[i - 1] + devMAT[i] / orgPlanet.Resources) / (5 + (1 / orgPlanet.Resources));
             if capProd > (devAvailableIndustry[i - 1] / 5) then capProd := devAvailableIndustry[i - 1] / 5;
             if devMAT[i] > 0 then
               devMAT[i] := devMAT[i] - 5 * capProd;
             if devMAT[i] < 0 then
               devMAT[i] := 0;
             devCAP[i] := devCAP[i] + capProd;
          end;
        end;
      P_MAT: begin
          if orgPlanet.Resources > 0 then
            devMAT[i] := devMAT[i] + devAvailableIndustry[i - 1] * orgPlanet.Resources;
           end;
    end;
    // calculate population
    devPopulation[i] := devpopulation[i - 1];
    if devPopulation[i] < orgPlanet.size then begin
      devPopulation[i] := devPopulation[i] + devCOL[i] * 8;
      if devPopulation[i] > size then
      begin
        devCOL[i] := devCOL[i] + (devPopulation[i] - size) / 8;
        devPopulation[i] := size;
      end;
    end;
    devPopulation[i] := devpopulation[i] + devpopulation[i] * 0.08;

    // add the coming groups that unloads their load this turn
    for j := 0 to devGroups.Count - 1 do
    begin
      devGroup := TdevGroup(devGroups.items[j]);
      if devGroup.devTo[i] = orgPlanet then
        if devGroup.devToGo[i] = 0 then
        begin
          if devGroup.devLoadedWith = P_COL then
            devPopulation[i] := devPopulation[i] + devGroup.devQuantity * 8 * devGroup.devAantal;
          if devGroup.devLoadedWith = P_CAP then
            devCAP[i] := devCAP[i] + devGroup.devQuantity * devGroup.devAantal;
          if devGroup.devLoadedWith = P_MAT then
            devMAT[i] := devMAT[i] + devGroup.devQuantity * devGroup.devAantal;
        end;
    end;
    // turn population greater than size into colonists
    if devPopulation[i] > size then
    begin
      devCOL[i] := devCOL[i] + (devPopulation[i] - size) / 8;
      devPopulation[i] := size;
    end;
    devIndustry[i] := devIndustry[i - 1];
    // increase industry by the CAP available (add CAP to industry)
    if devIndustry[i] < devPopulation[i] then
    begin
      if inivars.Gametype = gtGalaxyBlind then begin
        tmpCap1 := devPopulation[i] - devIndustry[i];
        tmpCap2 := devCAP[i] - tmpCap1 / 2;
        if tmpCap2 <= 0 then begin
          devIndustry[i] := devIndustry[i] + devCAP[i] * 2;
          devCAP[i] := 0;
        end else begin
          devIndustry[i] := devPopulation[i];
          devCAP[i] := tmpCap2;
        end;
      end else begin
        devIndustry[i] := devIndustry[i] + devCAP[i];
        devCAP[i] := 0;
        // place all industry above population into CAP
        if devIndustry[i] > devPopulation[i] then
        begin
          devCAP[i] := (devIndustry[i] - devPopulation[i]);
          devIndustry[i] := devPopulation[i];
        end;
      end;
    end;
    ////////
    devAvailableIndustry[i] := GetEffIndustry(i);
    if (i = 1) and blnWithLoadTurn1 then // only at develop turn 1
      for j := 0 to devGroups.Count - 1 do
      begin
        devGroup := TdevGroup(devGroups.items[j]);
        if ((devGroup.devTo[i] = orgPlanet) and (devGroup.devToGo[i] = 0)) or
          ((devGroup.devFrom[0] = orgPlanet) and (devGroup.PlanetDest = nil)) then
        begin
          for x := 1 to devGroup.devAantal do // repeat this for every ship in the group
          begin
            case devGroup.orgGroup._Type.CargoTypes of
              ctCAP: LoadShipWithCap(devGroup, i);
              ctCAPthenCOL: if not LoadShipWithCap(devGroup, i) then
                  LoadShipWithCol(devGroup, i);
              ctCOL: LoadShipWithCol(devGroup, i);
              ctCOLthenCAP: if not LoadShipWithCol(devGroup, i) then
                  LoadShipWithCap(devGroup, i);
            end;
          end;
        end;
      end;
  end;
end;

function TDevPlanet.GetEffIndustry(devTurn: integer): single;
begin
  Result := devIndustry[devTurn];
  if devPopulation[devTurn] > devIndustry[devTurn] then
    Result := Result + (devPopulation[devTurn] - devIndustry[devTurn]) / 4;
end;

procedure TDevPlanet.GetdevTurn(var population, industry: integer);
var
  size: single;
  i: integer;
begin
  size := orgPlanet.Size;
  population := 0;
  industry := 0;
  for i := 0 to MAXDEVELOPTURNS do
  begin
    if devPopulation[i] < size then inc(population);
    if devIndustry[i] < size then inc(industry);
  end;
  if Size = 0 then begin
    population := -1;
    industry := -1;
  end;
end;

procedure PutGroupsOnRoutes;
var
  i: integer;
  devGroup: TdevGroup;
  group: TGroup;
  devPlanet: TdevPlanet;
  route: TRoute;
  intNumber: integer;
begin
  if routes.Count = 0 then Exit;
  i := 0;
  while i < devGroups.Count do
  begin
    devGroup := TdevGroup(devGroups.items[i]);
    if Assigned(devGroup) and Assigned(devGroup.OrgGroup) and
       (devGroup.devType in [dtNone, dtToDo]) and not devGroup.Ignore then begin
      if (devGroup.devType in [dtLoadedSend, dtOnRoute]) or (devGroup.devToGo[0] > 0) or 
         Assigned(devGroup.devTo[1]) then begin
        inc(i);
        Continue;
      end;

      group := devGroup.OrgGroup;
      if group.EffCargo = 0 then
        Continue;
      devPlanet := FindDevPlanet(group.Planet);
      if Assigned(devPlanet) then begin
        route := FindRoute(group.Planet);
        if Assigned(route) then begin
            if Assigned(route.COLTo) and (devPlanet.devCOL[0] > 0) then begin
              intNumber := trunc(devPlanet.devCOL[0] / group.EffCargo) + 1;
              if intNumber > devGroup.devAantal then
                intNumber := devGroup.devAantal;
              if intNumber < devGroup.devAantal then
                BreakGroup(devGroup, intNumber); // Result the new Create group
              devGroup.devQuantity := devPlanet.devCOL[0] / intNumber;
              devPlanet.devCOL[0] := devPlanet.devCOL[0] - intNumber * group.EffCargo;
              if devPlanet.devCOL[0] < 0 then devPlanet.devCOL[0] := 0
              else
                devGroup.devQuantity := group.EffCargo;
              devGroup.devTo[1] := route.COLTo;
              devGroup.devType := dtOnRoute;
              devGroup.devLoadedWith := P_COL;
            end;
            devGroup := TdevGroup(devGroups.items[i]);
            if devGroup.devType in [dtLoadedSend, dtOnRoute] then continue;
            if Assigned(route.CAPTo) and (devPlanet.devCAP[0] > 0) then begin
              intNumber := trunc(devPlanet.devCAP[0] / group.EffCargo) + 1;
              if intNumber > devGroup.devAantal then
                intNumber := devGroup.devAantal;
              if intNumber < devGroup.devAantal then
                BreakGroup(devGroup, intNumber); // Result the new Create group
              devGroup.devQuantity := devPlanet.devCAP[0];
              devPlanet.devCAP[0] := devPlanet.devCAP[0] - intNumber * group.EffCargo;
              if devPlanet.devCAP[0] < 0 then devPlanet.devCAP[0] := 0
              else
                devGroup.devQuantity := intNumber * group.EffCargo;
              devGroup.devTo[1] := route.CAPTo;
              devGroup.devType := dtOnRoute;
              devGroup.devLoadedWith := P_CAP;
            end;
            devGroup := TdevGroup(devGroups.items[i]);
            if devGroup.devType in [dtLoadedSend, dtOnRoute] then continue;
            if Assigned(route.MATTo) and (devPlanet.devMAT[0] > 0) then begin
              intNumber := trunc(devPlanet.devMAT[0] / group.EffCargo) + 1;
              if intNumber > devGroup.devAantal then
                intNumber := devGroup.devAantal;
              if intNumber < devGroup.devAantal then
                BreakGroup(devGroup, intNumber); // Result the new Create group
              devGroup.devQuantity := devPlanet.devMAT[0];
              devPlanet.devMAT[0] := devPlanet.devMAT[0] - intNumber * group.EffCargo;
              if devPlanet.devMAT[0] < 0 then devPlanet.devMAT[0] := 0
              else
                devGroup.devQuantity := intNumber * group.EffCargo;
              devGroup.devTo[1] := route.MATTo;
              devGroup.devType := dtOnRoute;
              devGroup.devLoadedWith := P_MAT;
            end;
            devGroup := TdevGroup(devGroups.items[i]);
            if devGroup.devType in [dtLoadedSend, dtOnRoute] then continue;
            if Assigned(route.EMPTo) then begin
              devGroup.devQuantity := 0;
              devGroup.devTo[1] := route.EMPTo;
              devGroup.devType := dtOnRoute;
              devGroup.devLoadedWith := EMPTY;
            end;
        end;
      end;
    end;
    inc(i);
  end;
end;

procedure CalculateDevGroupsLists;
var
  i, k: integer;
  devGroup: TdevGroup;
  devPlanet: TdevPlanet;
  dest: TPlanet;
begin
  for i := 0 to devGroups.Count - 1 do
  begin
    devGroup := TdevGroup(devGroups.items[i]);
//    devGroup.devType := dtNone;
    if not devGroup.Ignore and
       ((devGroup.Upgraded and (devGroup.devType = dtOnRoute)) or
        not devGroup.Upgraded) then
    begin
      if devGroup.devTo[1] <> TPlanet(1) then
      begin
        devPlanet := FindDevPlanet(devGroup.OrgGroup.Planet);
        if Assigned(devPlanet) then begin
          devGroup.PlanetCOL := devplanet.devCOL[0];
          devGroup.PlanetCAP := devPlanet.devCap[0];
          devGroup.devCargo := devGroup.orgGroup.EffCargo;
        end else begin
          devGroup.PlanetCOL := devGroup.orgGroup.Planet.ColonistCur;
          devGroup.PlanetCAP := devGroup.orgGroup.Planet.CapitalCur;
          devGroup.devCargo := devGroup.devQuantity;
        end;
        devGroup.PlanetDist := devGroup.devToGo[1];

        dest := nil;
        for k := 0 to MAXDEVELOPTURNS do
          if devGroup.devTo[k] <> nil then
            dest := devGroup.devTo[k];

        if dest = nil then
        begin
          devGroup.devType := dtToDo;
          devGroup.PlanetDest := devGroup.devFrom[0];
        end
        else
        begin
          if devGroup.devType <> dtOnRoute then
            devGroup.devType := dtLoadedSend;
          devGroup.PlanetDest := dest;
        end;
      end
      else
        ShowMessage('planet 1 encountered!');
    end;
  end;
end;

procedure DoOutput;
var
  i: integer;
  devPlanet: TdevPlanet;
  devGroup: TDevGroup;
  intPop, intInd: integer;
  strpop, strInd, strTotal: string;
  intGroup,
    intAantal: integer;
  strCol, strCap, strDistance: string;
  listItem: TListItem;
  blntoDevelopingSortOrder: boolean;
  blntoDoListviewSortOrder: boolean;
  blnloadedSendListviewSortOrder: boolean;
  blntoTransportListviewSortOrder: boolean;
begin
// developing:  planeet, col turns, cap turns, total
// to transport: planeet, col nu, col 1, cap nu, cap 1
// to do: group, aantal, effcargo, speedloaded, cur planet, col nu, cap nu
// loaded send: group, aantal, cargo, speedloaded with cargo, destination, distance left after 1 turn
  toDevelopingListview.BeginUpdate;
  toTransportListview.BeginUpdate;
  toDoListview.BeginUpdate;
  loadedSendListview.BeginUpdate;
  blntoDevelopingSortOrder := toDevelopingListview.CurrentSortAscending;
  blntoDoListviewSortOrder := toDoListview.CurrentSortAscending;
  blnloadedSendListviewSortOrder := loadedSendListview.CurrentSortAscending;
  blntoTransportListviewSortOrder := toTransportListview.CurrentSortAscending;
  try
    toDevelopingListview.Items.Clear;
    toTransportListview.Items.Clear;
    for i := 0 to devPlanets.Count - 1 do
    begin
      devPlanet := TdevPlanet(devPlanets.Objects[i]);

      if devPlanet.developThisPlanet then
      begin
        if (devPLanet.orgPlanet.PlayerCur <> qReportPlayer) and
          (devPLanet.orgPlanet.PlayerCur <> nil) then
        begin
          intPop := 0;
          intInd := 0;
          strPop := 'Not Possible';
          strInd := 'Not Possible';
        end
        else
        begin
          devPlanet.GetdevTurn(intPop, intInd);
          if intPop = -1 then strPop := 'Unknown'
          else
            if (intPop = 0) then strPop := 'FULL'
            else
              if intPop > MAXDEVELOPTURNS then strpop := '>' + Format('%-d', [intPop - 1])
              else strPop := Format('%d', [intPop]);
          if intInd = -1 then strInd := 'Unknown'
          else
            if (intInd = 0) then strInd := 'FULL'
            else
              if intInd > MAXDEVELOPTURNS then strInd := '>' + Format('%-d', [intInd - 1])
              else strInd := Format('%d', [intInd]);
        end;
        if intPop <> 0 then
          strTotal := format('(%5.0f/%5.0f)', [devPlanet.orgPlanet.PopulationCur, devPlanet.orgPlanet.Size])
        else
          if intInd <> 0 then
            strTotal := format('(%5.0f/%5.0f)', [devPlanet.orgPlanet.IndustryCur, devPlanet.orgPlanet.Size])
          else
            strTotal := '';
        listItem := toDevelopingListview.Items.Add;
        listItem.Caption := devPlanet.orgPlanet.Name;
        listItem.SubItems.Add(strPop);
        listItem.SubItems.Add(strInd);
        listItem.SubItems.Add(strTotal);
        listitem.Data := devPlanet;
      end;
      if (devPlanet.devCOL[0] <> 0) or (devPlanet.devCAP[0] <> 0) or
        (devPlanet.devCOL[1] <> 0) or (devPlanet.devCAP[1] <> 0) then
      begin
        listItem := toTransportListview.Items.Add;
        listItem.Caption := devPlanet.orgPlanet.Name;
        listItem.SubItems.Add(format('%5.2f', [devPlanet.devCOL[0]]));
        listItem.SubItems.Add(format('%5.2f', [devPlanet.devCOL[1]]));
        listItem.SubItems.Add(format('%5.2f', [devPlanet.devCAP[0]]));
        listItem.SubItems.Add(format('%5.2f', [devPlanet.devCAP[1]]));
        listItem.SubItems.Add(format('%5.2f', [devPlanet.devMAT[0]]));
        listItem.SubItems.Add(format('%5.2f', [devPlanet.devMAT[1]]));
        listItem.Data := devPlanet;
      end;
    end;
    toDoListview.Items.Clear;
    loadedSendListview.Items.Clear;
    for i := 0 to devGroups.Count - 1 do
    begin
      listItem := nil;
      devGroup := TdevGroup(devGroups.items[i]);
      case devGroup.devType of
        dtToDo: listItem := toDoListview.Items.Add;
        dtLoadedSend: listItem := loadedSendListview.Items.Add;
        dtOnRoute: listItem := loadedSendListview.Items.Add;
      else
        continue;
      end;
      intGroup := devgroup.orgGroup.nr;
      intAantal := devGroup.devAantal;
      listItem.Caption := intToStr(intGroup);
      listItem.SubItems.Add(intToStr(intAantal));
      listItem.SubItems.Add(format('%5.2f', [devGroup.devCargo]));
      listItem.SubItems.Add(format('%5.2f', [devGroup.GetSpeed]));
      listItem.SubItems.Add(devGroup.PlanetDest.Name);
      strCol := format('%5.2f', [devgroup.PlanetCOL]);
      strCap := format('%5.2f', [devgroup.PlanetCAP]);
      strDistance := format('%5.2f', [devGroup.devToGo[1]]);
      case devGroup.devType of
        dtToDo:
          begin
            listItem.SubItems.Add(strCol);
            listItem.SubItems.Add(strCap);
            listItem.Data := devGroup;
          end;
        dtLoadedSend,
        dtOnRoute:
          begin
            listItem.SubItems.Add(strDistance);
            listItem.Data := devGroup;
            listItem.ImageIndex := trunc(devGroup.devToGo[0] / devGroup.GetSpeed);
          end;
      end;
    end;
  finally
    toDevelopingListview.EndUpdate;
    toTransportListview.EndUpdate;
    toDoListview.EndUpdate;
    loadedSendListview.EndUpdate;
  end;
  toDevelopingListview.CurrentSortAscending := blntoDevelopingSortOrder;
  toDoListview.CurrentSortAscending := blntoDoListviewSortOrder;
  loadedSendListview.CurrentSortAscending := blnloadedSendListviewSortOrder;
  toTransportListview.CurrentSortAscending := blntoTransportListviewSortOrder;
end;

function AnalyseerOrders(blnWithLoadTurn1, blnUseRoutesForDev: boolean): boolean;
var
  i: integer;
begin
  Result := True;

  SetNullDevTurn;
  DeleteNewShiptypes;
  ResetRoutes;
  GetProducingOrders;
  GetGroupOrders(blnUseRoutesForDev);
  CalculateDevGroupsLists;
  for i := 0 to devPlanets.Count - 1 do
    TDevPlanet(devPlanets.objects[i]).CalculateDevelopment(blnWithLoadTurn1);
  CalculateDevGroupsLists;
end;

procedure AnalyseerOrdersWithoutSending;
var
  i: integer;
begin
  SetNullDevTurn;
  DeleteNewShiptypes;
  ResetRoutes;
  GetProducingOrders;
  for i := 0 to devPlanets.Count - 1 do
    TDevPlanet(devPlanets.objects[i]).CalculateDevelopment(false);
end;

initialization
  devPlanets := TStringList.Create;
  devPlanets.Sorted := True;
  devGroups := TList.Create;
finalization
  ClearDevPlanets;
  devPlanets.Free;
  ClearDevGroups;
  devGroups.Free;
end.

