unit UDistLists;

interface

uses
  Windows, Messages, SysUtils, Classes;

type

{ TStringList class }
  PDStringItem = ^TDStringItem;
  TDStringItem = record
    FString: string;
    FObject: TObject;
    FDistance: integer;
  end;

  PDStringItemList = ^TDStringItemList;
  TDStringItemList = array[0..MaxListSize] of TDStringItem;

  TDStringList = class(TStrings)
  private
    FList: PDStringItemList;
    FCount: Integer;
    FCapacity: Integer;
    procedure Grow;
    procedure InsertItem(Index, Dist: Integer; const S: string);
  protected
    function Get(Index: Integer): string; override;
    function GetCapacity: Integer; override;
    function GetCount: Integer; override;
    function GetObject(Index: Integer): TObject; override;
    procedure Put(Index: Integer; const S: string); override;
    procedure PutObject(Index: Integer; AObject: TObject); override;
    procedure SetCapacity(NewCapacity: Integer); override;
  public
    destructor Destroy; override;
    function Add(const Dist: integer; const S: string): Integer;
    procedure Clear; override;
    procedure Delete(Index: Integer); override;
    function Find(const Dist: integer; var Index: Integer): Boolean;
    function FindRange(const Dist: integer; var Index, Count: Integer): Boolean;
    function IndexOf(const Dist: integer): Integer;
    procedure Insert(Index: Integer; const S: string); override;
  end;

implementation

uses
  Consts;

{ TDStringList }

destructor TDStringList.Destroy;
begin
  inherited Destroy;
  if FCount <> 0 then Finalize(FList^[0], FCount);
  FCount := 0;
  SetCapacity(0);
end;

function TDStringList.Add(const Dist: integer; const S: string): Integer;
begin
  Find(Dist, Result);
  InsertItem(Result, Dist, S);
end;

procedure TDStringList.Clear;
begin
  if FCount <> 0 then
  begin
    Finalize(FList^[0], FCount);
    FCount := 0;
    SetCapacity(0);
  end;
end;

procedure TDStringList.Delete(Index: Integer);
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Finalize(FList^[Index]);
  Dec(FCount);
  if Index < FCount then
    System.Move(FList^[Index + 1], FList^[Index],
      (FCount - Index) * SizeOf(TDStringItem));
end;

function TDStringList.Find(const Dist: integer; var Index: Integer): Boolean;
var
  L, H, I, C: Integer;
begin
  Result := False;
  L := 0;
  H := FCount - 1;
  while L <= H do
  begin
    I := (L + H) shr 1;
    if Dist < FList^[I].FDistance then C := 1
    else
      if Dist > FList^[I].FDistance then C := -1
      else
        C := 0;
    if C < 0 then L := I + 1 else
    begin
      H := I - 1;
      if C = 0 then
        Result := True;
    end;
  end;
  Index := L;
end;

function TDStringList.Get(Index: Integer): string;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Result := FList^[Index].FString;
end;

function TDStringList.GetCapacity: Integer;
begin
  Result := FCapacity;
end;

function TDStringList.GetCount: Integer;
begin
  Result := FCount;
end;

function TDStringList.GetObject(Index: Integer): TObject;
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  Result := FList^[Index].FObject;
end;

procedure TDStringList.Grow;
var
  Delta: Integer;
begin
  if FCapacity > 512 then Delta := FCapacity div 4 else
    if FCapacity > 64 then Delta := 128 else
      if FCapacity > 8 then Delta := 16 else
        Delta := 4;
  SetCapacity(FCapacity + Delta);
end;

function TDStringList.IndexOf(const Dist: integer): Integer;
begin
  if not Find(Dist, Result) then Result := -1;
end;

procedure TDStringList.Insert(Index: Integer; const S: string);
begin
  Error(SSortedListError, 0);
end;

procedure TDStringList.InsertItem(Index, Dist: Integer; const S: string);
begin
  if FCount = FCapacity then Grow;
  if Index < FCount then
    System.Move(FList^[Index], FList^[Index + 1],
      (FCount - Index) * SizeOf(TDStringItem));
  with FList^[Index] do
  begin
    Pointer(FString) := nil;
    FObject := nil;
    FString := S;
    FDistance := Dist;
  end;
  Inc(FCount);
end;

procedure TDStringList.Put(Index: Integer; const S: string);
begin
  Error(SSortedListError, 0);
end;

procedure TDStringList.PutObject(Index: Integer; AObject: TObject);
begin
  if (Index < 0) or (Index >= FCount) then Error(SListIndexError, Index);
  FList^[Index].FObject := AObject;
end;

procedure TDStringList.SetCapacity(NewCapacity: Integer);
begin
  ReallocMem(FList, NewCapacity * SizeOf(TDStringItem));
  FCapacity := NewCapacity;
end;

function TDStringList.FindRange(const Dist: integer; var Index,
  Count: Integer): Boolean;
begin
  Count := 0;
  Result := Find(Dist, Index);
  if Result then begin
    while FList^[index + Count].FDistance = Dist do inc(Count);
  end;
end;

end.
