unit UFiles;

interface

uses Windows, inifiles, Classes, UTypes;

type
  TGametype = (gtUnknown, gtGalaxyNormal, gtGalaxyNG, gtGalaxyBlind,
               gtGalaxyPLUS, gtGalaxyNGBlind);
const
  SGameStr: array[TGametype] of string =
    ('Unknown Galaxy type', 'Galaxy Normal', 'Galaxy NG', 'Galaxy Blind',
     'Galaxy PLUS', 'Galaxy NG Blind');

type
  TIniVars = record
    Gametype: TGametype;
    outbackType: boolean;
    bigGame: boolean;
    mapFaktor,
      mapScrollV,
      mapScrollH,
      mapFaktorOor,
      mapScrollVOor,
      mapScrollHOor: single;
    SkipPlanetsUntilTurn: integer;
    SkipBattlesUntilTurn: integer;
    EliteNameCreation: boolean;
  end;

var
  IniVars: TIniVars;
  FirstTurn: integer;
  LastTurn: integer;
  strSpelD,
    strSpelC: string;
  turn0HabitedPlanetsSize: single;
  turn0HabitedPlanetsSize2nd: single;
  strGameName: string;
  GameNr: integer;
  gPlanetNumber: Integer;

function ReadMKGalIni: boolean;
function ReadReports: boolean;
function GetGameStr: string;
function SetConsts(Gametype: TGametype): boolean;


implementation

uses SysUtils, UmkStrParse, MKstrUtils, Dialogs, xProcs, Math,
  mwFastTime, Forms, UfrmWait, UfrmMain, UFilesCountMethod, UAddTabs,
  UfrmConfigIni, HyperStr, mkRegEx;

const
  CSGAL_HEADERGalGame: string = '^'#9'+Galaxy Game \w+ \w+ \w+ \w+';
  CSGAL_HEADEROptions: string = '^'#9'+Your Options\W*$';
  CSGAL_HEADERStatus: string = '^'#9'+Status of Players\W*$';

  CSGAL_GalaxyInitString: string = 'Galaxy Game';
  CIGAL_GalaxyInitTurnPos: integer = 4;
  CIGAL_GalaxyInitGameNamePos: integer = 2;
  CIGAL_GalaxyInitPlayerNamePos: integer = -1;

  CSGAL_StatPlayersString: string = 'Status of Players';
  CIGAL_StatPlayersplayerNamePos: integer = 0;
  CIGAL_StatPlayersDrivePos: integer = 1;
  CIGAL_StatPlayersWeaponsPos: integer = 2;
  CIGAL_StatPlayersShieldPos: integer = 3;
  CIGAL_StatPlayersCargoPos: integer = 4;
  CIGAL_StatPlayersPopulationPos: integer = 5;
  CIGAL_StatPlayersIndustryPos: integer = 6;
  CIGAL_StatPlayersPlanetsPos: integer = 7;
  CIGAL_StatPlayersWarPos: integer = 8;

  CSGAL_PlayersPlanetsString: string = 'Your Planets';
  CIGAL_PlayersPlanetsPlanetNamePos: integer = 0;
  CIGAL_PlayersPlanetsXPos: integer = 1;
  CIGAL_PlayersPlanetsYPos: integer = 2;
  CIGAL_PlayersPlanetsSizePos: integer = 3;
  CIGAL_PlayersPlanetsPopulationPos: integer = 4;
  CIGAL_PlayersPlanetsIndustryPos: integer = 5;
  CIGAL_PlayersPlanetsResourcesPos: integer = 6;
  CIGAL_PlayersPlanetsProdPos: integer = 7;
  CIGAL_PlayersPlanetsCapitalPos: integer = 8;
  CIGAL_PlayersPlanetsMaterialPos: integer = 9;
  CIGAL_PlayersPlanetsColonistPos: integer = 10;

  CSGAL_OtherPlanetsUnidentString: string = 'Unidentified Planets';
  CSGAL_OtherPlanetsUninhabString: string = 'Uninhabited Planets';
  CIGAL_OtherPlanetsPlanetNamePos: integer = 0;
  CIGAL_OtherPlanetsPlanetNrPos: integer = 1; // blind type
  CIGAL_OtherPlanetsXPos: integer = 1;
  CIGAL_OtherPlanetsYPos: integer = 2;
  CIGAL_OtherPlanetsSizePos: integer = 3;
  CIGAL_OtherPlanetsResourcePos: integer = 4;

  CSGAL_PlayerPlanetsString: string = 'Planets';
  CIGAL_PlayerPlanetsPlanetNamePos: integer = 0;
  CIGAL_PlayerPlanetsXPos: integer = 1;
  CIGAL_PlayerPlanetsYPos: integer = 2;
  CIGAL_PlayerPlanetsSizePos: integer = 3;
  CIGAL_PlayerPlanetsPopulationPos: integer = 4;
  CIGAL_PlayerPlanetsIndustryPos: integer = 5;
  CIGAL_PlayerPlanetsResourcesPos: integer = 6;
  CIGAL_PlayerPlanetsProdPos: integer = 7;
  CIGAL_PlayerPlanetsCapitalPos: integer = 8;
  CIGAL_PlayerPlanetsMaterialPos: integer = 9;
  CIGAL_PlayerPlanetsColonistPos: integer = 10;

  CSGAL_ShipTypesYourString: string = 'Your Ship Types';
  CSGAL_ShipTypesOtherString: string = ' Ship Types';
  CIGAL_ShipTypesNamePos: integer = 0;
  CIGAL_ShipTypesDrivePos: integer = 1;
  CIGAL_ShipTypesAttacksPos: integer = 2;
  CIGAL_ShipTypesWeaponsPos: integer = 3;
  CIGAL_ShipTypesShieldsPos: integer = 4;
  CIGAL_ShipTypesCargoPos: integer = 5;

  CSGAL_BombingsString: string = 'Bombings';
  CIGAL_BombingsAttackerPos: integer = 0;
  CIGAL_BombingsDefenderPos: integer = 1;
  CIGAL_BombingsPlanetPos: integer = 2;
  CIGAL_BombingsPopulationPos: integer = 3;
  CIGAL_BombingsIndustryPos: integer = 4;
  CIGAL_BombingsProducingPos: integer = 5;
  CIGAL_BombingsCapPos: integer = 6;
  CIGAL_BombingsMatPos: integer = 7;
  CIGAL_BombingsColPos: integer = 8;

  CSGAL_BattlesString: string = 'Battle at';
  CIGAL_BattlesPlanetNamePos: integer = 2;
  CIGAL_BattlesPlanerNamePos: integer = 0;
  CIGAL_BattlesGrnLeftPos: integer = 8; // 0
  CIGAL_BattlesGrNrPos: integer = 0; // 1
  CIGAL_BattlesGrShipTypePos: integer = 1; // 2
  CIGAL_BattlesGrCargoTypePos: integer = 6; // 8
  CIGAL_BattlesCargoQuantityPos: integer = 7; // 9
  CIGAL_BattlesDrivePos: integer = 2; // 3
  CIGAL_BattlesWeaponPos: integer = 3; // 5
  CIGAL_BattlesShieldPos: integer = 4; // 6
  CIGAL_BattlesCargoPos: integer = 5; // 7

  CSGAL_ShipsInProcString: string = 'Ships In Production';
  CIGAL_ShipsInProcNamePos: integer = 0;
  CIGAL_ShipsInProcPlanetNrPos: integer = 1; // blind
  CIGAL_ShipsInProcShipTypePos: integer = 1;

  CIGAL_GroupRowNrPos: integer = 0;
  CIGAL_GroupRowNPos: integer = 0;
  CIGAL_GroupRowShipTypePos: integer = 1;
  CIGAL_GroupRowDrivePos: integer = 2;
  CIGAL_GroupRowWeaponPos: integer = 3; //4
  CIGAL_GroupRowShieldPos: integer = 4; //5
  CIGAL_GroupRowCargoPos: integer = 5; //6
  CIGAL_GroupRowCargoTypePos: integer = 6; //7
  CIGAL_GroupRowCargoQuantityPos: integer = 7; //8
  CIGAL_GroupRowDestPos: integer = 8; //12
  CIGAL_GroupRowRangePos: integer = 9; //13
  CIGAL_GroupRowLocationPos: integer = 10; //11
  CIGAL_GroupRowRangeCount: integer = 10; //14

  CSGAL_GroupsGroupsString: string = ' Groups';
  CSGAL_GroupsFleetsString: string = 'Fleet ';
  CIGAL_GroupsFleetsNamePos: integer = 1;
  CIGAL_GroupsFleetsSpeedPos: integer = 3;

  CSGAL_IncomGroupsString: string = 'Incoming Groups';
  CIGAL_IncomGroupsFromPos: integer = 0;
  CIGAL_IncomGroupsDestPos: integer = 1;
  CIGAL_IncomGroupsRangePos: integer = 2;
  CIGAL_IncomGroupsSpeedPos: integer = 3;
  CIGAL_IncomGroupsMassPos: integer = 4;

var
  strIniFileName: string;
  blindRepName: integer;
  TurnParse: TMkStrParse;
  curLine: integer;
  LastBattleLine: integer;
  Headers: TStringList;
  gamestr: string;
  lstReadedGroupsPlayers: TList;
  mkreExpr: TmkreExpr;

function SetConsts(Gametype: TGametype): boolean;
begin
  Result := False;
  case gameType of
    gtUnknown: Exit;
    gtGalaxyNormal, gtGalaxyNG: ;
    gtGalaxyNGBlind:
      begin
        CSGAL_HEADERGalGame := '^'#9'+GalaxyNG-Blind Game \w+ \w+ \w+ \w+';
        CSGAL_GalaxyInitString := 'GalaxyNG-Blind Game';
      end;
    gtGalaxyBlind:
      begin
        CSGAL_HEADERGalGame := '^'#9'+Blind Galaxy game \w+ \w+ \w+ \w+';
        CSGAL_HEADEROptions := '^'#9'+Options\W*$';
        CSGAL_HEADERStatus := '^'#9'+Status of Players\W*$';

        CSGAL_GalaxyInitString := 'Blind Galaxy game';
        CIGAL_GalaxyInitTurnPos := 4;
        CIGAL_GalaxyInitGameNamePos := 3;
        CIGAL_GalaxyInitPlayerNamePos := -3;

        CIGAL_StatPlayersPopulationPos := 6;
        CIGAL_StatPlayersIndustryPos := 7;
        CIGAL_StatPlayersPlanetsPos := 8;
        CIGAL_StatPlayersWarPos := 9;

        CSGAL_PlayersPlanetsString := 'Your Systems';
        CIGAL_PlayersPlanetsPlanetNamePos := 0;
        CIGAL_PlayersPlanetsXPos := 2;
        CIGAL_PlayersPlanetsYPos := 3;
        CIGAL_PlayersPlanetsSizePos := 4;
        CIGAL_PlayersPlanetsPopulationPos := 5;
        CIGAL_PlayersPlanetsIndustryPos := 6;
        CIGAL_PlayersPlanetsResourcesPos := 7;
        CIGAL_PlayersPlanetsProdPos := 8;
        CIGAL_PlayersPlanetsCapitalPos := 10;
        CIGAL_PlayersPlanetsMaterialPos := 11;
        CIGAL_PlayersPlanetsColonistPos := 9;

        CSGAL_OtherPlanetsUnidentString := 'Unknown Systems';
        CSGAL_OtherPlanetsUninhabString := 'Uninhabited Systems';
        CIGAL_OtherPlanetsPlanetNamePos := 0;
        CIGAL_OtherPlanetsPlanetNrPos := 1; // blind type
        CIGAL_OtherPlanetsXPos := 1;
        CIGAL_OtherPlanetsYPos := 2;
        CIGAL_OtherPlanetsSizePos := 3;
        CIGAL_OtherPlanetsResourcePos := 4;

        CSGAL_PlayerPlanetsString := 'Systems';
        CIGAL_PlayerPlanetsPlanetNamePos := 0;
        CIGAL_PlayerPlanetsXPos := 2;
        CIGAL_PlayerPlanetsYPos := 3;
        CIGAL_PlayerPlanetsSizePos := 4;
        CIGAL_PlayerPlanetsPopulationPos := 5;
        CIGAL_PlayerPlanetsIndustryPos := 6;
        CIGAL_PlayerPlanetsResourcesPos := 7;
        CIGAL_PlayerPlanetsProdPos := 8;
        CIGAL_PlayerPlanetsCapitalPos := 10;
        CIGAL_PlayerPlanetsMaterialPos := 11;
        CIGAL_PlayerPlanetsColonistPos := 9;

        CSGAL_BombingsString := 'Bombings';
        CIGAL_BombingsAttackerPos := 0;
        CIGAL_BombingsDefenderPos := 1;
        CIGAL_BombingsPlanetPos := 2;
        CIGAL_BombingsPopulationPos := 4;
        CIGAL_BombingsIndustryPos := 5;
        CIGAL_BombingsProducingPos := -1;
        CIGAL_BombingsCapPos := 7;
        CIGAL_BombingsMatPos := 8;
        CIGAL_BombingsColPos := 6;

        CSGAL_BattlesString := 'Battle at';
        CIGAL_BattlesPlanetNamePos := -1;
        CIGAL_BattlesGrnLeftPos := 0;
        CIGAL_BattlesGrNrPos := 1;
        CIGAL_BattlesGrShipTypePos := 2;
        CIGAL_BattlesGrCargoTypePos := 8;
        CIGAL_BattlesCargoQuantityPos := 9;
        CIGAL_BattlesDrivePos := 3;
        CIGAL_BattlesWeaponPos := 5;
        CIGAL_BattlesShieldPos := 6;
        CIGAL_BattlesCargoPos := 7;

        CSGAL_ShipsInProcString := 'Ships in Production';
        CIGAL_ShipsInProcNamePos := 0;
        CIGAL_ShipsInProcPlanetNrPos := 1; // blind
        CIGAL_ShipsInProcShipTypePos := 6;

        CIGAL_GroupRowNrPos := 0;
        CIGAL_GroupRowNPos := 0;
        CIGAL_GroupRowShipTypePos := 1;
        CIGAL_GroupRowDrivePos := 2;
        CIGAL_GroupRowWeaponPos := 4;
        CIGAL_GroupRowShieldPos := 5;
        CIGAL_GroupRowCargoPos := 6;
        CIGAL_GroupRowCargoTypePos := 7;
        CIGAL_GroupRowCargoQuantityPos := 8;
        CIGAL_GroupRowDestPos := 12;
        CIGAL_GroupRowRangePos := 13;
        CIGAL_GroupRowLocationPos := 11;
        CIGAL_GroupRowRangeCount := 14;

        CSGAL_IncomGroupsString := 'Incoming Groups';
        CIGAL_IncomGroupsFromPos := 1;
        CIGAL_IncomGroupsDestPos := 3;
        CIGAL_IncomGroupsRangePos := 4;
        CIGAL_IncomGroupsSpeedPos := 5;
        CIGAL_IncomGroupsMassPos := 6;
      end;
    gtGalaxyPLUS: exit;
  end;
end;

function TabExpand(strLine: string): string;
var
  i, j: integer;
begin
  Result := '';
  j := 0;
  for i := 1 to Length(strLine) do begin
    if strLine[i] <> #9 then Result := Result + strLine[i]
    else RPAD(Result, #32, ((j div 8) + 1) * 8);
    j := Length(Result) - 1;
  end;
end;

function GetXorYCoord(str: string): integer;
var
  i, len: integer;
  blnNegative: boolean;
begin
  Result := 0;
  i := 1;
  blnNegative := False;
  len := length(str);
  while (i <= len) and not (str[i] in ['0'..'9', '-']) do inc(i);
  if (i <= len) and (str[i] = '-') then begin
    blnNegative := True;
    inc(i);
  end;
  while (i <= len) and (str[i] in ['0'..'9', '.']) do
  begin
    if str[i] <> '.' then
      result := result * 10 + ord(str[i]) - ord('0');
    inc(i);
  end;
  if blnNegative then Result := -Result;
end;

function GetNumber(line: string; var index: integer): Single;
var
  start, code: integer;
  strTemp: string;
begin
  if Length(line) = 0 then begin
    Result := 0;
    Exit;
  end;
  start := scant(line, '0123456789.-', index);
  if start > 0 then begin
    index := scanT(line, ', '#160, start);
    if index = 0 then index := Length(line) + 1;
    strTemp := copy(line, start, index - start);
    Val(strTemp, Result, code);
  end else
    Result := 0;
end;

procedure ReadMap(const FileName: string; const turn: integer);
var
  intStart: integer;
  intEnd: integer;
  report: string;
  list: TStringList;
  minX, minY, maxX, maxY: single;
  strLine: string;
  index: integer;
  DX, DY: single;
  MapX, MapY: integer;
  Track: TTracks;
  i, iPos: integer;
  line: string;
begin
  report := MkStrFileLoad(FileName);

  SetLength(Report, DeleteC(report, #10));
  mkreExpr.Str := report;
  // maak zoekgebied kleiner
  MapX := Length('--------------------------------------------------------------------------------');
  mkreExpr.Pattern := '--------------------------------------------------------------------------------';
  intStart := mkreExpr.DoSearch(1);
  if intStart > 0 then begin
    intEnd := mkreExpr.DoSearch(intStart + 1);
    if intEnd > 0 then begin
      intStart := scanb(report, #13, intStart);
      if intStart < 1 then Exit;
      intStart := scanb(report, #13, intStart - 1);
      if intStart < 1 then Exit;
      inc(intStart);
      intEnd := scanc(report, #13, intEnd);
      if intEnd < 1 then Exit;
      intEnd := scanc(report, #13, intEnd + 1);
      if intEnd < 1 then Exit;

      report := copy(report, intStart, intEnd - intStart);
      list := TStringList.Create;
      try
        list.SetText(PChar(report));
        if list.Count > 4 then begin
          if list[1] <> '--------------------------------------------------------------------------------' then
            Exit;
          if list[list.Count - 2] <> '--------------------------------------------------------------------------------' then
            Exit;
          MapY := list.Count - 4;
          strLine := list[0];
          //0.00,0.00  400.00,0.00
          index := 1;
          minX := GetNumber(strLine, index);
          minY := GetNumber(strLine, index);
          strLine := list[list.count - 1];
          //0.00,0.00  400.00,0.00
          index := 1;
          GetNumber(strLine, index);
          GetNumber(strLine, index);
          maxX := GetNumber(strLine, index);
          maxY := GetNumber(strLine, index);
          Dx := (MaxX - minX) / Mapx;
          Dy := (MaxY - minY) / Mapy;

          for i := 2 to list.count - 3 do begin
            line := TabExpand(list[i]);
            iPos := scanc(line, '-', 1);
            while iPos > 0 do begin
              Track := TTracks.Create;
              Track.Turn := turn;
              Track.My := false;
              Track.X := trunc(minX + (iPos - 0.5) * Dx) * 100;
              Track.Y := trunc(minY + (i - 1.5) * Dy) * 100;
              TracksList.Add(Track);
              iPos := scanc(line, '-', iPos + 1);
            end;
          end;
        end;
      finally
        list.Free;
      end;
    end;
  end;
end;

function ReadGameType(strReportname: string): TGameType;
var
  report: TStringList;
  strNothing: string;
  FileStr: string;
begin
  report := TStringList.Create;
  try
    report.LoadFromFile(strReportname);
    FileStr := report.GetText;
    ReplaceC(FileStr, #$A0, #$20);
    report.SetText(PChar(FileStr));
    Result := SearchGameType(report, strNothing);
  finally
    report.Free;
  end;
end;

function GetGameStr: string;
begin
  Result := gamestr;
end;

procedure CalculateMaxPopulation(turn: integer);
var
  i, j: integer;
  player: Tplayer;
begin
  for i := 0 to players.Count - 1 do
  begin
    player := TPlayer(players.objects[i]);
    player.maxPopulation[turn] := 0;
    for j := 0 to planets.Count - 1 do
      if TPlanet(planets.Objects[j]).PlayerCur = player then
        player.maxPopulation[turn] := player.maxPopulation[turn] + TPlanet(planets.Objects[j]).Size;
  end;
end;

function ReadMKGalIni: boolean;
var
  inifile: TInifile;
  game: integer;
  strGameType: string;
  gmtp: TGameType;
begin
  DecimalSeparator := '.';
  ThousandSeparator := ',';
  Result := False;
  if FileExists(strIniFileName) then begin
    iniFile := TIniFile.Create(strIniFileName);
    try
      game := iniFile.ReadInteger('RUN', 'game', -1);
      if ParamCount > 0 then
      try
        game := StrToInt(ParamStr(1));
      except
        game := -1;
      end;
      if game <> -1 then
      begin
        gamestr := format('%d', [game]);
        strgameType := iniFile.ReadString(gamestr, 'GameType', '');
        inivars.Gametype := gtUnknown;
        for gmtp := low(TGameType) to high(TGameType) do
          if compareText(SGameStr[gmtp], strGameType) = 0 then
            inivars.Gametype := gmtp;
        if inivars.Gametype = gtUnknown then begin
          if iniFile.ReadBool(gamestr, 'blind', False) then begin
            inivars.Gametype := gtGalaxyBlind;
            iniFile.WriteString(gamestr, 'GameType', SGameStr[inivars.Gametype]);
          end;
        end;
        IniVars.outbackType := iniFile.ReadBool(gamestr, 'outbacktype', False);
        IniVars.bigGame := iniFile.ReadBool(gamestr, 'bigGame', True);
        strSpelD := iniFile.ReadString(gamestr, 'spelD', '');
        strSpelC := iniFile.ReadString(gamestr, 'spelC', '');
        IniVars.SkipPlanetsUntilTurn := iniFile.ReadInteger(gameStr, 'SkipPlanetsUntilTurn', 0);
        IniVars.SkipBattlesUntilTurn := iniFile.ReadInteger(gameStr, 'SkipBattlesUntilTurn', 0);
        IniVars.mapFaktor := StrToFloat(iniFile.ReadString(gamestr, 'mapFaktor', '2.5'));
        IniVars.mapScrollV := StrToFloat(iniFile.ReadString(gamestr, 'mapScrollV', '0'));
        IniVars.mapScrollH := StrToFloat(iniFile.ReadString(gamestr, 'mapScrollH', '0'));
        IniVars.mapFaktorOor := IniVars.mapFaktor;
        IniVars.mapScrollVOor := IniVars.mapScrollV;
        IniVars.mapScrollHOor := IniVars.mapScrollH;
        turn0HabitedPlanetsSize := iniFile.ReadInteger(gamestr, 'turn0HabitedPlanetsSize', 0);
        turn0HabitedPlanetsSize2nd := iniFile.ReadInteger(gamestr, 'turn0HabitedPlanetsSize2nd', 0);
        blindRepName := iniFile.ReadInteger(gamestr, 'blindRepName', 0);
        UseCountMethod := iniFile.ReadInteger(gamestr, 'UseCountMethod', 3);
        inivars.EliteNameCreation := iniFile.ReadBool('Run', 'EliteNameCreation', false); 
      end;
    finally
      iniFile.Free;
    end;
    if (strSpelD = '') or (strSpelC = '') then Exit;
    Result := True;
    SetConsts(IniVars.Gametype);
  end;
end;

procedure GetHeaders(strCurReportName: string);
var
  i: integer;
  str: string;
  status: integer;
begin
{  if pos('k034', strCurReportName) > 0 then
    ShowMessage('Problem Report');    }
  Headers.Clear;
  for i := 0 to TurnParse.Count - 1 do
  begin
    str := turnParse[i];
    if Length(str) > 2 then
      if str[1] = #9 then
        Headers.AddObject(turnParse[i], TObject(i));
  end;
  // check for errors
  i := headers.IndexOf(#9#9'MISTAKES');
  if i <> -1 then
    headers.Delete(i);

  i := 0;
  if inivars.Gametype = gtGalaxyBlind then status := 1
  else status := 0;
  while (i < headers.Count) do begin
    mkreExpr.Str := headers[i];
    case status of
      0: begin
          mkreExpr.Pattern := CSGAL_HEADERGalGame;
          if (Length(mkreExpr.Str) > 0) and (mkreExpr.DoSearch(1) > 0) then begin
            if inivars.Gametype = gtGalaxyBlind then status := 2
            else status := 1;
            inc(i);
            continue;
          end;
          headers.Delete(i);
        end;
      1: begin
          if inivars.Gametype = gtGalaxyNormal then begin
            status := 2;
            continue;
          end;

          mkreExpr.Pattern := CSGAL_HEADEROptions;
          if (Length(mkreExpr.Str) > 0) and (mkreExpr.DoSearch(1) > 0) then begin
            if inivars.Gametype = gtGalaxyBlind then status := 0
            else status := 2;
            inc(i);
            continue;
          end;
          headers.Delete(i);
        end;
      2: begin
          mkreExpr.Pattern := CSGAL_HEADERStatus;
          if (Length(mkreExpr.Str) > 0) and (mkreExpr.DoSearch(1) > 0) then begin
            if inivars.Gametype = gtGalaxyBlind then status := 3
            else status := 3;
            inc(i);
            continue;
          end;
          headers.Delete(i);
        end;
    else
      Exit;
    end;
  end;
end;

function GetHeaderLine(firstLine: integer; Search: string): integer;
var
  j: integer;
begin
  Result := -1;
  if headers.count = 0 then exit;
  if firstline < 0 then firstline := TurnParse.Count + firstline;
  mkreExpr.Pattern := Search;
  j := 0;
  while (j < headers.count) and (firstline > integer(Headers.Objects[j])) do
    inc(j);

  while j < headers.count do begin
    mkreExpr.Str := Headers.Strings[j];
    if (mkreExpr.DoSearchWithRange(1, 1) > 0) then
      result := integer(Headers.Objects[j]);
    if Result >= firstline then
      Exit;
    inc(j);
  end;
  Result := -1;
end;

procedure DeleteOldGroupsFromKnownPlanets(turn: integer);
var
  i, j, k: integer;
  player: TPlayer;
  planet: TPlanet;
begin
  for i := 0 to reportPlayer.Groups.Count - 1 do begin
    if TGroup(reportPlayer.groups.items[i]).Range = 0 then begin
      planet := TGroup(reportPlayer.groups.items[i]).planet;
      for j := 0 to players.Count - 1 do begin
        player := TPlayer(players.Objects[j]);
        if player <> reportPlayer then begin
          k := 0;
          while k < player.Groups.Count do begin
            if planet = TGroup(player.groups.items[k]).planet then begin
              if TGroup(player.groups.items[k]).turn < turn then begin
                player.groups.Delete(k);
                continue;
              end;
            end;
            inc(k);
          end;
        end;
      end;
    end;
  end;
  for j := 0 to players.Count - 1 do begin
    player := TPlayer(players.Objects[j]);
    if player <> reportPlayer then begin
      k := 0;
      while k < player.Groups.Count do begin
        if reportPlayer = TGroup(player.groups.items[k]).planet.PlayerCur then begin
          if TGroup(player.groups.items[k]).turn < turn then begin
            player.groups.Delete(k);
            continue;
          end;
        end;
        inc(k);
      end;
    end;
  end;
end;

function GetTurnNumberPlayerName(var turn: integer; var reportPlayerS: string;
  strCurReportName: string): boolean;
var
  strTemp: string;
  curLine: integer;
begin
  Result := False;
  // get turn number, Playername
  try
{    if pos('k008', strCurReportName) > 0 then
      ShowMessage('Problem Report');    }
    turn := -1;
    Result := False;
    strTemp := CSGAL_GalaxyInitString;
    curLine := GetHeaderLine(0, strTemp);
    if curLine > -1 then
    try
      // turn number

      if inivars.Gametype = gtGalaxyBlind then
        turn := TurnParse.ChildWords[curLine].AsInteger[-1] // Blind Galaxy
      else
        turn := TurnParse.ChildWords[curLine].AsInteger[CIGAL_GalaxyInitTurnPos]; // Gal NG
      // turn number
      strGameName := TurnParse.ChildWords[curLine][CIGAL_GalaxyInitGameNamePos]; // Gal NG

      // player Name
      if IniVars.outbackType then
      begin
        if TurnParse.ChildWords[curLine][-2] = 'Nation' then
          reportPlayerS := TurnParse.ChildWords[curLine][CIGAL_GalaxyInitPlayerNamePos - 1] +
            '_' + TurnParse.ChildWords[curLine][CIGAL_GalaxyInitPlayerNamePos]
        else
          reportPlayerS := TurnParse.ChildWords[curLine][CIGAL_GalaxyInitPlayerNamePos];
      end
      else
      begin
        if (TurnParse.ChildWords[curLine][CIGAL_GalaxyInitPlayerNamePos -1] = 'Nation') or
           (TurnParse.ChildWords[curLine][CIGAL_GalaxyInitPlayerNamePos -1] = 'Player') then
          reportPlayerS := TurnParse.ChildWords[curLine][CIGAL_GalaxyInitPlayerNamePos - 1] +
            '_' + TurnParse.ChildWords[curLine][CIGAL_GalaxyInitPlayerNamePos]
        else
          reportPlayerS := TurnParse.ChildWords[curLine][CIGAL_GalaxyInitPlayerNamePos];
      end
    except
      Turn := -1;
    end;
    if Turn = -1 then
  //    WriteLn(Format('Turn: %d Player: %s Report: %s !!!',
  //                   [turn, reportPlayerS, strCurReportName]))
    else
      Result := True;
  except
    ShowMessage(Format('Error - Turn: %d Player: %s Report: %s !'#13 +
      'While getting player name',
      [turn, reportPlayerS, strCurReportName]))
  end;
end;

function GetStatusOfPlayers(ntr, turn: integer; reportPlayerS: string): boolean;
var
  newPlayer: TPlayer;
  curLine,
    iPos1, i,
    plPosInFile,
    plPosInFileOpschuif: integer;
  nieuwePl: boolean;

  drive, weapons, shield, cargo, population, industry: single;
  planets: integer;
  name, lowername, lowerReportPlayerS, wr: string;
  war: boolean;

  function GetNewPlayerNr: integer;
  var
    j: integer;
  begin
    Result := 1;
    for j := 0 to players.Count - 1 do
    begin
      if TPlayer(players.objects[j]).Nr >= Result then
        Result := TPlayer(players.objects[j]).Nr + 1;
    end;
  end;
begin
  // get turn number, Playername
  Result := False;
  lowerReportPlayerS := lowercase(ReportPlayerS);
  curLine := GetHeaderLine(0, CSGAL_StatPlayersString);
  if curLine <> -1 then
  begin
    inc(curline);
    while TurnParse[curLine] = '' do inc(curline);
    inc(curline);
    plPosInFile := -1;
    plPosInFileOpschuif := -1;
    for i := 0 to players.Count - 1 do
      TPlayer(players.Objects[i]).InFile := False;
    while (curLine < TurnParse.Count) and (TurnParse[curLine] <> '') do
    begin
      inc(plPosInFile);
      inc(plPosInFileOpschuif);
      // presume that underscores are used!!
      iPos1 := 0;
      name := TurnParse.ChildWords[curLine][CIGAL_StatPlayersplayerNamePos];
      lowername := lowercase(name);
      drive := TurnParse.ChildWords[curLine].AsSingle[CIGAL_StatPlayersDrivePos];
      weapons := (TurnParse.ChildWords[curLine]).AsSingle[CIGAL_StatPlayersWeaponsPos];
      shield := TurnParse.ChildWords[curLine].AsSingle[CIGAL_StatPlayersShieldPos];
      cargo := TurnParse.ChildWords[curLine].AsSingle[CIGAL_StatPlayersCargoPos];
      population := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_StatPlayersPopulationPos];
      industry := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_StatPlayersIndustryPos];
      planets := TurnParse.ChildWords[curLine].AsInteger[iPos1 + CIGAL_StatPlayersPlanetsPos];
      // in blind geen war met jezelf
      if (inivars.Gametype in [gtGalaxyBlind, gtGalaxyNormal]) and
        (lowerReportPlayerS = lowername) then
        War := False
      else
      begin
        wr := TurnParse.ChildWords[curLine][iPos1 + CIGAL_StatPlayersWarPos];
        War := (wr = 'War') or (wr = 'W');
      end;
      nieuwePl := True; // we gaan er van uit dat dit een nieuwe speler is
      i := 0;
      while (i < players.Count) and nieuwePl do
      begin
        if not TPlayer(players.Objects[i]).LigtEruit then
          if lowercase(TPlayer(players.Objects[i]).Name) = lowername then
          begin
            nieuwePl := False;
            // al in file, onthoud waar hij stond in vorig rapport
            plPosInFileOpschuif := TPlayer(players.Objects[i]).PosInFile;
            TPlayer(players.Objects[i]).PosInFile := plPosInFile;
            TPlayer(players.Objects[i]).InFile := True;
          end;
        if nieuwePl then inc(i);
      end;

      if (ntr > 0) and ((inivars.Gametype <> gtGalaxyBlind)
                        or ((inivars.Gametype = gtGalaxyBlind) and (ntr <= 3))) then
        if nieuwePl then
        begin
          i := 0;
          while (i < players.Count) and nieuwePl do
          begin
            if not TPlayer(players.Objects[i]).LigtEruit then
              if (TPlayer(players.Objects[i]).PosInFile = plPosInFileOpschuif) then
              begin
                nieuwePl := False;
                TPlayer(players.Objects[i]).PosInFile := plPosInFile;
                TPlayer(players.Objects[i]).Name := name;
                TPlayer(players.Objects[i]).InFile := True;
              end;
            if nieuwePl then inc(i);
          end;
        end;
      if nieuwePl then
      begin
        newPlayer := TPlayer.Create;
        players.AddObject(lowername, newPlayer);
      end
      else
      begin
        newPlayer := TPlayer(players.Objects[i]);
        if players.Strings[i] <> lowername then
        begin
          players.delete(i);
          players.AddObject(lowername, newPlayer);
        end;
      end;
      //gtGalaxyNGBlind
      if inivars.Gametype = gtGalaxyNGBlind then begin
        //only a update when the player was not available
        if newPlayer.Industry[turn] = 0 then begin
          newPlayer.SetPlayer(turn, name, drive, weapons, shield, cargo, population,
            industry, planets, war);
        end;
      end else
        newPlayer.SetPlayer(turn, name, drive, weapons, shield, cargo, population,
          industry, planets, war);
      newPlayer.PosInFile := plPosInFile;
      newPLayer.InFile := True;
      inc(curLine);
    end;
    for i := 0 to players.Count - 1 do
    begin
      if not TPlayer(players.Objects[i]).InFile then
        TPlayer(players.Objects[i]).LigtEruit := True;
      if TPlayer(players.Objects[i]).Nr = -1 then
        TPlayer(players.Objects[i]).Nr := GetNewPlayerNr;
    end;
  end;
end;

function SearchForPlanet(const x, y: integer;
  const lowerplanetname: string;
  const turn: integer): TPlanet;
var
  sinX, sinY: Single;
  DistToNull: Single;
  indexDist, CountDist: integer;
  i: integer;
  extDist, extX, extY: Int64;
  pl, pl2: TPlanet;
  extX1, extY1, extDist1: Int64;
begin
  sinX := x;
  sinY := y;
  DistToNull := sqrt(sinX * sinX + sinY * sinY);
  planetsDist.FindRange(trunc(DistToNull), indexDist, CountDist);
  i := 0;
  while i < CountDist do begin
    if (TPlanet(planetsDist.Objects[indexDist + i]).x = x) and
      (TPlanet(planetsDist.Objects[indexDist + i]).y = y) then
      break
    else
      inc(i);
  end;
  if i < CountDist then begin
    inc(indexDist, i);
    if Planets.Find(planetsDist[indexDist], i) then
      Planets.Delete(i)
    else
      raise Exception.CreateFmt('Error while reading planets - Turn: %d', [turn]);
    Result := TPlanet(planetsDist.objects[indexDist]);
    planetsDist.Delete(indexDist);
  end else begin
    if inivars.Gametype = gtGalaxyNGBlind then begin
      Result := TPlanet.Create;
    end else begin
      if firstturn <> turn then begin
        extDist := high(INT64);
        extX := X;
        extY := Y;
        pl2 := nil;
        i := 0;
        while (i < Planets.Count) do begin
          pl := TPlanet(Planets.Objects[i]);
          extX1 := pl.X;
          extY1 := pl.y;
          extDist1 := (extX1 - extX) * (extX1 - extX) + (extY1 - extY) * (extY1 - extY);
          begin
            if extDist1 < extDist then begin
              extDist := extDist1;
              pl2 := pl;
            end;
            inc(i);
          end;
        end;
        if Assigned(pl2) then begin
          Result := pl2;
          if Planets.Find(lowercase(Result.Name), i) then
            Planets.Delete(i)
          else
            raise Exception.CreateFmt('Error while reading planets - Turn: %d '#13+
                                      'Planet %s not found', [turn,Result.Name]);
          indexDist := planetsDist.IndexOfObject(Result);
          planetsDist.Delete(indexDist);
        end else
          raise Exception.CreateFmt('Error while reading planets - Turn: %d', [turn]);
      end else
        Result := TPlanet.Create;
    end;
  end;
  Planets.AddObject(lowerPlanetName, Result);
  i := planetsDist.Add(trunc(DistToNull), lowerPlanetName);
  planetsDist.objects[i] := Result;
end;

function GetTurnPlayerPlanets(turn: integer; reportPlayerS, strCurReportName: string): boolean;
var
  curLine,
    iPos1, iPos2, i: integer;

  x, y: integer;
  size, population, industry, resources: single;
  name, lowerPlanetName, prod, ProduceType: string;
  producing: integer;
  capital, material, colonist: single;
  nr: integer;
  newPlanet: TPlanet;

begin
  //
  Result := False;

  curLine := GetHeaderLine(0, CSGAL_PlayersPlanetsString);
  if curLine > -1 then
  try
    inc(curline);
    while TurnParse[curLine] = '' do inc(curline);
    inc(curline);
    while (curLine < TurnParse.Count) and
      (TurnParse[curLine] <> '') do
    begin
      producing := 0;
      nr := -1;
      iPos1 := 0;
      name := TurnParse.ChildWords[curLine][CIGAL_PlayersPlanetsPlanetNamePos];
      if inivars.Gametype = gtGalaxyBlind then
      begin
        nr := TurnParse.ChildWords[curLine].AsInteger[CIGAL_PlayersPlanetsPlanetNamePos + 1];
        lowerPlanetName := mkItoa(nr);
      end else begin
        lowerPlanetName := lowercase(Name);
        if turn = 0 then begin
          nr := mkAToI(name);
        end else begin
          nr := gPlanetNumber;
          inc(gPlanetNumber);
        end;
      end;
      //ipos1 = 0 / 1
      x := GetXorYCoord(TurnParse.ChildWords[curLine].AsString[iPos1 + CIGAL_PlayerPlanetsXPos]);
      y := GetXorYCoord(TurnParse.ChildWords[curLine].AsString[iPos1 + CIGAL_PlayerPlanetsYPos]);

      size := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_PlayersPlanetsSizePos]; //4
      population := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_PlayersPlanetsPopulationPos]; //5
      industry := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_PlayersPlanetsIndustryPos]; //6
      resources := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_PlayersPlanetsResourcesPos]; //7
      prod := TurnParse.ChildWords[curLine].AsString[iPos1 + CIGAL_PlayersPlanetsProdPos]; //8

      iPos2 := 0;
      capital := TurnParse.ChildWords[curLine].AsSingle[iPos2 + CIGAL_PlayersPlanetsCapitalPos]; //10
      material := TurnParse.ChildWords[curLine].AsSingle[iPos2 + CIGAL_PlayersPlanetsMaterialPos]; //11
      colonist := TurnParse.ChildWords[curLine].AsSingle[iPos2 + CIGAL_PlayersPlanetsColonistPos]; //9

      ProduceType := prod;
      if CompareText(prod, 'cap') = 0 then Producing := P_CAP
      else
        if CompareText(prod, 'mat') = 0 then Producing := P_MAT;

      if not Planets.Find(lowerPlanetName, i) then i := -1;

      if i = -1 then
        newPlanet := SearchForPlanet(x, y, lowerPlanetName, turn)
      else
        newPLanet := TPlanet(Planets.Objects[i]);
      newPlanet.SetPlanet(turn, name, ProduceType, x, y, size, population, industry,
        resources, producing, capital, material, colonist);
      newPlanet.player[turn] := reportPlayer;
      if (turn = FirstTurn) then newPlanet.Nr := nr;
      inc(curline);
    end;
    Result := True;
  except
    on e: Exception do
    ShowMessage(Format('Error - Turn: %d Player: %s Report: %s !'#13 +
      'While getting the planets of this player'#13+
      e.message,
      [turn, reportPlayerS, strCurReportName]))
  end;
end;

function GetOtherPlanets(turn: integer; reportPlayerS, strCurReportName: string): boolean;
var
  strTemp: string;
  curLine: integer;
  x, y: integer;
  size, resources: single;
  name, lowerPlanetName: string;
  nr, i: integer;
  newPlanet: TPlanet;
  iUp, iPos1, n: integer;
begin
  //
  Result := False;

  for iUp := 0 to 1 do
  begin
    case iUp of
      0: strTemp := CSGAL_OtherPlanetsUnidentString;
      1: strTemp := CSGAL_OtherPlanetsUninhabString;
    end;
    curLine := GetHeaderLine(0, strTemp);
    if curLine > -1 then
    try
      inc(curline);
      while TurnParse[curLine] = '' do inc(curline);
      inc(curline);
      while (curLine < TurnParse.Count) and
        (TurnParse.ChildWords[curLine].Count >= 3) do
      begin
        nr := -1;
        iPos1 := 0;
        size := 0;
        resources := 0;
        name := TurnParse.ChildWords[curLine][CIGAL_OtherPlanetsPlanetNamePos];
        if inivars.Gametype = gtGalaxyBlind then begin
          if (iUp = 1) then begin
            nr := TurnParse.ChildWords[curLine].AsInteger[CIGAL_OtherPlanetsPlanetNrPos];
            lowerPlanetName := mkItoa(nr);
            inc(iPos1);
          end else begin
            nr := mkAtoi(Name)
          end;
          lowerPlanetName := mkItoa(nr)
        end else begin
          lowerPlanetName := lowercase(name);
          if turn = 0 then
            nr := mkAToI(name)
          else begin
            nr := gPlanetNumber;
            inc(gPlanetNumber);
          end;
        end;

        x := GetXorYCoord(TurnParse.ChildWords[curLine].AsString[iPos1 + CIGAL_OtherPlanetsXPos]);
        y := GetXorYCoord(TurnParse.ChildWords[curLine].AsString[iPos1 + CIGAL_OtherPlanetsYPos]);
        if iUp = 1 then
        begin
          if TurnParse.ChildWords[curLine].Count >= CIGAL_OtherPlanetsResourcePos then
          begin
            size := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_OtherPlanetsSizePos];
            resources := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_OtherPlanetsResourcePos];
          end;
        end;
        if Planets.Find(lowerPlanetName, i) then
          newPlanet := TPlanet(Planets.Objects[i])
        else
          newPlanet := SearchForPlanet(x, y, lowerPlanetName, turn);
        newPlanet.Name := name;
        if turn = FirstTurn then
          newPlanet.Nr := nr;
        newPlanet.X := x;
        newPlanet.y := y;
        if (GameNr = 1) then
          newPlanet.ProduceType := '';
        //gtGalaxyNGBlind
        if (inivars.Gametype <> gtGalaxyBlind)
           and (inivars.Gametype <> gtGalaxyNGBlind) then begin
          if (iUp = 0) then begin
            if (not Assigned(newPlanet.Player[turn])) or
              (newPlanet.Player[turn] = TPlayer(1)) then
              newPlanet.Player[turn] := TPlayer(1);
          end else
            newPlanet.Player[turn] := nil;
        end else
          newPlanet.Player[turn] := nil;
        if size > 0 then begin
          newPlanet.Size := size;
          newPlanet.Resources := resources;
        end;
{ uitgezet, alleen maar verwarrend}
{        if (turn = 0) and (iniVars.gametype <> gtGalaxyBlind) and (iUp = 0) then
          if not iniVars.outbackType then begin
            if (turn0HabitedPlanetsSize > 0) then
              if Assigned(newPlanet.PlayerCur) then
                newPlanet.Size := turn0HabitedPlanetsSize;
          end else
            for i := 0 to 1 do begin
              n := mkAtoi(name) - 1;
              for nr := 0 to players.Count - 1 do
                if mkAtoi(TPlayer(players.Objects[nr]).name) = n then begin
                  newPlanet.Player[turn] := TPlayer(players.Objects[nr]);
                  newPlanet.Size := 1000;
                  newPlanet.Population[turn] := 1000;
                  newPlanet.Industry[turn] := 1000;
                  newPlanet.Resources := 10;
                end;
            end;}
        inc(curline);
      end;
      Result := True;
    except
    on e: Exception do
      ShowMessage(Format('Error - Turn: %d Player: %s Report: %s !'#13 +
        'While getting the unknown planets of other players'#13+
        e.Message,
        [turn, reportPlayerS, strCurReportName]))
    end;
  end;
end;

function GetPlayerPlanets(turn: integer; reportPlayerS, strCurReportName: string): boolean;
var
  strTemp: string;
  curLine,
    iPos1, iPos2, i, j: integer;

  x, y: integer;
  size, population, industry, resources: single;
  name, lowerPlanetName, prod: string;
  ProduceType: string;
  producing: integer;
  capital, material, colonist: single;
  nr: integer;
  sinX, sinY, DistToNull: Single;
  indexDist, countDist: integer;

  newPlanet: TPlanet;
  tmpPlanet: TPlanet;
  curPlayer: TPlayer;
{$IFNDEF PERFORM}
  mwFastTime: TmwFastTime;
{$ENDIF}
begin
  //
  Result := False;
  for j := 0 to players.Count - 1 do
  begin
    curPlayer := TPlayer(players.Objects[j]);
    strTemp := curPlayer.Name + ' ' + CSGAL_PlayerPlanetsString;
    curLine := GetHeaderLine(0, strTemp);
    if curLine > -1 then
    try
      inc(curline);
      while TurnParse.AsString[curLine] = '' do inc(curline);
      inc(curline);
      while (curLine < TurnParse.Count) and
        (TurnParse.AsString[curLine] <> '') do
      begin
        producing := 0;
        nr := -1;
        iPos1 := 0;
        name := TurnParse.ChildWords[curLine].AsString[CIGAL_PlayerPlanetsPlanetNamePos];
        if inivars.Gametype = gtGalaxyBlind then
        begin
          nr := TurnParse.ChildWords[curLine].AsInteger[CIGAL_PlayerPlanetsPlanetNamePos + 1];
          lowerPlanetName := mkItoa(nr);
        end
        else
        begin
          lowerPlanetName := lowercase(name);
          if turn = 0 then
            Nr := StrToInt(name)
          else begin
            Nr := gPlanetNumber;
            inc(gPlanetNumber);
          end;
        end;
        x := GetXorYCoord(TurnParse.ChildWords[curLine].AsString[iPos1 + CIGAL_PlayerPlanetsXPos]);
        y := GetXorYCoord(TurnParse.ChildWords[curLine].AsString[iPos1 + CIGAL_PlayerPlanetsYPos]);

        size := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_PlayerPlanetsSizePos];
        population := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_PlayerPlanetsPopulationPos];
        industry := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_PlayerPlanetsIndustryPos];
        resources := TurnParse.ChildWords[curLine].AsSingle[iPos1 + CIGAL_PlayerPlanetsResourcesPos];
        prod := TurnParse.ChildWords[curLine].AsString[iPos1 + CIGAL_PlayerPlanetsProdPos];
        iPos2 := 0;
        capital := TurnParse.ChildWords[curLine].AsSingle[iPos2 + CIGAL_PlayerPlanetsCapitalPos];
        material := TurnParse.ChildWords[curLine].AsSingle[iPos2 + CIGAL_PlayerPlanetsMaterialPos];
        colonist := TurnParse.ChildWords[curLine].AsSingle[iPos2 + CIGAL_PlayerPlanetsColonistPos];

        ProduceType := prod;
        if prod = 'cap' then Producing := P_CAP
        else
          if prod = 'mat' then Producing := P_MAT;
        if Planets.Find(lowerPlanetname, i) then
          newPLanet := TPlanet(Planets.Objects[i])
        else
          newPlanet := SearchForPlanet(x, y, lowerPlanetName, turn);
        newPlanet.SetPlanet(turn, name, ProduceType, x, y, size, population, industry,
          resources, producing, capital, material, colonist);
        newPlanet.player[turn] := curPlayer;
        if turn = FirstTurn then
          newPlanet.Nr := nr;
        inc(curline);
      end;
      Result := True;
    except
    on e: Exception do
      ShowMessage(Format('Error - Turn: %d Player: %s Report: %s !'#13 +
        'While getting the known planets of other players'#13+
        e.Message,
        [turn, reportPlayerS, strCurReportName]))
    end;
  end;
end;

function GetShipTypes(turn: integer; reportPlayerS, strCurReportName: string): boolean;
var
  strTemp: string;
  curLine,
    i, j: integer;

  Name: string;
  drive, weapons, shields, cargo, mass: single;
  attacks: integer;

  curPlayer: TPlayer;
  st: TShipType;
begin
  Result := False;
  for j := 0 to players.Count do // 1 more than normally, used for 'your'
  begin
    if j = players.Count then
    begin
      strTemp := CSGAL_ShipTypesYourString;
      curPlayer := reportPlayer;
    end
    else
    begin
      curPlayer := TPlayer(players.Objects[j]);
      strTemp := curPlayer.Name + CSGAL_ShipTypesOtherString;
    end;
    curLine := GetHeaderLine(0, strTemp);
    if curLine > -1 then
    try
      inc(curline);
      while TurnParse[curLine] = '' do inc(curline);
      inc(curline);
      while (curLine < TurnParse.Count) and
        (TurnParse[curLine] <> '') do
      begin
        name := TurnParse.ChildWords[curLine][CIGAL_ShipTypesNamePos];
        drive := TurnParse.ChildWords[curLine].AsSingle[CIGAL_ShipTypesDrivePos];
        attacks := TurnParse.ChildWords[curLine].Asinteger[CIGAL_ShipTypesAttacksPos];
        weapons := TurnParse.ChildWords[curLine].AsSingle[CIGAL_ShipTypesWeaponsPos];
        shields := TurnParse.ChildWords[curLine].AsSingle[CIGAL_ShipTypesShieldsPos];
        cargo := TurnParse.ChildWords[curLine].AsSingle[CIGAL_ShipTypesCargoPos];
        if (inivars.Gametype = gtGalaxyNGBlind) then begin
          mass := TurnParse.ChildWords[curLine].AsSingle[6];
        end else begin
          mass := 0;
        end;

        if not curplayer.ShipTypes.Find(Name, i) then begin
          st := TShipType.Create;
          curPlayer.ShipTypes.AddObject(Name, st);
          st.Name := name;
          st.Drive := round(drive * 100) / 100;
          st.Attacks := attacks;
          st.Weapons := weapons;
          st.Shields := shields;
          st.Cargo := cargo;
          st.IsNew := false;
          st.NGBlindMass := mass;
        end;
        inc(curline);
      end;
      Result := True;
    except
    on e: Exception do
      ShowMessage(Format('Error - Turn: %d Player: %s Report: %s !'#13 +
        'While getting shiptypes'#13+
        e.Message,
        [turn, reportPlayerS, strCurReportName]))
    end;
  end;
end;

function GetBombings(turn: integer; reportPlayerS, strCurReportName: string): boolean;
var
  curLine: integer;
  bombing: TBombing;
  strBombing,
    strProducing: string;
  intProducing: integer;
begin
  Result := True;
  curLine := GetHeaderLine(0, CSGAL_BombingsString);
  if curLine > -1 then
  begin
    // Skip empty rows
    inc(CurLine, 2);

    while (curLine < TurnParse.Count) and
      (TurnParse[curLine] = '') do
      inc(curLine);
    inc(curLine);
    while TurnParse[curLine] <> '' do
    begin
      bombing := TBombing.Create;
      try
        bombing.Turn := turn;
        bombing.Attacker := FindPlayerWithName(TurnParse.ChildWords[curLine].AsString[CIGAL_BombingsAttackerPos]);
        bombing.Defender := FindPlayerWithName(TurnParse.ChildWords[curLine].AsString[CIGAL_BombingsDefenderPos]);
        bombing.Planet := FindPlanetWithName(TurnParse.ChildWords[curLine].AsString[CIGAL_BombingsPlanetPos]);
        bombing.Population := TurnParse.ChildWords[curLine].AsSingle[CIGAL_BombingsPopulationPos];
        bombing.Industry := TurnParse.ChildWords[curLine].AsSingle[CIGAL_BombingsIndustryPos];
        if inivars.Gametype <> gtGalaxyBlind then begin
          strProducing := TurnParse.ChildWords[curLine].AsString[CIGAL_BombingsProducingPos];
          intProducing := 0;
          if strProducing = 'CAP' then intProducing := P_CAP
          else
            if strProducing = 'COL' then intProducing := P_COL
            else
              if strProducing = 'MAT' then intProducing := P_MAT;
          bombing.Producing := intProducing;
        end;
        bombing.Cap := TurnParse.ChildWords[curLine].AsSingle[CIGAL_BombingsCapPos];
        bombing.Mat := TurnParse.ChildWords[curLine].AsSingle[CIGAL_BombingsMatPos];
        bombing.Col := TurnParse.ChildWords[curLine].AsSingle[CIGAL_BombingsColPos];
        strBombing := Format('%d$%s', [turn, TurnParse.ChildWords[curLine].AsString[2]]);
        if (Bombings.IndexOf(strBombing) = -1) and
          Assigned(bombing.Attacker) and Assigned(bombing.Defender) and
          Assigned(bombing.Planet) then
        begin
          Bombings.AddObject(strBombing, bombing);
          if bombing.Planet.Size = 0 then
            bombing.Planet.Size := bombing.Population;

          if integer(bombing.Planet.PlayerCur) = 1 then
          try
            bombing.Planet.PopulationCur := (bombing.Population / 4) * 1.08;
            bombing.Planet.IndustryCur := bombing.Industry / 4;
          except
            ;
          end;
          if Assigned(bombing.Planet.Player[turn]) then
            bombing.Planet.Player[turn] := bombing.Attacker;
        end
        else
          Bombing.Free;
      except
        bombing.Free;
      end;
      inc(CurLine);
    end;
  end;
end;

{
We will need this when we are skipping the battles, but not the groups

procedure GetLastBattleLine;
var
  i: integer;
begin
  // Get LastBattleLine
  LastBattleLine := 0;
  i := 0;
  mkreExpr.Pattern := '(Your Planets)|(Unidentified Planets)|(Unknown Systems)';
  while i < headers.Count do begin
    mkreExpr.Str := headers[i];
    if mkreExpr.DoSearchWithRange(1, 1) > 0 then
      LastBattleLine := integer(Headers.Objects[i]);
    inc(i);
  end;
end; }

function GetBattles(turn: integer; reportPlayerS, strCurReportName: string): boolean;
var
  curLine, nextBattleLine,
    i, j: integer;

  name: string;
  drive, weapons, shields, cargo: single;
  attacks: integer;

  // general
  nleft, nr, aw: integer;
  shiptype, cargoType: string;
  cargoQuantity: single;
  // blind
  effDrive, effWeapon, effShield, effCargo: single;
  // normal
  driveTech, weaponTech, shieldTech, cargoTech: single;


  bBestaat,
    blnBattleExists: boolean;
  curPlayer,
    lastKnownPlayer,
    MostMassPlayer, tmpPlayer: TPlayer;
  mostMass, Mass: single;
  curPlanet, planet: TPlanet;
  bt: TBattle;
  gr: TGroup;
  planetName, lowerPlanetName, lowerPlayerName, playerName: string;
  st: TShipType;
  player: TPlayer;
  group: TGroup;
begin
  //
  Result := False;
{  if pos('k040', strCurReportName) > 0 then
    ShowMessage('Problem Report');   }
  LastBattleLine := 0;
  curLine := GetHeaderLine(0, CSGAL_BattlesString);
  while curLine > -1 do
  try
    planetName := TurnParse.ChildWords[curLine][CIGAL_BattlesPlanetNamePos];
    if inivars.Gametype = gtGalaxyBlind then begin
      planetName := TurnParse.ChildWords[curLine][CIGAL_BattlesPlanetNamePos];
      if planetName[1] = '(' then
        planetName := TurnParse.ChildWords[curLine][CIGAL_BattlesPlanetNamePos - 1];
    end;
    lowerPlanetName := lowercase(planetName);
    curPlanet := nil;
    blnBattleExists := Battles.Find(mkItoa(turn) + #9 + lowerPlanetName, i);
    if Planets.Find(lowerPlanetName, i) then
    begin
      bt := nil;
      if not blnBattleExists then
      begin
        curPlanet := TPlanet(Planets.Objects[i]); //  TPlanet(Planets.Items[i]);
        // alle schepen op die planeet weggooien; behalve van qReportPlayer en
        // reportPlayer, ook alle oude groupen van deze planeet verwijderen
        for i := 0 to players.Count - 1 do begin
          curPlayer := TPlayer(Players.Objects[i]);
          j := 0;
          while j < curPlayer.Groups.Count do begin
            group := TGroup(curPlayer.Groups.items[j]);
            if group.planet = curPlanet then begin
              curPlayer.Groups.Delete(j);
              Continue;
            end;
            inc(j);
          end;
        end;
        bt := TBattle.Create;
        Battles.AddObject(mkItoa(turn) + #9 + planetname, bt);
        bt.Turn := turn;
        bt.Planet := curPlanet;
      end;
      // get the next battle , if it exists
      inc(curLine, 2);
      nextBattleLine := GetHeaderLine(curLine, CSGAL_BattlesString);
      if nextBattleLine = -1 then nextBattleLine := TurnParse.Count; // just greater than curLine;
      while (curLine < TurnParse.Count) and
        (nextBattleLine > curLine) and
        (TurnParse[curLine] = '') do
        inc(curLine);
      while (curLine < TurnParse.Count) and
        (nextBattleLine > curLine) and
        (Pos('Groups', TurnParse.AsString[curLine]) > 0) do
      begin
        playerName := TurnParse.ChildWords[curLine].AsString[CIGAL_BattlesPlanerNamePos];
        if inivars.Gametype = gtGalaxyNGBlind then
          if CompareText(playerName, 'Incoming') = 0 then
            break;
        if playerName = 'Your' then curPlayer := reportPlayer
        else
        begin
          curplayer := nil;
          if Players.Find(lowercase(playerName), i) then
            curPlayer := TPlayer(Players.Objects[i])
          else
            raise Exception.Create('No such player!!');
        end;
        inc(curLine);
        while (curLine < TurnParse.Count) and
          (TurnParse[curLine] = '') do inc(curline); // skip empty rows
        inc(curline); // skip header # N etc
        while (curLine < TurnParse.Count) and
          (TurnParse[curLine] <> '') do
        begin
                                              // start reading this player
          if not blnBattleExists then
          begin
            nleft := TurnParse.ChildWords[curLine].AsInteger[CIGAL_BattlesGrnLeftPos];
            nr := TurnParse.ChildWords[curLine].AsInteger[CIGAL_BattlesGrNrPos];
            shiptype := lowercase(TurnParse.ChildWords[curLine][CIGAL_BattlesGrShipTypePos]);
            cargoType := TurnParse.ChildWords[curLine][CIGAL_BattlesGrCargoTypePos];
            driveTech := TurnParse.ChildWords[curLine].AsSingle[CIGAL_BattlesDrivePos];
            weaponTech := TurnParse.ChildWords[curLine].AsSingle[CIGAL_BattlesWeaponPos];
            shieldTech := TurnParse.ChildWords[curLine].AsSingle[CIGAL_BattlesShieldPos];
            cargoTech := TurnParse.ChildWords[curLine].AsSingle[CIGAL_BattlesCargoPos];
            st := nil;
            if curplayer.shiptypes.Find(lowercase(shiptype), i) then
              st := TShipType(curplayer.shiptypes.Objects[i])
            else
              raise Exception.Create('Shiptype missing!!!');

            if inivars.gameType = gtGalaxyBlind then
            begin
              if st.Drive > 0 then drivetech := drivetech / st.Drive
              else driveTech := 0;
              if st.weapons > 0 then weapontech := weapontech / st.weapons
              else weaponTech := 0;
              if st.Shields > 0 then
                Shieldtech := (ShieldTech * (power(st.Mass, 0.33333333)) / 3.107232506) / st.Shields
              else ShieldTech := 0;
              if (st.Cargo + st.Cargo * st.Cargo / 10) > 0 then
                cargotech := cargotech / (st.Cargo + st.Cargo * st.Cargo / 10)
              else cargotech := 0;
            end;

            if nLeft > 0 then begin
              if ((curPlayer <> qReportPlayer) and (curPlayer <> reportPlayer)) then begin
                gr := TGroup.Create;
                curPlayer.Groups.Add(gr);

                gr.Bat := 1;
                gr.AantalBattle := nr;
                gr.Aantal := nLeft;
                gr._Type := st;

                gr.DriveTech := drivetech;
                gr.WeaponsTech := weapontech;
                gr.ShieldsTech := Shieldtech;
                gr.CargoTech := cargotech;
                gr.Planet := curPlanet;
                gr.player := curPlayer;
                gr.Turn := turn;
              end;
            end;
            // nu een groep maken voor een battle
            gr := TGroup.Create;
            bt.Groups.Add(gr);
            gr.AantalBattle := nLeft;
            gr.Aantal := nr;
            gr._Type := st;
            gr.DriveTech := drivetech;
            gr.WeaponsTech := weapontech;
            gr.ShieldsTech := Shieldtech;
            gr.CargoTech := cargotech;
            gr.Planet := curPlanet;
            gr.player := curPlayer;
            gr.Turn := turn;
          end;
          inc(curLine);
        end;
        // groups from 1 player parsed
        inc(curLine);
        while (curLine < TurnParse.Count) and
          (nextBattleLine > curLine) and
          (TurnParse[curLine] = '') do
          inc(curLine);
      end;
      // battle parsed
      // player is unknow
      // als er een battle is geweest kun je daaruit afleiden van wie de planeet waarschijnlijk is
      if not blnBattleExists and Assigned(bt) and Assigned(bt.planet) and
        (bt.planet.PlayerCur = TPlayer(1)) then
      begin
        planet := bt.Planet;
        lastKnownPlayer := nil;
        for i := 0 to turn - 1 do
          if (bt.Planet.Player[i] <> nil) and (bt.Planet.Player[i] <> TPlayer(1)) then
            lastKnownPlayer := bt.Planet.Player[i];
        bBestaat := False;
        if Assigned(lastKnownPlayer) then
        begin
          i := 0;
          while (i < bt.Groups.Count) and not bBestaat do
          begin
            if (TGroup(bt.Groups.Items[i]).Player = lastKnownPlayer) and
              (TGroup(bt.Groups.Items[i]).AantalBattle > 0) then
              bBestaat := True;
            inc(i);
          end;
        end;
        if not bBestaat then // laatst bekende speler was aanwezig bij deze battle
                             // en had nog schepen over
        begin
          // zoeken naar speler met meeste massa
          mostMass := 0;
          Mass := 0;
          MostMassPlayer := nil;
          tmpPlayer := nil;
          for i := 0 to bt.Groups.Count - 1 do
          begin
            if TGroup(bt.Groups.Items[i]).Player <> tmpPlayer then
            begin
              if (Mass > mostMass) then
              begin
                MostMassPlayer := tmpPlayer;
                mostMass := Mass;
              end;
              tmpPlayer := TGroup(bt.Groups.Items[i]).Player;
              Mass := 0;
            end;
            Mass := Mass + TGroup(bt.Groups.Items[i]).AantalBattle *
              TGroup(bt.Groups.Items[i])._type.Mass;
          end;
          if (Mass > mostMass) then
            lastKnownPlayer := tmpPlayer
          else
            lastKnownPlayer := MostMassPlayer;
        end;
        bt.Planet.LastOwnerObject := lastKnownPlayer;
        if Assigned(lastKnownPlayer) then
          bt.planet.LastOwnerName := lastKnownPlayer.Name;
        bt.Planet.LastOwnerType := 2;
        bt.Planet.LastOwnerTurn := turn;
      end;
    end
    else
      inc(curLine);
    LastBattleLine := curLine; // onthouden waar de laatste battle gelezen was,
                               // hierna beginnen ergens de groups.
    // find next battle
    curLine := GetHeaderLine(curLine, CSGAL_BattlesString);
  except
    on e: Exception do
    ShowMessage(Format('Error - Turn: %d Player: %s Report: %s !'#13 +
      'While reading battles'#13+
      e.Message,
      [turn, reportPlayerS, strCurReportName]))
  end;
end;

procedure GetRoutes(turn: integer; reportPlayerS, strCurReportName: string);
var
  curLine: integer;
  i: integer;
  strFrom, strCOL, strCAP, strMAT, strEMP: string;
  planet: TPlanet;
  Route: TRoute;
  strName: string;
begin
  for i := 0 to routes.Count - 1 do
    TRoute(routes.objects[i]).Free;
  routes.Clear;
  curLine := GetHeaderLine(1, #9#9 + 'Your Routes');
  if (curLine > -1) and (curLine < TurnParse.Count) then begin
    inc(curLine);
    while (curLine < TurnParse.Count) and
      (TurnParse.AsString[curLine] = '') do inc(curline); // skip empty rows
    inc(curline); // skip header # N etc
    while (curLine < TurnParse.Count) and (TurnParse.ChildWords[curline].Count > 4) do
    begin
    // blind: N C $ M E
      if inivars.Gametype = gtGalaxyBlind then begin
        strFrom := TurnParse.ChildWords[curLine][0];
        strCAP := TurnParse.ChildWords[curLine][2];
        strMAT := TurnParse.ChildWords[curLine][3];
        strCOL := TurnParse.ChildWords[curLine][1];
        strEMP := TurnParse.ChildWords[curLine][4];
      end else begin
        strFrom := TurnParse.ChildWords[curLine][0];
        strCAP := TurnParse.ChildWords[curLine][1];
        strMAT := TurnParse.ChildWords[curLine][2];
        strCOL := TurnParse.ChildWords[curLine][3];
        strEMP := TurnParse.ChildWords[curLine][4];
      end;
      planet := FindPlanetWithName(strFrom);
      if Assigned(planet) then begin
        Route := TRoute.Create(planet);
        strName := lowercase(planet.Name);
        try
          if strCol <> '-' then begin
            planet := FindPlanetWithName(strCol);
            Route.OrgCOLTo := planet;
          end;
          if strCap <> '-' then begin
            planet := FindPlanetWithName(strCap);
            Route.OrgCAPTo := planet;
          end;
          if strMat <> '-' then begin
            planet := FindPlanetWithName(strMat);
            Route.OrgMATTo := planet;
          end;
          if strEmp <> '-' then begin
            planet := FindPlanetWithName(strEmp);
            Route.OrgEMPTo := planet;
          end;
          route.RestoreOrgValues;
          Routes.AddObject(strName, route);
        except
          Route.Free;
        end;
      end;
      inc(curLine);
    end;
  end;
end;

procedure ShipsInProductionCount;
var
  planet: TPlanet;
  shiptype: TShipType;
  number: integer;
  numberLastTech: integer;
  i: integer;
  j: integer;
  group: TGroup;
begin
  for i := 0 to ShipsInProduction.count -1 do begin
    planet := FindPlanetWithName(TShipInProduction(ShipsInProduction[i]).PlanetName);
    shiptype := FindShipType(qReportPlayer, TShipInProduction(ShipsInProduction[i]).ShipTypeName);
    if assigned(planet) and Assigned(shiptype) then begin
      number := 0;
      numberLastTech := 0;
      for j := 0 to qReportPlayer.Groups.Count -1 do begin
        group := TGroup(qReportPlayer.Groups[j]);
        if (group.Planet = planet)
           and (group.Range = 0)
           and (group._Type = shiptype) then begin
          if ((shiptype.Drive = 0)
              or ((shiptype.Drive > 0)
                  and (group.DriveTech = qReportPlayer.Drive[LastTurn -1])))
             and
             ((shiptype.Weapons = 0)
              or ((shiptype.Weapons > 0)
                  and (group.WeaponsTech = qReportPlayer.Weapons[LastTurn -1])))
             and
             ((shiptype.Shields = 0)
              or ((shiptype.Shields > 0)
                  and (group.ShieldsTech = qReportPlayer.Shield[LastTurn -1])))
             and
             ((shiptype.Cargo = 0)
              or ((shiptype.Cargo > 0)
                  and (group.CargoTech = qReportPlayer.Cargo[LastTurn -1]))) then begin
            numberLastTech := numberLastTech + group.Aantal;
          end;

          number := number + group.Aantal;
        end;
        TShipInProduction(ShipsInProduction[i]).NumberOfShipsPresent := number;
        TShipInProduction(ShipsInProduction[i]).NumberOfShipsPresentLastTech := numberLastTech;
      end;
    end;
  end;
end;

procedure GetShipsInProduction(turn: integer; reportPlayerS, strCurReportName: string);
var
  curLine: integer;
  ShipInProduction: TShipInProduction;
  planetname, ShipTypeName: string;
  planetNr: integer;
  i: integer;
begin
  try
    for i := 0 to ShipsInProduction.Count -1 do
    begin
      TShipInProduction(ShipsInProduction.Items[i]).Free;
    end;
    ShipsInProduction.Clear;
    curLine := GetHeaderLine(1, CSGAL_ShipsInProcString);
    if (curLine > -1) and (curLine < TurnParse.Count) then begin
      inc(curLine);
      while (curLine < TurnParse.Count) and
        (TurnParse.AsString[curLine] = '') do inc(curline); // skip empty rows
      inc(curline); // skip header # N etc
      while (curLine < TurnParse.Count) and (TurnParse.AsString[curLine] <> '') do
      begin
        if inivars.Gametype = gtGalaxyBlind then
          planetNr := TurnParse.ChildWords[curLine].AsInteger[1]
        else
          planetNr := -1;
        planetname := TurnParse.ChildWords[curLine][CIGAL_ShipsInProcNamePos];
        ShipTypeName := TurnParse.ChildWords[curLine][CIGAL_ShipsInProcShipTypePos];
        if ShipTypeName <> '-' then begin
          ShipInProduction := TShipInProduction.Create;
          ShipInProduction.PlanetName := planetname;
          ShipInProduction.PlanetNr := planetNr;
          ShipInProduction.ShipTypeName := ShipTypeName;
          ShipInProduction.NumberOfShipsPresent := 0;
          ShipInProduction.NumberOfShipsPresentLastTech := 0;
          ShipsInProduction.Add(ShipInProduction);
        end;
        inc(curLine);
      end;
    end;
  except
    on e: Exception do
    ShowMessage(Format('Error - Turn: %d Player: %s Report: %s !'#13 +
      'While reading production of planets'#13+
      e.Message,
      [turn, reportPlayerS, strCurReportName]))
  end;
end;

procedure ParseGroupRow(curLine, iPlayer, turn: integer; player: TPlayer;
  fleetName: string; fleetSpeed: single);
var
  iPos: integer;
  nr, n: integer;
  shiptypename, cargoType: string;
  cargoQuantity: single;
  range: single;
  location, dest: string;
  destNr: integer;
  driveTech, weaponTech, shieldTech, cargoTech: single;
  mass: Single;
  curPlanet, locPlanet: TPlanet;
  i: integer;
  gr: TGroup;
begin
  iPos := 0;
  if iPlayer = Players.Count then
  begin
    nr := TurnParse.ChildWords[curLine].AsInteger[CIGAL_GroupRowNrPos];
    inc(iPos);
  end
  else nr := 0;
  n := TurnParse.ChildWords[curLine].AsInteger[iPos + CIGAL_GroupRowNPos];
  shiptypename := lowercase(TurnParse.ChildWords[curLine].AsString[iPos + CIGAL_GroupRowShipTypePos]);
  driveTech := TurnParse.ChildWords[curLine].AsSingle[iPos + CIGAL_GroupRowDrivePos];
  weaponTech := TurnParse.ChildWords[curLine].AsSingle[iPos + CIGAL_GroupRowWeaponPos];
  shieldTech := TurnParse.ChildWords[curLine].AsSingle[iPos + CIGAL_GroupRowShieldPos];
  cargoTech := TurnParse.ChildWords[curLine].AsSingle[iPos + CIGAL_GroupRowCargoPos];
  cargoType := TurnParse.ChildWords[curLine].AsString[iPos + CIGAL_GroupRowCargoTypePos];
  cargoQuantity := TurnParse.ChildWords[curLine].AsSingle[iPos + CIGAL_GroupRowCargoQuantityPos];
  dest := TurnParse.ChildWords[curLine].AsString[iPos + CIGAL_GroupRowDestPos];
        //n,type,driveTech,weaponTech,shieldTech,cargoTech,cargoType,cargoQuantity,dest
  if TurnParse.ChildWords[curLine].Count >= (iPos + CIGAL_GroupRowRangeCount) then
  begin
    range := TurnParse.ChildWords[curLine].AsSingle[iPos + CIGAL_GroupRowRangePos];
    if TurnParse.ChildWords[curLine].Count > iPos + CIGAL_GroupRowLocationPos then
      location := TurnParse.ChildWords[curLine].AsString[iPos + CIGAL_GroupRowLocationPos]
    else
      location := '';
  end
  else
    range := 0;
  if inivars.Gametype = gtGalaxyBlind then begin
    destNr := TurnParse.ChildWords[curLine].AsInteger[iPos + CIGAL_GroupRowDestPos];
    curPlanet := FindPlanetNr(destNr);
    if not Assigned(curPlanet) then
      curPlanet := NullPlanet;
//      raise Exception.Create('Planet does not exists: ' + TurnParse.AsString[curLine]);
  end else begin
    if Planets.Find(lowercase(dest), i) then
      curPlanet := TPlanet(Planets.Objects[i]) //  TPlanet(Planets.Items[i])
    else
      raise Exception.Create('Planet does not exists: ' + TurnParse.AsString[curLine]);
  end;
  locPlanet := nil;
  if Planets.Find(lowercase(location), i) then
    locPlanet := TPlanet(Planets.Objects[i]);
  gr := TGroup.Create;
  player.Groups.Add(gr);
  gr.LocPlanet := locPlanet;
  gr.AantalBattle := n;
  gr.Aantal := n;
  if player.ShipTypes.Find(shiptypename, i) then
    gr._Type := TShiptype(player.ShipTypes.Objects[i]);

  if inivars.Gametype = gtGalaxyBlind then
  begin
    if gr._Type.Drive > 0 then driveTech := driveTech / gr._Type.Drive
    else driveTech := 0;
    if gr._Type.weapons > 0 then weapontech := weaponTech / gr._Type.weapons
    else weaponTech := 0;
    if gr._Type.Shields > 0 then
      Shieldtech := (Shieldtech * (power(gr._Type.Mass, 0.33333333)) / 3.107232506) / gr._Type.Shields
    else ShieldTech := 0;
    if (gr._Type.Cargo + gr._Type.Cargo * gr._Type.Cargo / 10) > 0 then
      cargotech := cargotech / (gr._Type.Cargo + gr._Type.Cargo * gr._Type.Cargo / 10)
    else cargotech := 0;
  end;
  gr.DriveTech := driveTech;
  gr.WeaponsTech := weapontech;
  gr.ShieldsTech := Shieldtech;
  gr.CargoTech := cargotech;
  gr.Planet := curplanet;
  gr.Player := player;
  gr.Nr := Nr;
  gr.Range := range;
  gr.Lading := cargoQuantity;
  gr.LadingType := 0;
  if lowercase(cargotype) = 'cap' then gr.LadingType := P_CAP;
  if lowercase(cargotype) = 'mat' then gr.LadingType := P_MAT;
  if lowercase(cargotype) = 'col' then gr.LadingType := P_COL;
  gr.Turn := turn;
  gr.FleetName := Fleetname;
  gr.FleetSpeed := FleetSpeed;
  if Assigned(locPlanet) then begin
    gr.SourcePlanet := locPlanet;
    gr.DestPlanet := curplanet;
    gr.ETA := trunc(range / gr.EffSpeedNow) + 1;
  end else begin
    gr.SourcePlanet := curplanet;
    gr.DestPlanet := nil;
    gr.ETA := 0;
  end;
end;

function GetGroups(turn: integer; reportPlayerS, strCurReportName: string): boolean;
var
  strTemp, fleetName: string;
  curLine, iPlayer: integer;
  strFleetSpeed: string;
  fleetspeed: single;
  player: TPlayer;
begin
  //
{  if pos('k034', strCurReportName) > 0 then
    ShowMessage('Problem Report');    }
  Result := False;
  for iPlayer := 0 to Players.Count do
  try
    if iPlayer = Players.Count then begin
      strTemp := 'Your';
      player := reportPlayer;
      while player.Groups.Count > 0 do begin
        TGroup(player.Groups.Items[0]).free;
        player.Groups.Delete(0);
      end;
    end else begin
      player := TPlayer(players.Objects[iPlayer]);
      strTemp := player.Name;
      if lstReadedGroupsPlayers.IndexOf(player) <> -1 then
        Continue;
    end;
    strTemp := strTemp + CSGAL_GroupsGroupsString;
    curLine := GetHeaderLine(LastBattleLine, strTemp);
    if (curLine > -1) and (curLine < TurnParse.Count) then begin
      inc(curLine);
      while (curLine < TurnParse.Count) and
        (TurnParse.ChildWords[curLine].Count = 0) do inc(curline); // skip empty rows
      inc(curline); // skip header # N etc
      while (curLine < TurnParse.Count) and (TurnParse.ChildWords[curLine].Count > 0) do
      begin
        ParseGroupRow(curLine, iPlayer, turn, player, '', 0.0);
        inc(curLine);
      end;
    end;
  except
    on e: Exception do
    ShowMessage(Format('Error - Turn: %d Player: %s Report: %s !'#13 +
      'While reading groups'#13+
      e.Message,
      [turn, reportPlayer.Name, strCurReportName]))
  end;
  try
    curLine := GetHeaderLine(1, CSGAL_GroupsFleetsString);
    while (curLine > -1) and (curLine < TurnParse.Count) do begin
      fleetName := TurnParse.ChildWords[curLine][CIGAL_GroupsFleetsNamePos];
      try
        strFleetSpeed := TurnParse.ChildWords[curLine][CIGAL_GroupsFleetsSpeedPos];
        if strFleetSpeed[Length(strFleetSpeed)] = ')' then
          strFleetSpeed := copy(strFleetSpeed, 1, Length(strFleetSpeed) - 1);
        fleetspeed := StrToFloat(strFleetSpeed);
      except
        fleetName := '';
        fleetspeed := 0.0;
      end;
      inc(curLine);
      while (curLine < TurnParse.Count) and
        (TurnParse.AsString[curLine] = '') do inc(curline); // skip empty rows
      inc(curline); // skip header # N etc
      while (curLine < TurnParse.Count) and (Length(TurnParse.AsString[curLine]) > 0 { <> ''}) do
      begin
        ParseGroupRow(curLine, Players.Count, turn, reportPlayer, fleetname, fleetspeed);
        inc(curLine);
      end;
      curLine := GetHeaderLine(curLine, CSGAL_GroupsFleetsString);
    end;
  except
    on e: Exception do
    ShowMessage(Format('Error - Turn: %d Player: %s Report: %s !!!'#13 +
      'While reading fleets'#13+
      e.Message,
      [turn, reportPlayer.Name, strCurReportName]))
  end;
end;

//Get incoming groups
//O   D   R     S     M
//72 34 39.03 46.00 1.00

function GetIncomingGroups(turn: integer; reportPlayerS, strCurReportName: string): boolean;
var
  curLine, i, iPos: integer;
  strFrom, strDest: string;
  planetNrFrom, planetNrTo: integer;
  range, speed, mass: single;
  fromPlanet,
    destPlanet: TPlanet;
  IncomingGroup: TIncomingGroup;
begin
  curLine := GetHeaderLine(0, CSGAL_IncomGroupsString);
  if (curLine > -1) and (curLine < TurnParse.Count) then
  try
    inc(curLine);
    while (curLine < TurnParse.Count) and
      (TurnParse.AsString[curLine] = '') do inc(curline); // skip empty rows
    inc(curline); // skip header # N etc
    while (curLine < TurnParse.Count) and
      (TurnParse.AsString[curLine] <> '') do
    begin
      iPos := 0;
      strFrom := TurnParse.ChildWords[curLine].AsString[iPos + CIGAL_IncomGroupsFromPos];
      strDest := TurnParse.ChildWords[curLine].AsString[iPos + CIGAL_IncomGroupsDestPos];
//      if inivars.Gametype = gtGalaxyBlind then begin
//        planetNrFrom := TurnParse.ChildWords[curLine].AsInteger[iPos + CIGAL_IncomGroupsFromPos];
//        planetNrTo := TurnParse.ChildWords[curLine].AsInteger[iPos + CIGAL_IncomGroupsDestPos];
//      end;
      range := TurnParse.ChildWords[curLine].AsSingle[iPos + CIGAL_IncomGroupsRangePos];
      speed := TurnParse.ChildWords[curLine].AsSingle[iPos + CIGAL_IncomGroupsSpeedPos];
      mass := TurnParse.ChildWords[curLine].AsSingle[iPos + CIGAL_IncomGroupsMassPos];
      fromPlanet := nil;
      if Planets.Find(lowercase(strFrom), i) then
        fromPlanet := TPlanet(Planets.Objects[i]) //  TPlanet(Planets.Items[i])
      else begin
        if inivars.Gametype = gtGalaxyBlind then
          fromPlanet := NullPlanet
        else
          raise Exception.Create('PLanet does not exists: ' + TurnParse.AsString[curLine]);
      end;
      destPlanet := nil;
      if Planets.Find(lowercase(strDest), i) then
        destPlanet := TPlanet(Planets.Objects[i]) //  TPlanet(Planets.Items[i])
      else
        raise Exception.Create('PLanet does not exists: ' + TurnParse.AsString[curLine]);
      IncomingGroup := TIncomingGroup.Create;
      IncomingGroup.FromPlanet := fromPlanet;
      IncomingGroup.DestPlanet := destPlanet;
      IncomingGroup.Player := reportPlayer;
      IncomingGroup.range := range;
      IncomingGroup.speed := speed;
      IncomingGroup.mass := mass;
      IncomingGroup.Turn := turn;
      IncomingGroups.Add(IncomingGroup);
      inc(curLine);
    end;
  except
    on e: Exception do
    ShowMessage(Format('Error - Turn: %d Player: %s Report: %s !'#13 +
      'While reading incoming groups'#13+
      e.Message,
      [turn, reportPlayerS, strCurReportName]))
  end;
  Result := true;
end;

procedure DeleteDoubleGroups;
var
  tmlStringList: TStringList;
  i, j, index: integer;
  player: TPlayer;
  group: TGroup;
  tempString: string;
begin
  tmlStringList := TStringList.Create;
  try
    for i := 0 to players.Count - 1 do begin
      tmlStringList.Clear;
      player := Tplayer(players.Objects[i]);
      if Assigned(player) then begin
        j := 0;
        while j < player.Groups.Count do begin
          group := TGroup(player.Groups.Items[j]);
          if Assigned(group) then begin
            tempString := IntToStr(group.Turn) + '$';
            if Assigned(group._Type) then
              tempString := tempString + lowercase(group._Type.Name) + '%';
            if Assigned(group.Planet) then
              tempString := tempString + lowercase(group.Planet.Name) + '$';
            if Assigned(group.LocPlanet) then
              tempString := tempString + lowercase(group.LocPlanet.Name) + '%';
            tempString := tempString + IntToStr(group.Aantal) + '$';
            tempString := tempString + FloatToStr(group.Range) + '$';
            tempString := tempString + FloatToStr(group.DriveTech) + '$';
            tempString := tempString + FloatToStr(group.WeaponsTech) + '$';
            tempString := tempString + FloatToStr(group.ShieldsTech) + '$';
            tempString := tempString + FloatToStr(group.Lading) + '$';
            tempString := tempString + FloatToStr(group.LadingType) + '$';
            tempString := tempString + FloatToStr(group.CargoTech);
            if (player = qreportPlayer) and (group.Nr = 0) then begin
              TGroup(player.Groups.Items[j]).Free;
              player.Groups.Delete(j);
            end else begin
              tmlStringList.AddObject(tempString, group);
              inc(j);
            end;
          end else
            inc(j);
        end;
      end;
      tmlStringList.Sort;
      j := 0;
      while j < tmlStringList.Count - 1 do begin
        if CompareText(tmlStringList[j], tmlStringList[j + 1]) = 0 then begin
          if TGroup(tmlStringList.Objects[j]).FleetName = '' then begin
            index := player.Groups.IndexOf(tmlStringList.Objects[j]);
            if index <> -1 then begin
              TGroup(player.Groups.Items[index]).Free;
              player.Groups.Delete(index);
            end;
          end;
        end;
        inc(j);
      end;
    end;
  finally
    tmlStringList.Free;
  end;
end;

procedure CreateMyTracks(turn: integer);
var
  group: TGroup;
  track: TTracks;
  distAll: Single;
  ratioD: Single;
  j: integer;
  dx, dy: Single;
begin
  for j := 0 to QReportPlayer.Groups.Count - 1 do begin
    group := TGroup(QReportPlayer.Groups[j]);
    if (group.Turn = turn) and (group.Range > 0) then begin
      if not Assigned(group.LocPlanet) or not Assigned(group.Planet) then
        continue;
      try
        dx := group.LocPlanet.X - group.Planet.X;
        dy := group.LocPlanet.Y - group.Planet.Y;
        distAll := sqrt(dx * dx + dy * dy);
      except
        Continue;
      end;
      if distAll = 0 then continue;

      Track := TTracks.Create;
      Track.Turn := turn;
      Track.My := True;
      ratioD := (distAll - (group.range * 100)) / distAll;
      Track.X := trunc(group.LocPlanet.X + (ratioD * (group.Planet.X - group.LocPlanet.X)));
      Track.Y := trunc(group.LocPlanet.Y + (ratioD * (group.Planet.Y - group.LocPlanet.Y)));
      TracksList.Add(Track);
    end;
  end;
end;

function ReadReports: boolean;
var
  strCurReportName,
    reportPlayerS,
    qReportPlayerS: string;
  gameN, turn: integer;
  ntr, i: integer;
  iniFile: TIniFile;
begin
  Result := False;
  LastTurn := -1;
  TurnParse := TMkStrParse.Create(nil);
  lstReadedGroupsPlayers := TList.Create;
  try
    TurnParse.CRLFDelimeter := True;
    TurnParse.CreateChild('SD');

    for i := 0 to 255 do
      case i of
        ord('0')..ord('9'),
          ord('A')..ord('Z'),
          ord('a')..ord('z'),
          ord('!'), ord('@'), ord('#'), ord('$'), ord('%'), ord('^'), ord('&'),
          ord('*'), ord('('), ord(')'), ord('-'), ord('+'), ord('='), ord('|'),
          ord('\'), ord('{'), ord('}'), ord('['), ord(']'), ord(';'), ord(':'),
          ord('"'), 39, ord('<'), ord('.'), ord('>'), ord('?'),
          ord('/'),
          ord('_'): TurnParse.Child.CharDelimeters[char(i)] := False;
      else
        TurnParse.Child.CharDelimeters[char(i)] := True;
      end;

    if inivars.Gametype = gtGalaxyNGBlind then
      players.Duplicates := dupAccept;

    for ntr := 0 to MAX_TURNS do
    begin
      lstReadedGroupsPlayers.Clear;
      for gameN := 1 to length(strSpelC) do
      begin
        GameNr := gameN;
        strCurReportName := format(strSpelD, [strSpelC[gameN], ntr]);
        if FileExists(strCurReportName) then
        begin
{          if (strSpelC[gameN] = 'p') and (ntr = 18) then
            ShowMessage('Problem Report');   }

          if inivars.Gametype = gtUnKnown then begin
            inivars.Gametype := ReadGameType(strCurReportName);
            iniFile := TIniFile.Create(strIniFileName);
            try
              iniFile.WriteString(gamestr, 'GameType', SGameStr[inivars.Gametype]);
            finally
              iniFile.Free;
            end;
          end;
          frmWait.CurrentWork := 'Reading report';
          TurnParse.StringToParse := MkStrFileLoad(strCurReportName);
          GetHeaders(strCurReportName);
          if Headers.Count < 5 then begin
            frmWait.CurrentWork := 'Adding Tabs';
            AddTabsToReportFile(strCurReportName);
            TurnParse.StringToParse := MkStrFileLoad(strCurReportName);
            GetHeaders(strCurReportName);
          end;
          curLine := 0;
          // get turn number and player name
          if GetTurnNumberPlayerName(turn, reportPlayerS, strCurReportName) then
          begin
            // get status of players
            //gtGalaxyNGBlind
            if (gameN = 1) or (inivars.Gametype = gtGalaxyNGBlind) then // alleen bij hoofdspeler lezen
            begin
              if (gameN = 1) then
                qreportPlayerS := reportPlayerS;
              GetStatusOfPlayers(ntr, turn, reportPlayerS);
              if (gameN = 1) then begin
                LastTurn := turn;
                Application.Title := 'MKGal ' + strGameName + ' - ' + IntToStr(LastTurn) + ' ' + reportPlayerS;
                frmWait.Title := Application.Title;
                frmMkGal.Title := Application.Title;
              end;
            end;

            for i := 0 to Players.Count - 1 do
            begin
              if CompareText(TPlayer(Players.Objects[i]).Name, reportPlayerS) = 0 then
                reportPlayer := TPlayer(Players.Objects[i]);
              if gameN = 1 then
                if CompareText(TPlayer(Players.Objects[i]).Name, qreportPlayerS) = 0 then
                  qreportPlayer := TPlayer(Players.Objects[i]);
            end;
            if not Assigned(reportPlayer) then exit;
//
            if turn >= inivars.SkipPlanetsUntilTurn then begin
              if FirstTurn = -1 then
                FirstTurn := Turn;
              GetTurnPlayerPlanets(turn, reportPlayerS, strCurReportName);
              GetOtherPlanets(turn, reportPlayerS, strCurReportName);
              GetPlayerPlanets(turn, reportPlayerS, strCurReportName);

              if gameN = 1 then begin
                GetShipsInProduction(turn, reportPlayerS, strCurReportName);
                GetRoutes(turn, reportPlayerS, strCurReportName);
              end;

              if (turn = 0) and (gameN = 1) then
                CheckForPlayerAndPlanetsPatterns;
              GetShipTypes(turn, reportPlayerS, strCurReportName);

              if inivars.SkipBattlesUntilTurn <= turn then begin
                GetBattles(turn, reportPlayerS, strCurReportName);
                GetGroups(turn, reportPlayerS, strCurReportName);
                lstReadedGroupsPlayers.Add(reportPlayer);
              end;
              GetBombings(turn, reportPlayerS, strCurReportName);

              if gameN = 1 then // alleen bij hoofdspeler lezen
                GetIncomingGroups(turn, reportPlayerS, strCurReportName);
              DeleteOldGroupsFromKnownPlanets(turn);
              CalculateMaxPopulation(turn);
              if (gameN = 1) and (turn <= UseCountMethod) then
                DoCountMethod(turn);
            end;
//
          end;
        end;
      end;
    end;
    ShipsInProductionCount;
    if inivars.Gametype in [gtGalaxyNG, gtGalaxyNormal] then
      for i := LastTurn - 1 to LastTurn do begin
        for gameN := 1 to length(strSpelC) do begin
          strCurReportName := format(strSpelD, [strSpelC[gameN], i]);
          if FileExists(strCurReportName) then begin
            ReadMap(strCurReportName, i);
          end;
        end;
      end;
    if inivars.Gametype in [gtGalaxyNG] then
      for i := LastTurn - 1 to LastTurn do begin
        CreateMyTracks(i);
      end;
    frmWait.CurrentWork := 'Ready';
    DeleteDoubleGroups;
    Result := True;
  finally
    lstReadedGroupsPlayers.Free;
    TurnParse.Free;
  end;
  if players.Count = 0 then begin
    Result := False;
    Exit;
  end;
  if FileExists(ExtractFilePath(ParamStr(0)) + 'ExtrInf.ini') then
  begin
    ExtraPlanetInfoList.ReadInfo(ExtractFilePath(ParamStr(0)) + 'ExtrInf.ini');
    CalculateMaxPopulation(turn);
  end;
end;

initialization
  strIniFileName := ExtractFilePath(ParamStr(0)) + 'MKGal.ini';
  Headers := TStringList.Create;
  mkreExpr := TmkreExpr.Create(nil);
  FirstTurn := -1;
  gPlanetNumber := 0;
finalization
  mkreExpr.Free;
  Headers.Free;
end.

