unit UFilesCountMethod;

interface

uses
  Windows, Classes, UTypes;

type
  TCountMethodPlanet = record
    Planet: TPlanet;
    PlayerNr: integer;
    Player: TPlayer;
    FirstTurn: boolean;
  end;
  TCountMethodPlanetList = array of TCountMethodPlanet;

  TCountMethodCluster = record
    turns: array of integer;
    PlayerNr: integer;
    player: Tplayer;
  end;
  TCountMethodClusterList = array of TCountMethodCluster;

var
  UseCountMethod: integer;
  HomeWorlds: TCountMethodPlanetList;

procedure DoCountMethod(turn: integer);
procedure CheckForPlayerAndPlanetsPatterns;

implementation

uses
  SysUtils, Dialogs, UFiles, HyperStr, USortingAlgorithems, mkStrUtils;

var
  PlanetCountMethodClusterList: TCountMethodClusterList;
  PlayerCountMethodClusterList: TCountMethodClusterList;

procedure CheckForPlayerAndPlanetsPatterns;
var
  i, j: integer;
  PlanetsPerPlayer: integer;
  AllPlanetsPerPlayer: integer;
  intPlayer: integer;
  SizePattern: array of Single;
  ResourcePattern: array of Single;
  counter: integer;
  tmpCountMethodPlanet: TCountMethodPlanet;
  blnNo: boolean;
  player: TPlayer;
begin
  SetLength(HomeWorlds, 0);
  try
    // get all homeworlds
    for i := 0 to planets.Count - 1 do begin
      if assigned(TPlanet(planets.objects[i]).PlayerCur) then begin
        SetLength(HomeWorlds, length(HomeWorlds) + 1);
        HomeWorlds[high(HomeWorlds)].planet := TPlanet(planets.objects[i]);
        HomeWorlds[high(HomeWorlds)].PlayerNr := -1;
        if TPlanet(planets.objects[i]).PlayerCur <> TPlayer(1) then
          HomeWorlds[high(HomeWorlds)].Player := TPlanet(planets.objects[i]).PlayerCur
        else
          HomeWorlds[high(HomeWorlds)].Player := nil;
        HomeWorlds[high(HomeWorlds)].FirstTurn := true;
      end;
    end;
    //Sort planets on nr
    for i := 0 to high(HomeWorlds) do begin
      for j := high(HomeWorlds) downto i + 1 do begin
        if HomeWorlds[j].Planet.Nr < HomeWorlds[j - 1].Planet.Nr then begin
          tmpCountMethodPlanet := HomeWorlds[j];
          HomeWorlds[j] := HomeWorlds[j - 1];
          HomeWorlds[j - 1] := tmpCountMethodPlanet;
        end;

      end;
    end;
    //Create a pattern
    PlanetsPerPlayer := Length(HomeWorlds) div players.Count;
    SetLength(SizePattern, PlanetsPerPlayer);
    SetLength(ResourcePattern, PlanetsPerPlayer);
    counter := 0;
    for i := 0 to high(HomeWorlds) do begin
      if Assigned(HomeWorlds[i].Player) and (counter < PlanetsPerPlayer) then begin
        SizePattern[counter] := HomeWorlds[i].Planet.Size;
        ResourcePattern[counter] := HomeWorlds[i].Planet.Resources;
        inc(counter);
      end;
    end;
    // use pattern on all planets
    counter := 0;
    if (turn0HabitedPlanetsSize = 0) then
      for i := 0 to high(HomeWorlds) do begin
        if not Assigned(HomeWorlds[i].Player) then begin
          if (counter = PlanetsPerPlayer) then counter := 0;
          HomeWorlds[i].Planet.Size := SizePattern[counter];
          HomeWorlds[i].Planet.Resources := ResourcePattern[counter];
          inc(counter);
        end;
      end;
    // is there a know pattern for the players?
    AllPlanetsPerPlayer := planets.count div players.count;
    i := 0;
    blnNo := false;
    intPlayer := 0; // zero based
    while (i < Length(HomeWorlds)) and not blnNo do begin
      if HomeWorlds[i].Planet.Nr <> AllPlanetsPerPlayer * (intPlayer) + 1 then
        blnNo := true;
      inc(intPlayer);
      inc(i, PlanetsPerPlayer);
    end;
    i := 0;
    if not blnNo then begin
      intPlayer := 0; // zero based
      while (i < Length(HomeWorlds)) do begin
        player := nil;
        for j := 0 to players.Count - 1 do
          if mkAtoi(TPlayer(players.objects[j]).Name) = intPlayer + 1 then
            player := TPlayer(players.objects[j]);

        inc(intPlayer);
        for j := 0 to PlanetsPerPlayer - 1 do begin
          if assigned(player) and (integer(HomeWorlds[i].Planet.PlayerCur) <= 1) then
            HomeWorlds[i].Planet.Player[lastturn] := player;
          inc(i);
        end;
      end;
    end;
  finally
    SetLength(HomeWorlds, 0);
    SetLength(SizePattern, 0);
    SetLength(ResourcePattern, 0);
  end;
end;

procedure SetupCountMethodClusterList(var list: TCountMethodClusterList; blnSetPlanetNr: boolean);
var
  i: integer;
begin
  SetLength(list, players.Count);
  for i := 0 to High(list) do begin
    SetLength(list[i].turns, UseCountMethod);
    list[i].PlayerNr := -1;
    if blnSetPlanetNr then list[i].PlayerNr := i
    else list[i].Player := Tplayer(players.Objects[i]);
  end;
end;

procedure DestroyCountMethodClusterList(var list: TCountMethodClusterList);
var
  i: integer;
begin
  for i := 0 to High(list) do
    SetLength(list[i].turns, 0);
  SetLength(list, 0);
end;

procedure CalculatePlayerClusters(var list: TCountMethodClusterList; turn: integer);
var
  i: integer;
begin
  for i := 0 to High(list) do begin
    list[i].turns[turn] := list[i].player.Planets[turn];
  end;
end;

procedure CalculatePlanetClusters(var ClusterList: TCountMethodClusterList;
  PlanetList: TCountMethodPlanetList;
  turn: integer);
var
  i: integer;
begin
  for i := 0 to High(ClusterList) do begin
    ClusterList[i].turns[turn] := 0;
  end;
  for i := 0 to High(PlanetList) do begin
    if PlanetList[i].PlayerNr <> -1 then
      inc(ClusterList[PlanetList[i].PlayerNr].turns[turn])
    else
      ShowMessage(PlanetList[i].Planet.Name);
  end;
end;

function FindPlanetInMethodPlanetList(List: TCountMethodPlanetList; planet: TPlanet): integer;
begin
  Result := 0;
  while Result < Length(List) do
    if list[Result].Planet = planet then Exit
    else inc(Result);
  Result := -1;
end;

procedure DeleteUnhabitedPlanets(var List: TCountMethodPlanetList);
var
  i: integer;
  curInc: integer;
begin
  i := 0;
  curInc := -1;
  while i < length(list) do begin
    if Assigned(list[i].Planet.PlayerCur) then begin
      inc(curInc);
      list[curInc] := list[i];
    end;
    inc(i);
  end;
  SetLength(list, curInc + 1);
end;

procedure MatchClusterLists(turn: integer; var PlanetCountMethodClusterList,
  PlayerCountMethodClusterList: TCountMethodClusterList);
var
  Occured, firstPlayer, firstPlanet: integer;
  indexPlanets, indexPlayers: integer;

  function MatchTurns(turns1, turns2: array of integer): boolean;
  var
    intTurn: integer;
  begin
    Result := True;
    for intTurn := 0 to turn do
      if turns1[intTurn] <> turns2[intTurn] then result := False;
  end;

begin
  for indexPlanets := 0 to high(PlanetCountMethodClusterList) do begin
    if not Assigned(PlanetCountMethodClusterList[indexPlanets].player) then begin
      Occured := 0;
      firstPlanet := -1;
      firstPlayer := -1;
      for indexPlayers := 0 to high(PlayerCountMethodClusterList) do begin
        if PlayerCountMethodClusterList[indexPlayers].playerNr = -1 then begin
          if MatchTurns(PlanetCountMethodClusterList[indexPlanets].turns,
            PlayerCountMethodClusterList[indexPlayers].turns) then
          begin
            if firstPlanet = -1 then firstPlanet := indexPlanets;
            if firstPlayer = -1 then firstPlayer := indexPlayers;
            inc(Occured);
          end;
        end;
      end;
      if (Occured = 1) and
        not Assigned(PlanetCountMethodClusterList[indexPlanets].player) and
        (PlayerCountMethodClusterList[indexPlanets].playerNr = -1) then begin
        PlanetCountMethodClusterList[indexPlanets].player := PlayerCountMethodClusterList[firstPlayer].player;
        PlayerCountMethodClusterList[indexPlanets].playerNr := PlanetCountMethodClusterList[firstPlanet].playerNr;
      end;
    end;
  end;

end;

procedure DoCountMethod(turn: integer);
const
  MinDistanceHalf: integer = high(integer); // absolutely not true
  HomePlanetsPerPlayer: integer = 1;
var
  i, j: integer;
  x, y: integer;
  tempSortList: TList;
  incNummer: integer;
  tmpPlayerNr, tmpIndex: integer;
begin
  if UseCountMethod = turn then begin
    SetLength(HomeWorlds, 0);
    DestroyCountMethodClusterList(PlanetCountMethodClusterList);
    DestroyCountMethodClusterList(PlayerCountMethodClusterList);
    Exit;
  end;
  if turn = 0 then begin
    SetupCountMethodClusterList(PlanetCountMethodClusterList, true);
    SetupCountMethodClusterList(PlayerCountMethodClusterList, false);
  end;
  if turn = 1 then Exit;
  CalculatePlayerClusters(PlayerCountMethodClusterList, turn);
  tempSortList := TList.Create;
  try
    DeleteUnhabitedPlanets(HomeWorlds);
    // Add planets to HomeWorlds that are habited
    for i := 0 to planets.Count - 1 do begin
      if assigned(TPlanet(planets.objects[i]).PlayerCur) then begin
        if FindPlanetInMethodPlanetList(Homeworlds, TPlanet(planets.objects[i])) = -1 then begin
          SetLength(HomeWorlds, length(HomeWorlds) + 1);
          HomeWorlds[high(HomeWorlds)].planet := TPlanet(planets.objects[i]);
          HomeWorlds[high(HomeWorlds)].PlayerNr := -1;
          HomeWorlds[high(HomeWorlds)].FirstTurn := (turn = 0) or (turn = 1);
        end;
      end;
    end;
    if turn = 0 then begin
      HomePlanetsPerPlayer := Length(HomeWorlds) div players.Count;
    end;
    // FindNearest planets
    incNummer := 0;
    for i := 0 to high(HomeWorlds) do begin
      if HomeWorlds[i].PlayerNr = -1 then begin
        // in turn 0 we had to give the starting numbers of planets for each race
        if (turn = 0) then begin
          HomeWorlds[i].PlayerNr := incNummer;
          inc(incNummer);
        end;
        tempSortList.Clear;
        x := HomeWorlds[i].Planet.X;
        y := HomeWorlds[i].Planet.Y;
        tmpPlayerNr := HomeWorlds[i].PlayerNr;
        tmpIndex := i;
        for j := 0 to high(HomeWorlds) do begin
          HomeWorlds[j].Planet.SetDistanceFrom(x, y);
          tempSortList.Add(pointer(j));
        end;
        tempSortList.Sort(@CompareHomeWorlds);
        // in turn 0 or 1 everyone has equal number of planets
        if (turn = 0) or (turn = 1) then begin
          j := 0;
          while (j < HomePlanetsPerPlayer) and (j < tempSortList.Count) do begin
            if HomeWorlds[integer(tempSortList.Items[j])].PlayerNr = -1 then
              HomeWorlds[integer(tempSortList.Items[j])].PlayerNr := tmpPlayerNr;
            inc(j);
          end;
          if j < tempSortList.Count then
            if (HomeWorlds[integer(tempSortList.Items[j])].Planet.Distance div 2) < MinDistanceHalf then
              MinDistanceHalf := HomeWorlds[integer(tempSortList.Items[j])].Planet.Distance div 2;
        end else begin
          // this planet is the center of the world, now we find the nearest
          // 'first turn' planet, within an radius of half the minimum distance
          // between clusters
          j := 0;
          while (j < tempSortList.Count) and
            (HomeWorlds[integer(tempSortList.Items[j])].Planet.Distance <= MinDistanceHalf) do
          begin
            if (j <> tmpIndex) and HomeWorlds[integer(tempSortList.Items[j])].FirstTurn then begin
              HomeWorlds[tmpIndex].PlayerNr := HomeWorlds[integer(tempSortList.Items[j])].PlayerNr;
              break;
            end;
            inc(j);
          end;
          // no home planet found within the radius of half the minimum distance
          // between cluster, so pick the nearest colonized planet
          if HomeWorlds[tmpIndex].PlayerNr = -1 then begin
            j := 0;
            repeat
              HomeWorlds[tmpIndex].PlayerNr := HomeWorlds[integer(tempSortList.Items[j])].PlayerNr;
              inc(j);
            until (j < tempSortList.Count) and (HomeWorlds[tmpIndex].PlayerNr <> -1);
          end;
        end;
      end;
    end;

    if (turn = 0) and (turn0HabitedPlanetsSize2nd > 0) and (turn0HabitedPlanetsSize > 0) then begin
      tempSortList.Sort(@CompareHomeWorldsOnNrAndName);
      i := 0;
      while i < tempSortList.Count do begin
        j := 0;
        repeat

          if j <> (HomePlanetsPerPlayer - 1) then
            HomeWorlds[integer(tempSortList.Items[i])].Planet.Size := turn0HabitedPlanetsSize2nd;
          inc(j);
          inc(i);
        until (j = HomePlanetsPerPlayer) or (i = tempSortList.Count);
      end;
    end;

    CalculatePlanetClusters(PlanetCountMethodClusterList, HomeWorlds, turn);

    MatchClusterLists(turn, PlanetCountMethodClusterList, PlayerCountMethodClusterList);
    try
      for i := 0 to high(HomeWorlds) do begin
        if HomeWorlds[i].Planet.PlayerCur = TPlayer(1) then begin
          tmpPlayerNr := HomeWorlds[i].PlayerNr;
          if tmpPlayerNr <> -1 then begin
            for j := 0 to high(PlanetCountMethodClusterList) do begin
              if (PlanetCountMethodClusterList[j].PlayerNr = tmpPlayerNr) and
                Assigned(PlanetCountMethodClusterList[j].Player) then begin
                HomeWorlds[i].Planet.PlayerCur := PlanetCountMethodClusterList[j].Player;
                HomeWorlds[i].Player := PlanetCountMethodClusterList[j].Player;

                HomeWorlds[i].Planet.LastOwnerName := HomeWorlds[i].Player.Name;
                HomeWorlds[i].Planet.LastOwnerType := 2;
                HomeWorlds[i].Planet.LastOwnerTurn := Turn;

                break;
              end;
            end;
          end;
        end
      end;
    except

    end;
    try
      // set names for the habited planets
      for i := 0 to high(HomeWorlds) do begin
        if HomeWorlds[i].Planet.PlayerCur = TPlayer(1) then begin
          if (HomeWorlds[i].Planet.LastOwnerType = 0) and Assigned(HomeWorlds[i].Planet.LastOwnerObject) then begin
            HomeWorlds[i].Planet.LastOwnerName := HomeWorlds[i].Planet.LastOwnerObject.Name;
          end else begin
            HomeWorlds[i].Planet.LastOwnerName := IntToStr(HomeWorlds[i].PlayerNr);
            HomeWorlds[i].Planet.LastOwnerType := 2;
            HomeWorlds[i].Planet.LastOwnerTurn := Turn;
          end;
        end else begin
          if Assigned(HomeWorlds[i].Planet.PlayerCur) then begin
            HomeWorlds[i].Planet.LastOwnerName := HomeWorlds[i].Planet.PlayerCur.Name;
            HomeWorlds[i].Planet.LastOwnerType := 0;
            HomeWorlds[i].Planet.LastOwnerTurn := Turn;
            HomeWorlds[i].Planet.LastOwnerObject := HomeWorlds[i].Planet.PlayerCur;
          end;
        end;
      end;
    except
      ;
    end;
  finally
    tempSortList.Clear;
    tempSortList.Free;
  end;
end;

end.
