unit UfrmBattleSimulator;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, Grids, UTypes, StdCtrls, Menus, RXSpin,
  OvcBase, OvcState;

type
  TfrmBattleSimulator = class(TForm)
    PageControl1: TPageControl;
    tshGroups: TTabSheet;
    tshSimulate: TTabSheet;
    sgrDefenders: TStringGrid;
    Splitter1: TSplitter;
    Panel1: TPanel;
    Panel2: TPanel;
    sgrAttackers: TStringGrid;
    btnClearDefenders: TButton;
    btnClearAttackers: TButton;
    pmDefenders: TPopupMenu;
    mniDeleteGroupD: TMenuItem;
    Memo1: TMemo;
    pmAttackers: TPopupMenu;
    mniDeleteGroupA: TMenuItem;
    Panel3: TPanel;
    btnSimulate: TButton;
    cbxShowResults: TCheckBox;
    edtMax_Battles: TRxSpinEdit;
    OvcFormState1: TOvcFormState;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnClearAttackersClick(Sender: TObject);
    procedure btnClearDefendersClick(Sender: TObject);
    procedure mniDeleteGroupDClick(Sender: TObject);
    procedure mniDeleteGroupAClick(Sender: TObject);
    procedure pmDefendersPopup(Sender: TObject);
    procedure pmAttackersPopup(Sender: TObject);
    procedure btnSimulateClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    battle,
      ResultBattle: TBattle;
    procedure DeleteGroups(gr: TList);
    procedure SetGridsHeaders;
    procedure ChangeLoadA(Sender: TObject);
    procedure ChangeLoadD(Sender: TObject);
    { Private declarations }
  public
    procedure AddGroupToDefenders(gr: TGroup);
    procedure AddGroupToAttackers(gr: TGroup);
    procedure NewBattle;
    procedure UpdateBattleSim;
    { Public declarations }
  end;

var
  frmBattleSimulator: TfrmBattleSimulator;

implementation

uses
  UBatSim2, UfrmMain;

{$R *.DFM}

procedure TfrmBattleSimulator.ChangeLoadA(Sender: TObject);
var
  group: Tgroup;
  index: integer;
  newLoad: string;
  Load: single;
begin
  index := sgrAttackers.Selection.Top;
  if (index > 0) and (index < (sgrAttackers.RowCount - 1)) then
  begin
    group := battle.GroupA.Items[index - 1];
    newLoad := InputBox('Load', 'Enter Ship load', Format('%5.2f', [group.effCargo]));
    try
      Load := StrToFloat(newLoad);
    except
      Load := 0;
    end;
    if (Load >= 0) and (Load <= group.EffCargo) then group.Lading := Load;
    UpdateBattleSim;
  end;
end;

procedure TfrmBattleSimulator.ChangeLoadD(Sender: TObject);
var
  group: Tgroup;
  index: integer;
  newLoad: string;
  Load: single;
begin
  index := sgrDefenders.Selection.Top;
  if (index > 0) and (index < (sgrDefenders.RowCount - 1)) then
  begin
    group := battle.GroupD.Items[index - 1];
    newLoad := InputBox('Load', 'Enter Ship load', Format('%5.2f', [group.effCargo]));
    try
      Load := StrToFloat(newLoad);
    except
      Load := 0;
    end;
    if (Load >= 0) and (Load <= group.EffCargo) then group.Lading := Load;
    UpdateBattleSim;
  end;
end;

procedure TfrmBattleSimulator.DeleteGroups(gr: TList);
var
  group: TGroup;
begin
  if btnSimulate.Tag <> 1 then Exit;
  while gr.Count > 0 do
  begin
    group := gr.items[0];
    group._Type.Free;
    group.Free;
    gr.Delete(0);
  end;
end;

procedure TfrmBattleSimulator.UpdateBattleSim;
var
  i: integer;
  group: TGroup;
begin
  SetGridsHeaders;
  sgrDefenders.RowCount := battle.GroupD.Count + 2;
  sgrDefenders.FixedRows := 1;
  for i := 0 to battle.GroupD.Count - 1 do
  begin
    group := TGroup(battle.GroupD.items[i]);
    sgrDefenders.Rows[i + 1].Clear;
    sgrDefenders.Rows[i + 1].Add(IntToStr(group.Aantal));
    if Assigned(group.Player) then
      sgrDefenders.Rows[i + 1].Add(group.Player.Name)
    else
      sgrDefenders.Rows[i + 1].Add('Defender');
    sgrDefenders.Rows[i + 1].Add(group._Type.Name);
    sgrDefenders.Rows[i + 1].Add(IntToStr(group._Type.Attacks));
    sgrDefenders.Rows[i + 1].Add(Format('%05.2f', [group.EffWeapon]));
    if group.Lading > 0 then
      sgrDefenders.Rows[i + 1].Add(Format('%05.2f', [group.EffShieldLoaded]))
    else
      sgrDefenders.Rows[i + 1].Add(Format('%05.2f', [group.EffShield]));
    sgrDefenders.Rows[i + 1].Add(Format('%05.2f', [group.EffCargo]));
    sgrDefenders.Rows[i + 1].Add(Format('%05.2f', [group.Lading]));
    sgrDefenders.Rows[i + 1].Add(Format('%05.2f', [group._Type.Mass + group.Lading]));
  end;
  sgrDefenders.Rows[battle.GroupD.Count + 1].Clear;
  sgrAttackers.RowCount := battle.GroupA.Count + 2;
  sgrAttackers.FixedRows := 1;
  for i := 0 to battle.GroupA.Count - 1 do
  begin
    group := TGroup(battle.GroupA.items[i]);
    sgrAttackers.Rows[i + 1].Clear;
    sgrAttackers.Rows[i + 1].Add(IntToStr(group.Aantal));
    if Assigned(group.Player) then
      sgrAttackers.Rows[i + 1].Add(group.Player.Name)
    else
      sgrAttackers.Rows[i + 1].Add('Attacker');
    sgrAttackers.Rows[i + 1].Add(group._Type.Name);
    sgrAttackers.Rows[i + 1].Add(IntToStr(group._Type.Attacks));
    sgrAttackers.Rows[i + 1].Add(Format('%05.2f', [group.EffWeapon]));
    sgrAttackers.Rows[i + 1].Add(Format('%05.2f', [group.GetEffShieldWithLoad(group.Lading)]));
    sgrAttackers.Rows[i + 1].Add(Format('%05.2f', [group.EffCargo]));
    sgrAttackers.Rows[i + 1].Add(Format('%05.2f', [group.Lading]));
    sgrAttackers.Rows[i + 1].Add(Format('%05.2f', [group._Type.Mass + group.Lading]));
  end;
  sgrAttackers.Rows[battle.GroupA.Count + 1].Clear;
end;

procedure TfrmBattleSimulator.AddGroupToDefenders(gr: TGroup);
begin
  if btnSimulate.Tag <> 1 then Exit;
  battle.GroupD.Add(gr);
end;

procedure TfrmBattleSimulator.AddGroupToAttackers(gr: TGroup);
begin
  if btnSimulate.Tag <> 1 then Exit;
  battle.GroupA.Add(gr);
end;

procedure TfrmBattleSimulator.NewBattle;
begin
  if btnSimulate.Tag <> 1 then Exit;
  DeleteGroups(battle.GroupA);
  DeleteGroups(battle.GroupD);
  battle.Free;
  battle := TBattle.Create;
end;

procedure TfrmBattleSimulator.SetGridsHeaders;
begin
  sgrDefenders.ColCount := 9;
  sgrDefenders.Rows[0].Clear;
  sgrDefenders.Rows[0].Add('#');
  sgrDefenders.Rows[0].Add('Owner');
  sgrDefenders.Rows[0].Add('Name');
  sgrDefenders.Rows[0].Add('Attacks');
  sgrDefenders.Rows[0].Add('Eff Weapon');
  sgrDefenders.Rows[0].Add('Eff Shield');
  sgrDefenders.Rows[0].Add('Eff Cargo');
  sgrDefenders.Rows[0].Add('Load');
  sgrDefenders.Rows[0].Add('Mass');

  sgrAttackers.ColCount := 9;
  sgrAttackers.Rows[0].Clear;
  sgrAttackers.Rows[0].Add('#');
  sgrAttackers.Rows[0].Add('Owner');
  sgrAttackers.Rows[0].Add('Name');
  sgrAttackers.Rows[0].Add('Attacks');
  sgrAttackers.Rows[0].Add('Eff Weapon');
  sgrAttackers.Rows[0].Add('Eff Shield');
  sgrAttackers.Rows[0].Add('Eff Cargo');
  sgrAttackers.Rows[0].Add('Load');
  sgrAttackers.Rows[0].Add('Mass');
end;

procedure TfrmBattleSimulator.FormCreate(Sender: TObject);
begin
  battle := TBattle.Create;
  ResultBattle := TBattle.Create;
  SetGridsHeaders;
end;

procedure TfrmBattleSimulator.FormDestroy(Sender: TObject);
begin
  DeleteGroups(battle.GroupA);
  DeleteGroups(battle.GroupD);
  battle.Free;
  ResultBattle.Free;
end;

procedure TfrmBattleSimulator.btnClearAttackersClick(Sender: TObject);
begin
  if btnSimulate.Tag <> 1 then Exit;
  DeleteGroups(battle.GroupA);
  UpdateBattleSim;
end;

procedure TfrmBattleSimulator.btnClearDefendersClick(Sender: TObject);
begin
  if btnSimulate.Tag <> 1 then Exit;
  DeleteGroups(battle.GroupD);
  UpdateBattleSim;
end;

procedure TfrmBattleSimulator.mniDeleteGroupDClick(Sender: TObject);
var
  group: Tgroup;
  index: integer;
begin
  if btnSimulate.Tag <> 1 then Exit;
  index := sgrDefenders.Selection.Top;
  if (index > 0) and (index < (sgrDefenders.RowCount - 1)) then
  begin
    group := battle.GroupD.Items[index - 1];
    group._Type.Free;
    group.Free;
    battle.GroupD.Delete(index - 1);
    UpdateBattleSim;
  end;
end;

procedure TfrmBattleSimulator.mniDeleteGroupAClick(Sender: TObject);
var
  group: Tgroup;
  index: integer;
begin
  if btnSimulate.Tag <> 1 then Exit;
  index := sgrAttackers.Selection.Top;
  if (index > 0) and (index < (sgrAttackers.RowCount - 1)) then
  begin
    group := battle.GroupA.Items[index - 1];
    group._Type.Free;
    group.Free;
    battle.GroupA.Delete(index - 1);
    UpdateBattleSim;
  end;
end;

procedure TfrmBattleSimulator.pmDefendersPopup(Sender: TObject);
var
  group: Tgroup;
  index: integer;
begin
  if btnSimulate.Tag <> 1 then Exit;
  index := sgrDefenders.Selection.Top;
  while pmDefenders.Items.Count > 1 do
    pmDefenders.Items.Delete(1);
  if (index > 0) and (index < (sgrDefenders.RowCount - 1)) then
  begin
    group := battle.GroupD.Items[index - 1];
    if group.EffCargo > 0 then
      pmDefenders.Items.Add(newItem('Load', 0, False, True, ChangeLoadD, 0, 'mniLoadD'));
  end;
end;

procedure TfrmBattleSimulator.pmAttackersPopup(Sender: TObject);
var
  group: Tgroup;
  index: integer;
begin
  if btnSimulate.Tag <> 1 then Exit;
  index := sgrAttackers.Selection.Top;
  while pmAttackers.Items.Count > 1 do
    pmAttackers.Items.Delete(1);
  if (index > 0) and (index < (sgrAttackers.RowCount - 1)) then
  begin
    group := battle.GroupA.Items[index - 1];
    if group.EffCargo > 0 then
      pmAttackers.Items.Add(newItem('Load', 0, False, True, ChangeLoadA, 0, 'mniLoadD'));
  end;
end;

procedure TfrmBattleSimulator.btnSimulateClick(Sender: TObject);
begin
  if btnSimulate.Tag = 1 then
  begin
    if (Battle.GroupA.Count = 0) or (Battle.GroupD.Count = 0) then Exit;
    btnSimulate.Tag := 2;
    btnSimulate.caption := 'Stop Simulation';
    BattleSim := TBatSim.Create;
    tshGroups.TabVisible := False;
    BattleSim.groupA := Battle.GroupA;
    BattleSim.groupD := Battle.GroupD;
    BattleSim.Priority := tpIdle;
    BattleSim.max_battle := edtMax_Battles.AsInteger;
    BattleSim.verb := cbxShowResults.Checked;
    Memo1.Lines.Clear;
    BattleSim.RoundResults := Memo1.Lines;
    BattleSim.Resume;
  end
  else
    if btnSimulate.Tag = 2 then
    begin
      btnSimulate.Tag := 0;
      btnSimulate.caption := 'Stopping Simulation';
      if BattleSim <> nil then
      begin
        BattleSim.Terminate;

        BattleSim.WaitFor;
        BattleSim.Free;
      end;
      BattleSim := nil;
      btnSimulate.Tag := 1;
      btnSimulate.caption := 'Start Simulation';
      tshGroups.TabVisible := True;
    end;
end;

procedure TfrmBattleSimulator.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  WindowState := wsMinimized;
  CanClose := false;
end;

end.
