unit UfrmEditor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Tabs, Menus, ActnList, ImgList, ComCtrls, ToolWin,
  OvcBase, OvcState, Buttons, ClipBrd, RichEdit, RichEdit2, RXCombos;

const
  RulerAdj = 4/3;
  GutterWid = 6;

type
  TfrmEditor = class(TForm)
    StandardToolBar: TToolBar;
    PrintButton: TToolButton;
    ToolButton5: TToolButton;
    CutButton: TToolButton;
    CopyButton: TToolButton;
    PasteButton: TToolButton;
    UndoButton: TToolButton;
    ToolButton10: TToolButton;
    FontName: TComboBox;
    ToolButton11: TToolButton;
    FontSize: TEdit;
    UpDown1: TUpDown;
    ToolButton2: TToolButton;
    BoldButton: TToolButton;
    ItalicButton: TToolButton;
    UnderlineButton: TToolButton;
    ToolButton16: TToolButton;
    LeftAlign: TToolButton;
    CenterAlign: TToolButton;
    RightAlign: TToolButton;
    ToolButton20: TToolButton;
    BulletsButton: TToolButton;
    ToolbarImages: TImageList;
    ActionList2: TActionList;
    EditUndoCmd: TAction;
    EditCutCmd: TAction;
    EditCopyCmd: TAction;
    EditPasteCmd: TAction;
    EditFontCmd: TAction;
    ActionList3: TActionList;
    FilePrintCmd: TAction;
    FontDialog1: TFontDialog;
    PrintDialog: TPrintDialog;
    MainMenu: TMainMenu;
    FileMenu: TMenuItem;
    FilePrintItem: TMenuItem;
    EditMenu: TMenuItem;
    EditUndoItem: TMenuItem;
    N2: TMenuItem;
    EditCutItem: TMenuItem;
    EditCopyItem: TMenuItem;
    EditPasteItem: TMenuItem;
    N5: TMenuItem;
    miEditFont: TMenuItem;
    TabSet1: TTabSet;
    Ruler: TPanel;
    FirstInd: TLabel;
    LeftInd: TLabel;
    RulerLine: TBevel;
    RightInd: TLabel;
    Bevel1: TBevel;
    OvcFormState1: TOvcFormState;
    actSelectColor: TAction;
    Editor: TRichEdit98;
    Panel1: TPanel;
    Label1: TLabel;
    ColorComboBox1: TColorComboBox;
    Label2: TLabel;
    ColorComboBox2: TColorComboBox;
    ColorDialog: TColorDialog;
    procedure EditUndoCmdExecute(Sender: TObject);
    procedure EditCutCmdExecute(Sender: TObject);
    procedure EditCopyCmdExecute(Sender: TObject);
    procedure EditPasteCmdExecute(Sender: TObject);
    procedure EditFontCmdExecute(Sender: TObject);
    procedure SelectionChange(Sender: TObject);
    procedure Editor2Change(Sender: TObject);
    procedure BoldButtonClick(Sender: TObject);
    procedure ItalicButtonClick(Sender: TObject);
    procedure UnderlineButtonClick(Sender: TObject);
    procedure LeftAlignClick(Sender: TObject);
    procedure BulletsButtonClick(Sender: TObject);
    procedure FontNameChange(Sender: TObject);
    procedure FontSizeChange(Sender: TObject);
    procedure LeftIndMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure LeftIndMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure LeftIndMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FirstIndMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure RightIndMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ActionList2Update(Action: TBasicAction;
      var Handled: Boolean);
    procedure FilePrintCmdExecute(Sender: TObject);
    procedure RulerResize(Sender: TObject);
    procedure TabSet1Change(Sender: TObject; NewTab: Integer;
      var AllowChange: Boolean);
    procedure actSelectColorExecute(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormHide(Sender: TObject);
    procedure ColorComboBox1Change(Sender: TObject);
    procedure ColorComboBox2Change(Sender: TObject);
    procedure EditorURLClick(Sender: TObject; URL: String);
  private
    FUpdating: boolean;
    FDragOfs: Integer;
    FDragging: boolean;
    function CurrText: TTextAttributes98;
    procedure GetFontNames;
    procedure SetEditRect;
    procedure SetModified(Value: Boolean);
    procedure SetupRuler;
    procedure UpdateCursorPos;
    { Private declarations }
  public
    function GetCurNotesFile: string;
    function GetNotesFile(aTab: integer): string;
    procedure DoLoad; overload;
    procedure DoLoad(aTab: integer); overload;
    procedure DoSave; overload;
    procedure DoSave(aTab: integer); overload;
    { Public declarations }
  end;

var
  frmEditor: TfrmEditor;

implementation

uses
  UFiles, ShellApi;

{$R *.DFM}

procedure TfrmEditor.EditUndoCmdExecute(Sender: TObject);
begin
  with Editor do
    if HandleAllocated then SendMessage(Handle, EM_UNDO, 0, 0);
end;

procedure TfrmEditor.EditCutCmdExecute(Sender: TObject);
begin
  Editor.CutToClipboard;
end;

procedure TfrmEditor.EditCopyCmdExecute(Sender: TObject);
begin
  Editor.CopyToClipboard;
end;

procedure TfrmEditor.EditPasteCmdExecute(Sender: TObject);
begin
  Editor.PasteFromClipboard;
end;

procedure TfrmEditor.EditFontCmdExecute(Sender: TObject);
begin
  FontDialog1.Font.Assign(Editor.SelAttributes);
  if FontDialog1.Execute then
    CurrText.Assign(FontDialog1.Font);
  SelectionChange(Self);
  Editor.SetFocus;
end;

procedure TfrmEditor.SelectionChange(Sender: TObject);
begin
  with Editor.Paragraph do
  try
    FUpdating := True;
    FirstInd.Left := Trunc(FirstIndent*RulerAdj)-4+GutterWid;
    LeftInd.Left := Trunc((LeftIndent+FirstIndent)*RulerAdj)-4+GutterWid;
    RightInd.Left := Ruler.ClientWidth-6-Trunc((RightIndent+GutterWid)*RulerAdj);

    BoldButton.Down := fsBold in Editor.SelAttributes.Style;
    ItalicButton.Down := fsItalic in Editor.SelAttributes.Style;
    UnderlineButton.Down := fsUnderline in Editor.SelAttributes.Style;
    BulletsButton.Down := Boolean(Numbering);
    FontSize.Text := IntToStr(Editor.SelAttributes.Size);
    FontName.Text := Editor.SelAttributes.Name;
    case Ord(Alignment) of
      0: LeftAlign.Down := True;
      1: RightAlign.Down := True;
      2: CenterAlign.Down := True;
    end;
    ColorComboBox1.ColorValue := Editor.SelAttributes.Color;
    ColorComboBox2.ColorValue := Editor.SelAttributes.BackColor;

    UpdateCursorPos;
  finally
    FUpdating := False;
  end;
end;

procedure TfrmEditor.Editor2Change(Sender: TObject);
begin
  SetModified(Editor.Modified);
end;

function TfrmEditor.CurrText: TTextAttributes98;
begin
  if Editor.SelLength > 0 then Result := Editor.SelAttributes
  else Result := Editor.DefAttributes;
end;

procedure TfrmEditor.SetModified(Value: Boolean);
begin

end;

procedure TfrmEditor.UpdateCursorPos;
begin

end;

procedure TfrmEditor.BoldButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  if BoldButton.Down then
    CurrText.Style := CurrText.Style + [fsBold]
  else
    CurrText.Style := CurrText.Style - [fsBold];
end;

procedure TfrmEditor.ItalicButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  if ItalicButton.Down then
    CurrText.Style := CurrText.Style + [fsItalic]
  else
    CurrText.Style := CurrText.Style - [fsItalic];
end;

procedure TfrmEditor.UnderlineButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  if UnderlineButton.Down then
    CurrText.Style := CurrText.Style + [fsUnderline]
  else
    CurrText.Style := CurrText.Style - [fsUnderline];
end;

procedure TfrmEditor.LeftAlignClick(Sender: TObject);
begin
  if FUpdating then Exit;
  Editor.Paragraph.Alignment := TAlignment98(TControl(Sender).Tag);
end;

procedure TfrmEditor.BulletsButtonClick(Sender: TObject);
begin
  if FUpdating then Exit;
  Editor.Paragraph.Numbering := TNumberingStyle98(BulletsButton.Down);
end;

procedure TfrmEditor.FontNameChange(Sender: TObject);
begin
  if FUpdating then Exit;
  CurrText.Name := FontName.Items[FontName.ItemIndex];
end;

procedure TfrmEditor.FontSizeChange(Sender: TObject);
begin
  if FUpdating then Exit;
  CurrText.Size := StrToInt(FontSize.Text);
end;

procedure TfrmEditor.LeftIndMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FDragOfs := (TLabel(Sender).Width div 2);
  TLabel(Sender).Left := TLabel(Sender).Left+X-FDragOfs;
  FDragging := True;
end;

procedure TfrmEditor.LeftIndMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  if FDragging then begin
    TLabel(Sender).Left :=  TLabel(Sender).Left+X-FDragOfs;
    if TLabel(Sender).Left < 0 then
      TLabel(Sender).Left := 0;
  end;
end;

procedure TfrmEditor.LeftIndMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDragging := False;
  Editor.Paragraph.LeftIndent := Trunc((LeftInd.Left+FDragOfs-GutterWid) / RulerAdj)-Editor.Paragraph.FirstIndent;
  SelectionChange(Sender);
end;

procedure TfrmEditor.FirstIndMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDragging := False;
  Editor.Paragraph.FirstIndent := Trunc((FirstInd.Left+FDragOfs-GutterWid) / RulerAdj);
  LeftIndMouseUp(Sender, Button, Shift, X, Y);
end;

procedure TfrmEditor.RightIndMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FDragging := False;
  Editor.Paragraph.RightIndent := Trunc((Ruler.ClientWidth-RightInd.Left+FDragOfs-2) / RulerAdj)-2*GutterWid;
  SelectionChange(Sender);
end;

procedure TfrmEditor.FormCreate(Sender: TObject);
var
  ScreenLogPixels: Integer;
  DC: HDC;
begin
  GetFontNames;
  SetupRuler;
  SelectionChange(Self);
  try
    CurrText.Name := 'Verdana';
  except
    CurrText.Name := DefFontData.Name;
  end;
  try
    CurrText.Size := 10;
  except
    DC := GetDC(0);
    ScreenLogPixels := GetDeviceCaps(DC, LOGPIXELSY);
    CurrText.Size := -MulDiv(DefFontData.Height, 72, ScreenLogPixels);
  end;
end;

procedure TfrmEditor.FormPaint(Sender: TObject);
begin
  SetEditRect;
end;

procedure TfrmEditor.FormResize(Sender: TObject);
begin
  SetEditRect;
  SelectionChange(Sender);
end;

procedure TfrmEditor.FormShow(Sender: TObject);
begin
  frmEditor.DoLoad;
  UpdateCursorPos;
  Editor2Change(nil);
  Editor.SetFocus;
end;

function EnumFontsProc(var LogFont: TLogFont; var TextMetric: TTextMetric;
  FontType: Integer; Data: Pointer): Integer; stdcall;
begin
  TStrings(Data).Add(LogFont.lfFaceName);
  Result := 1;
end;

procedure TfrmEditor.GetFontNames;
var
  DC: HDC;
begin
  DC := GetDC(0);
  EnumFonts(DC, nil, @EnumFontsProc, Pointer(FontName.Items));
  ReleaseDC(0, DC);
  FontName.Sorted := True;
end;

procedure TfrmEditor.SetupRuler;
var
  I: Integer;
  S: String;
begin
  SetLength(S, 201);
  I := 1;
  while I < 200 do
  begin
    S[I] := #9;
    S[I+1] := '|';
    Inc(I, 2);
  end;
  Ruler.Caption := S;
end;

procedure TfrmEditor.SetEditRect;
var
  R: TRect;
begin
  with Editor do
  begin
    R := Rect(GutterWid, 0, ClientWidth-GutterWid, ClientHeight);
    SendMessage(Handle, EM_SETRECT, 0, Longint(@R));
  end;
end;

procedure TfrmEditor.ActionList2Update(Action: TBasicAction;
  var Handled: Boolean);
begin
 { Update the status of the edit commands }
  EditCutCmd.Enabled := Editor.SelLength > 0;
  EditCopyCmd.Enabled := EditCutCmd.Enabled;
  if Editor.HandleAllocated then
  begin
    EditUndoCmd.Enabled := Editor.Perform(EM_CANUNDO, 0, 0) <> 0;
    EditPasteCmd.Enabled := Editor.Perform(EM_CANPASTE, 0, 0) <> 0;
  end;
end;

procedure TfrmEditor.FilePrintCmdExecute(Sender: TObject);
var
  vCaption: string;
begin
  vCaption := Tabset1.Tabs[Tabset1.TabIndex];
  if PrintDialog.Execute then
    Editor.Print(vCaption);
end;

procedure TfrmEditor.RulerResize(Sender: TObject);
begin
  RulerLine.Width := Ruler.ClientWidth - (RulerLine.Left*2);
end;

procedure TfrmEditor.DoSave;
begin
  Editor.Lines.SaveToFile(GetCurNotesFile);
  Editor.Modified := false;
end;

function TfrmEditor.GetCurNotesFile: string;
begin
  Result := GetNotesFile(tabset1.TabIndex);
end;

procedure TfrmEditor.DoLoad;
begin
  if FileExists(GetCurNotesFile) then
    Editor.Lines.LoadFromFile(GetCurNotesFile)
  else
    Editor.Lines.Clear;
  Editor.SetFocus;
  Editor.Modified := false;
end;

procedure TfrmEditor.TabSet1Change(Sender: TObject; NewTab: Integer;
  var AllowChange: Boolean);
begin
  DoSave;
  DoLoad(newTab);
end;

function TfrmEditor.GetNotesFile(aTab: integer): string;
begin
  Result := ExtractFilePath(ParamStr(0));
  case aTab of
    0: Result := Result + 'Notes_General.doc';
    1: Result := Result + 'Notes_General_OtherPlayers.doc';
    2: Result := Result + 'Notes_General_Servers.doc';
    4: Result := Result + 'Notes_FAQ.doc';
    5: Result := Result + 'Notes_Help.doc';
  else
    Result := Result + 'Notes_' + GetGameStr + '.doc';
  end;
end;

procedure TfrmEditor.DoLoad(aTab: integer);
begin
  if FileExists(GetNotesFile(aTab)) then
    Editor.Lines.LoadFromFile(GetNotesFile(aTab))
  else begin
    FUpdating := true;
    Editor.Lines.Clear;
    Editor.DefAttributes.Style := [];
    Editor.DefAttributes.Size := 10;
    Editor.DefAttributes.Color := clBlack;
    Editor.DefAttributes.BackColor := clWhite;
    SelectionChange(nil)
  end;
  Editor.SetFocus;
  Editor.Modified := false;
end;

procedure TfrmEditor.DoSave(aTab: integer);
begin
  Editor.Lines.SaveToFile(GetNotesFile(aTab));
  Editor.Modified := false;
end;

procedure TfrmEditor.actSelectColorExecute(Sender: TObject);
begin
  ColorDialog.Color := CurrText.Color;
  if ColorDialog.Execute then begin
    CurrText.Color := ColorDialog.Color;
  end;
end;

procedure TfrmEditor.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  WindowState := wsMinimized;
  CanClose := false;
end;

procedure TfrmEditor.FormHide(Sender: TObject);
begin
  frmEditor.DoSave;
end;

procedure TfrmEditor.ColorComboBox1Change(Sender: TObject);
begin
  if FUpdating then Exit;
  Editor.SelAttributes.Color := ColorComboBox1.ColorValue;
  Editor.SetFocus;
end;

procedure TfrmEditor.ColorComboBox2Change(Sender: TObject);
begin
  if FUpdating then Exit;
  Editor.SelAttributes.BackColor := ColorComboBox2.ColorValue;
  Editor.SetFocus;
end;

procedure TfrmEditor.EditorURLClick(Sender: TObject; URL: String);
begin
  SHellExecute(Application.handle, 'Open', PChar(URL), nil, nil, SW_SHOWNORMAL);
end;

end.
