unit UfrmGraph;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, UTypes, UFiles, TeeProcs, TeEngine, Chart, Series,
  OvcBase, OvcState;

type
  TfrmGraph = class(TForm)
    Panel1: TPanel;
    cbxPlayer: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    cbxSubject: TComboBox;
    lblTopPlayer: TLabel;
    Chart1: TChart;
    OvcFormState1: TOvcFormState;
    procedure FormShow(Sender: TObject);
    procedure cbxSubjectChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbxPlayerChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    FTopPlayer: TPlayer;
    FPlayersList: TList;
    FSubject: integer;
    procedure FillcbxPlayer;
    procedure FillPlayersList;
    function GetPlayerData(player: TPlayer; turn: integer): single;
    { Private declarations }
  public
    property PlayerList: TList read FPlayersList;
    procedure ReCalculate;
    procedure ClearPlayersList;
    { Public declarations }
  end;

var
  frmGraph: TfrmGraph;

implementation

uses
  UfrmMain;

{$R *.DFM}

var
  GraphPoints: array[0..MAX_PLAYERS, 0..MAX_TURNS] of Single;

procedure TfrmGraph.FillcbxPlayer;
var
  i: integer;
begin
  cbxPlayer.Clear;
  cbxPlayer.Items.AddObject('All', TObject(1));
  cbxPlayer.Items.AddObject('Top Six', TObject(2));
  for i := 0 to players.Count - 1 do
    cbxPlayer.Items.AddObject(TPlayer(players.Objects[i]).Name, players.Objects[i]);
end;

function SortEffIndustry(Item1, Item2: Pointer): Integer;
begin
  if TPlayer(item1).EffIndustryCur < TPlayer(item2).EffIndustryCur then
    Result := 1
  else
    if TPlayer(item1).EffIndustryCur > TPlayer(item2).EffIndustryCur then
      Result := -1
    else
      Result := 0;
end;

procedure TfrmGraph.FillPlayersList;
var
  player: TPlayer;
  i, j: integer;
  PlayList: TList;
begin
  ClearPlayersList;
  PlayList := TList.Create;
  try
    for i := 0 to players.Count - 1 do
    begin
      PlayList.Add(players.Objects[i]);
    end;
    PlayList.Sort(SortEffIndustry);
    if cbxPlayer.ItemIndex <> -1 then
    begin
      player := TPlayer(cbxPlayer.Items.Objects[cbxPlayer.ItemIndex]);
      if player = TPlayer(1) then
      begin
        for i := 0 to PlayList.Count - 1 do
        begin
          FPlayersList.Add(PlayList.Items[i]);
        end;
      end
      else
        if player = TPlayer(2) then
        begin
          j := PlayList.Count - 1;
          if j > 5 then j := 5;
          for i := 0 to j do
          begin
            FPlayersList.Add(PlayList.Items[i]);
          end;

        end
        else
        begin
          FPlayersList.Add(player);
          FPlayersList.Add(FTopPlayer);
        end;
    end;
  finally
    PlayList.Free;
  end;
end;

procedure TfrmGraph.FormShow(Sender: TObject);
var
  i: integer;
  MaxEffInd: single;
begin
  MaxEffInd := 0;
  FTopPlayer := nil;
  for i := 0 to players.Count - 1 do
  begin
    if Tplayer(players.Objects[i]).EffIndustryCur > MaxEffInd then
    begin
      MaxEffInd := Tplayer(players.Objects[i]).EffIndustryCur;
      FTopPlayer := TPlayer(players.Objects[i]);
    end;
  end;
  if Assigned(FTopPlayer) then lblTopPlayer.Caption := FTopPlayer.Name;
  FillcbxPlayer;
  cbxPlayer.ItemIndex := 0;
  cbxSubject.ItemIndex := 0;
  FSubject := 0;
  FillPlayersList;
  ReCalculate;
end;

{
Effective Industry
Effective Industry Delta
Industry
Industry Delta
Population
Population Delta
Population Percent Growth
Planets
Planets Delta
Total Technology
Total Technology Delta
Drive Technology
Drive Technology Delta
Weapons Technology
Weapons Technology Delta
Shield Technology
Shield Technology Delta
Cargo Technology
Cargo Technology Delta
Max Population
Max Population Delta
}

function TfrmGraph.GetPlayerData(player: TPlayer; turn: integer): single;
begin
  case FSubject of
    0: Result := player.EffIndustry[turn]; // Effective Industry
    1: Result := player.EffIndustryD[turn]; // Effective Industry Delta
    2: Result := player.Industry[turn]; // Industry
    3: Result := player.IndustryD[turn]; // Industry Delta
    4: Result := player.Population[turn]; // Population
    5: Result := player.PopulationD[turn]; // Population Delta
    6: Result := player.PopulationPercent[turn]; // Population Percent Growth
    7: Result := player.Planets[turn]; // Planets
    8: Result := player.PlanetsD[turn]; // Planets Delta
    9: Result := player.Drive[turn] +
      player.Weapons[turn] +
        player.Shield[turn] +
        player.Cargo[turn]; // Total Technology
    10: Result := player.DriveD[turn] +
      player.WeaponsD[turn] +
        player.ShieldD[turn] +
        player.CargoD[turn]; // Total Technology Delta
    11: Result := player.Drive[turn]; // Drive Technology
    12: Result := player.DriveD[turn]; // Drive Technology Delta
    13: Result := player.Weapons[turn]; // Weapons Technology
    14: Result := player.WeaponsD[turn]; // Weapons Technology Delta
    15: Result := player.Shield[turn]; // Shield Technology
    16: Result := player.ShieldD[turn]; // Shield Technology Delta
    17: Result := player.Cargo[turn]; // Cargo Technology
    18: Result := player.CargoD[turn]; // Cargo Technology Delta
    19: Result := player.maxPopulation[turn]; // MAX Population
    20: Result := player.maxPopulationD[turn]; // MAX Population Delta
  else
    Result := 0;
  end;
end;

procedure TfrmGraph.ClearPlayersList;
begin
  while FPlayersList.Count > 0 do
    FPlayersList.Delete(0);
end;

procedure TfrmGraph.ReCalculate;
const
  MAX_COL = 13;
var
  i, j: integer;
  SeriesList: TFastLineSeries;
  ColorTable: array[0..MAX_COL] of TColor;
begin
  Chart1.SeriesList.Clear;
  ColorTable[0] := clYellow;
  ColorTable[1] := clRed;
  ColorTable[2] := clBlue;
  ColorTable[3] := clLime;
  ColorTable[4] := clGreen;
  ColorTable[5] := clAqua;
  ColorTable[6] := clBlack;
  ColorTable[7] := clDkGray;
  ColorTable[8] := clLtGray;
  ColorTable[9] := clMaroon;
  ColorTable[10] := clNavy;
  ColorTable[11] := clOlive;
  ColorTable[12] := clPurple;
  ColorTable[13] := clTeal;
  for i := 0 to MAX_PLAYERS do
    for j := 0 to MAX_TURNS do
      GraphPoints[i, j] := 0;
  for i := 0 to FPlayersList.Count - 1 do
    for j := 0 to LastTurn do
      GraphPoints[i, j] := GetPlayerData(TPlayer(FPlayersList.Items[i]), j);
  if FPlayersList.Count < 1 then Exit;
  for i := 0 to FPlayersList.Count - 1 do
  begin
    SeriesList := TFastLineSeries.Create(Chart1);
    Chart1.AddSeries(SeriesList);
    SeriesList.Title := TPlayer(FPlayersList.items[i]).Name;
    SeriesList.SeriesColor := ColorTable[i mod 14];
    for j := 0 to LastTurn do
    begin
      SeriesList.AddXY(j, GraphPoints[i, j], IntToStr(j), clBlack);
    end;
  end;
end;

procedure TfrmGraph.cbxSubjectChange(Sender: TObject);
begin
  if FSubject <> cbxSubject.ItemIndex then
  begin
    FSubject := cbxSubject.ItemIndex;
    ReCalculate;
  end;
end;

procedure TfrmGraph.FormCreate(Sender: TObject);
begin
  FPlayersList := TList.Create;
end;

procedure TfrmGraph.FormDestroy(Sender: TObject);
begin
  ClearPlayersList;
  FPlayersList.Free;
end;

procedure TfrmGraph.cbxPlayerChange(Sender: TObject);
begin
  FillPlayersList;
  ReCalculate;
end;

procedure TfrmGraph.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  WindowState := wsMinimized;
  CanClose := false;
end;

end.
