unit UfrmGroupFilter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  UTypes, StdCtrls, ComCtrls, RXCtrls, RXSpin, Menus, OvcFiler, OvcStore,
  OvcState, OvcBase;

type
  TGroupFilter = class;

  TFilterPlayer = class
  private
    FPlayer: TPlayer;
    FSelected: boolean;
    procedure SetSelected(const Value: boolean);
  protected
    GroupFilter: TGroupFilter;
  public
    property Player: TPlayer read FPlayer write FPlayer;
    property Selected: boolean read FSelected write SetSelected;
  end;
  TFilterPlayers = array of TFilterPlayer;

  TFilterShipType = class
  private
    FSelected: boolean;
    FShipType: TShipType;
    FFilterPlayer: TFilterPlayer;
    procedure SetSelected(const Value: boolean);
  protected
    GroupFilter: TGroupFilter;
  public
    property FilterPlayer: TFilterPlayer read FFilterPlayer write FFilterPlayer;
    property Selected: boolean read FSelected write SetSelected;
    property ShipType: TShipType read FShipType write FShipType;
  end;
  TFilterShipTypes = array of TFilterShipType;

  TFilterShipOption = class
  private
    FUseAttacksMax: boolean;
    FUseDriveMax: boolean;
    FUseMassMax: boolean;
    FUseMassMin: boolean;
    FUseWeaponsMax: boolean;
    FUseAttacksMin: boolean;
    FUseDriveMin: boolean;
    FUseShieldsMin: boolean;
    FUseShieldsMax: boolean;
    FUseWeaponsMin: boolean;
    FdblAttacksMax: integer;
    FdblShieldsMin: single;
    FdblMassMax: single;
    FdblDriveMax: single;
    FdblShieldsMax: single;
    FdblWeaponsMin: single;
    FdblDriveMin: single;
    FdblWeaponsMax: single;
    FdblMassMin: single;
    FdblAttacksMin: integer;
  protected
    GroupFilter: TGroupFilter;
  public
    property dblAttacksMax: integer read FdblAttacksMax write FdblAttacksMax;
    property dblAttacksMin: integer read FdblAttacksMin write FdblAttacksMin;
    property dblDriveMax: single read FdblDriveMax write FdblDriveMax;
    property dblDriveMin: single read FdblDriveMin write FdblDriveMin;
    property dblMassMax: single read FdblMassMax write FdblMassMax;
    property dblMassMin: single read FdblMassMin write FdblMassMin;
    property dblShieldsMax: single read FdblShieldsMax write FdblShieldsMax;
    property dblShieldsMin: single read FdblShieldsMin write FdblShieldsMin;
    property dblWeaponsMax: single read FdblWeaponsMax write FdblWeaponsMax;
    property dblWeaponsMin: single read FdblWeaponsMin write FdblWeaponsMin;
    property UseAttacksMax: boolean read FUseAttacksMax write FUseAttacksMax;
    property UseAttacksMin: boolean read FUseAttacksMin write FUseAttacksMin;
    property UseDriveMax: boolean read FUseDriveMax write FUseDriveMax;
    property UseDriveMin: boolean read FUseDriveMin write FUseDriveMin;
    property UseMassMax: boolean read FUseMassMax write FUseMassMax;
    property UseMassMin: boolean read FUseMassMin write FUseMassMin;
    property UseWeaponsMax: boolean read FUseWeaponsMax write FUseWeaponsMax;
    property UseWeaponsMin: boolean read FUseWeaponsMin write FUseWeaponsMin;
    property UseShieldsMax: boolean read FUseShieldsMax write FUseShieldsMax;
    property UseShieldsMin: boolean read FUseShieldsMin write FUseShieldsMin;
  end;

  TGroupFilter = class
  private
    FFiltername: string;
    FFilterPlayers: TFilterPlayers;
    FFilterShipOption: TFilterShipOption;
    FFilterShipTypes: TFilterShipTypes;
    FThisTurn: boolean;
    FUseShipTypes: boolean;
    FUseShipCountMin: boolean;
    FUseShipCountMax: boolean;
    FShipCountMax: integer;
    FArriving: integer;
    FShipCountMin: integer;
    FOnFilterPlayerChanged: TNotifyEvent;
    FOnFilterShipTypeChanged: TNotifyEvent;
    FUpdating: boolean;
    FChangedShipTypes: boolean;
    FChangedPlayers: boolean;
    FUseArriving: boolean;
    FGroupsOnPlanets: TList;
    procedure SetUpdating(const Value: boolean);
    procedure SetChangedPlayers(const Value: boolean);
    procedure SetChangedShipTypes(const Value: boolean);
  protected
    procedure ClearFilterPlayers(var FilterPlayers: TFilterPlayers);
    procedure ClearFilterShipTypes(var FilterShipTypes: TFilterShipTypes);
    procedure CreateFilterPlayers(var FilterPlayers: TFilterPlayers);
    procedure CreateFilterShipTypes(FilterPlayers: TFilterPlayers; var FilterShipTypes: TFilterShipTypes);

    procedure FilterPlayerChanged;
    procedure FilterShipTypesChanged;
  public
    property Filtername: string read FFiltername write FFiltername;
    property FilterPlayers: TFilterPlayers read FFilterPlayers;
    property FilterShipOption: TFilterShipOption read FFilterShipOption;
    property FilterShipTypes: TFilterShipTypes read FFilterShipTypes;
    property ThisTurn: boolean read FThisTurn write FThisTurn;
    property UseShipTypes: boolean read FUseShipTypes write FUseShipTypes;
    property UseShipCountMax: boolean read FUseShipCountMax write FUseShipCountMax;
    property UseShipCountMin: boolean read FUseShipCountMin write FUseShipCountMin;
    property ShipCountMax: integer read FShipCountMax write FShipCountMax;
    property ShipCountMin: integer read FShipCountMin write FShipCountMin;
    property Arriving: integer read FArriving write FArriving;
    property UseArriving: boolean read FUseArriving write FUseArriving;

    property ChangedPlayers: boolean read FChangedPlayers write SetChangedPlayers;
    property ChangedShipTypes: boolean read FChangedShipTypes write SetChangedShipTypes;
    property Updating: boolean read FUpdating write SetUpdating;
    property OnFilterPlayerChanged: TNotifyEvent read FOnFilterPlayerChanged write FOnFilterPlayerChanged;
    property OnFilterShipTypeChanged: TNotifyEvent read FOnFilterShipTypeChanged write FOnFilterShipTypeChanged;

    property GroupsOnPlanets: TList read FGroupsOnPlanets;

    constructor Create;
    destructor Destroy; override;

    procedure CalculateGroupsOnPlanet;
    function IsGroupValid(group: TGroup): boolean;
    function PlayerOfGroupSelected(group: TGroup): boolean;
    function ShipTypeOfGroupSelected(group: TGroup): boolean;
    function ShipOptionsOfGroupValid(group: TGroup): boolean;
  end;

  TfrmGroupFilter = class(TForm)
    edtFiltername: TEdit;
    lbxPlayers: TRxCheckListBox;
    Label1: TLabel;
    lbxShiptypes: TRxCheckListBox;
    rbtShipTypes: TRadioButton;
    rbtShipOptions: TRadioButton;
    cbxDriveMin: TCheckBox;
    edtDriveMin: TRxSpinEdit;
    edtDriveMax: TRxSpinEdit;
    edtMinNumberShips: TRxSpinEdit;
    edtAttacksMin: TRxSpinEdit;
    edtAttacksMax: TRxSpinEdit;
    Label2: TLabel;
    Label3: TLabel;
    edtWeaponsMax: TRxSpinEdit;
    edtShieldsMin: TRxSpinEdit;
    edtShieldsMax: TRxSpinEdit;
    edtWeaponsMin: TRxSpinEdit;
    cbxAttacksMin: TCheckBox;
    cbxWeaponsMin: TCheckBox;
    cbxShieldsMin: TCheckBox;
    cbxDriveMax: TCheckBox;
    cbxAttacksMax: TCheckBox;
    cbxWeaponsMax: TCheckBox;
    cbxShieldsMax: TCheckBox;
    Label4: TLabel;
    edtMaxNumberShips: TRxSpinEdit;
    cbxThisTurn: TCheckBox;
    edtMassMin: TRxSpinEdit;
    edtMassMax: TRxSpinEdit;
    cbxMassMin: TCheckBox;
    cbxMassMax: TCheckBox;
    cbxArriving: TComboBox;
    cbxShipCountMax: TCheckBox;
    cbxShipCountMin: TCheckBox;
    PopupMenu1: TPopupMenu;
    btnSetLastChanges: TButton;
    btnClose: TButton;
    btnDiscardLastChanges: TButton;
    cbxUseArriving: TCheckBox;
    OvcFormState1: TOvcFormState;
    OvcComponentState1: TOvcComponentState;
    mniCheckSelected: TMenuItem;
    mniUncheckSelected: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure lbxPlayersClickCheck(Sender: TObject);
    procedure lbxShiptypesClickCheck(Sender: TObject);
    procedure btnDiscardLastChangesClick(Sender: TObject);
    procedure btnSetLastChangesClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure mniCheckSelectedClick(Sender: TObject);
  private
    FGroupFilter: TGroupFilter;
    procedure FillLbxPlayers(GroupFilter: TGroupFilter);
    procedure FillLbxShipTypes(GroupFilter: TGroupFilter);
    procedure FilterToOptionsOnForm(GroupFilter: TGroupFilter);
    procedure FilterFromOptionsOnForm(GroupFilter: TGroupFilter);
    procedure ExtraInfoToForm(GroupFilter: TGroupFilter);
    procedure ExtraInfoFromForm(GroupFilter: TGroupFilter);
    procedure SetGroupFilter(const Value: TGroupFilter);
    { Private declarations }
  public
    property GroupFilter: TGroupFilter read FGroupFilter write SetGroupFilter;
    { Public declarations }
  end;

var
  frmGroupFilter: TfrmGroupFilter;

implementation

{$R *.DFM}

uses
  UFiles, UfrmMain;

{ TfrmGroupFilter }

procedure TfrmGroupFilter.FillLbxPlayers(GroupFilter: TGroupFilter);
var
  i, index: integer;
begin
  lbxplayers.Items.BeginUpdate;
  try
    lbxPlayers.Clear;
    for i := 0 to high(GroupFilter.FilterPlayers) do begin
      index := lbxplayers.Items.AddObject(GroupFilter.FilterPlayers[i].Player.Name, GroupFilter.FilterPlayers[i]);
      lbxplayers.Checked[index] := GroupFilter.FilterPlayers[i].Selected;
    end;
  finally
    lbxplayers.Items.EndUpdate;
  end;
end;

procedure TfrmGroupFilter.FillLbxShipTypes(GroupFilter: TGroupFilter);
var
  i, index: integer;
begin
  lbxShiptypes.Items.BeginUpdate;
  try
    lbxShiptypes.Items.Clear;
    for i := 0 to high(GroupFilter.FilterShipTypes) do begin
      if GroupFilter.FilterShipTypes[i].FilterPlayer.Selected then begin
        index := lbxShiptypes.Items.AddObject(GroupFilter.FilterShipTypes[i].ShipType.Name, GroupFilter.FilterShipTypes[i]);
        lbxShiptypes.Checked[index] := GroupFilter.FilterShipTypes[i].Selected;
      end;
    end;
  finally
    lbxShiptypes.Items.EndUpdate;
  end;
end;

procedure TfrmGroupFilter.SetGroupFilter(const Value: TGroupFilter);
begin
  FGroupFilter := Value;
end;

procedure TfrmGroupFilter.FilterFromOptionsOnForm(
  GroupFilter: TGroupFilter);
begin
  GroupFilter.FilterShipOption.UseDriveMin := cbxDriveMin.Checked;
  GroupFilter.FilterShipOption.UseDriveMax := cbxDriveMax.Checked;
  GroupFilter.FilterShipOption.UseAttacksMin := cbxAttacksMin.Checked;
  GroupFilter.FilterShipOption.UseAttacksMax := cbxAttacksMax.Checked;
  GroupFilter.FilterShipOption.UseWeaponsMin := cbxWeaponsMin.Checked;
  GroupFilter.FilterShipOption.UseWeaponsMax := cbxWeaponsMax.Checked;
  GroupFilter.FilterShipOption.UseShieldsMin := cbxShieldsMin.Checked;
  GroupFilter.FilterShipOption.UseShieldsMax := cbxShieldsMax.Checked;
  GroupFilter.FilterShipOption.UseMassMin := cbxMassMin.Checked;
  GroupFilter.FilterShipOption.UseMassMax := cbxMassMax.Checked;
  GroupFilter.FilterShipOption.dblDriveMin := edtDriveMin.Value;
  GroupFilter.FilterShipOption.dblDriveMax := edtDriveMax.Value;
  GroupFilter.FilterShipOption.dblAttacksMin := edtAttacksMin.AsInteger;
  GroupFilter.FilterShipOption.dblAttacksMax := edtAttacksMax.AsInteger;
  GroupFilter.FilterShipOption.dblWeaponsMin := edtWeaponsMin.Value;
  GroupFilter.FilterShipOption.dblWeaponsMax := edtWeaponsMax.Value;
  GroupFilter.FilterShipOption.dblShieldsMin := edtShieldsMin.Value;
  GroupFilter.FilterShipOption.dblShieldsMax := edtShieldsMax.Value;
  GroupFilter.FilterShipOption.dblMassMin := edtMassMin.Value;
  GroupFilter.FilterShipOption.dblMassMax := edtMassMax.Value;
end;

procedure TfrmGroupFilter.FilterToOptionsOnForm(GroupFilter: TGroupFilter);
begin
  cbxDriveMin.Checked := GroupFilter.FilterShipOption.UseDriveMin;
  cbxDriveMax.Checked := GroupFilter.FilterShipOption.UseDriveMax;
  cbxAttacksMin.Checked := GroupFilter.FilterShipOption.UseAttacksMin;
  cbxAttacksMax.Checked := GroupFilter.FilterShipOption.UseAttacksMax;
  cbxWeaponsMin.Checked := GroupFilter.FilterShipOption.UseWeaponsMin;
  cbxWeaponsMax.Checked := GroupFilter.FilterShipOption.UseWeaponsMax;
  cbxShieldsMin.Checked := GroupFilter.FilterShipOption.UseShieldsMin;
  cbxShieldsMax.Checked := GroupFilter.FilterShipOption.UseShieldsMax;
  cbxMassMin.Checked := GroupFilter.FilterShipOption.UseMassMin;
  cbxMassMax.Checked := GroupFilter.FilterShipOption.UseMassMax;
  edtDriveMin.Value := GroupFilter.FilterShipOption.dblDriveMin;
  edtDriveMax.Value := GroupFilter.FilterShipOption.dblDriveMax;
  edtAttacksMin.Value := GroupFilter.FilterShipOption.dblAttacksMin;
  edtAttacksMax.Value := GroupFilter.FilterShipOption.dblAttacksMax;
  edtWeaponsMin.Value := GroupFilter.FilterShipOption.dblWeaponsMin;
  edtWeaponsMax.Value := GroupFilter.FilterShipOption.dblWeaponsMax;
  edtShieldsMin.Value := GroupFilter.FilterShipOption.dblShieldsMin;
  edtShieldsMax.Value := GroupFilter.FilterShipOption.dblShieldsMax;
  edtMassMin.Value := GroupFilter.FilterShipOption.dblMassMin;
  edtMassMax.Value := GroupFilter.FilterShipOption.dblMassMax;
end;

procedure TfrmGroupFilter.ExtraInfoFromForm(GroupFilter: TGroupFilter);
begin
  GroupFilter.Filtername := edtFilterName.Text;
  GroupFilter.UseShipTypes := rbtShipTypes.Checked;
  GroupFilter.ThisTurn := cbxThisTurn.Checked;
  GroupFilter.UseShipCountMax := cbxShipCountMax.Checked;
  GroupFilter.UseShipCountMin := cbxShipCountMin.Checked;
  GroupFilter.ShipCountMax := edtMaxNumberShips.AsInteger;
  GroupFilter.ShipCountMin := edtMinNumberShips.AsInteger;
  GroupFilter.Arriving := cbxArriving.ItemIndex;
  GroupFilter.UseArriving := cbxUseArriving.Checked;
end;

procedure TfrmGroupFilter.ExtraInfoToForm(GroupFilter: TGroupFilter);
begin
  edtFilterName.Text := GroupFilter.Filtername;
  rbtShipTypes.Checked := GroupFilter.UseShipTypes;
  cbxThisTurn.Checked := GroupFilter.ThisTurn;
  cbxShipCountMax.Checked := GroupFilter.UseShipCountMax;
  cbxShipCountMin.Checked := GroupFilter.UseShipCountMin;
  edtMaxNumberShips.AsInteger := GroupFilter.ShipCountMax;
  edtMinNumberShips.AsInteger := GroupFilter.ShipCountMin;
  cbxArriving.ItemIndex := GroupFilter.Arriving;
  cbxUseArriving.Checked := GroupFilter.UseArriving;
end;

{ TGroupFilter }

procedure TGroupFilter.CalculateGroupsOnPlanet;
var
  indexPl, index: integer;
begin
  FGroupsOnPlanets.Clear;
  for indexPl := 0 to high(FFilterPlayers) do begin
    if FFilterPlayers[indexPl].Selected then begin
      for index := 0 to FFilterPlayers[indexPl].Player.Groups.Count - 1 do begin
        if IsGroupValid(TGroup(FFilterPlayers[indexPl].Player.Groups.Items[index])) then
          FGroupsOnPlanets.Add(TGroup(FFilterPlayers[indexPl].Player.Groups.Items[index]).Planet);
      end;
    end;
  end;
end;

procedure TGroupFilter.ClearFilterPlayers(var FilterPlayers: TFilterPlayers);
var
  indexPl: integer;
begin
  for indexPl := 0 to high(FFilterPlayers) do
    FFilterPlayers[indexPl].Free;
  SetLength(FFilterPlayers, 0);
end;

procedure TGroupFilter.ClearFilterShipTypes(var FilterShipTypes: TFilterShipTypes);
var
  indexST: integer;
begin
  for indexST := 0 to high(FFilterShipTypes) do
    FFilterShipTypes[indexST].Free;
  SetLength(FFilterShipTypes, 0);
end;

constructor TGroupFilter.Create;
begin
  Updating := True;
  try
    CreateFilterPlayers(FFilterPlayers);
    CreateFilterShipTypes(FFilterPlayers, FFilterShipTypes);
    FFilterShipOption := TFilterShipOption.Create;
    FFilterShipOption.GroupFilter := Self;
    FUseShipTypes := True;
    FThisTurn := True;

    FGroupsOnPlanets := TList.Create;
  finally
    Updating := False;
  end;
end;

procedure TGroupFilter.CreateFilterPlayers(var FilterPlayers: TFilterPlayers);
var
  indexPl: integer;
  intCount: integer;
begin
  ClearFilterPlayers(FFilterPlayers);
  intCount := 0;
  for indexPl := 0 to players.Count - 1 do
    if TPlayer(players.Objects[indexPl]).Groups.Count > 0 then inc(intCount);
  SetLength(FFilterPlayers, intCount);
  intCount := 0;
  for indexPl := 0 to players.Count - 1 do begin
    if TPlayer(players.Objects[indexPl]).Groups.Count > 0 then begin
      FFilterPlayers[intCount] := TFilterPlayer.Create;
      FFilterPlayers[intCount].GroupFilter := Self;
      FFilterPlayers[intCount].Selected := qreportPlayer = TPlayer(players.Objects[indexPl]);
      FFilterPlayers[intCount].Player := TPlayer(players.Objects[indexPl]);
      inc(intCount);
    end;
  end;
end;

procedure TGroupFilter.CreateFilterShipTypes(FilterPlayers: TFilterPlayers; var FilterShipTypes: TFilterShipTypes);
var
  indexPl, indexST, indexSTLastPlayer: integer;
  player: TPlayer;
begin
  ClearFilterShipTypes(FFilterShipTypes);
  indexSTLastPlayer := 0;
  for indexPl := 0 to high(FilterPlayers) do begin
    player := FilterPlayers[indexPl].Player;
    SetLength(FFilterShipTypes, indexSTLastPlayer + player.ShipTypes.Count);
    for indexST := 0 to player.ShipTypes.Count - 1 do begin
      FFilterShipTypes[indexSTLastPlayer + indexST] := TFilterShipType.Create;
      FFilterShipTypes[indexSTLastPlayer + indexST].GroupFilter := Self;
      FFilterShipTypes[indexSTLastPlayer + indexST].Selected := True;
      FFilterShipTypes[indexSTLastPlayer + indexST].FilterPlayer := FilterPlayers[indexPl];
      FFilterShipTypes[indexSTLastPlayer + indexST].ShipType := TShipType(player.ShipTypes.Objects[indexST]);
    end;
    indexSTLastPlayer := indexSTLastPlayer + player.ShipTypes.Count;
  end;
end;

destructor TGroupFilter.Destroy;
begin
  FUpdating := True;
  ClearFilterPlayers(FFilterPlayers);
  ClearFilterShipTypes(FFilterShipTypes);
  FFilterShipOption.Free;
  FGroupsOnPlanets.Free;
end;

procedure TfrmGroupFilter.FormCreate(Sender: TObject);
begin
  GroupFilter := TGroupFilter.Create;
end;

procedure TfrmGroupFilter.FormDestroy(Sender: TObject);
begin
  GroupFilter.Free;
end;

procedure TfrmGroupFilter.FormShow(Sender: TObject);
begin
  FGroupFilter.Updating := true;
  try
    FillLbxPlayers(FGroupFilter);
    FillLbxShipTypes(FGroupFilter);
    FilterToOptionsOnForm(FGroupFilter);
  finally
    FGroupFilter.Updating := False;
  end;
end;

procedure TfrmGroupFilter.lbxPlayersClickCheck(Sender: TObject);
begin
  if lbxPlayers.ItemIndex <> -1 then begin
    TFilterPlayer(lbxPlayers.Items.Objects[lbxPlayers.ItemIndex]).Selected := lbxPlayers.Checked[lbxPlayers.ItemIndex];
    FillLbxShipTypes(FGroupFilter);
  end;
end;

procedure TfrmGroupFilter.lbxShiptypesClickCheck(Sender: TObject);
begin
  if lbxShiptypes.ItemIndex <> -1 then
    TFilterShipType(lbxShiptypes.Items.Objects[lbxShiptypes.ItemIndex]).Selected := lbxShiptypes.Checked[lbxShiptypes.ItemIndex];
end;

procedure TfrmGroupFilter.btnDiscardLastChangesClick(Sender: TObject);
begin
  FilterToOptionsOnForm(FGroupFilter);
  ExtraInfoToForm(FGroupFilter);
end;

procedure TfrmGroupFilter.btnSetLastChangesClick(Sender: TObject);
begin
  FilterFromOptionsOnForm(FGroupFilter);
  ExtraInfoFromForm(FGroupFilter);
end;

procedure TGroupFilter.FilterPlayerChanged;
begin
  if Assigned(FOnFilterPlayerChanged) then FOnFilterPlayerChanged(Self);
  FChangedPlayers := False;
end;

procedure TGroupFilter.FilterShipTypesChanged;
begin
  if Assigned(FOnFilterShipTypeChanged) then FOnFilterShipTypeChanged(Self);
  FChangedPlayers := False;
end;

function TGroupFilter.IsGroupValid(group: TGroup): boolean;
begin
  Result := False;
  if UseShipCountMax then if ShipCountMax > group.Nr then Exit;
  if UseShipCountMin then if ShipCountMin < group.Nr then Exit;
  Result := (not FThisTurn or (FThisTurn and (group.Turn = LastTurn))) and
    PlayerOfGroupSelected(group) and
    (((not FUseShipTypes) and ShipOptionsOfGroupValid(group)) or (FUseShipTypes and ShipTypeOfGroupSelected(group)));
  if Result then begin
    if UseArriving and (group.EffSpeedNow > 0) then
      Result := (Arriving = trunc(group.Range / group.EffSpeedNow));
  end;
end;

function TGroupFilter.PlayerOfGroupSelected(group: TGroup): boolean;
var
  index: integer;
begin
  Result := False;
  index := 0;
  while index < Length(FFilterPlayers) do begin
    if FFilterPlayers[index].Player = group.Player then begin
      Result := FFilterPlayers[index].Selected;
      Exit;
    end;
    inc(index);
  end;
end;

procedure TGroupFilter.SetChangedPlayers(const Value: boolean);
begin
  if FChangedPlayers <> Value then begin
    FChangedPlayers := Value;
    if not FUpdating then
      FilterPlayerChanged;
  end;
end;

procedure TGroupFilter.SetChangedShipTypes(const Value: boolean);
begin
  if FChangedShipTypes <> Value then begin
    FChangedShipTypes := Value;
    if not FUpdating then
      FilterShipTypesChanged;
  end;
end;

procedure TGroupFilter.SetUpdating(const Value: boolean);
begin
  if (Value = False) and FUpdating then begin
    if FChangedPlayers then FilterPlayerChanged;
    if FChangedShipTypes then FilterShipTypesChanged;
  end;
  FUpdating := Value;
end;

function TGroupFilter.ShipOptionsOfGroupValid(group: TGroup): boolean;
begin
  Result := False;
  if FFilterShipOption.UseDriveMax then if group._Type.Drive > FFilterShipOption.dblDriveMax then Exit;
  if FFilterShipOption.UseDriveMin then if group._Type.Drive < FFilterShipOption.dblDriveMin then Exit;
  if FFilterShipOption.UseAttacksMax then if group._Type.Attacks > FFilterShipOption.dblAttacksMax then Exit;
  if FFilterShipOption.UseAttacksMin then if group._Type.Attacks < FFilterShipOption.dblAttacksMin then Exit;
  if FFilterShipOption.UseWeaponsMax then if group._Type.Weapons > FFilterShipOption.dblWeaponsMax then Exit;
  if FFilterShipOption.UseWeaponsMin then if group._Type.Weapons < FFilterShipOption.dblWeaponsMin then Exit;
  if FFilterShipOption.UseShieldsMax then if group._Type.Shields > FFilterShipOption.dblShieldsMax then Exit;
  if FFilterShipOption.UseShieldsMin then if group._Type.Shields < FFilterShipOption.dblShieldsMin then Exit;
  if FFilterShipOption.UseMassMax then if group._Type.Mass > FFilterShipOption.dblMassMax then Exit;
  if FFilterShipOption.UseMassMin then if group._Type.Mass < FFilterShipOption.dblMassMin then Exit;
  Result := True;
end;

function TGroupFilter.ShipTypeOfGroupSelected(group: TGroup): boolean;
var
  index: integer;
begin
  Result := False;
  index := 0;
  while index < Length(FFilterShipTypes) do begin
    if (FFilterShipTypes[index].FilterPlayer.Player = group.Player) and (FFilterShipTypes[index].ShipType = group._Type) then begin
      Result := FFilterShipTypes[index].Selected;
      Exit;
    end;
    inc(index);
  end;
end;

{ TFilterPlayer }

procedure TFilterPlayer.SetSelected(const Value: boolean);
begin
  if FSelected <> Value then begin
    FSelected := Value;
    GroupFilter.ChangedPlayers := True;
  end;
end;

{ TFilterShipType }

procedure TFilterShipType.SetSelected(const Value: boolean);
begin
  if FSelected <> Value then begin
    FSelected := Value;
    GroupFilter.ChangedShipTypes := True;
  end;
end;

procedure TfrmGroupFilter.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmGroupFilter.mniCheckSelectedClick(Sender: TObject);
var
  index: integer;
begin
  if not (ActiveControl is TRxCheckListBox) then Exit;
  if not (Sender is TMenuItem) then Exit;
  with ActiveControl as TRxCheckListBox do begin
    for index := 0 to Items.Count - 1 do begin
      if Selected[index] then Checked[index] := TMenuItem(Sender).Tag = 1;
    end;
  end;
  FillLbxShipTypes(FGroupFilter);
end;

end.
