unit UfrmNewMap;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  UTypes, Menus, OvcBase, OvcState, SysMenu, TopWnd, widget;

type
  TMap = class;

  TMapBookMark = class(TComponent)
  private
    FValid: boolean;
    FYPos: integer;
    FXPos: integer;
    FRatio: integer;
    FYRange: integer;
    FXRange: integer;
    procedure SetRatio(const Value: integer);
    procedure SetValid(const Value: boolean);
    procedure SetXPos(const Value: integer);
    procedure SetYPos(const Value: integer);
    procedure SetXRange(const Value: integer);
    procedure SetYRange(const Value: integer);
  published
    property Ratio: integer read FRatio write SetRatio;
    property XPos: integer read FXPos write SetXPos;
    property YPos: integer read FYPos write SetYPos;
    property Valid: boolean read FValid write SetValid;
    property XRange: integer read FXRange write SetXRange;
    property YRange: integer read FYRange write SetYRange;
  end;

  TStoreMapBookmarks = class(TComponent)
  private
    FBM0: TMapBookMark;
    FBM6: TMapBookMark;
    FBM1: TMapBookMark;
    FBM9: TMapBookMark;
    FBM8: TMapBookMark;
    FBM5: TMapBookMark;
    FBM4: TMapBookMark;
    FBM7: TMapBookMark;
    FBM3: TMapBookMark;
    FBM2: TMapBookMark;
    FLastPos: TMapBookMark;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property BM0: TMapBookMark read FBM0;
    property BM1: TMapBookMark read FBM1;
    property BM2: TMapBookMark read FBM2;
    property BM3: TMapBookMark read FBM3;
    property BM4: TMapBookMark read FBM4;
    property BM5: TMapBookMark read FBM5;
    property BM6: TMapBookMark read FBM6;
    property BM7: TMapBookMark read FBM7;
    property BM8: TMapBookMark read FBM8;
    property BM9: TMapBookMark read FBM9;
    property LastPos: TMapBookMark read FLastPos;
  end;

  TShowIncomings = class
  private
    FromX, FromY: integer;
    ToX, ToY: integer;
    CurX, CurY: integer;
    FCanvas: TCanvas;
    FMap: TMap;
  public
    property Canvas: TCanvas read FCanvas write FCanvas;
    property Map: TMap read FMap write FMap;

    procedure SetIncoming(const FromPlanet, DestPlanet: TPlanet; const range, speed: single);
    procedure DrawCursor;
  end;

  TGroupCursor = class
  private
    FX, FY: integer;
    FSpeed: single;
    FSpeed2: single;
    FCanvas: TCanvas;
    FMap: TMap;
    FDontShow: boolean;
  public
    property Canvas: TCanvas read FCanvas write FCanvas;
    property Map: TMap read FMap write FMap;
    property DontShow: boolean read FDontShow write FDontShow;
    procedure DrawCursor;
    procedure SetGroup(const x, y: integer; const speed, speed2: single);
  end;

  TPlanetCursor = class
  private
    intX, intY: integer; // position on map
    FX1, FY1, FX2, FY2: integer;
    FMapRange: integer;
    FShow: boolean;
    FPlanet: TPlanet;
    FMap: TMap;
    FFormCanvas: TCanvas;
    FShowRanges: boolean;
    FRange: integer;
    procedure SetFormCanvas(const Value: TCanvas);
  protected
    function MustPaint: boolean;
  public
    procedure Recalc; virtual;
    procedure Draw; virtual;
    property Planet: TPlanet read FPlanet write FPlanet;
    property Show: boolean read FShow write FShow;
    property ShowRanges: boolean read FShowRanges write FShowRanges;
    property Range: integer read FRange write FRange;
    property Map: TMap read FMap write FMap;
    property FormCanvas: TCanvas read FFormCanvas write SetFormCanvas;
  end;

  TPlanet2Cursor = class(TPlanetCursor)
    procedure Draw; override;
  end;

  TPlanetPCursor = class(TPlanetCursor)
    procedure Draw; override;
  end;

  TPlanetPCursorRed = class(TPlanetPCursor)
    procedure Draw; override;
  end;

  TPlanetPCursorBlue = class(TPlanetPCursor)
    procedure Draw; override;
  end;

  TPlanetPCursorClass = class of TPlanetPCursor;

  TCursorState = (csNone, csCtrl, csAlt);

  TMap = class
  private
    FPercent: integer;
    FMaxMapY: integer;
    FMaxMapX: integer;
    FScreenClientY: integer;
    FScreenClientX: integer;
    FOffsetY: integer;
    FOffsetX: integer;
    FUpdateCount: integer;
    FShowGrowth: boolean;
    FShowNames: boolean;
    FShowIncomings: boolean;
    procedure SetPercent(const Value: integer);
    procedure SetMaxMapY(const Value: integer);
    procedure SetMaxMapX(const Value: integer);
    procedure SetScreenClientX(const Value: integer);
    procedure SetScreenClientY(const Value: integer);
    procedure SetOffsetX(const Value: integer);
    procedure SetOffsetY(const Value: integer);
    procedure SetShowGrowth(const Value: boolean);
    procedure SetShowNames(const Value: boolean);
    procedure SetShowIncomings(const Value: boolean);
  protected
    procedure ScreenPosToMapPos(intScreenX, intScreenY: integer; out intMapX, intMapY: integer); virtual;
    procedure MapPosToScreenPos(intMapX, intMapY: integer; out intScreenX, intScreenY: integer); virtual;
    procedure MaxMapChanged;
    procedure DoChanges;
    procedure CalculatePercent;
  public
    property OffsetX: integer read FOffsetX write SetOffsetX;
    property OffsetY: integer read FOffsetY write SetOffsetY;
    property Percent: integer read FPercent write SetPercent;
    property MaxMapX: integer read FMaxMapX write SetMaxMapX;
    property MaxMapY: integer read FMaxMapY write SetMaxMapY;
    property ScreenClientX: integer read FScreenClientX write SetScreenClientX;
    property ScreenClientY: integer read FScreenClientY write SetScreenClientY;

    property ShowGrowth: boolean read FShowGrowth write SetShowGrowth;
    property ShowNames: boolean read FShowNames write SetShowNames;
    property ShowIncomings: boolean read FShowIncomings write SetShowIncomings;
    procedure BeginUpdate;
    procedure EndUpdate;

    procedure PaintPlanet(const canvas: TCanvas; const Name: string; const MapX, MapY, size, intSize, pop, indus: integer; const color: TColor);
    procedure SetMaxMapXY(const X, Y: integer);
    procedure SetOffsetXY(const X, Y: integer);
    procedure SetScreenClientXY(const X, Y: integer);
  end;

  TAreaSelectionCursor = class
  private
    FCursor: TRect;
  public
    constructor Create;
    procedure CalculateSelectionCursor(PosRect: TRect);
    procedure DrawSelectionCursor(canvas: TCanvas);
    procedure RemoveSelectionCursor(canvas: TCanvas);
  end;

  TOverlayItem = class
  private
    FMap: TMap;
  public
    property Map: TMap read FMap write FMap;
    procedure paint(Canvas: TCanvas); virtual; abstract;
    procedure SaveToStream(stream: TStream); virtual; abstract;
    procedure LoadFromStream(stream: TStream); virtual; abstract;
  end;

  TOverlayCircle = class(TOverlayItem)
  private
    FY: integer;
    FX: integer;
    FRadius: Integer;
  public
    property X: integer read FX write FX;
    property Y: integer read FY write FY;
    property Radius: Integer read FRadius write FRadius;
    procedure paint(Canvas: TCanvas); override;
    procedure SaveToStream(stream: TStream); override;
    procedure LoadFromStream(stream: TStream); override;
  end;

  TfrmNewMap = class(TForm)
    OvcFormState1: TOvcFormState;
    MapSysMenu: TPopupMenu;
    SystemMenu1: TSystemMenu;
    TopMostWindow1: TTopMostWindow;
    mniTopMost: TMenuItem;
    WidgetFullExtends: TWidget;
    WidgetPlus: TWidget;
    WidgetMin: TWidget;
    WidgetMarkBlue: TWidget;
    WidgetMarkRed: TWidget;
    WidgetShowGrowth: TWidget;
    WidgetShowNames: TWidget;
    WidgetShowIncomings: TWidget;
    OvcPersistentState1: TOvcPersistentState;
    DisableGroupCircles1: TMenuItem;
    OvcComponentState1: TOvcComponentState;
    MapMenu: TPopupMenu;
    Viewplanetinorders1: TMenuItem;
    RenamePlanet1: TMenuItem;
    InsertDevelopTag1: TMenuItem;
    ClaimOwnerShip1: TMenuItem;
    Produceonplanet1: TMenuItem;
    N1: TMenuItem;
    N2: TMenuItem;
    SetNewMapArea1: TMenuItem;
    N3: TMenuItem;
    TracksCurrentturn1: TMenuItem;
    TracksPreviousturn1: TMenuItem;
    TracksNone1: TMenuItem;
    TracksNone2: TMenuItem;
    TracksCurrentturn2: TMenuItem;
    TracksPreviousturn2: TMenuItem;
    Tracks1: TMenuItem;
    Routes1: TMenuItem;
    CAP1: TMenuItem;
    COL1: TMenuItem;
    MAT1: TMenuItem;
    EMP1: TMenuItem;
    Map1: TMenuItem;
    ClearRedMarks1: TMenuItem;
    ClearBlueMarks1: TMenuItem;
    DisableGroupCircles2: TMenuItem;
    ClearPlayerMarks1: TMenuItem;
    DisablePlanetCircles1: TMenuItem;
    MapRange101: TMenuItem;
    mniSpecialValue: TMenuItem;
    Viewplanetscentered1: TMenuItem;
    ChangeColors1: TMenuItem;
    mniOverlayMode: TMenuItem;
    mniShowOverlay: TMenuItem;
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormDblClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormActivate(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure mniTopMostClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure mniFullExtendsClick(Sender: TObject);
    procedure mniMarkPlanetsClick(Sender: TObject);
    procedure WidgetMinClick(Sender: TObject);
    procedure WidgetPlusClick(Sender: TObject);
    procedure WidgetShowGrowthClick(Sender: TObject);
    procedure WidgetShowIncomingsClick(Sender: TObject);
    procedure WidgetShowNamesClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure DisableGroupCircles1Click(Sender: TObject);
    procedure Viewplanetinorders1Click(Sender: TObject);
    procedure RenamePlanet1Click(Sender: TObject);
    procedure MapMenuPopup(Sender: TObject);
    procedure InsertDevelopTag1Click(Sender: TObject);
    procedure ClaimOwnerShip1Click(Sender: TObject);
    procedure Produceonplanet1Click(Sender: TObject);
    procedure SetNewMapArea1Click(Sender: TObject);
    procedure TracksCurrentturn1Click(Sender: TObject);
    procedure TracksNone1Click(Sender: TObject);
    procedure TracksPreviousturn1Click(Sender: TObject);
    procedure Routes1Click(Sender: TObject);
    procedure CAP1Click(Sender: TObject);
    procedure COL1Click(Sender: TObject);
    procedure MAT1Click(Sender: TObject);
    procedure EMP1Click(Sender: TObject);
    procedure ClearPlayerMarks1Click(Sender: TObject);
    procedure ClearRedMarks1Click(Sender: TObject);
    procedure ClearBlueMarks1Click(Sender: TObject);
    procedure DisablePlanetCircles1Click(Sender: TObject);
    procedure Map1Click(Sender: TObject);
    procedure MapRange101Click(Sender: TObject);
    procedure Viewplanetscentered1Click(Sender: TObject);
    procedure ChangeColors1Click(Sender: TObject);
  private
    FTrackturn: integer;
    FAreaSelectionCursor: TAreaSelectionCursor;
    FMap: TMap;
    FCursorState: TCursorState;
    FCursorRect: TRect;
    FPlanetCursor: TPlanetCursor;
    FPlanetCursor2: TPlanet2Cursor;
    FCursors: TList;
    FCursorsRed: Tlist;
    FCursorsBlue: Tlist;
    FGroupCursor: TGroupCursor;
    FMouseDownEvent: boolean;
    FDoNotpointPlanet: boolean;
    FDoDrag: boolean;
    FDragX: integer;
    FDragY: integer;
    FIncomings: TList;
    FStoreMapBookmarks: TStoreMapBookmarks;
    FOverlayItems: array of TOverlayItem;
    procedure WMHScroll(var Message: TWMHScroll); message WM_HSCROLL;
    procedure WMVScroll(var Message: TWMVScroll); message WM_VSCROLL;
    procedure SetStoreMapBookmarks(const Value: TStoreMapBookmarks);
  protected
    procedure CreateInComings;
    procedure StoreBookMark(MapBookMark: TMapBookMark);
    procedure RestoreBookMark(MapBookMark: TMapBookMark);
    procedure DrawTracks(turn: integer);
    procedure ClearMarks(Tag: integer);
    { Private declarations }
  public
    property DoNotpointPlanet: boolean read FDoNotpointPlanet write FDoNotpointPlanet;
    property Cursors: Tlist read FCursors;
    property StoreMapBookmarks: TStoreMapBookmarks read FStoreMapBookmarks write SetStoreMapBookmarks;
    property GroupCursor: TGroupCursor read FGroupCursor;
    function FindPlanetAtPos(x, y: integer): TPlanet;
    procedure CalculateMapSize;
    procedure DrawPlanets;
    procedure DrawPlanet(planet: TPlanet);
    procedure FullScreen;
    procedure Pointplanet(planet: TPlanet; const centered: boolean);
    procedure Pointplanet2(planet: TPlanet; const centered: boolean);
    procedure SelectPlanets(player: TPlayer);
    procedure RemoveCursors;
    procedure AddCursor(planet: TPlanet);
    { Public declarations }
  end;


var
  frmNewMap: TfrmNewMap;

implementation

{$R *.DFM}

uses
  UFiles, UfrmPlanets, UfrmPlanetSelect, UfrmShipDesign, mkStrUtils,
  UfrmExpressionFilters, UfrmOrders, UfrmMain, xprocs, UfrmRoutes,
  UfrmChangeColors;

{ TMap }

procedure TMap.MapPosToScreenPos(intMapX, intMapY: integer; out intScreenX,
  intScreenY: integer);
begin
  if inivars.Gametype = gtGalaxyBlind then begin
    intMapX := intMapX + (FMaxMapX div 2);
    intMapY := intMapY + (FMaxMapY div 2);
  end;
  intScreenX := round((intMapX * FPercent) / 10000) + 10 - FOffsetX;
  intScreenY := round((intMapY * FPercent) / 10000) + 10 - FOffsetY;
end;

procedure TMap.ScreenPosToMapPos(intScreenX, intScreenY: integer; out intMapX,
  intMapY: integer);
begin
  intMapX := intScreenX + FOffsetX - 10;
  intMapX := (intMapX * 10000) div FPercent;
  intMapY := intScreenY + FOffsetY - 10;
  intMapY := (intMapY * 10000) div FPercent;
  if inivars.Gametype = gtGalaxyBlind then begin
    intMapX := intMapX - (FMaxMapX div 2);
    intMapY := intMapY - (FMaxMapY div 2);
  end;
end;

procedure TMap.MaxMapChanged;
begin
  CalculatePercent;
end;

procedure TMap.SetOffsetXY(const X, Y: integer);
begin
  FOffsetX := X;
  FOffsetY := Y;
end;

procedure TMap.SetMaxMapX(const Value: integer);
begin
  FMaxMapX := Value;
  MaxMapChanged;
end;

procedure TMap.SetMaxMapXY(const X, Y: integer);
begin
  FMaxMapX := X;
  FMaxMapY := Y;
  MaxMapChanged;
end;

procedure TMap.SetMaxMapY(const Value: integer);
begin
  FMaxMapY := Value;
  MaxMapChanged;
end;

procedure TMap.SetOffsetX(const Value: integer);
begin
  FOffsetX := Value;
end;

procedure TMap.SetOffsetY(const Value: integer);
begin
  FOffsetY := Value;
end;

procedure TMap.SetPercent(const Value: integer);
begin
  if FPercent <> Value then begin
    if Value < 10 then
      FPercent := 10
    else
      if Value > 50000 then
        FPercent := 50000
      else
        FPercent := Value;
  end;
end;

procedure TfrmNewMap.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  planet: TPlanet;
begin
  FCursorRect.Top := Y;
  FCursorRect.Bottom := Y;
  FCursorRect.Left := X;
  FCursorRect.Right := X;
  if (ssCTRL in Shift) then
    FCursorState := csCTRL
  else
    if (ssALT in Shift) and
      ((frmPlanetSelect.Showing and frmPlanetSelect.SelectMulti) or not frmPlanetSelect.Showing) then
      FCursorState := csALT
    else begin
      FCursorState := csNone;
      // Do select planet at cursor pos
      planet := FindPlanetAtPos(x, y);
      if planet = nil then Exit;
      if mbLeft = Button then Pointplanet(planet, Viewplanetscentered1.checked);
      if mbRight = Button then Pointplanet2(planet, Viewplanetscentered1.checked);
      FMouseDownEvent := True;
      try
        frmPlanets.ShowPlanet2(planet, Viewplanetscentered1.Checked);
        if (mbLeft = Button) and (frmPlanetSelect.Showing) then
          frmPlanetSelect.ShowPlanet(planet);
        if mbLeft = Button then frmShipDesign.SetBaseMassPlanet(planet);
      finally
        FMouseDownEvent := False;
      end;
    end;
  Update;
end;

procedure TfrmNewMap.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  tmpX, tmpY: integer;
begin
  if (csCTRL = FCursorState) or (csALT = FCursorState) then begin
    // First remove cursor
    FAreaSelectionCursor.DrawSelectionCursor(Canvas);
  end;

  if (not (ssCTRL in Shift) and (csCTRL = FCursorState)) or
    (not (ssALT in Shift) and (csALT = FCursorState)) then begin
    FAreaSelectionCursor.RemoveSelectionCursor(Canvas);
    FCursorState := csNone;
  end;

  if FDoDrag then begin
    if (FDragX <> -1) and (FDragY <> -1) then begin
      tmpX := (X - FDragX) * 3;
      tmpY := (Y - FDragY) * 3;
      if HorzScrollBar.Position + tmpX < 0 then
        HorzScrollBar.Position := 0
      else
        HorzScrollBar.Position := HorzScrollBar.Position + tmpX;
      if VertScrollBar.Position + tmpY < 0 then
        VertScrollBar.Position := 0
      else
        VertScrollBar.Position := VertScrollBar.Position + tmpY;
      FMap.FOffsetX := HorzScrollBar.Position;
      FMap.FOffsetY := VertScrollBar.Position;
      Update;
    end;
    FDragX := X;
    FDragY := Y;
  end;

  if (csCTRL = FCursorState) or (csALT = FCursorState) then begin
    FCursorRect.Right := X;
    FCursorRect.Bottom := Y;
    FAreaSelectionCursor.CalculateSelectionCursor(FCursorRect);
    FAreaSelectionCursor.DrawSelectionCursor(Canvas);
  end;
end;

procedure TfrmNewMap.FormMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  intXMin, intYMin, intXMax, intYMax: integer;
  tmpXScale, tmpYScale: single;
  intScreenX, intScreenY: integer;
  tmpScreenX, tmpScreenY: integer;
  i: integer;
  planet: TPlanet;
begin
  if csCTRL = FCursorState then begin
    // calculate      FCursorRect
    try
      tmpXScale := (Clientwidth / (FAreaSelectionCursor.FCursor.Right - FAreaSelectionCursor.FCursor.Left));
      tmpYScale := (ClientHeight / (FAreaSelectionCursor.FCursor.Bottom - FAreaSelectionCursor.FCursor.Top));
    except
      FCursorState := csNone;
      FAreaSelectionCursor.DrawSelectionCursor(Canvas);
      FAreaSelectionCursor.RemoveSelectionCursor(Canvas);
      Exit;
    end;

    // Get center of selected area
    FMap.ScreenPosToMapPos((FAreaSelectionCursor.FCursor.Left + FAreaSelectionCursor.FCursor.Right) div 2,
      (FAreaSelectionCursor.FCursor.Top + FAreaSelectionCursor.FCursor.Bottom) div 2, intXMin, intYMin);

    // First remove cursor
    FAreaSelectionCursor.DrawSelectionCursor(Canvas);
    FAreaSelectionCursor.RemoveSelectionCursor(Canvas);


    if tmpXScale < 0 then tmpXScale := -1 / tmpXScale;
    if tmpYScale < 0 then tmpYScale := -1 / tmpYScale;

    if tmpXScale < tmpYScale then
      FMap.Percent := round(FMap.Percent * tmpXScale)
    else
      FMap.Percent := round(FMap.Percent * tmpYScale);

    FMap.SetOffsetXY(0, 0);
    FMap.MapPosToScreenPos(intXMin, intYMin, tmpScreenX, tmpScreenY);

    if inivars.Gametype = gtGalaxyBlind then
      FMap.MapPosToScreenPos(FMap.MaxMapX div 2, FMap.MaxMapY div 2, intScreenX, intScreenY)
    else
      FMap.MapPosToScreenPos(FMap.MaxMapX, FMap.MaxMapY, intScreenX, intScreenY);
    VertScrollBar.Range := intScreenX + 10;
    HorzScrollBar.Range := intScreenY + 10;
    tmpScreenX := tmpScreenX - ClientWidth div 2;
    if tmpScreenX < 0 then tmpScreenX := 0;
    tmpScreenY := tmpScreenY - ClientHeight div 2;
    if tmpScreenY < 0 then tmpScreenY := 0;
    VertScrollBar.Position := tmpScreenY;
    HorzScrollBar.Position := tmpScreenX;
    FMap.SetOffsetXY(HorzScrollBar.Position, VertScrollBar.Position);
    Repaint;
  end;
  if csALT = FCursorState then begin
    FMap.ScreenPosToMapPos(FAreaSelectionCursor.FCursor.Left, FAreaSelectionCursor.FCursor.Top, intXMin, intYMin);
    FMap.ScreenPosToMapPos(FAreaSelectionCursor.FCursor.Right, FAreaSelectionCursor.FCursor.Bottom, intXMax, intYMax);
    // First remove cursor
    FAreaSelectionCursor.DrawSelectionCursor(Canvas);
    FAreaSelectionCursor.RemoveSelectionCursor(Canvas);
    FDoNotpointPlanet := True;
    try
      if frmPlanetSelect.Showing then begin
        for i := 0 to frmPlanetSelect.mkListView1.Items.Count - 1 do begin
          planet := PSmallPlanetData(frmPlanetSelect.mkListView1.viewlist[i]).planet;
          if (planet.X > intXMin) and (planet.X < intXMax) and
            (planet.Y > intYMin) and (planet.Y < intYMax) then
            frmPlanetSelect.mkListView1.Items[i].Selected := True;
        end;
      end else begin
        for i := 0 to frmPlanets.mkListView1.Items.Count - 1 do begin
          planet := PSmallPlanetData(frmPlanets.mkListView1.viewlist[i]).planet;
          if (planet.X > intXMin) and (planet.X < intXMax) and
            (planet.Y > intYMin) and (planet.Y < intYMax) then
            frmPlanets.mkListView1.Items[i].Selected := True;
        end;
      end;
    finally
      FDoNotpointPlanet := False;
    end;
  end;
  FCursorState := csNone;
end;

{ TSelectionCursor }

procedure TAreaSelectionCursor.CalculateSelectionCursor(PosRect: TRect);
begin
  if PosRect.Left > PosRect.Right then begin
    FCursor.Left := PosRect.Right;
    FCursor.Right := PosRect.Left;
  end else begin
    FCursor.Left := PosRect.Left;
    FCursor.Right := PosRect.Right;
  end;
  if PosRect.Top > PosRect.Bottom then begin
    FCursor.Top := PosRect.Bottom;
    FCursor.Bottom := PosRect.Top;
  end else begin
    FCursor.Top := PosRect.Top;
    FCursor.Bottom := PosRect.Bottom;
  end;
end;

constructor TAreaSelectionCursor.Create;
begin
  FCursor.Left := -1;
  FCursor.Top := -1;
  FCursor.Right := -1;
  FCursor.Bottom := -1;
end;

procedure TAreaSelectionCursor.DrawSelectionCursor(canvas: TCanvas);
begin
  if FCursor.Left <> -1 then begin
    Canvas.Brush.Color := clWhite;
    Canvas.DrawFocusRect(FCursor);
  end;
end;

procedure TAreaSelectionCursor.RemoveSelectionCursor(canvas: TCanvas);
begin
  FCursor.Left := -1;
  FCursor.Top := -1;
  FCursor.Right := -1;
  FCursor.Bottom := -1;
end;

procedure TfrmNewMap.FormCreate(Sender: TObject);
begin
  FDoNotpointPlanet := False;
  FMap := TMap.Create;
  FPlanetCursor := TPlanetCursor.Create;
  FPlanetCursor.Show := False;
  FPlanetCursor.Map := FMap;
  FPlanetCursor.FormCanvas := Canvas;
  FPlanetCursor.ShowRanges := false;
  FPlanetCursor.Range := 10;
  FPlanetCursor2 := TPlanet2Cursor.Create;
  FPlanetCursor2.Show := False;
  FPlanetCursor2.Map := FMap;
  FPlanetCursor2.FormCanvas := Canvas;
  FPlanetCursor2.ShowRanges := false;
  FPlanetCursor2.Range := 0;
  FCursors := TList.Create;
  FCursorsRed := TList.Create;
  FCursorsBlue := TList.Create;
  FMouseDownEvent := False;
  FAreaSelectionCursor := TAreaSelectionCursor.Create;
  FGroupCursor := TGroupCursor.Create;
  FGroupCursor.Map := FMap;
  FGroupCursor.Canvas := Canvas;
  FStoreMapBookmarks := TStoreMapBookmarks.Create(nil);
  FStoreMapBookmarks.FBM0.Valid := False;
  FStoreMapBookmarks.FBM1.Valid := False;
  FStoreMapBookmarks.FBM2.Valid := False;
  FStoreMapBookmarks.FBM3.Valid := False;
  FStoreMapBookmarks.FBM4.Valid := False;
  FStoreMapBookmarks.FBM5.Valid := False;
  FStoreMapBookmarks.FBM6.Valid := False;
  FStoreMapBookmarks.FBM7.Valid := False;
  FStoreMapBookmarks.FBM8.Valid := False;
  FStoreMapBookmarks.FBM9.Valid := False;
  FIncomings := TList.Create;
end;

procedure TfrmNewMap.FormDestroy(Sender: TObject);
begin
  FAreaSelectionCursor.Free;
  FAreaSelectionCursor := nil;
  FMap.Free;
  FMap := nil;
  FPlanetCursor.Free;
  FPlanetCursor := nil;
  FPlanetCursor2.Free;
  FPlanetCursor2 := nil;
  while FCursors.Count > 0 do
  begin
    TObject(FCursors.Items[0]).Free;
    FCursors.Delete(0);
  end;
  FCursors.Free;
  while FCursorsRed.Count > 0 do
  begin
    TObject(FCursorsRed.Items[0]).Free;
    FCursorsRed.Delete(0);
  end;
  FCursorsRed.Free;
  while FCursorsBlue.Count > 0 do
  begin
    TObject(FCursorsBlue.Items[0]).Free;
    FCursorsBlue.Delete(0);
  end;
  FCursorsBlue.Free;
  while FInComings.Count > 0 do begin
    TObject(FInComings.Items[0]).Free;
    FInComings.Delete(0);
  end;
  FInComings.Free;
  FStoreMapBookmarks.Free;
  FGroupCursor.Free;
end;

procedure TMap.SetScreenClientX(const Value: integer);
begin
  FScreenClientX := Value;
end;

procedure TMap.SetScreenClientY(const Value: integer);
begin
  FScreenClientY := Value;
end;

procedure TMap.SetScreenClientXY(const X, Y: integer);
begin
  FScreenClientX := X;
  FScreenClientY := Y;
end;

procedure TMap.BeginUpdate;
begin
  inc(FUpdateCount);
end;

procedure TMap.EndUpdate;
begin
  if FUpdateCount > 0 then
    dec(FUpdateCount);
  DoChanges;
end;

procedure TMap.DoChanges;
begin
  if FUpdateCount > 0 then Exit;
end;

procedure TMap.PaintPlanet(const canvas: TCanvas; const Name: string; const MapX, MapY, size, intSize, pop, indus: integer; const color: TColor);
var
  ScreenX, ScreenY: integer;
  FHandle: HDC;
  i: integer;
begin
  MapPosToScreenPos(MapX, MapY, ScreenX, ScreenY);
  if (ScreenX < (Canvas.ClipRect.Left - 16)) or (ScreenX > (Canvas.ClipRect.Right + 16)) or
    (ScreenY < (Canvas.ClipRect.Top - 16)) or (ScreenY > (Canvas.ClipRect.Bottom + 16)) then
    Exit;

  FHandle := Canvas.Handle;
  Windows.SetPixel(FHandle, ScreenX, ScreenY, color);

  if size = 2 then begin
    Windows.SetPixel(FHandle, ScreenX + 1, ScreenY, color);
    Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 1, color);
    Windows.SetPixel(FHandle, ScreenX, ScreenY + 1, color);
  end else
    if size = 3 then begin
      Windows.SetPixel(FHandle, ScreenX, ScreenY - 1, color);
      Windows.SetPixel(FHandle, ScreenX - 1, ScreenY, color);
      Windows.SetPixel(FHandle, ScreenX + 1, ScreenY, color);
      Windows.SetPixel(FHandle, ScreenX, ScreenY + 1, color);
    end else
      if size = 4 then begin
        Windows.SetPixel(FHandle, ScreenX, ScreenY - 1, color);
        Windows.SetPixel(FHandle, ScreenX + 1, ScreenY - 1, color);
        Windows.SetPixel(FHandle, ScreenX - 1, ScreenY, color);
        Windows.SetPixel(FHandle, ScreenX + 1, ScreenY, color);
        Windows.SetPixel(FHandle, ScreenX + 2, ScreenY, color);
        Windows.SetPixel(FHandle, ScreenX - 1, ScreenY + 1, color);
        Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 1, color);
        Windows.SetPixel(FHandle, ScreenX + 2, ScreenY + 1, color);
        Windows.SetPixel(FHandle, ScreenX, ScreenY + 2, color);
        Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 2, color);
      end else
        if size = 5 then begin
          Windows.SetPixel(FHandle, ScreenX, ScreenY - 2, color);
          Windows.SetPixel(FHandle, ScreenX - 1, ScreenY - 1, color);
          Windows.SetPixel(FHandle, ScreenX, ScreenY - 1, color);
          Windows.SetPixel(FHandle, ScreenX + 1, ScreenY - 1, color);
          Windows.SetPixel(FHandle, ScreenX - 2, ScreenY, color);
          Windows.SetPixel(FHandle, ScreenX - 1, ScreenY, color);
          Windows.SetPixel(FHandle, ScreenX + 1, ScreenY, color);
          Windows.SetPixel(FHandle, ScreenX + 2, ScreenY, color);
          Windows.SetPixel(FHandle, ScreenX - 1, ScreenY + 1, color);
          Windows.SetPixel(FHandle, ScreenX, ScreenY + 1, color);
          Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 1, color);
          Windows.SetPixel(FHandle, ScreenX, ScreenY + 2, color);
        end else
          if size = 6 then begin
            Windows.SetPixel(FHandle, ScreenX, ScreenY - 2, color);
            Windows.SetPixel(FHandle, ScreenX + 1, ScreenY - 2, color);
            Windows.SetPixel(FHandle, ScreenX - 1, ScreenY - 1, color);
            Windows.SetPixel(FHandle, ScreenX, ScreenY - 1, color);
            Windows.SetPixel(FHandle, ScreenX + 1, ScreenY - 1, color);
            Windows.SetPixel(FHandle, ScreenX + 2, ScreenY - 1, color);
            Windows.SetPixel(FHandle, ScreenX - 2, ScreenY, color);
            Windows.SetPixel(FHandle, ScreenX - 1, ScreenY, color);
            Windows.SetPixel(FHandle, ScreenX + 1, ScreenY, color);
            Windows.SetPixel(FHandle, ScreenX + 2, ScreenY, color);
            Windows.SetPixel(FHandle, ScreenX + 3, ScreenY, color);
            Windows.SetPixel(FHandle, ScreenX - 2, ScreenY + 1, color);
            Windows.SetPixel(FHandle, ScreenX - 1, ScreenY + 1, color);
            Windows.SetPixel(FHandle, ScreenX, ScreenY + 1, color);
            Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 1, color);
            Windows.SetPixel(FHandle, ScreenX + 2, ScreenY + 1, color);
            Windows.SetPixel(FHandle, ScreenX + 3, ScreenY + 1, color);
            Windows.SetPixel(FHandle, ScreenX - 1, ScreenY + 2, color);
            Windows.SetPixel(FHandle, ScreenX, ScreenY + 2, color);
            Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 2, color);
            Windows.SetPixel(FHandle, ScreenX + 2, ScreenY + 2, color);
            Windows.SetPixel(FHandle, ScreenX, ScreenY + 3, color);
            Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 3, color);
          end else
            if size = 7 then begin
              Windows.SetPixel(FHandle, ScreenX - 1, ScreenY - 3, color);
              Windows.SetPixel(FHandle, ScreenX, ScreenY - 3, color);
              Windows.SetPixel(FHandle, ScreenX + 1, ScreenY - 3, color);
              Windows.SetPixel(FHandle, ScreenX - 2, ScreenY - 2, color);
              Windows.SetPixel(FHandle, ScreenX - 1, ScreenY - 2, color);
              Windows.SetPixel(FHandle, ScreenX, ScreenY - 2, color);
              Windows.SetPixel(FHandle, ScreenX + 1, ScreenY - 2, color);
              Windows.SetPixel(FHandle, ScreenX + 2, ScreenY - 2, color);
              Windows.SetPixel(FHandle, ScreenX - 3, ScreenY - 1, color);
              Windows.SetPixel(FHandle, ScreenX - 2, ScreenY - 1, color);
              Windows.SetPixel(FHandle, ScreenX - 1, ScreenY - 1, color);
              Windows.SetPixel(FHandle, ScreenX, ScreenY - 1, color);
              Windows.SetPixel(FHandle, ScreenX + 1, ScreenY - 1, color);
              Windows.SetPixel(FHandle, ScreenX + 2, ScreenY - 1, color);
              Windows.SetPixel(FHandle, ScreenX + 3, ScreenY - 1, color);
              Windows.SetPixel(FHandle, ScreenX - 3, ScreenY, color);
              Windows.SetPixel(FHandle, ScreenX - 2, ScreenY, color);
              Windows.SetPixel(FHandle, ScreenX - 1, ScreenY, color);
              Windows.SetPixel(FHandle, ScreenX + 1, ScreenY, color);
              Windows.SetPixel(FHandle, ScreenX + 2, ScreenY, color);
              Windows.SetPixel(FHandle, ScreenX + 3, ScreenY, color);
              Windows.SetPixel(FHandle, ScreenX - 3, ScreenY + 1, color);
              Windows.SetPixel(FHandle, ScreenX - 2, ScreenY + 1, color);
              Windows.SetPixel(FHandle, ScreenX - 1, ScreenY + 1, color);
              Windows.SetPixel(FHandle, ScreenX, ScreenY + 1, color);
              Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 1, color);
              Windows.SetPixel(FHandle, ScreenX + 2, ScreenY + 1, color);
              Windows.SetPixel(FHandle, ScreenX + 3, ScreenY + 1, color);
              Windows.SetPixel(FHandle, ScreenX - 2, ScreenY + 2, color);
              Windows.SetPixel(FHandle, ScreenX - 1, ScreenY + 2, color);
              Windows.SetPixel(FHandle, ScreenX, ScreenY + 2, color);
              Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 2, color);
              Windows.SetPixel(FHandle, ScreenX + 2, ScreenY + 2, color);
              Windows.SetPixel(FHandle, ScreenX - 1, ScreenY + 3, color);
              Windows.SetPixel(FHandle, ScreenX, ScreenY + 3, color);
              Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 3, color);
            end else
              if size = 8 then begin
                Windows.SetPixel(FHandle, ScreenX - 1, ScreenY - 3, color);
                Windows.SetPixel(FHandle, ScreenX, ScreenY - 3, color);
                Windows.SetPixel(FHandle, ScreenX + 1, ScreenY - 3, color);
                Windows.SetPixel(FHandle, ScreenX + 2, ScreenY - 3, color);
                Windows.SetPixel(FHandle, ScreenX - 2, ScreenY - 2, color);
                Windows.SetPixel(FHandle, ScreenX - 1, ScreenY - 2, color);
                Windows.SetPixel(FHandle, ScreenX, ScreenY - 2, color);
                Windows.SetPixel(FHandle, ScreenX + 1, ScreenY - 2, color);
                Windows.SetPixel(FHandle, ScreenX + 2, ScreenY - 2, color);
                Windows.SetPixel(FHandle, ScreenX + 3, ScreenY - 2, color);
                Windows.SetPixel(FHandle, ScreenX - 3, ScreenY - 1, color);
                Windows.SetPixel(FHandle, ScreenX - 2, ScreenY - 1, color);
                Windows.SetPixel(FHandle, ScreenX - 1, ScreenY - 1, color);
                Windows.SetPixel(FHandle, ScreenX, ScreenY - 1, color);
                Windows.SetPixel(FHandle, ScreenX + 1, ScreenY - 1, color);
                Windows.SetPixel(FHandle, ScreenX + 2, ScreenY - 1, color);
                Windows.SetPixel(FHandle, ScreenX + 3, ScreenY - 1, color);
                Windows.SetPixel(FHandle, ScreenX + 4, ScreenY - 1, color);
                Windows.SetPixel(FHandle, ScreenX - 3, ScreenY, color);
                Windows.SetPixel(FHandle, ScreenX - 2, ScreenY, color);
                Windows.SetPixel(FHandle, ScreenX - 1, ScreenY, color);
                Windows.SetPixel(FHandle, ScreenX, ScreenY, color);
                Windows.SetPixel(FHandle, ScreenX + 1, ScreenY, color);
                Windows.SetPixel(FHandle, ScreenX + 2, ScreenY, color);
                Windows.SetPixel(FHandle, ScreenX + 3, ScreenY, color);
                Windows.SetPixel(FHandle, ScreenX + 4, ScreenY, color);
                Windows.SetPixel(FHandle, ScreenX - 3, ScreenY + 1, color);
                Windows.SetPixel(FHandle, ScreenX - 2, ScreenY + 1, color);
                Windows.SetPixel(FHandle, ScreenX - 1, ScreenY + 1, color);
                Windows.SetPixel(FHandle, ScreenX, ScreenY + 1, color);
                Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 1, color);
                Windows.SetPixel(FHandle, ScreenX + 2, ScreenY + 1, color);
                Windows.SetPixel(FHandle, ScreenX + 3, ScreenY + 1, color);
                Windows.SetPixel(FHandle, ScreenX + 4, ScreenY + 1, color);
                Windows.SetPixel(FHandle, ScreenX - 3, ScreenY + 2, color);
                Windows.SetPixel(FHandle, ScreenX - 2, ScreenY + 2, color);
                Windows.SetPixel(FHandle, ScreenX - 1, ScreenY + 2, color);
                Windows.SetPixel(FHandle, ScreenX, ScreenY + 2, color);
                Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 2, color);
                Windows.SetPixel(FHandle, ScreenX + 2, ScreenY + 2, color);
                Windows.SetPixel(FHandle, ScreenX + 3, ScreenY + 2, color);
                Windows.SetPixel(FHandle, ScreenX + 4, ScreenY + 2, color);
                Windows.SetPixel(FHandle, ScreenX - 2, ScreenY + 3, color);
                Windows.SetPixel(FHandle, ScreenX - 1, ScreenY + 3, color);
                Windows.SetPixel(FHandle, ScreenX, ScreenY + 3, color);
                Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 3, color);
                Windows.SetPixel(FHandle, ScreenX + 2, ScreenY + 3, color);
                Windows.SetPixel(FHandle, ScreenX + 3, ScreenY + 3, color);
                Windows.SetPixel(FHandle, ScreenX - 1, ScreenY + 4, color);
                Windows.SetPixel(FHandle, ScreenX, ScreenY + 4, color);
                Windows.SetPixel(FHandle, ScreenX + 1, ScreenY + 4, color);
                Windows.SetPixel(FHandle, ScreenX + 2, ScreenY + 4, color);
              end else begin
                Canvas.Pen.Color := color;
                Canvas.Pen.Width := size;
                Canvas.MoveTo(ScreenX, ScreenY);
                Canvas.LineTo(ScreenX, ScreenY);
              end;

  if FShowGrowth and (intSize > 0) then begin
    for i := ScreenY - intSize + 2 to ScreenY + 2 do
      Windows.SetPixel(FHandle, ScreenX + Size, i, CurrentColorSchema.BarSize);
  end;

  if FShowGrowth and (pop > 0) then begin
    for i := ScreenY - pop + 2 to ScreenY + 2 do
      Windows.SetPixel(FHandle, ScreenX + Size + 2, i, CurrentColorSchema.BarPopulation);
  end;
  if FShowGrowth and (indus > 0) then begin
    for i := ScreenY - pop + 2 to ScreenY + 2 do
      Windows.SetPixel(FHandle, ScreenX + Size + 4, i, CurrentColorSchema.BarIndustry);
  end;
  if FShowNames and (Name <> '') then begin
    Canvas.Font.Color := CurrentColorSchema.NameOfPlanets;
    Canvas.TextOut(ScreenX, ScreenY + Size, Name);
  end;
end;

procedure TfrmNewMap.DrawPlanet(planet: TPlanet);
var
  PlanetColor: TColor;
  blnOwned,
    blnHomePlanet,
    blnPlayer: boolean;
  intPlanetSize: integer;
  intSize, intPop, intIndus: integer;
begin
  blnOwned := Assigned(planet.PlayerCur);
  blnHomePlanet := Assigned(planet.Player[0]);
  blnPlayer := (planet.PlayerCur = qreportplayer);

  if Planet.Size > 0 then PlanetColor := CurrentColorSchema.UnInSizeKnown
  else PlanetColor := CurrentColorSchema.UnInSizeUnk;
  if blnOwned then
  begin
    if Planet.Size > 0 then begin
      if (CurrentColorSchema.UseOneColorOtherPlayers or (planet.LastOwnerObject = nil)) then
        PlanetColor := CurrentColorSchema.OtherPlayersPlanets
      else
        PlanetColor := planet.LastOwnerObject.Color;
    end else begin
      if (CurrentColorSchema.UseOneColorOtherPlayers or (planet.LastOwnerObject = nil)) then
        PlanetColor := CurrentColorSchema.HabSizeUnk
      else
        PlanetColor := planet.LastOwnerObject.Color;
    end;
  end;
  if blnPlayer then PlanetColor := CurrentColorSchema.PlayerPlanets;
  if blnHomePlanet and (CurrentColorSchema.UseColorsFirstTurn) then PlanetColor := CurrentColorSchema.FirstTurnPlanets;
  intPlanetSize := round(planet.Size / 200) + 3;
  if planet.Size > 0 then
    intSize := round(planet.Size / 100) + 1
  else
    intSize := 0;
  if planet.PopulationCur > 0 then
    intPop := round(planet.PopulationCur / 100) + 1
  else
    intPop := 0;
  if planet.IndustryCur > 0 then
    intIndus := round(planet.IndustryCur / 100) + 1
  else
    intIndus := 0;
  FMap.PaintPlanet(Canvas, planet.Name, planet.X, planet.Y, intPlanetSize, intSize, intPop, intIndus, PlanetColor);
end;

procedure TfrmNewMap.DrawPlanets;
var
  intPlanets: integer;
  planet: TPlanet;
begin
  if Planets.Count > 0 then
  begin
    Canvas.Brush.Style := bsClear;
    SetTextAlign(Canvas.Handle, TA_CENTER);
    for intPlanets := 0 to Planets.Count - 1 do
    begin
      planet := TPlanet(planets.Objects[intPLanets]);
      DrawPlanet(planet);
    end;
  end;
end;

procedure TfrmNewMap.FormShow(Sender: TObject);
begin
  FTrackturn := LastTurn;
  CreateInComings;
  FullScreen;
  ovcPersistentState1.RestoreState(FStoreMapBookmarks, GetGameStr + '_MapBM');
  RestoreBookMark(FStoreMapBookmarks.LastPos);
  FGroupCursor.DontShow := DisableGroupCircles1.Checked;
  FPlanetCursor.ShowRanges := not DisablePlanetCircles1.Checked;
  if mkAToI(mniSpecialValue.Caption) > 0 then
    FPlanetCursor.Range := mkAToI(mniSpecialValue.Caption);
end;

procedure TfrmNewMap.FormPaint(Sender: TObject);
var
  i: integer;
begin
  Color := CurrentColorSchema.BackGround;
  Canvas.Brush.Style := bsClear;
  Canvas.Pen.Width := 1;
  Canvas.Pen.Mode := pmCopy;
  DrawTracks(FTrackturn);

  if FMap.ShowIncomings then
    for i := 0 to FIncomings.Count - 1 do begin
      TShowIncomings(FIncomings[i]).DrawCursor;
    end;
  DrawPlanets;
  //DrawCursors;
  Canvas.Pen.Width := 1;
  FGroupCursor.DrawCursor;
  if FPlanetCursor2.Show then
    FPlanetCursor2.Draw;
  if FPlanetCursor.Show then
    FPlanetCursor.Draw;
  for i := 0 to FCursors.Count - 1 do
    TPlanetPCursor(FCursors.Items[i]).Draw;
  for i := 0 to FCursorsRed.Count - 1 do
    TPlanetPCursor(FCursorsRed.Items[i]).Draw;
  for i := 0 to FCursorsBlue.Count - 1 do
    TPlanetPCursor(FCursorsBlue.Items[i]).Draw;
end;

procedure TfrmNewMap.FormResize(Sender: TObject);
begin
  FMap.FOffsetX := HorzScrollBar.Position;
  FMap.FOffsetY := VertScrollBar.Position;
  Repaint;
end;

procedure TfrmNewMap.CalculateMapSize;
var
  i: integer;
  maxX, maxY,
    minX, minY: integer;
begin
  maxX := 0;
  maxY := 0;
  minX := 0;
  minY := 0;
  for i := 0 to planets.Count - 1 do
  begin
    if TPlanet(planets.Objects[i]).x > maxX then maxX := TPlanet(planets.Objects[i]).x;
    if TPlanet(planets.Objects[i]).y > maxY then maxY := TPlanet(planets.Objects[i]).y;
    if TPlanet(planets.Objects[i]).x < minX then minX := TPlanet(planets.Objects[i]).x;
    if TPlanet(planets.Objects[i]).y < minY then minY := TPlanet(planets.Objects[i]).y;
  end;
  if (-minX) > maxX then FMap.MaxMapX := round(-minX)
  else FMap.MaxMapX := round(maxX);
  if (-minY) > maxY then FMap.MaxMapY := round(-minY)
  else FMap.MaxMapY := round(maxY);
  //gtGalaxyNGBlind
  if inivars.GameType = gtGalaxyNGBlind then begin
    if maxX > maxY then maxY := maxX;
    FMap.MaxMapX := maxY;
    FMap.MaxMapY := maxY;
  end;
  if inivars.Gametype = gtGalaxyBlind then
  begin
    FMap.MaxMapX := FMap.MaxMapX * 2;
    FMap.MaxMapY := FMap.MaxMapY * 2;
  end;
end;

procedure TMap.CalculatePercent;
var
  intTmpPercentX: integer;
  intTmpPercentY: integer;
begin
  if (FScreenClientX > 0) and (FScreenClientY > 0) and
    (FMaxMapX > 0) and (FMaxMapY > 0) then
  begin
    intTmpPercentX := FScreenClientX * 100 div (FMaxMapX div 100);
    intTmpPercentY := FScreenClientY * 100 div (FMaxMapY div 100);
    if intTmpPercentY < intTmpPercentX then
      FPercent := intTmpPercentY
    else
      FPercent := intTmpPercentX;
  end;
end;

procedure TfrmNewMap.WMHScroll(var Message: TWMHScroll);
begin
  inherited;
  FMap.FOffsetX := HorzScrollBar.Position;
  Update;
end;

procedure TfrmNewMap.WMVScroll(var Message: TWMVScroll);
begin
  inherited;
  FMap.FOffsetY := VertScrollBar.Position;
  Update;
end;

procedure TfrmNewMap.FullScreen;
var
  intScreenX,
    intScreenY: integer;
begin
  VertScrollBar.Range := 0;
  HorzScrollBar.Range := 0;
  FMap.SetScreenClientXY(ClientHeight - 25, ClientWidth - 25);
  FMap.SetOffsetXY(0, 0);
  CalculateMapSize;
  if inivars.Gametype = gtGalaxyBlind then
    FMap.MapPosToScreenPos(FMap.MaxMapX div 2, FMap.MaxMapY div 2, intScreenX, intScreenY)
  else
    FMap.MapPosToScreenPos(FMap.MaxMapX, FMap.MaxMapY, intScreenX, intScreenY);
  VertScrollBar.Position := 0;
  HorzScrollBar.Position := 0;
  VertScrollBar.Range := intScreenX + 10;
  HorzScrollBar.Range := intScreenY + 10;
end;

procedure TfrmNewMap.FormDblClick(Sender: TObject);
begin
  FullScreen;
  Repaint;
end;

procedure TMap.SetShowGrowth(const Value: boolean);
begin
  FShowGrowth := Value;
end;

procedure TMap.SetShowNames(const Value: boolean);
begin
  FShowNames := Value;
end;

procedure TMap.SetShowIncomings(const Value: boolean);
begin
  FShowIncomings := Value;
end;

{ TPlanetCursor }

procedure TPlanetCursor.Draw;
var
  i: integer;
  x1, y1, x2, y2: integer;
begin
  Recalc;
  if MustPaint then begin
    FFormCanvas.Pen.Color := CurrentColorSchema.Cursor;
    FFormCanvas.Pen.Mode := pmCopy;
    FFormCanvas.MoveTo(FX1, FY1);
    FFormCanvas.LineTo(FX1 + 24, FY1);
    FFormCanvas.MoveTo(FX2, FY2);
    FFormCanvas.LineTo(FX2, FY2 + 24);
    if FShowRanges and (FMapRange > 0) then begin
      FFormCanvas.Pen.Color := CurrentColorSchema.PlanetCircles;
      for i := 1 to 4 do begin
        x1 := intX - Trunc(i * FMapRange);
        y1 := intY - Trunc(i * FMapRange);
        x2 := intX + Trunc(i * FMapRange);
        y2 := intY + Trunc(i * FMapRange);
        FFormCanvas.Ellipse(x1, y1, x2, y2);
      end;
    end;
  end;
end;

function TPlanetCursor.MustPaint: boolean;
begin
  Result := False;
  if (intX < 0) or (intX > frmNewMap.ClientWidth) or (intY < 0) or (intY > frmNewMap.ClientHeight) then Exit;
  Result := True;
end;

procedure TPlanetCursor.Recalc;
begin
  if not Assigned(Fplanet) then exit;
  FMapRange := round(((FRange * 100) * FMap.Percent) / 10000);
  FMap.MapPosToScreenPos(Fplanet.X, Fplanet.Y, intX, intY);
  FX1 := intX - 2 - 10;
  FY1 := intY + 10 - 10;
  FX2 := intX + 10 - 10;
  FY2 := intY - 2 - 10;
end;

procedure TPlanetCursor.SetFormCanvas(const Value: TCanvas);
begin
  FFormCanvas := Value;
end;

{ TPlanet2Cursor }

procedure TPlanet2Cursor.Draw;
begin
  Recalc;
  if MustPaint then begin
    FFormCanvas.Pen.Color := CurrentColorSchema.Cursor;
    FFormCanvas.Pen.Mode := pmCopy;
    FFormCanvas.MoveTo(FX1, FY2);
    FFormCanvas.LineTo(FX1 + 24, FY2 + 24);
    FFormCanvas.MoveTo(FX1, FY2 + 24);
    FFormCanvas.LineTo(FX1 + 24, FY2);
  end;
end;

{ TPlanetPCursor }

procedure TPlanetPCursor.Draw;
begin
  Recalc;
  if MustPaint then begin
    FFormCanvas.Pen.Color := CurrentColorSchema.Cursor;
    FFormCanvas.Pen.Mode := pmCopy;
    FFormCanvas.Polyline([point(FX1 + 7, FY2 + 7), point(FX1 + 17, FY2 + 7),
      point(FX1 + 17, FY2 + 17), point(FX1 + 7, FY2 + 17),
        point(FX1 + 7, FY2 + 7)]);
  end;
end;

function TfrmNewMap.FindPlanetAtPos(x, y: integer): TPlanet;
var
  i: integer;
  x1, y1: integer;
  ptx, pty, mapx, mapy, dist, dist1: double;
  planet: TPlanet;
begin
  dist := high(INT64); // 1000000.0 * 1000000.0;
  FMap.ScreenPosToMapPos(x, y, x1, y1);
  mapx := x1;
  mapy := y1;
  Result := nil;
  for i := 0 to planets.count - 1 do
  begin
    planet := Tplanet(planets.Objects[i]);
    ptx := planet.x;
    pty := planet.y;
    dist1 := (ptx - mapx) * (ptx - mapx) +
      (pty - mapy) * (pty - mapy);
    if dist1 < dist then
    begin
      dist := dist1;
      Result := planet;
    end;
  end;
end;

procedure TfrmNewMap.Pointplanet(planet: TPlanet; const centered: boolean);
var
  tmpScreenX: integer;
  tmpScreenY: integer;
begin
  if Application.Terminated then
    Exit;
  if FMouseDownEvent then Exit;
  if FDoNotpointPlanet then Exit;
  if not Assigned(FPlanetCursor) then Exit;
  if not Assigned(planet) then
    FPlanetCursor.Show := False
  else
  begin
    if centered then begin
      FMap.SetOffsetXY(0,0);
      FMap.MapPosToScreenPos(planet.X, planet.Y, tmpScreenX, tmpScreenY);
      tmpScreenX := tmpScreenX - ClientWidth div 2;
      if tmpScreenX < 0 then tmpScreenX := 0;
      tmpScreenY := tmpScreenY - ClientHeight div 2;
      if tmpScreenY < 0 then tmpScreenY := 0;
      VertScrollBar.Position := tmpScreenY;
      HorzScrollBar.Position := tmpScreenX;
      FMap.SetOffsetXY(HorzScrollBar.Position, VertScrollBar.Position);
    end;
    planet1 := planet;
    FPlanetCursor.Planet := Planet;
    FPlanetCursor.Recalc;
    FPlanetCursor.Show := True;
  end;
  Repaint;
end;

procedure TfrmNewMap.Pointplanet2(planet: TPlanet; const centered: boolean);
var
  tmpScreenX: integer;
  tmpScreenY: integer;
begin
  if FMouseDownEvent then Exit;
  if FDoNotpointPlanet then Exit;
  if not Assigned(FPlanetCursor2) then Exit;
  if not Assigned(planet) then
    FPlanetCursor2.Show := False
  else
  begin
    if centered then begin
      FMap.SetOffsetXY(0,0);
      FMap.MapPosToScreenPos(planet.X, planet.Y, tmpScreenX, tmpScreenY);
      tmpScreenX := tmpScreenX - ClientWidth div 2;
      if tmpScreenX < 0 then tmpScreenX := 0;
      tmpScreenY := tmpScreenY - ClientHeight div 2;
      if tmpScreenY < 0 then tmpScreenY := 0;
      VertScrollBar.Position := tmpScreenY;
      HorzScrollBar.Position := tmpScreenX;
      FMap.SetOffsetXY(HorzScrollBar.Position, VertScrollBar.Position);
    end;
    planet2 := planet;
    FPlanetCursor2.Planet := Planet;
    FPlanetCursor2.Recalc;
    FPlanetCursor2.Show := True;
  end;
  Repaint;
end;

procedure TfrmNewMap.SelectPlanets(player: TPlayer);
var
  i: integer;
begin
  // delete old cursors
  RemoveCursors;

  for i := 0 to planets.Count - 1 do
    if TPlanet(planets.Objects[i]).LastOwnerObject = player then
      AddCursor(TPlanet(planets.Objects[i]));
  Repaint;
end;

procedure TfrmNewMap.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  WindowState := wsMinimized;
  CanClose := false;
end;

procedure TfrmNewMap.FormActivate(Sender: TObject);
begin
  Repaint;
end;

procedure TfrmNewMap.FormKeyPress(Sender: TObject; var Key: Char);
var
  intXMin, intYMin: integer;
  tmpScreenX, tmpScreenY: integer;
  intScreenX, intScreenY: integer;
  blnDoZoom: boolean;
  tmpScale: single;
begin
  blnDoZoom := False;
  case Key of
    '-': begin tmpScale := 0.90; blnDoZoom := True; end;
    '=': begin tmpScale := 1.10; blnDoZoom := True; end;
  else
    tmpScale := 1;
  end;

  if blnDoZoom then begin
    FMap.ScreenPosToMapPos(ClientWidth div 2, ClientHeight div 2, intXMin, intYMin);
    FMap.percent := round(FMap.percent * tmpScale);
    FMap.SetOffsetXY(0, 0);
    FMap.MapPosToScreenPos(intXMin, intYMin, tmpScreenX, tmpScreenY);

    if inivars.Gametype = gtGalaxyBlind then
      FMap.MapPosToScreenPos(FMap.MaxMapX div 2, FMap.MaxMapY div 2, intScreenX, intScreenY)
    else
      FMap.MapPosToScreenPos(FMap.MaxMapX, FMap.MaxMapY, intScreenX, intScreenY);
    VertScrollBar.Range := intScreenX + 10;
    HorzScrollBar.Range := intScreenY + 10;
    tmpScreenX := tmpScreenX - ClientWidth div 2;
    if (tmpScreenX < 0) or (ClientWidth > intScreenX) then tmpScreenX := 0;
    tmpScreenY := tmpScreenY - ClientHeight div 2;
    if (tmpScreenY < 0) or (ClientHeight > intScreenY) then tmpScreenY := 0;
    VertScrollBar.Position := tmpScreenY;
    HorzScrollBar.Position := tmpScreenX;
    FMap.SetOffsetXY(HorzScrollBar.Position, VertScrollBar.Position);
    Repaint;
  end;
end;

procedure TfrmNewMap.mniFullExtendsClick(Sender: TObject);
begin
  FullScreen;
  Repaint;
end;

procedure TfrmNewMap.ClearMarks(Tag: integer);
var
  cursorList: TList;
  PlanetPCursor: TPlanetPCursor;
begin
  case Tag of
    1: cursorList := FCursorsRed;
    2: cursorList := FCursorsBlue;
  else
    exit;
  end;

  while cursorList.Count > 0 do
  begin
    PlanetPCursor := TPlanetPCursor(cursorList.Items[0]);
    PlanetPCursor.Free;
    cursorList.Delete(0);
  end;
end;

procedure TfrmNewMap.mniMarkPlanetsClick(Sender: TObject);
var
  i: integer;
  PlanetPCursor: TPlanetPCursor;
  cursorList: TList;
  CursorClass: TPlanetPCursorClass;
begin
  // delete old cursors
  ClearMarks(TWidget(Sender).Tag);

  case TWidget(Sender).Tag of
    1: begin cursorList := FCursorsRed; CursorClass := TPlanetPCursorRed; end;
    2: begin cursorList := FCursorsBlue; CursorClass := TPlanetPCursorBlue; end;
  else
    exit;
  end;
  if frmExpressionFilters.ShowModal = mrOK then begin
    for i := 0 to planets.Count - 1 do
      if frmExpressionFilters.Filteredplanets.IndexOf(planets.Objects[i]) <> -1 then
      begin
        PlanetPCursor := CursorClass.Create;
        PlanetPCursor.Map := FMap;
        PlanetPCursor.FormCanvas := Canvas;
        PlanetPCursor.planet := TPlanet(planets.Objects[i]);
        PlanetPCursor.Show := True;
        PlanetPCursor.Recalc;
        cursorList.Add(PlanetPCursor);
      end;
  end;
  Repaint;
end;

{ TPlanetPCursorBlue }

procedure TPlanetPCursorBlue.Draw;
begin
  Recalc;
  if MustPaint then begin
    FFormCanvas.Pen.Color := clBlue;   {# }
    FFormCanvas.Pen.Mode := pmCopy;
    FFormCanvas.Polyline([point(FX1 + 6, FY2 + 6), point(FX1 + 18, FY2 + 6),
      point(FX1 + 18, FY2 + 18), point(FX1 + 6, FY2 + 18),
        point(FX1 + 6, FY2 + 6)]);
  end;
end;

{ TPlanetPCursorRed }

procedure TPlanetPCursorRed.Draw;
begin
  Recalc;
  if MustPaint then begin
    FFormCanvas.Pen.Color := clRed;   {# }
    FFormCanvas.Pen.Mode := pmCopy;
    FFormCanvas.Polyline([point(FX1 + 8, FY2 + 8), point(FX1 + 16, FY2 + 8),
      point(FX1 + 16, FY2 + 16), point(FX1 + 8, FY2 + 16),
        point(FX1 + 8, FY2 + 8)]);
  end;
end;

procedure TfrmNewMap.AddCursor(planet: TPlanet);
var
  PlanetPCursor: TPlanetPCursor;
begin
  PlanetPCursor := TPlanetPCursor.Create;
  PlanetPCursor.Map := FMap;
  PlanetPCursor.FormCanvas := Canvas;
  PlanetPCursor.planet := planet;
  PlanetPCursor.Show := True;
  PlanetPCursor.Recalc;
  FCursors.Add(PlanetPCursor);
end;

procedure TfrmNewMap.RemoveCursors;
var
  PlanetPCursor: TPlanetPCursor;
begin
  while FCursors.Count > 0 do
  begin
    PlanetPCursor := TPlanetPCursor(FCursors.Items[0]);
    PlanetPCursor.Free;
    FCursors.Delete(0);
  end;
end;

procedure TfrmNewMap.mniTopMostClick(Sender: TObject);
begin
  mniTopMost.Checked := not mniTopMost.Checked;
  TopMostWindow1.AlwaysOnTop := mniTopMost.Checked;
end;

procedure TfrmNewMap.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (ssShift in Shift) and (ssCTRL in Shift) and
    ((key >= ord('0')) and (key <= ord('9'))) then begin
    case key of
      ord('0'): StoreBookMark(FStoreMapBookmarks.FBM0);
      ord('1'): StoreBookMark(FStoreMapBookmarks.FBM1);
      ord('2'): StoreBookMark(FStoreMapBookmarks.FBM2);
      ord('3'): StoreBookMark(FStoreMapBookmarks.FBM3);
      ord('4'): StoreBookMark(FStoreMapBookmarks.FBM4);
      ord('5'): StoreBookMark(FStoreMapBookmarks.FBM5);
      ord('6'): StoreBookMark(FStoreMapBookmarks.FBM6);
      ord('7'): StoreBookMark(FStoreMapBookmarks.FBM7);
      ord('8'): StoreBookMark(FStoreMapBookmarks.FBM8);
      ord('9'): StoreBookMark(FStoreMapBookmarks.FBM9);
    end;
    exit;
  end;
  if (ssCTRL in Shift) and
    ((key >= ord('0')) and (key <= ord('9'))) then begin
    case key of
      ord('0'): RestoreBookMark(FStoreMapBookmarks.FBM0);
      ord('1'): RestoreBookMark(FStoreMapBookmarks.FBM1);
      ord('2'): RestoreBookMark(FStoreMapBookmarks.FBM2);
      ord('3'): RestoreBookMark(FStoreMapBookmarks.FBM3);
      ord('4'): RestoreBookMark(FStoreMapBookmarks.FBM4);
      ord('5'): RestoreBookMark(FStoreMapBookmarks.FBM5);
      ord('6'): RestoreBookMark(FStoreMapBookmarks.FBM6);
      ord('7'): RestoreBookMark(FStoreMapBookmarks.FBM7);
      ord('8'): RestoreBookMark(FStoreMapBookmarks.FBM8);
      ord('9'): RestoreBookMark(FStoreMapBookmarks.FBM9);
    end;
    exit;
  end;

  if ssCTRL in Shift then
    case Key of
      ord('U'): RemoveCursors;
    end;
  case Key of
    ord('D'): begin
        FDragX := -1;
        FDragY := -1;
        FDoDrag := True;
        cursor := crHandPoint;
      end
  end;
  if (Key = VK_F5) then
    Repaint;
  if (Key = VK_F6) then
    frmOrders.DoAnalyseButton;
end;

procedure TfrmNewMap.WidgetPlusClick(Sender: TObject);
var
  Key: char;
begin
  Key := '=';
  FormKeyPress(Sender, Key);
end;

procedure TfrmNewMap.WidgetMinClick(Sender: TObject);
var
  Key: char;
begin
  Key := '-';
  FormKeyPress(Sender, Key);
end;

procedure TfrmNewMap.WidgetShowGrowthClick(Sender: TObject);
begin
  WidgetShowGrowth.Down := not WidgetShowGrowth.Down;
  FMap.ShowGrowth := WidgetShowGrowth.Down;
  Repaint;
end;

procedure TfrmNewMap.WidgetShowNamesClick(Sender: TObject);
begin
  WidgetShowNames.Down := not WidgetShowNames.Down;
  FMap.ShowNames := WidgetShowNames.Down;
  Repaint;
end;

procedure TfrmNewMap.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    ord('D'): begin
        FDoDrag := False;
        cursor := crDefault;
      end;
  end;
end;

{ TShowIncomings }

procedure TShowIncomings.DrawCursor;
var
  tFromX: integer;
  tFromY: integer;
  tToX: integer;
  tToY: integer;
  tCurX: integer;
  tCurY: integer;
begin
  FMap.MapPosToScreenPos(FromX, FromY, tFromX, tFromY);
  FMap.MapPosToScreenPos(ToX, ToY, tToX, tToY);
  FMap.MapPosToScreenPos(CurX, CurY, tCurX, tCurY);
  FCanvas.Pen.Style := psDash;
  FCanvas.Pen.Color := CurrentColorSchema.IncomingGroupsTrav;
  FCanvas.MoveTo(tFromX, tFromY);
  FCanvas.LineTo(tCurX, tCurY);
  FCanvas.Pen.Style := psSolid;
  FCanvas.Pen.Color := CurrentColorSchema.IncomingGroups;
  FCanvas.LineTo(tToX, tToY);
end;

procedure TShowIncomings.SetIncoming(const FromPlanet, DestPlanet: TPlanet;
  const range, speed: single);
var
  distAll: single;
  ratioD: single;
  tmpFromX, tmpFromY, tmpToX, tmpToY: single;
begin
  FromX := FromPlanet.X;
  FromY := FromPlanet.Y;
  ToX := DestPlanet.X;
  ToY := DestPlanet.Y;
  tmpFromX := FromX;
  tmpFromY := FromY;
  tmpToX := ToX;
  tmpToY := ToY;
  distAll := sqrt(sqr(tmpFromX - tmpToX) + sqr(tmpFromY - tmpToY));
  if distAll = 0 then Exit;
  ratioD := (distAll - (range * 100)) / distAll;
  CurX := trunc(FromX + (ratioD * (ToX - FromX)));
  CurY := trunc(FromY + (ratioD * (ToY - FromY)));
end;

procedure TfrmNewMap.CreateInComings;
var
  i: integer;
  IncomingGroup: TIncomingGroup;
  ShowIncomings: TShowIncomings;
begin
  for i := 0 to IncomingGroups.Count - 1 do begin
    IncomingGroup := TIncomingGroup(IncomingGroups.Items[i]);
    if IncomingGroup.Turn = LastTurn then begin
      ShowIncomings := TShowIncomings.Create;
      ShowIncomings.SetIncoming(IncomingGroup.FromPlanet, IncomingGroup.DestPlanet, IncomingGroup.range, IncomingGroup.speed);
      ShowIncomings.Canvas := Canvas;
      ShowIncomings.Map := FMap;
      FIncomings.Add(ShowIncomings);
    end;
  end;
end;

procedure TfrmNewMap.WidgetShowIncomingsClick(Sender: TObject);
begin
  WidgetShowIncomings.Down := not WidgetShowIncomings.Down;
  FMap.ShowIncomings := WidgetShowIncomings.Down;
  Repaint;
end;

{ TStoreMapBookmarks }

constructor TStoreMapBookmarks.Create(AOwner: TComponent);
begin
  inherited Create(Aowner);
  FBM0 := TMapBookMark.Create(Self);
  FBM6 := TMapBookMark.Create(Self);
  FBM1 := TMapBookMark.Create(Self);
  FBM9 := TMapBookMark.Create(Self);
  FBM8 := TMapBookMark.Create(Self);
  FBM5 := TMapBookMark.Create(Self);
  FBM4 := TMapBookMark.Create(Self);
  FBM7 := TMapBookMark.Create(Self);
  FBM3 := TMapBookMark.Create(Self);
  FBM2 := TMapBookMark.Create(Self);
  FLastPos := TMapBookMark.Create(Self);
end;

destructor TStoreMapBookmarks.Destroy;
begin
  FBM0.Free;
  FBM6.Free;
  FBM1.Free;
  FBM9.Free;
  FBM8.Free;
  FBM5.Free;
  FBM4.Free;
  FBM7.Free;
  FBM3.Free;
  FBM2.Free;
  FLastPos.Free;
  inherited Destroy;
end;

procedure TfrmNewMap.SetStoreMapBookmarks(const Value: TStoreMapBookmarks);
begin
  FStoreMapBookmarks := Value;
end;

procedure TfrmNewMap.FormHide(Sender: TObject);
begin
  StoreBookMark(FStoreMapBookmarks.LastPos);
  ovcPersistentState1.SaveState(FStoreMapBookmarks, GetGameStr + '_MapBM');
end;

{ TMapBookMark }

procedure TMapBookMark.SetRatio(const Value: integer);
begin
  FRatio := Value;
end;

procedure TMapBookMark.SetValid(const Value: boolean);
begin
  FValid := Value;
end;

procedure TMapBookMark.SetXPos(const Value: integer);
begin
  FXPos := Value;
end;

procedure TMapBookMark.SetXRange(const Value: integer);
begin
  FXRange := Value;
end;

procedure TMapBookMark.SetYPos(const Value: integer);
begin
  FYPos := Value;
end;

procedure TfrmNewMap.RestoreBookMark(MapBookMark: TMapBookMark);
begin
  if not MapBookMark.Valid then Exit;
  FMap.SetPercent(MapBookMark.Ratio);
  HorzScrollBar.Range := MapBookMark.XRange;
  VertScrollBar.Range := MapBookMark.YRange;
  HorzScrollBar.Position := MapBookMark.XPos;
  VertScrollBar.Position := MapBookMark.YPos;
  FMap.SetOffsetXY(MapBookMark.XPos, MapBookMark.YPos);
  Repaint;
end;

procedure TfrmNewMap.StoreBookMark(MapBookMark: TMapBookMark);
begin
  MapBookMark.XPos := FMap.OffsetX;
  MapBookMark.YPos := FMap.OffsetY;
  MapBookMark.Ratio := FMap.Percent;
  MapBookMark.XRange := HorzScrollBar.Range;
  MapBookMark.YRange := VertScrollBar.Range;
  MapBookMark.Valid := True;
end;

procedure TMapBookMark.SetYRange(const Value: integer);
begin
  FYRange := Value;
end;

{ TGroupCursor }

procedure TGroupCursor.DrawCursor;
var
  i, intX, intY: integer;
  x1, y1, x2, y2, dist, dist2: integer;
begin
  if FSpeed = 0 then Exit;
  if FDontShow then Exit;
  dist := round(((FSpeed * 100) * FMap.Percent) / 10000); //intX - intY;
  if FSpeed2 <> 0 then
    dist2 := round(((FSpeed2 * 100) * FMap.Percent) / 10000)
  else
    dist2 := 0;
  FMap.MapPosToScreenPos(FX, FY, intX, intY);
  Canvas.Pen.Color := CurrentColorSchema.GroupsUnLoaded;
  for i := 1 to 4 do begin
    x1 := intX - Trunc(i * dist);
    y1 := intY - Trunc(i * dist);
    x2 := intX + Trunc(i * dist);
    y2 := intY + Trunc(i * dist);
    FCanvas.Ellipse(x1, y1, x2, y2);
  end;
  Canvas.Pen.Mode := pmMerge;
  Canvas.Pen.Color := CurrentColorSchema.GroupsLoaded;
  if dist2 <> 0 then
    for i := 1 to 4 do begin
      x1 := intX - Trunc(i * dist2);
      y1 := intY - Trunc(i * dist2);
      x2 := intX + Trunc(i * dist2);
      y2 := intY + Trunc(i * dist2);
      FCanvas.Ellipse(x1, y1, x2, y2);
    end;
  Canvas.Pen.Mode := pmCopy;
end;

procedure TGroupCursor.SetGroup(const x, y: integer; const speed, speed2: single);
begin
  FX := X;
  FY := Y;
  FSpeed := speed;
  if speed <> speed2 then
    FSpeed2 := speed2
  else
    FSpeed2 := 0;
end;

procedure TfrmNewMap.DisableGroupCircles1Click(Sender: TObject);
begin
  DisableGroupCircles1.Checked := not DisableGroupCircles1.Checked;
  DisableGroupCircles2.Checked := DisableGroupCircles1.Checked;
  if Assigned(FGroupCursor) then
    FGroupCursor.DontShow := DisableGroupCircles1.Checked;
end;

procedure TfrmNewMap.Viewplanetinorders1Click(Sender: TObject);
begin
  if Assigned(planet2) then
    frmOrders.FindFirstPlanetInOrders(planet2.Name);
end;

procedure TfrmNewMap.RenamePlanet1Click(Sender: TObject);
var
  strNewPlanetName: string;
  index: integer;
begin
  if Assigned(planet2) then begin
    strNewPlanetName := planet2.Name;
    if Inputquery('Create Order', 'Enter new planet name', strNewPlanetName) then begin
      for index := 0 to planets.Count - 1 do
        if (CompareText(strNewPlanetName, Planets[index]) = 0) and
          (Planets.objects[index] <> planet2) then begin
          ShowMessage('Planet Name already used!');
          Exit;
        end;
      frmOrders.InsertPlanetCommand('N ' + planet2.Name + ' ' + strNewPlanetName);
    end;
  end;
end;

procedure TfrmNewMap.MapMenuPopup(Sender: TObject);
begin
  ClaimOwnerShip1.Visible := inivars.Gametype <> gtGalaxyBlind;
  Viewplanetinorders1.Enabled := Assigned(planet2);
  RenamePlanet1.Enabled := Assigned(planet2) and (planet2.PlayerCur = qReportPlayer);
  InsertDevelopTag1.Enabled := Assigned(planet2);
  ClaimOwnerShip1.Enabled := Assigned(planet2);
  Produceonplanet1.Enabled := Assigned(planet2) and (planet2.PlayerCur = qReportPlayer);

  CAP1.Enabled := Assigned(planet2) and Assigned(planet1) and (planet1.PlayerCur = qReportPlayer);
  COL1.Enabled := Assigned(planet2) and Assigned(planet1) and (planet1.PlayerCur = qReportPlayer);
  MAT1.Enabled := Assigned(planet2) and Assigned(planet1) and (planet1.PlayerCur = qReportPlayer);
  EMP1.Enabled := Assigned(planet2) and Assigned(planet1);
end;

procedure TfrmNewMap.InsertDevelopTag1Click(Sender: TObject);
begin
  frmOrders.InsertPlanetCommand(';P ' + planet2.Name);
end;

procedure TfrmNewMap.ClaimOwnerShip1Click(Sender: TObject);
begin
  frmOrders.InsertPlanetCommand('V ' + planet2.Name);
end;

procedure TfrmNewMap.Produceonplanet1Click(Sender: TObject);
begin
  AddPlanetToClipBoardToproduce(planet2.Name)
end;

procedure TfrmNewMap.SetNewMapArea1Click(Sender: TObject);
var
  intXMin, intYMin, intXMax, intYMax: integer;
  x, y, size: integer;
begin
  FMap.ScreenPosToMapPos(0, 0, intXMin, intYMin);
  FMap.ScreenPosToMapPos(ClientWidth, ClientHeight, intXMax, intYMax);
  x := intXMin div 100;
  y := intYMin div 100;
  size := intMax(abs(intXMax - intXMin), abs(intYMax - intYMin)) div 100;
  if inivars.Gametype = gtGalaxyBlind then begin
    x := intMax(-FMap.MaxMapX div 200, x);
    y := intMax(-FMap.MaxMapY div 200, y);
    size := intMin(abs(FMap.MaxMapX div 200 - x), size);
    size := intMin(abs(FMap.MaxMapY div 200 - y), size);
  end else begin
    x := intMax(0, x);
    y := intMax(0, y);
    size := intMin(FMap.MaxMapX div 100 - x, size);
    size := intMin(FMap.MaxMapY div 100 - y, size);
  end;

  if question('Create M(ap) command for the Currency view Area?' + #13 + #10 +
    Format('X: %d Y: %d Size: %d', [x, y, size])) then
  begin
    frmOrders.InsertOtherCommand(Format('M %d %d %d', [x, y, size]));
  end;
end;

procedure TfrmNewMap.DrawTracks(turn: integer);
var
  i: integer;
  x, y: integer;
  track: TTracks;
begin
  for i := 0 to TracksList.Count - 1 do begin
    track := TTracks(TracksList[i]);
    if track.Turn = turn then begin
      FMap.MapPosToScreenPos(track.X, track.Y, x, y);
      if track.My then
        Canvas.Pixels[x, y] := CurrentColorSchema.MyTracks
      else
        Canvas.Pixels[x, y] := CurrentColorSchema.OtherTracks;
    end;
  end;
end;

procedure TfrmNewMap.TracksCurrentturn1Click(Sender: TObject);
begin
  TracksCurrentturn1.Checked := true;
  TracksCurrentturn2.Checked := true;
  FTrackturn := LastTurn;
  Repaint;
end;

procedure TfrmNewMap.TracksNone1Click(Sender: TObject);
begin
  TracksNone1.Checked := true;
  TracksNone2.Checked := true;
  FTrackturn := -1;
  Repaint;
end;

procedure TfrmNewMap.TracksPreviousturn1Click(Sender: TObject);
begin
  TracksPreviousturn1.Checked := true;
  TracksPreviousturn2.Checked := true;
  FTrackturn := LastTurn - 1;
  Repaint;
end;

procedure TfrmNewMap.Routes1Click(Sender: TObject);
var
  d: Single;
  dx, dy: Single;
begin
  if not Assigned(planet1) or not Assigned(planet2) then Exit;
  dx := planet1.x - planet2.x;
  dy := planet1.y - planet2.y;
  d := sqrt(dx * dx + dy * dy);
  CAP1.Caption := Format('Create CAP Route from %s to %s (dist: %4.2f)', [planet1.Name, planet2.Name, d / 100]);
  COL1.Caption := Format('Create COL Route from %s to %s (dist: %4.2f)', [planet1.Name, planet2.Name, d / 100]);
  MAT1.Caption := Format('Create MAT Route from %s to %s (dist: %4.2f)', [planet1.Name, planet2.Name, d / 100]);
  EMP1.Caption := Format('Create EMP Route from %s to %s (dist: %4.2f)', [planet1.Name, planet2.Name, d / 100]);
end;

procedure TfrmNewMap.CAP1Click(Sender: TObject);
var
  i: integer;
  route: TRoute;
begin
  frmOrders.InsertRouteCommand(Format('R %s CAP %s', [planet1.Name, planet2.Name]));
  i := routes.IndexOf(lowercase(planet1.Name));
  if i = -1 then begin
    route := TRoute.Create(planet1);
    routes.addobject(lowercase(planet1.Name), route);
  end else
    route := TRoute(routes.objects[i]);
  route.CAPTo := planet2;
  frmRoutes.DoFillLvRoutes;
end;

procedure TfrmNewMap.COL1Click(Sender: TObject);
var
  i: integer;
  route: TRoute;
begin
  frmOrders.InsertRouteCommand(Format('R %s COL %s', [planet1.Name, planet2.Name]));
  i := routes.IndexOf(lowercase(planet1.Name));
  if i = -1 then begin
    route := TRoute.Create(planet1);
    routes.addobject(lowercase(planet1.Name), route);
  end else
    route := TRoute(routes.objects[i]);
  route.COLTo := planet2;
  frmRoutes.DoFillLvRoutes;
end;

procedure TfrmNewMap.MAT1Click(Sender: TObject);
var
  i: integer;
  route: TRoute;
begin
  frmOrders.InsertRouteCommand(Format('R %s MAT %s', [planet1.Name, planet2.Name]));
  i := routes.IndexOf(lowercase(planet1.Name));
  if i = -1 then begin
    route := TRoute.Create(planet1);
    routes.addobject(lowercase(planet1.Name), route);
  end else
    route := TRoute(routes.objects[i]);
  route.MATTo := planet2;
  frmRoutes.DoFillLvRoutes;
end;

procedure TfrmNewMap.EMP1Click(Sender: TObject);
var
  i: integer;
  route: TRoute;
begin
  frmOrders.InsertRouteCommand(Format('R %s EMP %s', [planet1.Name, planet2.Name]));
  i := routes.IndexOf(lowercase(planet1.Name));
  if i = -1 then begin
    route := TRoute.Create(planet1);
    routes.addobject(lowercase(planet1.Name), route);
  end else
    route := TRoute(routes.objects[i]);
  route.EMPTo := planet2;
  frmRoutes.DoFillLvRoutes;
end;

procedure TfrmNewMap.ClearPlayerMarks1Click(Sender: TObject);
begin
  RemoveCursors;
end;

procedure TfrmNewMap.ClearRedMarks1Click(Sender: TObject);
begin
  ClearMarks(1);
end;

procedure TfrmNewMap.ClearBlueMarks1Click(Sender: TObject);
begin
  ClearMarks(2);
end;

procedure TfrmNewMap.DisablePlanetCircles1Click(Sender: TObject);
begin
  DisablePlanetCircles1.Checked := not DisablePlanetCircles1.Checked;
  if Assigned(FPlanetCursor) then
    FPlanetCursor.ShowRanges := not DisablePlanetCircles1.Checked;
end;

procedure TfrmNewMap.Map1Click(Sender: TObject);
begin
  MapRange101.Caption := 'Map Range ' + mkItoA(FPlanetCursor.range) + ' ...';
end;

procedure TfrmNewMap.MapRange101Click(Sender: TObject);
var
  strRange: string;
  intRange: integer;
begin
  strRange := mkIToA(FPlanetCursor.Range);
  if InputQuery('Planet Range', 'Enter new planet range:', strRange) then begin
    intRange := mkAToI(strRange);
    if intRange > 0 then begin
      FPlanetCursor.Range := intRange;
      mniSpecialValue.Caption := strRange;
      Repaint;
    end;
  end;
end;

procedure TfrmNewMap.Viewplanetscentered1Click(Sender: TObject);
begin
  Viewplanetscentered1.Checked := not Viewplanetscentered1.Checked;
end;

procedure TfrmNewMap.ChangeColors1Click(Sender: TObject);
var
  tmpColorSchema: TColorSchema;
begin
  tmpColorSchema := nil;
  try
    tmpColorSchema := TColorSchema.Create(nil);
    tmpColorSchema.Assign(CurrentColorSchema);
    if frmChangeColors.ShowModal <> mrOK then
      CurrentColorSchema.Assign(tmpColorSchema);
  finally
    tmpColorSchema.Free;
  end;
  Repaint;
end;

{ TOverlayCircle }

procedure TOverlayCircle.LoadFromStream(stream: TStream);
begin

end;

procedure TOverlayCircle.paint(Canvas: TCanvas);
var
  vFx, vFy: Integer;
  vRadius: integer;
  vTemp: Integer;
begin
  FMap.MapPosToScreenPos(FX, FY, vFx, vFy);
  FMap.MapPosToScreenPos(0, FRadius, vTemp, vRadius);
  Canvas.Ellipse(vFx - vRadius, vFy - vRadius, vFx + vRadius, vFy + vRadius);
end;

procedure TOverlayCircle.SaveToStream(stream: TStream);
begin

end;

end.
