unit UfrmPlanetSelect;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, RzPanel, UTypes, StdCtrls, EnhListView,
  ExtListView, OvcBase, OvcState, RzCommon, RzLookup, TopWnd, mkListView;

type
  TfrmPlanetSelect = class(TForm)
    RzPanel1: TRzPanel;
    RzPanel2: TRzPanel;
    btnOK: TButton;
    btnCancel: TButton;
    chbFilter: TCheckBox;
    cbxInverse: TCheckBox;
    OvcFormState1: TOvcFormState;
    btnGroupFilters: TButton;
    OvcComponentState1: TOvcComponentState;
    btnExpressionFilters: TButton;
    TimerMayChange: TTimer;
    btnAdvanced: TButton;
    cbxDoIntercept: TCheckBox;
    cbxLoadBeforeSend: TCheckBox;
    RzLookupDialog1: TRzLookupDialog;
    mkListView1: TmkListView;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure chbFilterClick(Sender: TObject);
    procedure ListView1Change(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure btnCancelClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure cbxInverseClick(Sender: TObject);
    procedure btnGroupFiltersClick(Sender: TObject);
    procedure btnExpressionFiltersClick(Sender: TObject);
    procedure TimerMayChangeTimer(Sender: TObject);
    procedure btnAdvancedClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure cbxDoInterceptClick(Sender: TObject);
    procedure mkListView1Data(Sender: TObject; Item: TListItem);
    procedure mkListView1ColumnClick(Sender: TObject; Column: TListColumn);
    procedure mkListView1DblClick(Sender: TObject);
  private
    FSelectMulti,
      FInitializing,
      FWorking: boolean;
    FSelectedPlanets: TStringList;
    FSelectedGroups: TStringList;
    LastIndex,
      FMaxShips: integer;
    function CreateMultiPlanets: string;
    function CreateSinglePlanet: string;
    { Private declarations }
  public
    property SelectMulti: boolean read FSelectMulti write FSelectMulti;
    property Working: boolean read FWorking write FWorking;
    property MaxShips: integer read FMaxShips write FMaxShips;
    property SelectedPlanets: TStringList read FSelectedPlanets;
    property SelectedGroups: TStringList read FSelectedGroups;
    function Execute: boolean;
    procedure ShowPlanet(planet: TPlanet);
    { Public declarations }
  end;

var
  frmPlanetSelect: TfrmPlanetSelect;

implementation

{$R *.DFM}

uses
  UfrmPLanets, clipbrd, UfrmOrders, UfrmMain,
  UfrmGroupFilter, UFiles, UfrmNewMap, UfrmExpressionFilters, UAdvancedSend,
  UfrmGroups, UfrmPlanetslist;

procedure TfrmPlanetSelect.ShowPlanet(planet: TPlanet);
var
  listItem: TListItem;
  i: integer;
begin
  if not FWorking then Exit;
  DoSmallPlanetDataShowPlanet(mkListView1, planet, false);
{  i := 0;
  while i < mkListView1.ViewList.Count do begin
    if planet = PSmallPlanetData(mkListView1.ViewList[i]).planet then begin
      listItem := mkListView1.Items[i];
      if listItem = nil then Exit;
      listItem.MakeVisible(False);
      listItem.Selected := not listItem.Selected;
      Exit;
    end;
    inc(i);
  end;}
end;

function TfrmPlanetSelect.Execute: boolean;
begin
  btnAdvanced.Enabled := FSelectMulti;
  Show;
  Result := True;
end;

procedure TfrmPlanetSelect.FormCreate(Sender: TObject);
begin
  mkListView1.ViewList.CompareFunc := DoSmallPlanetDataComp;
  FSelectedPlanets := TStringList.Create;
  FSelectedGroups := TStringList.Create;
  OvcComponentState1.Section := 'TfrmPlanetSelect_' + GetGameStr;
end;

procedure TfrmPlanetSelect.FormDestroy(Sender: TObject);
begin
  FSelectedPlanets.Free;
  FSelectedGroups.Free;
end;

procedure TfrmPlanetSelect.FormShow(Sender: TObject);
begin
  mkListView1.Items.BeginUpdate;
  Screen.Cursor := crHourGlass;
  try
    mkListView1.Items.Clear;
    FInitializing := True;
    try
      frmExpressionFilters.ClearFilteredPlanets;
      DoDataFillLvPlanets(mkListView1, chbFilter.Checked, cbxInverse.Checked);
    finally
      FInitializing := False;
    end;
    DoSmallPlanetDataDoubleClick(mkListView1, 0);
    mkListView1.MultiSelect := FSelectMulti;
    LastIndex := -1;
  finally
    Screen.Cursor := crDefault;
    mkListView1.Items.EndUpdate;
  end;
end;

procedure TfrmPlanetSelect.chbFilterClick(Sender: TObject);
begin
  mkListView1.Items.BeginUpdate;
  frmNewMap.DoNotpointPlanet := True;
  try
    DoDataFillLvPlanets(mkListView1, chbFilter.Checked, cbxInverse.Checked);
  finally
    mkListView1.Items.EndUpdate;
    frmNewMap.DoNotpointPlanet := False;
  end;
end;

procedure TfrmPlanetSelect.ListView1Change(Sender: TObject;
  Item: TListItem; Change: TItemChange);
begin
  if Application.Terminated then Exit;
  if not (Assigned(Item) and Assigned(frmNewMap) and Assigned(frmPlanets)) then Exit;
  if FInitializing then Exit;
  if frmNewMap.DoNotpointPlanet then Exit;
  if (Change = ctState) then begin
    if not TimerMayChange.Enabled then
      if LastIndex <> Item.Index then begin
        LastIndex := Item.Index;
        frmPlanets.ShowPlanet(PSmallPlanetData(mkListView1.ViewList[LastIndex]).planet, true);
        TimerMayChange.Enabled := True;
      end;
    if not TimerMayChange.Enabled and Item.Selected and SelectMulti and (mkListView1.SelCount > MaxShips) then
    begin
      MessageBeep($FFFF);
      Item.Selected := not Item.Selected;
      Exit;
    end;
  end;
end;

procedure TfrmPlanetSelect.btnCancelClick(Sender: TObject);
begin
  working := False;
  Close;
end;

function TfrmPlanetSelect.CreateMultiPlanets: string;
var
  i: integer;
  intGroups: integer;
  intPlanets: integer;
  intShipsInGroup: integer;
  strCurGroup: string;
  group: TGroup;
  strLoadType: string;
begin
  Result := '';
  intPlanets := 0;
  if cbxLoadBeforeSend.Checked then begin
    strLoadType := 'COL';
    RzLookupDialog1.SearchString := strLoadType;
    if RzLookupDialog1.Execute then
      strLoadType := RzLookupDialog1.SelectedItem;
  end;
  for intGroups := 0 to SelectedGroups.Count - 1 do begin
    strCurGroup := SelectedGroups[intGroups];
    group := TGroup(SelectedGroups.Objects[intGroups]);
    if not cbxDoIntercept.Checked and cbxLoadBeforeSend.Checked and (group.EffCargo > 0) then begin
      if Result <> '' then Result := Result + #13#10;
      Result := Result + 'L ' + strCurGroup + ' ' + strLoadType;
    end;
    intShipsInGroup := GetMaxToSend(TGroup(SelectedGroups.Objects[intGroups])); //.Aantal;
    i := 0;
    while (intPlanets < SelectedPlanets.Count) and (i < intShipsInGroup - 1) do begin
      if Result <> '' then Result := Result + #13#10;
      if cbxDoIntercept.Checked then
        Result := Result + 'I ' + strCurGroup + ' ' + SelectedPlanets[intPlanets] + ' 1'
      else
        Result := Result + 'S ' + strCurGroup + ' ' + SelectedPlanets[intPlanets] + ' 1';
      inc(i);
      inc(intPlanets);
    end;
    if intPlanets = SelectedPlanets.Count then Exit;
    if Result <> '' then Result := Result + #13#10;
    if cbxDoIntercept.Checked then
      Result := Result + 'I ' + strCurGroup + ' ' + SelectedPlanets[intPlanets]
    else
      Result := Result + 'S ' + strCurGroup + ' ' + SelectedPlanets[intPlanets];
    inc(intPlanets);
  end;
end;

function TfrmPlanetSelect.CreateSinglePlanet: string;
var
  sLine: string;
  i: integer;
  strLoadType: string;
  group: TGroup;
begin
  Result := '';
  if cbxLoadBeforeSend.Checked then begin
    strLoadType := 'COL';
    RzLookupDialog1.SearchString := strLoadType;
    if RzLookupDialog1.Execute then
      strLoadType := RzLookupDialog1.SelectedItem;
  end;
  if cbxDoIntercept.Checked then
    sLine := 'I %s ' + SelectedPlanets.Strings[0];
  for i := 0 to SelectedGroups.Count - 1 do
  begin
    group := TGroup(SelectedGroups.Objects[i]);
    if not cbxDoIntercept.Checked then begin
      if cbxLoadBeforeSend.Checked and (group.EffCargo > 0) then
        sLine := 'L %s ' + strLoadType + #13#10'S %s ' + SelectedPlanets.Strings[0]
      else
        sLine := 'S %s ' + SelectedPlanets.Strings[0];
    end;
    Result := Result + format(sLine, [SelectedGroups.Strings[i], SelectedGroups.Strings[i]]);
    if i < (SelectedGroups.Count - 1) then Result := Result + #13#10;
  end;
end;

procedure TfrmPlanetSelect.btnOKClick(Sender: TObject);
var
  i: integer;
begin
  SelectedPlanets.Clear;
  for i := 0 to mkListView1.items.Count - 1 do
    if mkListView1.Items[i].selected then
      SelectedPlanets.Add(mkListView1.Items[i].Caption);
  if SelectedPlanets.Count > 0 then
  begin
    if FSelectMulti then
      frmOrders.InsertCommand(CreateMultiPlanets)
    else
      frmOrders.InsertCommand(CreateSinglePlanet);
    frmOrders.AnalyseAndDoOutput;
  end;
  working := False;
  Close;
end;

procedure TfrmPlanetSelect.cbxInverseClick(Sender: TObject);
begin
  if chbFilter.Checked then
    DoDataFillLvPlanets(mkListView1, chbFilter.Checked, cbxInverse.Checked);
end;

procedure TfrmPlanetSelect.btnGroupFiltersClick(Sender: TObject);
begin
  frmGroupFilter.ShowModal;
  if chbFilter.Checked then begin
    frmNewMap.DoNotpointPlanet := False;
    try
      DoDataFillLvPlanets(mkListView1, chbFilter.Checked, cbxInverse.Checked);
    finally
      frmNewMap.DoNotpointPlanet := True;
    end;
  end;
end;

procedure TfrmPlanetSelect.btnExpressionFiltersClick(Sender: TObject);
begin
  frmExpressionFilters.ClearFilteredPlanets;
  if frmExpressionFilters.ShowModal = mrOK then begin
    frmNewMap.DoNotpointPlanet := True;
    try
      DoDataFillLvPlanets(mkListView1, chbFilter.Checked, cbxInverse.Checked);
    finally
      frmNewMap.DoNotpointPlanet := False;
    end;
  end;
end;

procedure TfrmPlanetSelect.TimerMayChangeTimer(Sender: TObject);
begin
  TimerMayChange.Enabled := False;
end;

procedure TfrmPlanetSelect.btnAdvancedClick(Sender: TObject);
var
  frmAdvancedSend: TfrmAdvancedSend;
var
  i: integer;
begin
  SelectedPlanets.Clear;
  for i := 0 to mkListView1.items.Count - 1 do
    if mkListView1.Items[i].selected then
      SelectedPlanets.AddObject(mkListView1.Items[i].Caption, PSmallPlanetData(mkListView1.ViewList[i]).Planet);
  if SelectedPlanets.Count > 0 then begin
    frmAdvancedSend := TfrmAdvancedSend.Create(nil);
    try
      frmAdvancedSend.SelectedPlanets := FSelectedPlanets;
      frmAdvancedSend.SelectedGroups := FSelectedGroups;
      frmAdvancedSend.DoIntercept := cbxDoIntercept.Checked;
      if frmAdvancedSend.ShowModal = mrOK then begin
        frmOrders.InsertCommand(frmAdvancedSend.CommandString);
        frmOrders.AnalyseAndDoOutput;
        working := False;
        Close;
      end;
    finally
      frmAdvancedSend.Free;
    end;
  end;
end;

procedure TfrmPlanetSelect.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  working := False;
end;

procedure TfrmPlanetSelect.cbxDoInterceptClick(Sender: TObject);
begin
  cbxLoadBeforeSend.Enabled := not cbxDoIntercept.Checked;
end;

procedure TfrmPlanetSelect.mkListView1Data(Sender: TObject;
  Item: TListItem);
begin
  Item.Caption := PSmallPlanetData(mkListView1.ViewList[Item.Index]).Name;
  Item.SubItems.Add(PSmallPlanetData(mkListView1.ViewList[Item.Index]).Distance);
  Item.SubItems.Add(PSmallPlanetData(mkListView1.ViewList[Item.Index]).Owner);
end;

procedure TfrmPlanetSelect.mkListView1ColumnClick(Sender: TObject;
  Column: TListColumn);
begin
  mkListview1.ViewList.SortColumn := Column.Index;
end;

procedure TfrmPlanetSelect.mkListView1DblClick(Sender: TObject);
begin
  if Assigned(mkListView1.Selected) then
    DoSmallPlanetDataDoubleClick(mkListView1, mkListView1.Selected.Index);
end;

end.
