unit UfrmTurnReports;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, RXCtrls, OvcBase, OvcState, ComCtrls, Menus,
  mwCustomEdit;

type
  TfrmTurnReports = class(TForm)
    Splitter1: TSplitter;
    Panel1: TPanel;
    RzListBox1: TTextListBox;
    OvcFormState1: TOvcFormState;
    PopupMenu1: TPopupMenu;
    mniSave: TMenuItem;
    mniReload: TMenuItem;
    Panel2: TPanel;
    mmoReport: TRichEdit;
    Panel3: TPanel;
    cbxShowPart: TComboBox;
    procedure FormShow(Sender: TObject);
    procedure RzListBox1Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure mniReloadClick(Sender: TObject);
    procedure mniSaveClick(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure cbxShowPartClick(Sender: TObject);
  private
    FLastFile: string;
    FShowPart: integer;
    procedure DoClick(index: integer);
    procedure SetShowPart(const Value: integer);
    { Private declarations }
  public
    property ShowPart: integer read FShowPart write SetShowPart;

    procedure DoShowPart(FileName: string; Value: integer);
    { Public declarations }
  end;

var
  frmTurnReports: TfrmTurnReports;

procedure AddTabsToAllReports;

implementation

{$R *.DFM}

uses
  UTypes, UFiles, UfrmMain, xProcs, HyperStr, StabUtil, mkRegEx, UAddTabs;

const
  SPPERSONAL = 0;
  SPGLOBAL = 1;
  SPWHOLE = 2;

procedure AddTabsToAllReports;
var
  ntr,
    gameN: integer;
  strCurReportName: string;
  reports: TStringList;
  blnRemoveReadOnly,
    blnRemoveAllReadOnly: boolean;
  FileStr: string;
  mkreExpr: TmkreExpr;
  i: integer;
  strOrgCaption: string;
begin
  blnRemoveAllReadOnly := False;
  if not Question('Parse all reports and add Tabs when needed?'#13 +
    'It will take some time, so be patient') then Exit;
  strOrgCaption := frmMKGal.Caption;
  mkreExpr := TmkreExpr.Create(nil);
  reports := TstringList.Create;
  try
    for ntr := 0 to MAX_TURNS do
    begin
      for gameN := 1 to length(strSpelC) do
      begin
        strCurReportName := format(strSpelD, [strSpelC[gameN], ntr]);
        if FileExists(strCurReportName) then
        begin
          blnRemoveReadOnly := True;
          if IsFileReadOnly(strCurReportName) then begin
            if not blnRemoveAllReadOnly then begin
              case MessageDlg('Remove readonly for file?'#13 + strCurReportName, mtConfirmation, [mbYes, mbNo, mbAll], 0) of
                mrYes: blnRemoveReadOnly := True;
                mrNo: blnRemoveReadOnly := False;
                mrAll: begin
                    blnRemoveAllReadOnly := True;
                    blnRemoveReadOnly := True;
                  end;
              else
                blnRemoveReadOnly := False;
              end;
            end;
            if blnRemoveReadOnly or blnRemoveAllReadOnly then
              MakeFileReadOnly(strCurReportName, False);
          end;
          if blnRemoveReadOnly or blnRemoveAllReadOnly then begin
            frmMKGal.Caption := 'MKGal adding tabs: ' + strCurReportName;
            frmMKGal.Repaint;
            CopyFile(PChar(strCurReportName), PChar(strCurReportName + '.TabBackup'), True);
            reports.LoadFromFile(strCurReportName);
            FileStr := reports.GetText;
            ReplaceC(FileStr, #$A0, #$20);
            reports.SetText(PChar(FileStr));
            mkreExpr.Pattern := SearchLines;
            for i := 0 to reports.Count - 1 do begin
              mkreExpr.Str := reports[i];
              if mkreExpr.DoSearch(1) > 0 then
                reports[i] := #9#9 + Trim(reports[i]);
            end;
            try
              reports.SaveToFile(strCurReportName);
            except
              ShowMessage(Format('I was not able to save %s to disk, please check if it is write protected, read only or if the disk is full', [strCurReportName]));
            end;
          end;
        end;
      end;
    end;
  finally
    reports.Free;
    mkreExpr.Free;
    frmMKGal.Caption := strOrgCaption;
  end;
end;

procedure TfrmTurnReports.FormShow(Sender: TObject);
var
  ntr,
    gameN: integer;
  strCurReportName: string;
begin
  if inivars.Gametype = gtGalaxyBlind then begin
    cbxShowPart.ItemIndex := SPWHOLE;
    FShowPart := SPWHOLE;
    cbxShowPart.Enabled := False;
  end else begin
    cbxShowPart.ItemIndex := SPPERSONAL;
    FShowPart := SPPERSONAL;
  end;
  RzListBox1.Items.Clear;
  for ntr := 0 to MAX_TURNS do
  begin
    for gameN := 1 to length(strSpelC) do
    begin
      strCurReportName := format(strSpelD, [strSpelC[gameN], ntr]);
      if FileExists(strCurReportName) then
      begin
        RzListBox1.Items.Add(strCurReportName);
      end;
    end;
  end;
  if LastTurn >= 0 then
  begin
    strCurReportName := format(strSpelD, [strSpelC[1], LastTurn]);
    FLastFile := strCurReportName;
    DoClick(RzListBox1.Items.IndexOf(strCurReportName));
  end;

end;

procedure TfrmTurnReports.DoClick(index: integer);
begin
  mmoReport.Lines.Clear;
  if index <> -1 then
  begin
    FLastFile := RzListBox1.Items[index];
    DoShowPart(FLastFile, ShowPart);
  end;
end;

procedure TfrmTurnReports.RzListBox1Click(Sender: TObject);
begin
  DoClick(RzListBox1.ItemIndex);
end;

procedure TfrmTurnReports.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  WindowState := wsMinimized;
  CanClose := false;
end;

procedure TfrmTurnReports.mniReloadClick(Sender: TObject);
begin
  if FileExists(FLastFile) then
    DoShowPart(FLastFile, ShowPart);
end;

procedure TfrmTurnReports.mniSaveClick(Sender: TObject);
begin
  if (FLastFile <> '') and Question('Are you sure to save?') then
    mmoReport.Lines.SaveToFile(FLastFile);
end;

procedure TfrmTurnReports.SetShowPart(const Value: integer);
begin
  if FShowPart = Value then Exit;
  FShowPart := Value;
  DoShowPart(FLastFile, Value);
end;

procedure TfrmTurnReports.DoShowPart(FileName: string; Value: integer);
var
  report: TStringList;
  i: integer;
  curPart: integer;
begin
  if Value = -1 then Exit;
  if not FileExists(FileName) then Exit;
  mmoReport.ReadOnly := Value <> SPWHOLE;
  report := TStringList.Create;
  try
    report.LoadFromFile(FileName);
    if Value <> SPWHOLE then begin
      report.LoadFromFile(FileName);
      curPart := SPWHOLE;
      i := 0;
      while i < report.Count do begin
        case curPart of
          SPWHOLE: if pos('PERSONAL Messages', report[i]) > 0 then begin
              curPart := SPPERSONAL;
            end else begin
              if pos('GLOBAL Messages', report[i]) > 0 then begin
                curPart := SPGLOBAL;
              end else
                report.Delete(i);
            end;
          SPPERSONAL: if pos('GLOBAL Messages', report[i]) > 0 then begin
              curPart := SPGLOBAL;
            end else begin
              if (pos('ORDERS RECEIVED', report[i]) > 0) or
                (pos('Status of Players', report[i]) > 0) then begin
                curPart := SPWHOLE;
              end else begin
                if Value <> SPPERSONAL then report.Delete(i)
                else inc(i);
              end;
            end;
          SPGLOBAL: if (pos('ORDERS RECEIVED', report[i]) > 0) or
            (pos('Status of Players', report[i]) > 0) then begin
              curPart := SPWHOLE;
            end else begin
              if pos('PERSONAL Messages', report[i]) > 0 then begin
                curPart := SPPERSONAL;
              end else begin
                if Value <> SPGLOBAL then report.Delete(i)
                else inc(i);
              end;
            end;
        end;
      end;
      mmoReport.Lines.BeginUpdate;
      try
        mmoReport.Lines.SetText(report.GetText);
      finally
        mmoReport.Lines.EndUpdate;
      end;
    end else
      mmoReport.Lines.LoadFromFile(FileName);
    //mmoReport.SetSelStart(0);
    mmoReport.SelStart := 0;
  finally
    report.Free;
  end;
end;

procedure TfrmTurnReports.PopupMenu1Popup(Sender: TObject);
begin
  mniSave.Enabled := ShowPart = SPWHOLE;
end;

procedure TfrmTurnReports.cbxShowPartClick(Sender: TObject);
begin
  if cbxShowPart.ItemIndex <> -1 then
    ShowPart := cbxShowPart.ItemIndex;
end;

end.
