unit UmkStrParse;

interface

uses Classes, SysUtils, Controls;

type
  TCharDelimiter = array[#0..#255] of ByteBool;

  PITokenArray = ^TITokenArray;
  TITokenArray = array[0..0] of PChar;

  TWordItem = record
    wordstart,
      wordend,
      nextword: integer;
  end;
  TWordItemArray = array[0..0] of TWordItem;

  TMkStrParse = class(TObject)
  private
    FCode: integer; // fout code van Val functie

    FChild,
      FParent: TMkStrParse;
    FLastChildWord: integer;
    FStringToParse: string;
    FStringToParseLen: integer;
    FCharDelimeters: TCharDelimiter;
    FSkipExtraDel: boolean;
    FCRLFDelimeter: boolean;
    FActueel: boolean;

    FWords: ^TWordItemArray;
    FWordsCount: integer;
    FWordsCapacity: integer;
    FWordsCapDelta: integer;
    procedure WordsArrayGrow;
    procedure WordsArrayClear;
    procedure WordsAdd(wordstart, wordend, nextword: integer);

    procedure DoParseNormal;
    procedure DoParseNormalWithSkip;
    procedure DoParseCRLF;
    function GetCount: integer;
    function GetCharDelimeter(index: char): boolean;
    procedure SetCharDelimeter(index: char; blnDelimeter: boolean);
    function GetChild: TMkStrParse;
    function GetChildWords(index: integer): TMkStrParse;
    procedure SetCRLFDelimeter(blnSet: boolean);
    procedure SetSkipExtraDel(blnSet: boolean);
    procedure SetStringToParse(strToParse: string);

    function GetWord(index: integer): string;
    procedure SetWord(index: integer; value: string);
    function GetAsString(index: integer): string;
    procedure SetAsString(index: integer; value: string);
    function GetAsInteger(index: integer): integer;
    procedure SetAsInteger(index, value: integer);
    function GetAsSingle(index: integer): single;
    procedure SetAsSingle(index: integer; value: single);
    function GetAsDate(index: integer): TDate;
    procedure SetAsDate(index: integer; value: TDate);

  public
    function Parse(var start, index: integer): integer;
    function ParseCRLF(var start, index: integer): integer;
    function ParseWord(var start, index: integer): integer;
    function FindNum(Num: integer): integer;
    function Fetch(index: integer): string;
    function FetchNum(Num: integer): string;

    property AsString[index: integer]: string read GetAsString write SetAsString; default;
    property AsInteger[index: integer]: integer read GetAsInteger write SetAsInteger;
    property AsSingle[index: integer]: Single read GetAsSingle write SetAsSingle;
    property AsDate[index: integer]: TDate read GetAsDate write SetAsDate;

    property CharDelimeters[index: char]: boolean read GetCharDelimeter write SetCharDelimeter;
    property CRLFDelimeter: boolean read FCRLFDelimeter write SetCRLFDelimeter;
    property SkipExtraDel: boolean read FSkipExtraDel write SetSkipExtraDel;
    property StringToParse: string read FStringToParse write SetStringToParse;

    property Child: TMkStrParse read GetChild;
    property ChildWords[index: integer]: TMkStrParse read GetChildWords;

    property Count: integer read GetCount;

    procedure CharDelimetersClear;
    constructor Create(Parent: TMkStrParse);
    function CreateChild(strOptions: string): TMkStrParse;
    destructor Destroy; override;
    procedure DoParsing;
    function ParseToTstring(strT: TStrings): integer;
    procedure ResetParsing;
  end;

implementation

uses
  mkStrUtils, hyperstr;

// private

function TMkStrParse.GetCharDelimeter(index: char): boolean;
begin
  Result := FCharDelimeters[index];
end;

procedure TMkStrParse.SetCharDelimeter(index: char; blnDelimeter: boolean);
begin
  if FCharDelimeters[index] <> blnDelimeter then
  begin
    FCharDelimeters[index] := blnDelimeter;
    ResetParsing;
  end;
end;

function TMkStrParse.GetChild: TMkStrParse;
begin
  Result := FChild;
end;

function TMkStrParse.GetChildWords(index: integer): TMkStrParse;
begin
  Result := FChild;
  if FLastChildWord <> index then
  begin
    FChild.StringToParse := GetWord(index);
    FLastChildWord := index;
  end;
end;

procedure TMkStrParse.SetCRLFDelimeter(blnSet: boolean);
begin
  if blnSet then
  begin
    CharDelimetersClear;
    FCharDelimeters[#10] := True;
    FCharDelimeters[#13] := True;
    FSkipExtraDel := False;
  end;
  FCRLFDelimeter := blnSet;
  ResetParsing;
end;

procedure TMkStrParse.SetSkipExtraDel(blnSet: boolean);
begin
  if FSkipExtraDel <> blnSet then
  begin
    if FCRLFDelimeter and blnSet then FCRLFDelimeter := False;
    FSkipExtraDel := blnSet;
    ResetParsing;
  end;
end;

procedure TMkStrParse.SetStringToParse(strToParse: string);
begin
  if FStringToParse <> strToParse then
  begin
    FStringToParse := strToParse;
    FStringToParseLen := Length(FStringToParse);
    ResetParsing;
  end;
end;

function TMkStrParse.GetWord(index: integer): string;
var
  startPChar: pointer;
  toPChar: pointer;
  Len: integer;
begin
  if not FActueel then DoParsing;
  if index < 0 then
    index := FWordsCount + index;
  if index >= FWordsCount then
    raise Exception.Create('Index out of bounds');
  startPChar := @FStringToParse[FWords[index].wordstart];
  len := FWords[index].wordend - FWords[index].wordstart + 1;
  SetLength(Result, len);
  toPChar := @Result[1];
  move(startPChar^, toPChar^, len);
end;

procedure TMkStrParse.SetWord(index: integer; value: string);
var
  StartIndex: integer;
begin
  if not FActueel then DoParsing;
  if index < 0 then
    index := FWordsCount + index;
  if index >= FWordsCount then
    raise Exception.Create('Index out of bounds');
  StartIndex := FWords[index].wordstart;
  delete(FStringToParse, StartIndex, (FWords[index].wordend - StartIndex + 1));
  insert(value, FStringToParse, StartIndex);
  ResetParsing;
end;

function TMkStrParse.GetAsString(index: integer): string;
begin
  Result := GetWord(index);
end;

procedure TMkStrParse.SetAsString(index: integer; value: string);
begin
  SetWord(index, value);
end;

function TMkStrParse.GetAsInteger(index: integer): integer;
begin
  Result := mkAtoI(GetWord(index));
end;

procedure TMkStrParse.SetAsInteger(index, value: integer);
begin
  SetWord(index, IntToStr(value));
end;

function TMkStrParse.GetAsSingle(index: integer): single;
begin
  Val(GetWord(index), Result, FCode);
end;

procedure TMkStrParse.SetAsSingle(index: integer; value: single);
begin
  SetWord(index, FloatToStr(value));
end;

function TMkStrParse.GetAsDate(index: integer): TDate;
begin
  Result := StrToDateTime(GetWord(index));
end;

procedure TMkStrParse.SetAsDate(index: integer; value: TDate);
begin
  SetWord(index, DateTimeToStr(value));
end;

// public

procedure TMkStrParse.CharDelimetersClear;
var
  i: char;
begin
  for i := #0 to #255 do FCharDelimeters[i] := False;
end;

constructor TMkStrParse.Create(Parent: TMkStrParse);
begin
  FParent := Parent;
  FChild := nil;
  FWords := nil;
  WordsArrayClear;
  CRLFDelimeter := True;
end;

function TMkStrParse.CreateChild(strOptions: string): TMkStrParse;
var
  i: integer;
begin
  if not Assigned(FChild) then
  begin
    FChild := TMkStrParse.Create(Self);
    for i := 1 to Length(strOptions) do
      case strOptions[i] of
        'C', 'c': FChild.CRLFDelimeter := True;
        'S', 's': FChild.SkipExtraDel := True;
      end;
    FLastChildWord := -1;
  end;
  Result := FChild;
end;

destructor TMkStrParse.Destroy;
begin
  WordsArrayClear;
  if Assigned(FChild) then
    FChild.Free;
  inherited;
end;

procedure TMkStrParse.DoParsing;
begin
  Resetparsing;
  if FStringToParseLen <> 0 then
  begin
    if FCRLFDelimeter then begin
      DoParseCRLF;
    end else begin
      if FSkipExtraDel then
        DoParseNormalWithSkip
      else
        DoParseNormal;
    end;
  end;
  FActueel := true;
end;

function TMkStrParse.GetCount: integer;
begin
  if not FActueel then DoParsing;
  Result := FWordsCount;
end;

procedure TMkStrParse.WordsArrayGrow;
begin
  FWordsCapacity := FWordsCapacity + FWordsCapDelta;

  ReallocMem(FWords, FWordsCapacity * sizeof(TWordItem));
  if (FWordsCapacity > 1024) and (FWordsCapacity <= 4096) then
    FWordsCapDelta := 512
  else
    if (FWordsCapacity > 4096) and (FWordsCapacity <= (1024 * 16)) then
      FWordsCapDelta := 1024
    else
      if (FWordsCapacity > (1024 * 16)) and (FWordsCapacity <= (1024 * 256)) then
        FWordsCapDelta := (1024 * 4);
end;

procedure TMkStrParse.WordsArrayClear;
begin
  if FWords <> nil then
  begin
    freemem(FWords);
    FWords := nil;
  end;
  FWordsCount := 0;
  FWordsCapacity := 0;
  FWordsCapDelta := 256;
end;

procedure TMkStrParse.WordsAdd(wordstart, wordend, nextword: integer);
begin
  if FWordsCount = FWordsCapacity then WordsArrayGrow;
  FWords[FWordsCount].wordstart := wordstart;
  FWords[FWordsCount].wordend := wordend;
  FWords[FWordsCount].nextword := nextword;
  inc(FWordsCount);
end;

function TMkStrParse.Parse(var start, index: integer): integer;
var
  runptr: PChar;
begin
  runptr := @FStringToParse[index];
  begin
    while (index <= FStringToParseLen) and (not FCharDelimeters[runptr^]) do
    begin
      inc(runptr); // zoek volgende delimeter
      inc(index);
    end;
  end;
  if index > FStringToParseLen then
  begin
    Result := FStringToParseLen;
    index := 0;
  end
  else
  begin
    Result := index - 1;
    inc(index);
  end;
end;

function TMkStrParse.ParseCRLF(var start, index: integer): integer;
var
  runptr: PChar;
begin
  Result := -1;
  runptr := @FStringToParse[index];
  if (runptr^ = #13) and (runptr[1] = #10) then // CR + LF assumed
  begin
    inc(index, 2);
    Result := start;
    inc(start);
    Exit;
  end;
  while (index <= FStringToParseLen) do
  begin
    if runptr^ = #13 then
    begin
      inc(index);
      inc(runptr);
      if (index <= FStringToParseLen) and (runptr^ = #10) then
      begin
        Result := index - 2;
        inc(index);
        break;
      end;
      if index > FStringToParseLen then
        break;
    end;
    inc(index);
    inc(runptr);
  end;
  if index > FStringToParseLen then
  begin
    Result := FStringToParseLen;
    index := 0;
  end;
end;

function TMkStrParse.ParseWord(var start, index: integer): integer;
var
  runptr: PChar;
  len: integer;
begin
  runptr := @FStringToParse[index];
  len := FStringToParseLen;
  while (index <= len) and (FCharDelimeters[runptr^]) do
  begin
    inc(runptr); // zoek volgende delimeter
    inc(index);
    start := index;
  end;
  Result := Parse(start, index);
  if index = 0 then Exit;
  runptr := @FStringToParse[index];
  while (index <= len) and (FCharDelimeters[runptr^]) do
  begin
    inc(runptr); // zoek volgende delimeter
    inc(index);
  end;
  if index > len then index := 0;
end;

function TMkStrParse.FindNum(Num: integer): integer;
var
  L, H, I: Integer;
begin
  L := 0;
  H := FWordsCount - 1;
  while L <= H do
  begin
    I := (L + H) shr 1;
    if FWords[I].wordend < NUM then L := I + 1
    else
      if FWords[I].wordstart > NUM then H := I - 1
      else
      begin
        Result := I;
        exit;
      end;
  end;
  Result := -1;
end;

function TMkStrParse.Fetch(index: integer): string;
var
  Num: integer;
begin
  Num := FindNum(index);
  if Num <> -1 then
    Result := GetWord(NUM)
  else
    Result := '';
end;

function TMkStrParse.FetchNum(Num: integer): string;
begin
  Result := GetWord(Num);
end;

function TMkStrParse.ParseToTstring(strT: TStrings): integer;
var
  i: integer;
begin
  strT.BeginUpdate;
  try
    if not FActueel then DoParsing;
    for i := 0 to FWordsCount - 1 do
      strT.Add(copy(FStringToParse, FWords[i].wordstart, FWords[i].wordend - FWords[i].wordstart + 1));
  finally
    strT.EndUpdate;
  end;
  Result := strT.Count;
end;

procedure TMkStrParse.ResetParsing;
begin
  WordsArrayClear;
  FActueel := False;
  FLastChildWord := -1;
end;

procedure TMkStrParse.DoParseCRLF;
var
  wordstart, wordend, nextword: integer;
begin
  wordstart := 1;
  nextword := 1;
  repeat
    wordend := ParseCRLF(wordstart, nextword);
    if wordstart <= FStringToParseLen then begin
      WordsAdd(wordstart, wordend, nextword);
      wordstart := nextword;
    end else begin
      wordstart := nextword;
      break;
    end;
  until (wordstart < 1);
end;

procedure TMkStrParse.DoParseNormal;
var
  wordstart, wordend, nextword: integer;
begin
  wordstart := 1;
  nextword := 1;
  repeat
    wordend := Parse(wordstart, nextword);
    if wordstart <= FStringToParseLen then begin
      WordsAdd(wordstart, wordend, nextword);
      wordstart := nextword;
    end else begin
      wordstart := nextword;
      break;
    end;
  until (wordstart < 1);
end;

procedure TMkStrParse.DoParseNormalWithSkip;
var
  wordstart, wordend, nextword: integer;
begin
  wordstart := 1;
  nextword := 1;
  repeat
    wordend := ParseWord(wordstart, nextword);
    if wordstart <= FStringToParseLen then begin
      WordsAdd(wordstart, wordend, nextword);
      wordstart := nextword;
    end else begin
      wordstart := nextword;
      break;
    end;
  until (wordstart < 1);
end;

end.

