unit Utypes;

interface

uses Windows, Classes, Math, UfastLists, UDistLists, Graphics;

const
  P_MAT = 2;
  DRIVE = 3;
  WEAPONS = 4;
  SHIELDS = 5;
  CARGO = 6;
  P_COL = 7;
  P_CAP = 8;
  SHIP = 9;
  EMPTY = 10;
  SHIPBASE = 100;

  MAX_TURNS = 200;
  MAX_PLAYERS = 60;

  MAX_DTURNS = 4;

type
  TPlanet = class;
  TPlayer = class;

  PSmallPlanetData = ^TSmallPlanetData;
  TSmallPlanetData = record
    Name: string;
    Owner: string;
    Distance: string;
    planet: TPlanet;
  end;
  PBigPlanetData = ^TBigPlanetData;
  TBigPlanetData = record
    Name: string;
    Nr: string;
    Owner: string;
    Distance: string;
    EffInd: string;
    ShipMass: string;
    Size: string;
    Res: string;
    Pop: string;
    Ind: string;
    COL: string;
    CAP: string;
    MAT: string;
    MyMass: string;
    FrMass: string;
    EnMass: string;
    SeMass: string;
    ProdType: string;
    ProdAmount: string;
    Buildability: string;
    planet: TPlanet;
  end;
  PBigGroupData = ^TBigGroupData;
  TBigGroupData = record
    GroupNr: string;
    Count: string;
    Avail: string;
    ETA: string;
    GRO: string;
    Planet: string;
    PRO: string;
    SourceP: string;
    ShipType: string;
    Distance: string;
    EffSpeed: string;
    Attacks: string;
    EffWeapons: string;
    EffShield: string;
    EffCargo: string;
    UpgradeCosts: string;
    Data: TObject;
  end;


  TRouteState = (rsReport, rsNew, rsDeleted);
  TRoute = class(TObject)
  private
    FPlanetFrom: TPlanet;
    FMATTo: TPlanet;
    FCOLTo: TPlanet;
    FCAPTo: TPlanet;
    FEMPTo: TPlanet;
    FOrgMATTo: TPlanet;
    FOrgCOLTo: TPlanet;
    FOrgCAPTo: TPlanet;
    FOrgEMPTo: TPlanet;
  public
    constructor Create(planet: TPlanet); virtual;
    procedure RestoreOrgValues;
    property PlanetFrom: TPlanet read FPlanetFrom write FPlanetFrom;
    property COLTo: TPlanet read FCOLTo write FCOLTo;
    property CAPTo: TPlanet read FCAPTo write FCAPTo;
    property MATTo: TPlanet read FMATTo write FMATTo;
    property EMPTo: TPlanet read FEMPTo write FEMPTo;
    property OrgCOLTo: TPlanet read FOrgCOLTo write FOrgCOLTo;
    property OrgCAPTo: TPlanet read FOrgCAPTo write FOrgCAPTo;
    property OrgMATTo: TPlanet read FOrgMATTo write FOrgMATTo;
    property OrgEMPTo: TPlanet read FOrgEMPTo write FOrgEMPTo;
  end;

  TCargoTypes = (ctDoNotUse, ctCOL, ctCAP, ctStationary, ctCOLThenCAP, ctCAPthenCOL);
  TShipType = class(TObject)
  private
    Fname: string;
    Fdrive: single;
    Fattacks: integer;
    Fweapons,
      Fshields,
      Fcargo: single;
    FCargoTypes: TCargoTypes;
    FIsNew: boolean;
    FNGBlindMass: Single;
    function GetMass: single;
    function GetIsStationary: boolean;
  public
    property CargoTypes: TCargoTypes read FCargoTypes write FCargoTypes;
    property Name: string read Fname write Fname;
    property Drive: single read Fdrive write Fdrive;
    property Attacks: integer read FAttacks write FAttacks;
    property Weapons: single read Fweapons write Fweapons;
    property Shields: single read Fshields write Fshields;
    property Cargo: single read Fcargo write Fcargo;
    property Mass: single read GetMass;
    property IsNew: boolean read FIsNew write FIsNew;
    property IsStationary: boolean read GetIsStationary;

    property NGBlindMass: single read FNGBlindMass write FNGBlindMass;
  end;

  TIncomingGroup = class(TObject)
  private
    FFromPlanet,
      FDestPlanet: TPlanet;
    FPlayer: TPlayer;
    Frange,
      Fspeed,
      Fmass: single;
    FTurn: integer;
  public
    property FromPlanet: TPlanet read FFromPlanet write FFromPlanet;
    property DestPlanet: TPlanet read FDestPlanet write FDestPlanet;
    property Player: TPlayer read FPlayer write FPlayer;
    property range: single read Frange write Frange;
    property speed: single read Fspeed write Fspeed;
    property mass: single read Fmass write Fmass;
    property Turn: integer read FTurn write FTurn;
  end;

  TTracks = class(TObject)
  private
    FTurn: integer;
    FY: integer;
    FX: integer;
    FMy: boolean;
  public
    property My: boolean read FMy write FMy;
    property X: integer read FX write FX;
    property Y: integer read FY write FY;
    property Turn: integer read FTurn write FTurn;
  end;

  TGroup = class(TObject)
  private
    Ftype: TShipType;
    Fplanet: TPlanet;
    Fplayer: TPlayer;
    FLocPlanet: TPlanet;
    FdriveTech, FweaponsTech, FshieldsTech, FcargoTech: single;
    Faantal: integer;
    Flading: single;
    FladingType: integer;
    FaantalBattle: integer;
    Fnr: integer;
    Frange: single;
    Fturn: integer;
    Fbat: integer;
    Ffleetname: string;
    FFleetSpeed: single;
    FETA: integer;
    FSourcePlanet: TPlanet;
    FDestPlanet: TPlanet;
    function GetEffCargo: single;
    function GetEffWeapon: single;
    function GetEffShield: single;
    function GetEffShieldLoaded: single;
    function GetEffSpeed: single;
    function GetEffSpeedLoaded: single;
    function GetEffSpeedNow: single;
    procedure SetFleetSpeed(const Value: single);
    function GetUpgradeCost: single;
  public
    constructor Create;
    destructor Destroy; override;
    property _Type: TShipType read Ftype write Ftype;
    property Planet: TPlanet read FPlanet write FPlanet;
    property SourcePlanet: TPlanet read FSourcePlanet write FSourcePlanet;
    property DestPlanet: TPlanet read FDestPlanet write FDestPlanet;
    property ETA: integer read FETA write FETA;
    property LocPlanet: TPlanet read FLocPlanet write FLocPlanet;
    property Player: TPlayer read FPlayer write FPlayer;
    property Nr: integer read Fnr write Fnr;
    property Turn: integer read Fturn write Fturn;
    property Bat: integer read Fbat write Fbat;
    property Range: single read Frange write Frange;
    property DriveTech: single read FdriveTech write FdriveTech;
    property WeaponsTech: single read FweaponsTech write FweaponsTech;
    property ShieldsTech: single read FshieldsTech write FshieldsTech;
    property CargoTech: single read FcargoTech write FcargoTech;
    property Aantal: integer read Faantal write Faantal;
    property AantalBattle: integer read FaantalBattle write FaantalBattle;
    property Lading: single read Flading write Flading;
    property LadingType: integer read FladingType write FladingType;
    property FleetName: string read FfleetName write FfleetName;
    property FleetSpeed: single read FFleetSpeed write SetFleetSpeed;
    property EffCargo: single read GetEffCargo;
    property EffWeapon: single read GetEffWeapon;
    property EffShield: single read GetEffShield;
    property EffShieldLoaded: single read GetEffShieldLoaded;
    property EffSpeed: single read GetEffSpeed;
    property EffSpeedLoaded: single read GetEffSpeedLoaded;
    property EffSpeedNow: single read GetEffSpeedNow;
    property UpgradeCost: single read GetUpgradeCost;

    function GetEffShieldWithLoad(load: single): single;
  end;

  TBattle = class
  private
    Fturn: integer;
    Fplanet: TPlanet;
    Fgroups: TList;
    FgroupA: TList; // attackers
    FgroupD: TList; // defenders
    function GetGroupA: Tlist;
    function GetGroupD: Tlist;
  public
    constructor Create;
    destructor Destroy; override;
    property Turn: integer read Fturn write Fturn;
    property Planet: TPlanet read Fplanet write Fplanet;
    property Groups: TList read Fgroups write Fgroups;
    property GroupA: TList read GetGroupA;
    property GroupD: TList read GetGroupD;
    procedure Simulate;
  end;

  TExtraPlanetInfo = class
  private
    FPlayerNr: integer;
    FPlanetNr: integer;
    FPlanet: TPlanet;
    FPlayer: TPlayer;
    procedure SetPlanetNr(const Value: integer);
    procedure SetPlayerNr(const Value: integer);
    procedure SetPlanet(const Value: TPlanet);
    procedure SetPlayer(const Value: TPlayer);
  public
    property Planet: TPlanet read FPlanet write SetPlanet;
    property PlanetNr: integer read FPlanetNr write SetPlanetNr;
    property Player: TPlayer read FPlayer write SetPlayer;
    property PlayerNr: integer read FPlayerNr write SetPlayerNr;
  end;

  TExtraPlanetInfoList = class
  private
    Fplanets: array of TExtraPlanetInfo;
    function GetPlanets(index: integer): TExtraPlanetInfo;
    procedure SetPlanets(index: integer; const Value: TExtraPlanetInfo);
  public
    property Planets[index: integer]: TExtraPlanetInfo read GetPlanets write SetPlanets;
    procedure ReadInfo(strFileName: string);
    procedure WriteInfo(strFileName: string);
    function AddPlanet(planet: TPlanet; player: TPlayer): integer;
    procedure DeletePlanet(planet: TPlanet);
    procedure SolveExtraInfo;
  end;

  TPlayer = class(Tobject)
  private
    Fst: TStringList; // list
    Fgr: TList; // list
    Fname: string;
    FDrive,
      FWeapons,
      Fshield,
      Fcargo,
      Fpopulation,
      Findustry,
      FmaxPopulation: array[0..MAX_TURNS + 2] of single;
    Fplanets: array[0..MAX_TURNS + 2] of integer;
    Fwar,
      FligtEruit,
      FinFile: boolean;
    FposInFile: integer;
    Fsortpos: integer;
    FgrafW: single;
    FNr: integer;
    FOldNames: string;
    FColor: TColor;

    function GetDrive(index: integer): single;
    function GetDriveD(index: integer): single;
    procedure SetDrive(index: integer; value: single);
    function GetWeapons(index: integer): single;
    function GetWeaponsD(index: integer): single;
    procedure SetWeapons(index: integer; value: single);
    function GetShield(index: integer): single;
    function GetShieldD(index: integer): single;
    procedure SetShield(index: integer; value: single);
    function GetCargo(index: integer): single;
    function GetCargoD(index: integer): single;
    procedure SetCargo(index: integer; value: single);
    function GetPopulation(index: integer): single;
    function GetPopulationD(index: integer): single;
    procedure SetPopulation(index: integer; value: single);
    function GetMaxPopulation(index: integer): single;
    function GetMaxPopulationD(index: integer): single;
    procedure SetMaxPopulation(index: integer; value: single);
    function GetEffIndustry(index: integer): single;
    function GetEffIndustryD(index: integer): single;
    function GetIndustry(index: integer): single;
    function GetIndustryD(index: integer): single;
    procedure SetIndustry(index: integer; value: single);
    function GetPlanets(index: integer): integer;
    function GetPlanetsD(index: integer): integer;
    procedure SetPlanets(index: integer; value: integer);
    procedure SetNr(const Value: integer);
    procedure SetName(const Value: string);
    procedure SetOldNames(const Value: string);
    function GetPopulationPercent(index: integer): single;
  public
    constructor Create;
    destructor Destroy; override;

    property ShipTypes: TStringList read Fst write Fst; // magic
    property Groups: TList read Fgr write Fgr; // magic
    property Name: string read Fname write SetName;
    property OldNames: string read FOldNames write SetOldNames;
    property Nr: integer read FNr write SetNr;
    property DriveCur: single index - 1 read GetDrive;
    property DriveDCur: single index - 1 read GetDriveD;
    property Drive[index: integer]: single read GetDrive write SetDrive;
    property DriveD[index: integer]: single read GetDriveD;
    property WeaponsCur: single index - 1 read GetWeapons;
    property WeaponsDCur: single index - 1 read GetWeaponsD;
    property Weapons[index: integer]: single read GetWeapons write SetWeapons;
    property WeaponsD[index: integer]: single read GetWeaponsD;
    property ShieldCur: single index - 1 read GetShield;
    property ShieldDCur: single index - 1 read GetShieldD;
    property Shield[index: integer]: single read GetShield write SetShield;
    property ShieldD[index: integer]: single read GetShieldD;
    property CargoCur: single index - 1 read GetCargo;
    property CargoDCur: single index - 1 read GetCargoD;
    property Cargo[index: integer]: single read GetCargo write SetCargo;
    property CargoD[index: integer]: single read GetCargoD;
    property PopulationCur: single index - 1 read GetPopulation;
    property PopulationDCur: single index - 1 read GetPopulationD;
    property Population[index: integer]: single read GetPopulation write SetPopulation;
    property PopulationD[index: integer]: single read GetPopulationD;
    property maxPopulationCur: single index - 1 read GetMaxPopulation;
    property maxPopulationCurD: single index - 1 read GetMaxPopulationD;
    property maxPopulation[index: integer]: single read GetMaxPopulation write SetMaxPopulation;
    property maxPopulationD[index: integer]: single read GetMaxPopulationD;
    property PopulationPercentCur: Single index -1 read GetPopulationPercent;
    property PopulationPercent[index: integer]: single read GetPopulationPercent;
    property EffIndustryCur: single index - 1 read GetEffIndustry;
    property EffIndustryDCur: single index - 1 read GetEffIndustryD;
    property EffIndustry[index: integer]: single read GetEffIndustry;
    property EffIndustryD[index: integer]: single read GetEffIndustryD;
    property IndustryCur: single index - 1 read GetIndustry;
    property IndustryDCur: single index - 1 read GetIndustryD;
    property Industry[index: integer]: single read GetIndustry write SetIndustry;
    property IndustryD[index: integer]: single read GetIndustryD;
    property PlanetsCur: integer index - 1 read GetPlanets;
    property PlanetsDCur: integer index - 1 read GetPlanetsD;
    property Planets[index: integer]: integer read GetPlanets write SetPlanets;
    property PlanetsD[index: integer]: integer read GetPlanetsD;
    property War: Boolean read FWar;

    property Color: TColor read FColor write FColor;
    property PosInFile: integer read FposInFile write FposInFile;
    property InFile: boolean read FinFile write FinFile;
    property LigtEruit: boolean read FligtErUit write FligtErUit;
    property SortPos: integer read FsortPos write FsortPos;
    property GrafW: single read FgrafW write FgrafW;

    procedure SetPlayer(Turn: integer;
      NameX: string;
      driveX, weaponsX, shieldX, cargoX: single;
      populationX, industryX: single;
      planetsX: integer;
      warX: boolean);
  end;

  Tplanet = class(TObject)
  private
    Fname: string;
    Fnr: integer;
    Fx, Fy: integer;
    FSize, FResources: single;
    FPopulation,
      FIndustry,
      FCapital,
      FMaterial,
      FColonist: array[0..MAX_TURNS + 2] of single;
    FProducing: array[0..MAX_TURNS + 2] of integer;
    Fplayer: array[0..MAX_TURNS + 1] of TPlayer;
    FplayerLast: integer;
    Fdistance: integer;
    FLastOwnerName: string;
    FLastOwnerType: integer;
    FLastOwnerObject: TPlayer;
    FLastOwnerTurn: integer;
    FFriendShipMass: single;
    FMyShipMass: single;
    FEnemyShipMass: single;
    FSendedShipMass: single;
    FProduceType: string;
    FOrgProduceType: string;
    FSendedShipMassEta2: single;
    FSendedShipMassEta2Plus: single;
    FBuildability: Single;

    function GetPopulation(index: integer): single;
    procedure SetPopulation(index: integer; value: single);
    function GetIndustry(index: integer): single;
    procedure SetIndustry(index: integer; value: single);
    function GetProducing(index: integer): integer;
    procedure SetProducing(index: integer; value: integer);
    function GetCapital(index: integer): single;
    procedure SetCapital(index: integer; value: single);
    function GetMaterial(index: integer): single;
    procedure SetMaterial(index: integer; value: single);
    function GetColonist(index: integer): single;
    procedure SetColonist(index: integer; value: single);

    function GetPlayer(index: integer): TPlayer;
    procedure SetPlayer(index: integer; value: TPlayer);
    function GetLastOwnerName: string;
    function GetLastOwnerType: integer;
    function GetLastOwnerObject: TPlayer;
    function GetLastOwnerTurn: integer;
    procedure SetNr(nr: integer);
    procedure SetLastOwnerName(const Value: string);
    procedure SetLastOwnerType(const Value: integer);
    procedure Setsize(const Value: single);
    function GetEffIndustry(index: integer): single;
    function GetEffMassProduce(index: integer): single;
    procedure SetEnemyShipMass(const Value: single);
    procedure SetFriendShipMass(const Value: single);
    procedure SetMyShipMass(const Value: single);
    procedure SetSendedShipMass(const Value: single);
    function GetProduceAmount: Single;
    function GetProduceAmountFull: Single;
    function CalculateProduceAmount(Effind: Single): Single;
    function GetProcucingType: integer;
    procedure SetSendedShipMassEta2(const Value: single);
    procedure SetSendedShipMassEta2Plus(const Value: single);
  public
    property Name: string read FName write FName;
    property Nr: integer read Fnr write SetNr;
    property X: integer read Fx write Fx;
    property Y: integer read Fy write Fy;
    property Size: single read Fsize write Setsize;
    property PopulationCur: single index - 1 read GetPopulation write SetPopulation;
    property Population[index: integer]: single read GetPopulation write SetPopulation;
    property IndustryCur: single index - 1 read GetIndustry write SetIndustry;
    property Industry[index: integer]: single read GetIndustry write SetIndustry;
    property EffIndustryCur: single index - 1 read GetEffIndustry;
    property EffIndustry[index: integer]: single read GetEffIndustry;
    property Resources: single read FResources write Fresources;
    property ProduceType: string read FProduceType write FProduceType;
    property OrgProduceType: string read FOrgProduceType write FOrgProduceType;
    property ProduceAmount: Single read GetProduceAmount;
    property ProduceAmountFull: Single read GetProduceAmountFull;
    property ProcucingType: integer read GetProcucingType;
    property ProducingCur: integer index - 1 read GetProducing write SetProducing;
    property Producing[index: integer]: integer read GetProducing write SetProducing;
    property EffMassProduceCur: single index - 1 read GetEffMassProduce;
    property EffMassProduce[index: integer]: single read GetEffMassProduce;
    property CapitalCur: single index - 1 read GetCapital write SetCapital;
    property Capital[index: integer]: single read GetCapital write SetCapital;
    property MaterialCur: single index - 1 read GetMaterial write SetMaterial;
    property Material[index: integer]: single read GetMaterial write SetMaterial;
    property ColonistCur: single index - 1 read GetColonist write SetColonist;
    property Colonist[index: integer]: single read GetColonist write SetColonist;

    property PlayerCur: TPlayer index - 1 read GetPlayer write SetPlayer;
    property Player[index: integer]: TPlayer read GetPlayer write SetPlayer;

    property Distance: integer read Fdistance;
    procedure SetDistanceFrom(x1, y1: integer);

    procedure SetLastOwner;
    property LastOwnerName: string read GetLastOwnerName write SetLastOwnerName;
    property LastOwnerType: integer read GetLastOwnerType write SetLastOwnerType;
    property LastOwnerObject: TPlayer read GetLastOwnerObject write FLastOwnerObject;
    property LastOwnerTurn: integer read GetLastOwnerTurn write FLastOwnerTurn;

    property MyShipMass: single read FMyShipMass write SetMyShipMass;
    property FriendShipMass: single read FFriendShipMass write SetFriendShipMass;
    property EnemyShipMass: single read FEnemyShipMass write SetEnemyShipMass;
    property SendedShipMass: single read FSendedShipMass write SetSendedShipMass;
    property SendedShipMassETA2: single read FSendedShipMassEta2 write SetSendedShipMassEta2;
    property SendedShipMassETA2Plus: single read FSendedShipMassEta2Plus write SetSendedShipMassEta2Plus;

    property Buildability: Single read FBuildability write FBuildability;

    procedure SetPlanet(const Turn: integer;
      const NameX, ProduceType: string;
      const xX, yX: integer; sizeX, populationX, industryX, resourcesX: single;
      const producingX: integer;
      const capitalX, materialX, colonistX: single);
  end;

  TBombing = class
  private
    FProducing: integer;
    FMat: single;
    FIndustry: single;
    FCap: single;
    FPopulation: single;
    FCol: single;
    FPlanet: TPlanet;
    FAttacker: TPlayer;
    FDefender: Tplayer;
    FTurn: integer;
    procedure SetAttacker(const Value: TPlayer);
    procedure SetCap(const Value: single);
    procedure SetCol(const Value: single);
    procedure SetDefender(const Value: Tplayer);
    procedure SetIndustry(const Value: single);
    procedure SetMat(const Value: single);
    procedure SetPlanet(const Value: TPlanet);
    procedure SetPopulation(const Value: single);
    procedure SetProducing(const Value: integer);
    procedure SetTurn(const Value: integer);
  public
    property Turn: integer read FTurn write SetTurn;
    property Attacker: TPlayer read FAttacker write SetAttacker;
    property Defender: Tplayer read FDefender write SetDefender;
    property Planet: TPlanet read FPlanet write SetPlanet;
    property Population: single read FPopulation write SetPopulation;
    property Industry: single read FIndustry write SetIndustry;
    property Producing: integer read FProducing write SetProducing;
    property Cap: single read FCap write SetCap;
    property Mat: single read FMat write SetMat;
    property Col: single read FCol write SetCol;
  end;

  TShipInProduction = class
  private
    FPlanetNr: integer;
    FPlanetName: string;
    FShipTypeName: string;
    FNumberOfShipsPresent: Integer;
    FNumberOfShipsPresentLastTech: Integer;
  public
    property PlanetName: string read FPlanetName write FPlanetName;
    property PlanetNr: integer read FPlanetNr write FPlanetNr;
    property ShipTypeName: string read FShipTypeName write FShipTypeName;
    property NumberOfShipsPresent: Integer read FNumberOfShipsPresent write FNumberOfShipsPresent;
    property NumberOfShipsPresentLastTech: Integer read FNumberOfShipsPresentLastTech write FNumberOfShipsPresentLastTech;
  end;

procedure AppendShipTypesToList(player: TPlayer; list: TStrings);

function NewBattle(battle: TBattle): TBattle;
function NewGroup(group: TGroup): TGroup;
function NewShipType(st: TShipType): TShipType;
function FindPlanetNr(nr: integer): TPlanet;
function FindPlanetWithName(name: string): Tplanet;
function FindPlayerNr(nr: integer): TPlayer;
function FindPlayerWithName(name: string): TPlayer;
function FindShipType(player: TPlayer; name: string): TShipType;

var
  players: TFStringList;
  reportPlayer: TPlayer;
  qreportPlayer: TPlayer;
  grafPlayer: TPlayer;
  ExtraPlanetInfoList: TExtraPlanetInfoList;

  planets: TFStringList;
  planetsDist: TDStringList;
  pl, planet1, planet2: TPlanet;

  battles: TFStringList;
  IncomingGroups: Tlist;
  Bombings: TFStringList;
  ShipsInProduction: TList;
  TracksList: TList;
  Routes: TFStringList;
  NullPlanet: TPlanet;

implementation

uses UFiles, Inifiles, SysUtils, Dialogs, Udevelop;

function NewGroup(group: TGroup): TGroup;
begin
  Result := TGroup.Create;
  Result.Fplayer := group.Fplayer;
  Result.Ftype := NewShipType(group.Ftype);
  Result.Fplanet := group.Fplanet;
  Result.FdriveTech := group.FdriveTech;
  Result.FweaponsTech := group.FweaponsTech;
  Result.FshieldsTech := group.FshieldsTech;
  Result.FcargoTech := group.FcargoTech;
  Result.Faantal := group.Faantal;
  Result.FladingType := group.FladingType;
  Result.FaantalBattle := group.FaantalBattle;
  Result.Fnr := group.Fnr;
  Result.Fturn := group.Fturn;
  Result.Frange := group.Frange;
  Result.Ffleetname := group.Ffleetname;
end;

function NewShipType(st: TShipType): TShipType;
begin
  Result := TShipType.Create;
  Result.Fname := st.Fname;
  Result.Fdrive := st.Fdrive;
  Result.Fattacks := st.Fattacks;
  Result.Fweapons := st.Fweapons;
  Result.Fshields := st.Fshields;
  Result.Fcargo := st.Fcargo;
end;

function NewBattle(battle: TBattle): TBattle;
var
  i: integer;
begin
  Result := TBattle.Create;
  Result.Fturn := battle.Fturn;
  Result.Fplanet := battle.Fplanet;
  Result.Fgroups := TList.Create;
  for i := 0 to battle.Fgroups.Count - 1 do
    Result.Fgroups.Add(NewGroup(TGroup(battle.Fgroups.items[i])));
  Result.FgroupA := TList.Create;
  for i := 0 to battle.FgroupA.Count - 1 do
    Result.FgroupA.Add(NewGroup(TGroup(battle.FgroupA.items[i])));
  Result.FgroupD := TList.Create;
  for i := 0 to battle.FgroupD.Count - 1 do
    Result.FgroupD.Add(NewGroup(TGroup(battle.FgroupD.items[i])));
end;

function TShipType.GetIsStationary: boolean;
begin
  Result := FCargoTypes = ctStationary;
end;

function TShipType.GetMass: single;
begin
  if (inivars.Gametype = gtGalaxyNGBlind) and (Fdrive < 0) then begin
    Result := FNGBlindMass;
  end else begin
    if Fattacks <= 1 then
      Result := Fdrive + Fweapons + Fshields + FCargo
    else
      Result := Fdrive + (1 + (FAttacks - 1) / 2) * Fweapons + Fshields + FCargo;
  end;
end;

// TGroup

constructor TGroup.Create;
begin
  inherited;
  FdriveTech := 1;
  FweaponsTech := 1;
  FshieldsTech := 1;
  FcargoTech := 1;
  FfleetName := '';
  Fplanet := nil;
  Fplayer := nil;
  Ftype := nil;

  FLocPlanet := nil;
  Faantal := 0;
  Flading := 0;
  FladingType := 0;
  FaantalBattle := 0;
  Fnr := -1;
  Frange := 0;
  Fturn := -1;
  Fbat := 0;
  FFleetSpeed := 0;
  FETA := 0;
  FSourcePlanet := nil;
  FDestPlanet := nil;
end;

destructor TGroup.Destroy;
begin
  inherited;
end;

function TGroup.GetEffCargo: single;
begin
  if CargoTech = -1 then begin
    Result := -1;
    exit;
  end;
  Result := CargoTech * (_Type.Cargo + _Type.Cargo * _Type.Cargo / 10);
end;

function TGroup.GetEffWeapon: single;
begin
  if WeaponsTech = -1 then begin
    Result := -1;
    exit;
  end;
  Result := WeaponsTech * _Type.Weapons;
end;

function TGroup.GetEffShield: single;
begin
  Result := 0;
  if ShieldsTech = 0 then exit;
  if ShieldsTech = -1 then begin
    Result := -1;
    exit;
  end;
  try
    Result := (ShieldsTech * _Type.Shields) /
      (power(_Type.Mass, 0.33333333)) *
      3.107232506;
  except

  end;
end;

function TGroup.GetEffShieldWithLoad(load: single): single;
begin
  if ShieldsTech = -1 then begin
    Result := -1;
    exit;
  end;
  try
    Result := (ShieldsTech * _Type.Shields) /
      (power((_Type.Mass + load), 0.33333333)) *
      3.107232506;
  except
    Result := 0;
  end;
end;

function TGroup.GetEffShieldLoaded: single;
begin
  if ShieldsTech = -1 then begin
    Result := -1;
    exit;
  end;
  try
    Result := (ShieldsTech * _Type.Shields) /
      (power((_Type.Mass + EffCargo / CargoTech), 0.33333333)) *
      3.107232506;
  except
    Result := 0;
  end;
end;

function TGroup.GetEffSpeed: single;
begin
  if DriveTech = -1 then begin
    Result := -1;
    exit;
  end;
  if FFleetname <> '' then Result := FFleetSpeed
  else
  try
    Result := 20 * (_Type.Drive * DriveTech) / _type.Mass;
  except
    Result := 0;
  end;
end;

function TGroup.GetEffSpeedLoaded: single;
begin
  if DriveTech = -1 then begin
    Result := -1;
    exit;
  end;
  if FFleetname <> '' then Result := FFleetSpeed
  else
  try
    if EffCargo > 0 then
      Result := 20 * (_Type.Drive * DriveTech) / (_Type.Mass + EffCargo / CargoTech)
    else
      Result := 20 * (_Type.Drive * DriveTech) / (_Type.Mass);
  except
    Result := 0;
  end;
end;

function TGroup.GetEffSpeedNow: single;
begin
  if DriveTech = -1 then begin
    Result := -1;
    exit;
  end;
  if FFleetname <> '' then Result := FFleetSpeed
  else
  try
    if Lading > 0 then
      Result := 20 * (_Type.Drive * DriveTech) / (_Type.Mass + Lading / CargoTech)
    else
      Result := 20 * (_Type.Drive * DriveTech) / (_Type.Mass);
  except
    Result := 0;
  end;
end;

procedure TGroup.SetFleetSpeed(const Value: single);
begin
  FFleetSpeed := Value;
end;

function TGroup.GetUpgradeCost: single;
var
  WeaponsMass: single;
begin
  Result := 0;
  if Player.DriveCur = 0 then exit;
  if Player.WeaponsCur = 0 then exit;
  if Player.ShieldCur = 0 then exit;
  if Player.CargoCur = 0 then exit;

  if Ftype.attacks <= 1 then
    WeaponsMass := Ftype.weapons
  else
    WeaponsMass := (1 + (Ftype.attacks - 1) / 2) * Ftype.weapons;

  Result := 10 * ((1 - FdriveTech / Player.DriveCur) * Ftype.Drive +
    (1 - FweaponsTech / Player.WeaponsCur) * WeaponsMass +
    (1 - FshieldsTech / Player.ShieldCur) * Ftype.Shields +
    (1 - FcargoTech / Player.CargoCur) * Ftype.Cargo);
end;

// TBattle

constructor TBattle.Create;
begin
  FGroups := Tlist.Create;
  FGroupA := nil;
  FGroupD := nil;
end;

destructor TBattle.Destroy;
begin
  while FGroups.Count > 0 do
    FGroups.Delete(0);
  if Assigned(FGroupA) then
  begin
    while FGroupA.Count > 0 do
    begin
      TGroup(FGroupA.Items[0]).Free;
      FGroupA.Delete(0);
    end;
    FGroupA.Free;
  end;
  if Assigned(FGroupD) then
  begin
    while FGroupD.Count > 0 do
    begin
      TGroup(FGroupD.Items[0]).Free;
      FGroupD.Delete(0);
    end;
    FGroupD.Free;
  end;
  FGroups.Free;
end;

function TBattle.GetGroupA: Tlist;
begin
  if not Assigned(FGroupA) then FGroupA := TList.Create;
  Result := FGroupA;
end;

function TBattle.GetGroupD: Tlist;
begin
  if not Assigned(FGroupD) then FGroupD := TList.Create;
  Result := FGroupD;
end;

procedure TBattle.Simulate;
begin

end;

//TPlayer

constructor TPlayer.Create;
begin
  inherited;
  Fst := TStringList.Create; // list
  Fst.Sorted := True;
  Fst.Duplicates := dupIgnore;
  Fgr := TList.Create; // list
  FNr := -1;
  fWar := True;
end;

destructor TPlayer.Destroy;
var
  i: Integer;
begin
  for i := 0 to Fst.Count - 1 do
    TShipType(Fst.Objects[i]).Free;
  Fst.Free;
  for i := 0 to Fgr.Count - 1 do
    TGroup(Fgr[i]).Free;
  Fgr.Free;
  inherited;
end;

function TPlayer.GetDrive(index: integer): single;
begin
  if index = -1 then Result := Fdrive[0]
  else Result := Fdrive[index + 2];
end;

function TPlayer.GetDriveD(index: integer): single;
begin
  if index = 0 then Result := 0 // eerste ronde, geen delta
  else
    if index = -1 then
      Result := Fdrive[0] - GetDrive(Round(Fdrive[1]) - 1)
    else
      Result := Fdrive[index + 2] - Fdrive[index + 1];
end;

procedure TPlayer.SetDrive(index: integer; value: single);
begin
  if index >= 0 then
  begin
    Fdrive[index + 2] := value;
    Fdrive[0] := value;
    Fdrive[1] := index;
  end
  else
    Fdrive[0] := value;
end;

function TPlayer.GetWeapons(index: integer): single;
begin
  if index = -1 then Result := Fweapons[0]
  else Result := Fweapons[index + 2];
end;

function TPlayer.GetWeaponsD(index: integer): single;
begin
  if index = 0 then Result := 0 // eerste ronde, geen delta
  else
    if index = -1 then Result := Fweapons[0] - GetWeapons(Round(Fweapons[1]) - 1)
    else Result := Fweapons[index + 2] - Fweapons[index + 1];
end;

procedure TPlayer.SetWeapons(index: integer; value: single);
begin
  if index >= 0 then
  begin
    Fweapons[index + 2] := value;
    Fweapons[0] := value;
    Fweapons[1] := index;
  end
  else
    Fweapons[0] := value;
end;

function TPlayer.GetShield(index: integer): single;
begin
  if index = -1 then Result := Fshield[0]
  else Result := Fshield[index + 2];
end;

function TPlayer.GetShieldD(index: integer): single;
begin
  if index = 0 then Result := 0 // eerste ronde, geen delta
  else
    if index = -1 then Result := Fshield[0] - GetShield(Round(Fshield[1]) - 1)
    else Result := Fshield[index + 2] - Fshield[index + 1];
end;

procedure TPlayer.SetShield(index: integer; value: single);
begin
  if index >= 0 then
  begin
    Fshield[index + 2] := value;
    Fshield[0] := value;
    Fshield[1] := index;
  end
  else
    Fshield[0] := value;
end;

function TPlayer.GetCargo(index: integer): single;
begin
  if index = -1 then Result := Fcargo[0]
  else Result := Fcargo[index + 2];
end;

function TPlayer.GetCargoD(index: integer): single;
begin
  if index = 0 then Result := 0 // eerste ronde, geen delta
  else
    if index = -1 then Result := Fcargo[0] - GetCargo(Round(Fcargo[1]) - 1)
    else Result := Fcargo[index + 2] - Fcargo[index + 1];
end;

procedure TPlayer.SetCargo(index: integer; value: single);
begin
  if index >= 0 then
  begin
    Fcargo[index + 2] := value;
    Fcargo[0] := value;
    Fcargo[1] := index;
  end
  else
    Fcargo[0] := value;
end;

function TPlayer.GetPopulation(index: integer): single;
begin
  if index = -1 then Result := Fpopulation[0]
  else Result := Fpopulation[index + 2];
end;

function TPlayer.GetPopulationD(index: integer): single;
begin
  if index = 0 then Result := 0 // eerste ronde, geen delta
  else
    if index = -1 then Result := Fpopulation[0] - GetPopulation(Round(Fpopulation[1]) - 1)
    else Result := Fpopulation[index + 2] - Fpopulation[index + 1];
end;

procedure TPlayer.SetPopulation(index: integer; value: single);
begin
  if index >= 0 then
  begin
    Fpopulation[index + 2] := value;
    Fpopulation[0] := value;
    Fpopulation[1] := index;
  end
  else
    Fpopulation[0] := value;
end;

function TPlayer.GetMaxPopulation(index: integer): single;
begin
  if index = -1 then Result := Fmaxpopulation[0]
  else Result := Fmaxpopulation[index + 2];
end;

function TPlayer.GetMaxPopulationD(index: integer): single;
begin
  if index = 0 then Result := 0 // eerste ronde, geen delta
  else
    if index = -1 then Result := Fmaxpopulation[0] - GetmaxPopulation(Round(Fmaxpopulation[1]) - 1)
    else Result := Fmaxpopulation[index + 2] - Fmaxpopulation[index + 1] //Fdrive[index + 2];
end;

procedure TPlayer.SetMaxPopulation(index: integer; value: single);
begin
  if index >= 0 then
  begin
    FmaxPopulation[index + 2] := value;
    FmaxPopulation[0] := value;
    FmaxPopulation[1] := index;
  end
  else
    FmaxPopulation[0] := value;
end;

function TPlayer.GetEffIndustry(index: integer): single;
var
  tmpInd, tmpPop: single;
begin
  if index = -1 then
  begin
    tmpInd := Findustry[0];
    tmpPop := Fpopulation[0];
  end
  else
  begin
    tmpInd := Findustry[index + 2];
    tmpPop := Fpopulation[index + 2];
  end;
  Result := tmpInd;
  if tmpPop > tmpInd then
    Result := Result + (tmpPop - tmpInd) / 4.0;
end;

function TPlayer.GetEffIndustryD(index: integer): single;
begin
  if index = 0 then Result := 0 // eerste ronde, geen delta
  else
    if index = -1 then Result := GetEffIndustry(-1) - GetEffIndustry(Round(Findustry[1]) - 1)
    else Result := GetEffIndustry(index) - GetEffIndustry(index - 1);
end;

function TPlayer.GetIndustry(index: integer): single;
begin
  if index = -1 then Result := Findustry[0]
  else Result := Findustry[index + 2];
end;

function TPlayer.GetIndustryD(index: integer): single;
begin
  if index = 0 then Result := 0 // eerste ronde, geen delta
  else
    if index = -1 then Result := Findustry[0] - GetIndustry(Round(Findustry[1]) - 1)
    else Result := Findustry[index + 2] - Findustry[index + 1];
end;

procedure TPlayer.SetIndustry(index: integer; value: single);
begin
  if index >= 0 then
  begin
    Findustry[index + 2] := value;
    Findustry[0] := value;
    Findustry[1] := index;
  end
  else
    Findustry[0] := value;
end;

function TPlayer.GetPlanets(index: integer): integer;
begin
  if index = -1 then Result := Fplanets[0]
  else Result := Fplanets[index + 2];
end;

function TPlayer.GetPlanetsD(index: integer): integer;
begin
  if index = 0 then Result := 0 // eerste ronde, geen delta
  else
    if index = -1 then Result := Fplanets[0] - GetPlanets(Round(Fplanets[1]) - 1)
    else Result := Fplanets[index + 2] - Fplanets[index + 1];
end;

procedure TPlayer.SetPlanets(index: integer; value: integer);
begin
  if index >= 0 then
  begin
    Fplanets[index + 2] := value;
    Fplanets[0] := value;
    Fplanets[1] := index;
  end
  else
    Fplanets[0] := value;
end;

procedure TPlayer.SetPlayer(Turn: integer;
  NameX: string;
  driveX, weaponsX, shieldX, cargoX: single;
  populationX, industryX: single;
  planetsX: integer;
  warX: boolean);
begin
  Fname := NameX;
  SetDrive(Turn, driveX);
  SetWeapons(Turn, weaponsX);
  SetShield(Turn, shieldX);
  SetCargo(Turn, cargoX);
  SetPopulation(Turn, populationX);
  SetIndustry(Turn, industryX);
  SetPlanets(Turn, planetsX);
  FWar := warX;
end;

procedure TPlayer.SetNr(const Value: integer);
begin
  FNr := Value;
end;

procedure TPlayer.SetName(const Value: string);
begin
  if CompareText(Fname, Value) <> 0 then
  begin
    if OldNames <> '' then
      OldNames := OldNames + ' ' + FName
    else
      OldNames := FName;
    Fname := Value;
  end;
end;

procedure TPlayer.SetOldNames(const Value: string);
begin
  FOldNames := Value;
end;

function TPlayer.GetPopulationPercent(index: integer): single;
begin
  if index = 0 then Result := 0 // eerste ronde
  else
    if index = -1 then begin
      if GetPopulation(Round(Fpopulation[1]) -1) > 0 then
        Result := (Fpopulation[0] - GetPopulation(Round(Fpopulation[1]) - 1)) / GetPopulation(Round(Fpopulation[1]) - 1)
      else
        Result := 0;
    end else begin
      if Fpopulation[index + 1] > 0 then
        Result := (Fpopulation[index + 2] - Fpopulation[index + 1]) / Fpopulation[index + 1]
      else
        Result := 0;
    end;
  Result := Result * 100;
end;

// TPlanet

function TPlanet.GetPopulation(index: integer): single;
begin
  if index = -1 then Result := FPopulation[0]
  else Result := FPopulation[index + 2];
end;

procedure TPlanet.SetPopulation(index: integer; value: single);
begin
  if index >= 0 then
  begin
    FPopulation[index + 2] := value;
    FPopulation[0] := value;
    FPopulation[1] := index;
  end
  else
    FPopulation[0] := value;
end;

function TPlanet.GetIndustry(index: integer): single;
begin
  if index = -1 then Result := Findustry[0]
  else Result := Findustry[index + 2];
end;

procedure TPlanet.SetIndustry(index: integer; value: single);
begin
  if index >= 0 then
  begin
    Findustry[index + 2] := value;
    Findustry[0] := value;
    Findustry[1] := index;
  end
  else
    Findustry[0] := value;
end;

function TPlanet.GetProducing(index: integer): integer;
begin
  if index = -1 then Result := Fproducing[0]
  else Result := Fproducing[index + 2];
end;

procedure TPlanet.SetProducing(index: integer; value: integer);
begin
  if index >= 0 then
  begin
    Fproducing[index + 2] := value;
    Fproducing[0] := value;
    Fproducing[1] := index;
  end
  else
    Fproducing[0] := value;
end;

function TPlanet.GetCapital(index: integer): single;
begin
  if index = -1 then Result := Fcapital[0]
  else Result := Fcapital[index + 2];
end;

procedure TPlanet.SetCapital(index: integer; value: single);
begin
  if index >= 0 then
  begin
    Fcapital[index + 2] := value;
    Fcapital[0] := value;
    Fcapital[1] := index;
  end
  else
    Fcapital[0] := value;
end;

function TPlanet.GetMaterial(index: integer): single;
begin
  if index = -1 then Result := Fmaterial[0]
  else Result := Fmaterial[index + 2];
end;

procedure TPlanet.SetMaterial(index: integer; value: single);
begin
  if index >= 0 then
  begin
    Fmaterial[index + 2] := value;
    Fmaterial[0] := value;
    Fmaterial[1] := index;
  end
  else
    Fmaterial[0] := value;
end;

function TPlanet.GetColonist(index: integer): single;
begin
  if index = -1 then Result := Fcolonist[0]
  else Result := Fcolonist[index + 2];
end;

procedure TPlanet.SetColonist(index: integer; value: single);
begin
  if index >= 0 then
  begin
    Fcolonist[index + 2] := value;
    Fcolonist[0] := value;
    Fcolonist[1] := index;
  end
  else
    Fcolonist[0] := value;
end;

function TPlanet.GetPlayer(index: integer): TPlayer;
begin
  if index = -1 then Result := Fplayer[0]
  else Result := Fplayer[index + 1];
end;

procedure TPlanet.SetPlayer(index: integer; value: TPlayer);
begin
  if index >= 0 then
  begin
    Fplayer[index + 1] := value;
    Fplayer[0] := value;
    Fplayerlast := index;
  end
  else
    Fplayer[0] := value;
end;

procedure TPlanet.setLastOwner;
var
  t: integer;
begin
  if not Assigned(Player[LastTurn]) then
  begin
    FLastOwnerName := '-';
    FLastOwnerType := 1;
    FLastOwnerObject := nil;
    FLastOwnerTurn := LastTurn;
  end
  else
    if Player[LastTurn] = TPlayer(1) then
    begin
      if FLastOwnerType = 0 then begin
        FLastOwnerType := 2;
        FLastOwnerName := '';
        FLastOwnerObject := nil;
        t := LastTurn - 1;
        repeat
          if Player[t] = nil then
            FLastOwnerType := 3
          else
            if Player[t] <> TPlayer(1) then
            begin
              FLastOwnerName := Player[t].Name;
              FLastOwnerObject := Player[t];
            end;
          dec(t);
        until (t < 0) or (FLastOwnerName <> '');
        if (t = -1) and (FLastOwnerName = '') then
        begin
          FLastOwnerTurn := -1;
          FLastOwnerType := 2;
        end
        else
          FLastOwnerTurn := t + 1;
      end;
    end
    else
    begin
      FLastOwnerName := Player[LastTurn].Name;
      FLastOwnerType := 1;
      FLastOwnerObject := Player[LastTurn];
      FLastOwnerTurn := LastTurn;
    end;
end;

function TPlanet.GetLastOwnerName: string;
begin
  Result := FLastOwnerName;
end;

function TPlanet.GetLastOwnerType: integer;
begin
  Result := FLastOwnerType;
end;

function TPlanet.GetLastOwnerObject: TPlayer;
begin
  Result := FLastOwnerObject;
end;

function TPlanet.GetLastOwnerTurn: integer;
begin
  Result := FLastOwnerTurn;
end;

procedure TPlanet.SetNr(nr: integer);
begin
  FNr := nr;
end;

procedure TPlanet.SetDistanceFrom(x1, y1: integer);
var
  extX, extY, extX1, extY1: single;
begin
  if inivars.bigGame then begin
    extX := X;
    extY := Y;
    extX1 := X1;
    extY1 := Y1;
    Fdistance := round(sqrt((extX - extX1) * (extX - extX1) + (extY - extY1) * (extY - extY1)));
  end else begin
    try
      Fdistance := round(sqrt((x - x1) * (x - x1) + (y - y1) * (y - y1)));
    except
      extX := X;
      extY := Y;
      extX1 := X1;
      extY1 := Y1;
      Fdistance := round(sqrt((extX - extX1) * (extX - extX1) + (extY - extY1) * (extY - extY1)));
    end;
  end;
end;

procedure TPlanet.SetPlanet(const Turn: integer;
  const NameX, ProduceType: string;
  const xX, yX: integer; sizeX, populationX, industryX, resourcesX: single;
  const producingX: integer;
  const capitalX, materialX, colonistX: single);
begin
  FName := NameX;
  if (GameNr = 1) or (ProduceType <> '') then begin
    FProduceType := ProduceType;
    FOrgProduceType := ProduceType;
  end;
  Fx := xX;
  Fy := yX;
  Fsize := sizeX;
  Fresources := resourcesX;
  SetPopulation(turn, populationX);
  SetIndustry(turn, industryX);
  SetProducing(turn, producingX);
  SetCapital(turn, capitalX);
  SetMaterial(turn, materialX);
  SetColonist(turn, colonistX);
end;

procedure Tplanet.SetLastOwnerName(const Value: string);
begin
  FLastOwnerName := Value;
end;

procedure Tplanet.SetLastOwnerType(const Value: integer);
begin
  FLastOwnerType := Value;
end;

procedure Tplanet.Setsize(const Value: single);
begin
  Fsize := Value;
end;

function Tplanet.GetEffIndustry(index: integer): single;
begin
  Result := Industry[index];
  if Population[index] > Industry[index] then
    Result := Result + (Population[index] - Industry[index]) / 4;
end;

function Tplanet.GetEffMassProduce(index: integer): single;
var
  devPlanet: Tdevplanet;
begin
  Result := 0;
  devPlanet := FindDevPlanet(Self);
  if Assigned(devPlanet) then begin
    if (Resources > 0) then begin
      Result := (devPlanet.devAvailableIndustry[0] + Material[index] / Resources) / (10 + (1 / Resources));
      if Result > (devPlanet.devAvailableIndustry[0] / 10) then Result := devPlanet.devAvailableIndustry[0] / 10;
    end;
  end else begin
    if (Resources > 0) then begin
      Result := (EffIndustry[index] + Material[index] / Resources) / (10 + (1 / Resources));
      if Result > (EffIndustry[index] / 10) then Result := EffIndustry[index] / 10;
    end;
  end;
end;

procedure Tplanet.SetEnemyShipMass(const Value: single);
begin
  FEnemyShipMass := Value;
end;

procedure Tplanet.SetFriendShipMass(const Value: single);
begin
  FFriendShipMass := Value;
end;

procedure Tplanet.SetMyShipMass(const Value: single);
begin
  FMyShipMass := Value;
end;

procedure Tplanet.SetSendedShipMass(const Value: single);
begin
  FSendedShipMass := Value;
end;

function Tplanet.GetProduceAmount: Single;
var
  devPlanet: Tdevplanet;
begin
  devPlanet := FindDevPlanet(Self);
  if Assigned(devPlanet) then 
    Result := CalculateProduceAmount(devPlanet.devAvailableIndustry[0])
  else
    Result := CalculateProduceAmount(EffindustryCur);
end;

function Tplanet.GetProcucingType: integer;
begin
  if CompareText(ProduceType, 'drive') = 0 then Result := DRIVE
  else
    if CompareText(ProduceType, 'weapons') = 0 then Result := WEAPONS
    else
      if CompareText(ProduceType, 'SHIELDS') = 0 then Result := SHIELDS
      else
        if CompareText(ProduceType, 'CARGO') = 0 then Result := CARGO
        else
          if CompareText(ProduceType, 'MAT') = 0 then Result := P_MAT
          else
            if CompareText(ProduceType, 'CAP') = 0 then Result := P_CAP
            else
              Result := SHIP;
end;

function Tplanet.CalculateProduceAmount(Effind: Single): Single;
var
  shiptype: TShipType;
  effMassProduce: Single;
begin
  Result := 0;
  if not Assigned(playerCur) or (integer(playerCur) = 1) then Exit;
  case GetProcucingType of
    DRIVE,
      WEAPONS,
      SHIELDS: Result := Effind / 5000;
    CARGO: Result := Effind / 2500;
    P_MAT: Result := Effind * FResources;
    P_CAP: //Result := Effind / (5 + 1 / FResources);
      begin
        Result := (Effind + Material[lastturn] / Resources) / (5 + (1 / Resources));
        if Result > (Effind / 5) then Result := Effind / 5;
      end;
    SHIP: begin
        shiptype := FindShipType(playerCur, ProduceType);
        if Assigned(shiptype) and (shiptype.Mass > 0) then begin
          effMassProduce := 0;
          if (Resources > 0) then begin
            effMassProduce := (Effind + Material[lastturn] / Resources) / (10 + (1 / Resources));
            if effMassProduce > (Effind / 10) then effMassProduce := Effind / 10;
          end;
          Result := effMassProduce / shiptype.Mass;
        end;
      end;
  end;
end;

function Tplanet.GetProduceAmountFull: Single;
begin
  Result := CalculateProduceAmount(Size);
end;

procedure Tplanet.SetSendedShipMassEta2(const Value: single);
begin
  FSendedShipMassEta2 := Value;
end;

procedure Tplanet.SetSendedShipMassEta2Plus(const Value: single);
begin
  FSendedShipMassEta2Plus := Value;
end;

{ TExtraPlanetInfo }

procedure TExtraPlanetInfo.SetPlanet(const Value: TPlanet);
begin
  FPlanet := Value;
end;

procedure TExtraPlanetInfo.SetPlanetNr(const Value: integer);
begin
  FPlanetNr := Value;
end;

procedure TExtraPlanetInfo.SetPlayer(const Value: TPlayer);
begin
  FPlayer := Value;
end;

procedure TExtraPlanetInfo.SetPlayerNr(const Value: integer);
begin
  FPlayerNr := Value;
end;

{ TExtraPlanetInfoList }

function TExtraPlanetInfoList.AddPlanet(planet: TPlanet; player: TPlayer): integer;
begin
  for Result := 0 to high(Fplanets) do
    if (Fplanets[Result].planet = planet) then
    begin
      Fplanets[Result].player := player;
      Fplanets[Result].playerNr := player.Nr;
      Exit;
    end;
  SetLength(Fplanets, Length(Fplanets) + 1);
  Result := high(Fplanets);
  Fplanets[Result] := TExtraPlanetInfo.Create;
  Fplanets[Result].planet := planet;
  Fplanets[Result].planetNr := planet.Nr;
  Fplanets[Result].player := player;
  Fplanets[Result].playerNr := player.Nr;
end;

procedure TExtraPlanetInfoList.DeletePlanet(planet: TPlanet);
var
  index, i: integer;
begin
  index := 0;
  while index < Length(Fplanets) do
  begin
    if (Fplanets[index].planet = planet) then
      break;
    inc(index);
  end;
  if index = Length(Fplanets) then Exit; // not found
  Fplanets[index].Free;
  for i := index + 1 to high(Fplanets) do
    Fplanets[i - 1] := Fplanets[i];
  SetLength(Fplanets, high(Fplanets));
end;

function TExtraPlanetInfoList.GetPlanets(index: integer): TExtraPlanetInfo;
begin
  Result := Fplanets[index]
end;

procedure TExtraPlanetInfoList.ReadInfo(strFileName: string);
var
  iniFile: TIniFile;
  i, count: integer;
begin
  iniFile := TIniFile.Create(strFileName);
  try
    for i := 0 to high(FPlanets) do
      FPlanets[i].Free;
    count := iniFile.ReadInteger('Totals_' + GetGameStr, 'Planets', 0);
    SetLength(FPlanets, count);
    for i := 0 to count - 1 do
    begin
      FPlanets[i] := TExtraPlanetInfo.Create;
      FPlanets[i].planetNr := iniFile.ReadInteger('Planets_' + GetGameStr,
        'planet_' + intToStr(i), 0);
      FPlanets[i].PlayerNr := iniFile.ReadInteger('Planets_' + GetGameStr,
        'Player_' + intToStr(i), 0);
    end;
    SolveExtraInfo;
  finally
    iniFile.Free;
  end;
end;

procedure TExtraPlanetInfoList.SetPlanets(index: integer;
  const Value: TExtraPlanetInfo);
begin
  Fplanets[index] := Value;
end;

procedure TExtraPlanetInfoList.SolveExtraInfo;
var
  i: integer;
  planet: TPlanet;
  player: TPlayer;
begin
  for i := 0 to high(Fplanets) do
  begin
    planet := FindPlanetNr(Fplanets[i].PlanetNr);
    if Assigned(planet) then
    begin
      Fplanets[i].planet := planet;
      player := FindPlayerNr(Fplanets[i].PlayerNr);
      if Assigned(player) and Assigned(planet.PlayerCur) then
      begin
        Fplanets[i].player := player;
        planet.PlayerCur := player;
      end
      else
        Fplanets[i].PlayerNr := 0;
    end;
  end;
end;

procedure TExtraPlanetInfoList.WriteInfo(strFileName: string);
var
  iniFile: TIniFile;
  i, Count: integer;
begin
  iniFile := TIniFile.Create(strFileName);
  try
    inifile.EraseSection('Planets_' + GetGameStr);
    Count := 0;
    for i := 0 to high(FPlanets) do
    begin
      if (FPlanets[i].PlanetNr > 0) and (FPlanets[i].PlayerNr > 0) then
      begin
        iniFile.WriteInteger('Planets_' + GetGameStr,
          'planet_' + intToStr(Count),
          FPlanets[Count].PlanetNr);
        iniFile.WriteInteger('Planets_' + GetGameStr,
          'player_' + intToStr(Count),
          FPlanets[Count].PlayerNr);
        inc(Count);
      end;
    end;
    iniFile.WriteInteger('Totals_' + GetGameStr, 'Planets', Count);
  finally
    iniFile.Free;
  end;
end;

procedure AppendShipTypesToList(player: TPlayer; list: TStrings);
var
  i: integer;
  strShipTypeName: string;
begin
  if Assigned(player) then begin
    for i := 0 to player.ShipTypes.Count - 1 do begin
      strShipTypeName := TShipType(player.ShipTypes.Objects[i]).Name;
      if list.IndexOf(strShipTypeName) = -1 then
        list.Add(strShipTypeName);
    end;
  end;
end;

function FindPlanetNr(nr: integer): TPlanet;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to planets.Count - 1 do
    if TPlanet(planets.Objects[i]).Nr = nr then
    begin
      Result := TPlanet(planets.Objects[i]);
      Exit;
    end;
end;

function FindPlayerNr(nr: integer): TPlayer;
var
  i: integer;
begin
  Result := nil;
  for i := 0 to players.Count - 1 do
    if TPlayer(players.Objects[i]).Nr = nr then
    begin
      Result := TPlayer(players.Objects[i]);
      Exit;
    end;
end;

function FindPlayerWithName(name: string): TPlayer;
var
  i: integer;
begin
  Result := nil;
  if players.Find(lowercase(Name), i) then
    Result := TPlayer(players.Objects[i]);
end;

function FindPlanetWithName(name: string): Tplanet;
var
  i: integer;
begin
  Result := nil;
  if planets.Find(lowercase(Name), i) then
    Result := TPlanet(planets.Objects[i]);
end;

function FindShipType(player: TPlayer; name: string): TShipType;
var
  i: integer;
begin
  Result := nil;
  if Assigned(player) then
    for i := 0 to player.ShipTypes.Count - 1 do
      if CompareText(TShipType(player.ShipTypes.Objects[i]).Name, Name) = 0 then
        Result := TShipType(player.ShipTypes.Objects[i]);
end;

{ TBombing }

procedure TBombing.SetAttacker(const Value: TPlayer);
begin
  FAttacker := Value;
end;

procedure TBombing.SetCap(const Value: single);
begin
  FCap := Value;
end;

procedure TBombing.SetCol(const Value: single);
begin
  FCol := Value;
end;

procedure TBombing.SetDefender(const Value: Tplayer);
begin
  FDefender := Value;
end;

procedure TBombing.SetIndustry(const Value: single);
begin
  FIndustry := Value;
end;

procedure TBombing.SetMat(const Value: single);
begin
  FMat := Value;
end;

procedure TBombing.SetPlanet(const Value: TPlanet);
begin
  FPlanet := Value;
end;

procedure TBombing.SetPopulation(const Value: single);
begin
  FPopulation := Value;
end;

procedure TBombing.SetProducing(const Value: integer);
begin
  FProducing := Value;
end;

procedure TBombing.SetTurn(const Value: integer);
begin
  FTurn := Value;
end;

{ TShipInProduction }

procedure CleanUpLists;
var
  i: integer;
begin
  for i := 0 to ShipsInProduction.Count - 1 do
    TShipInProduction(ShipsInProduction.Items[i]).Free;
  ShipsInProduction.Free;
  for i := 0 to IncomingGroups.Count - 1 do
    TIncomingGroup(IncomingGroups.Items[i]).Free;
  IncomingGroups.Free;
  for i := 0 to TracksList.Count - 1 do
    TTracks(TracksList.Items[i]).Free;
  TracksList.Free;
  for i := 0 to Routes.Count - 1 do
    TRoute(Routes.Objects[i]).Free;
  Routes.Free;
  for i := 0 to players.Count - 1 do
    TPlayer(players.Objects[i]).Free;
  players.Free;
  for i := 0 to planets.Count - 1 do
    TPlanet(planets.Objects[i]).Free;
  planets.Free;
  planetsDist.Free;
  for i := 0 to battles.Count - 1 do
    TBattle(battles.Objects[i]).Free;
  battles.Free;
  for i := 0 to bombings.Count - 1 do
    TBombing(bombings.Objects[i]).Free;
  Bombings.Free;
  ExtraPlanetInfoList.Free;
end;

{ TRoute }

constructor TRoute.Create(planet: TPlanet);
begin
  FPlanetFrom := planet;
  FMATTo := nil;
  FCOLTo := nil;
  FCAPTo := nil;
  FEMPTo := nil;
  FOrgMATTo := nil;
  FOrgCOLTo := nil;
  FOrgCAPTo := nil;
  FOrgEMPTo := nil;
end;

procedure TRoute.RestoreOrgValues;
begin
  FMATTo := FOrgMATTo;
  FCOLTo := FOrgCOLTo;
  FCAPTo := FOrgCAPTo;
  FEMPTo := FOrgEMPTo;
end;

initialization
  players := TFStringList.Create;
  players.Sorted := True;
  players.Duplicates := dupError;
  planets := TFStringList.Create;
  planets.Sorted := True;
  planets.Duplicates := dupError;
  planetsDist := TDStringList.Create;

  battles := TFStringList.Create;
  IncomingGroups := Tlist.Create;
  TracksList := TList.Create;
  Routes := TFStringList.Create;
  Routes.Sorted := True;
  Routes.Duplicates := dupError;
  Bombings := TFStringList.Create;
  Bombings.Sorted := True;
  Bombings.Duplicates := dupError;
  ExtraPlanetInfoList := TExtraPlanetInfoList.Create;
  ShipsInProduction := TList.Create;
  reportPlayer := nil;
  grafPlayer := nil;
  pl := nil;
  planet1 := nil;
  planet2 := nil;
  NullPlanet := TPlanet.Create;
  NullPlanet.Name := 'Unknown planet!';
finalization
  CleanUpLists;
  NullPlanet.Free;
end.

